
/*
 * GLCaml - Objective Caml interface for OpenGL 1.1, 1.2, 1.3, 1.4, 1.5, 2.0 and 2.1
 * and the following extensions: 
 * GL_ARB_imaging
 * GL_ARB_matrix_palette
 * GL_ARB_multitexture
 * GL_ARB_vertex_blend
 * GL_ATI_envmap_bumpmap
 * GL_ATI_map_object_buffer
 * GL_ATI_pn_triangles
 * GL_ATI_separate_stencil
 * GL_ATI_texture_compression_3dc
 * GL_ATI_vertex_streams
 * GL_ATIX_point_sprites
 * GL_ATIX_texture_env_combine3
 * GL_ATIX_texture_env_route
 * GL_ATIX_vertex_shader_output_point_size
 * GL_EXT_Cg_shader
 * GL_EXT_depth_bounds_test
 * GL_EXT_draw_range_elements
 * GL_EXT_fog_coord
 * GL_EXT_pixel_buffer_object
 * GL_EXT_secondary_color
 * GL_EXT_texture_cube_map
 * GL_EXT_texture_edge_clamp
 * GL_EXT_texture_rectangle
 * GL_EXT_vertex_shader
 * GL_KTX_buffer_region
 * GL_NV_fragment_program2
 * GL_NV_fragment_program_option
 * GL_NV_vertex_program2_option
 * GL_NV_vertex_program3
 *
 * Copyright (C) 2007 Elliott OTI
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 */
 
#include <caml/mlvalues.h>
#include <caml/memory.h>
#include <caml/alloc.h>
#include <caml/fail.h>
#include <caml/callback.h>
#include <caml/bigarray.h>

typedef unsigned int GLenum;
typedef unsigned char GLboolean;
typedef unsigned int GLbitfield;
typedef void GLvoid;
typedef char GLbyte;
typedef short GLshort;
typedef int GLint;
typedef unsigned char GLubyte;
typedef unsigned short GLushort;
typedef unsigned int GLuint;
typedef int GLsizei;
typedef float GLfloat;
typedef float GLclampf;
typedef double GLdouble;
typedef double GLclampd;
typedef char GLchar;
typedef ptrdiff_t GLsizeiptr;
typedef ptrdiff_t GLintptr;
typedef char* GLstring;

#ifdef _WIN32
#include <windows.h>

static HMODULE lib=NULL;

static void init_lib()
{
	if(lib)return;
	lib = LoadLibrary("opengl32.dll");
	if(lib == NULL) failwith("error loading opengl32.dll");
}

static void *get_proc_address(char *fname)
{
	return GetProcAddress(lib, fname);
}

#endif

#ifdef __linux__
#ifndef APIENTRY
#define APIENTRY
#endif
#include <dlfcn.h>
#include <stdio.h>

static void* lib=NULL;

static void init_lib()
{
	if(lib)return;
	lib = dlopen("libGL.so.1",RTLD_LAZY);
	if(lib == NULL) failwith("error loading libGL.so.1");
}

static void *get_proc_address(char *fname)
{
	return dlsym(lib, fname);
}

#endif

#if defined(__APPLE__) && defined(__GNUC__)
#ifndef APIENTRY
#define APIENTRY
#endif
#include <dlfcn.h>
#include <stdio.h>

static void* lib=NULL;

static void init_lib()
{
	if(lib)return;
	lib = dlopen("libGL.dylib",RTLD_LAZY);
	if(lib == NULL) failwith("error loading libGL.dylib");
}

static void *get_proc_address(char *fname)
{
	return dlsym(lib, fname);
}
#endif

static int fail_with_exception = 0;

void FailWith(char *msg)
{
	if(fail_with_exception)failwith(msg);
}

value glstub_set_failwith(value b)
{
	CAMLparam1(b);
	int fw = Bool_val(b);
	fail_with_exception = 0;
	if(fw)fail_with_exception = 1;
	CAMLreturn (Val_unit);
}

value glstub_init_lib (value u)
{
	CAMLparam1(u);
	init_lib ();
	CAMLreturn (Val_unit);	
}

static GLenum get_glenum(value v);

int get_glenum_list(value flag_list)
{
	CAMLparam1(flag_list);
	GLenum flag = 0;
	value l = flag_list;
	while (Is_block(l))
	{
 		 flag |= get_glenum(Field(l,0));
 		 l = Field(l,1);
	}
	CAMLreturn (flag);
}

value unsafe_coercion(value v)
{
        CAMLparam1(v);
        CAMLreturn(v);
}


/* OPenGL #defines */
#define GL_CONSTANT_COLOR 0x8001
#define GL_ONE_MINUS_CONSTANT_COLOR 0x8002
#define GL_CONSTANT_ALPHA 0x8003
#define GL_ONE_MINUS_CONSTANT_ALPHA 0x8004
#define GL_BLEND_COLOR 0x8005
#define GL_FUNC_ADD 0x8006
#define GL_MIN 0x8007
#define GL_MAX 0x8008
#define GL_BLEND_EQUATION 0x8009
#define GL_FUNC_SUBTRACT 0x800A
#define GL_FUNC_REVERSE_SUBTRACT 0x800B
#define GL_CONVOLUTION_1D 0x8010
#define GL_CONVOLUTION_2D 0x8011
#define GL_SEPARABLE_2D 0x8012
#define GL_CONVOLUTION_BORDER_MODE 0x8013
#define GL_CONVOLUTION_FILTER_SCALE 0x8014
#define GL_CONVOLUTION_FILTER_BIAS 0x8015
#define GL_REDUCE 0x8016
#define GL_CONVOLUTION_FORMAT 0x8017
#define GL_CONVOLUTION_WIDTH 0x8018
#define GL_CONVOLUTION_HEIGHT 0x8019
#define GL_MAX_CONVOLUTION_WIDTH 0x801A
#define GL_MAX_CONVOLUTION_HEIGHT 0x801B
#define GL_POST_CONVOLUTION_RED_SCALE 0x801C
#define GL_POST_CONVOLUTION_GREEN_SCALE 0x801D
#define GL_POST_CONVOLUTION_BLUE_SCALE 0x801E
#define GL_POST_CONVOLUTION_ALPHA_SCALE 0x801F
#define GL_POST_CONVOLUTION_RED_BIAS 0x8020
#define GL_POST_CONVOLUTION_GREEN_BIAS 0x8021
#define GL_POST_CONVOLUTION_BLUE_BIAS 0x8022
#define GL_POST_CONVOLUTION_ALPHA_BIAS 0x8023
#define GL_HISTOGRAM 0x8024
#define GL_PROXY_HISTOGRAM 0x8025
#define GL_HISTOGRAM_WIDTH 0x8026
#define GL_HISTOGRAM_FORMAT 0x8027
#define GL_HISTOGRAM_RED_SIZE 0x8028
#define GL_HISTOGRAM_GREEN_SIZE 0x8029
#define GL_HISTOGRAM_BLUE_SIZE 0x802A
#define GL_HISTOGRAM_ALPHA_SIZE 0x802B
#define GL_HISTOGRAM_LUMINANCE_SIZE 0x802C
#define GL_HISTOGRAM_SINK 0x802D
#define GL_MINMAX 0x802E
#define GL_MINMAX_FORMAT 0x802F
#define GL_MINMAX_SINK 0x8030
#define GL_TABLE_TOO_LARGE 0x8031
#define GL_COLOR_MATRIX 0x80B1
#define GL_COLOR_MATRIX_STACK_DEPTH 0x80B2
#define GL_MAX_COLOR_MATRIX_STACK_DEPTH 0x80B3
#define GL_POST_COLOR_MATRIX_RED_SCALE 0x80B4
#define GL_POST_COLOR_MATRIX_GREEN_SCALE 0x80B5
#define GL_POST_COLOR_MATRIX_BLUE_SCALE 0x80B6
#define GL_POST_COLOR_MATRIX_ALPHA_SCALE 0x80B7
#define GL_POST_COLOR_MATRIX_RED_BIAS 0x80B8
#define GL_POST_COLOR_MATRIX_GREEN_BIAS 0x80B9
#define GL_POST_COLOR_MATRIX_BLUE_BIAS 0x80BA
#define GL_POST_COLOR_MATRIX_ALPHA_BIAS 0x80BB
#define GL_COLOR_TABLE 0x80D0
#define GL_POST_CONVOLUTION_COLOR_TABLE 0x80D1
#define GL_POST_COLOR_MATRIX_COLOR_TABLE 0x80D2
#define GL_PROXY_COLOR_TABLE 0x80D3
#define GL_PROXY_POST_CONVOLUTION_COLOR_TABLE 0x80D4
#define GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE 0x80D5
#define GL_COLOR_TABLE_SCALE 0x80D6
#define GL_COLOR_TABLE_BIAS 0x80D7
#define GL_COLOR_TABLE_FORMAT 0x80D8
#define GL_COLOR_TABLE_WIDTH 0x80D9
#define GL_COLOR_TABLE_RED_SIZE 0x80DA
#define GL_COLOR_TABLE_GREEN_SIZE 0x80DB
#define GL_COLOR_TABLE_BLUE_SIZE 0x80DC
#define GL_COLOR_TABLE_ALPHA_SIZE 0x80DD
#define GL_COLOR_TABLE_LUMINANCE_SIZE 0x80DE
#define GL_COLOR_TABLE_INTENSITY_SIZE 0x80DF
#define GL_IGNORE_BORDER 0x8150
#define GL_CONSTANT_BORDER 0x8151
#define GL_WRAP_BORDER 0x8152
#define GL_REPLICATE_BORDER 0x8153
#define GL_CONVOLUTION_BORDER_COLOR 0x8154
#define GL_MATRIX_PALETTE_ARB 0x8840
#define GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB 0x8841
#define GL_MAX_PALETTE_MATRICES_ARB 0x8842
#define GL_CURRENT_PALETTE_MATRIX_ARB 0x8843
#define GL_MATRIX_INDEX_ARRAY_ARB 0x8844
#define GL_CURRENT_MATRIX_INDEX_ARB 0x8845
#define GL_MATRIX_INDEX_ARRAY_SIZE_ARB 0x8846
#define GL_MATRIX_INDEX_ARRAY_TYPE_ARB 0x8847
#define GL_MATRIX_INDEX_ARRAY_STRIDE_ARB 0x8848
#define GL_MATRIX_INDEX_ARRAY_POINTER_ARB 0x8849
#define GL_TEXTURE0_ARB 0x84C0
#define GL_TEXTURE1_ARB 0x84C1
#define GL_TEXTURE2_ARB 0x84C2
#define GL_TEXTURE3_ARB 0x84C3
#define GL_TEXTURE4_ARB 0x84C4
#define GL_TEXTURE5_ARB 0x84C5
#define GL_TEXTURE6_ARB 0x84C6
#define GL_TEXTURE7_ARB 0x84C7
#define GL_TEXTURE8_ARB 0x84C8
#define GL_TEXTURE9_ARB 0x84C9
#define GL_TEXTURE10_ARB 0x84CA
#define GL_TEXTURE11_ARB 0x84CB
#define GL_TEXTURE12_ARB 0x84CC
#define GL_TEXTURE13_ARB 0x84CD
#define GL_TEXTURE14_ARB 0x84CE
#define GL_TEXTURE15_ARB 0x84CF
#define GL_TEXTURE16_ARB 0x84D0
#define GL_TEXTURE17_ARB 0x84D1
#define GL_TEXTURE18_ARB 0x84D2
#define GL_TEXTURE19_ARB 0x84D3
#define GL_TEXTURE20_ARB 0x84D4
#define GL_TEXTURE21_ARB 0x84D5
#define GL_TEXTURE22_ARB 0x84D6
#define GL_TEXTURE23_ARB 0x84D7
#define GL_TEXTURE24_ARB 0x84D8
#define GL_TEXTURE25_ARB 0x84D9
#define GL_TEXTURE26_ARB 0x84DA
#define GL_TEXTURE27_ARB 0x84DB
#define GL_TEXTURE28_ARB 0x84DC
#define GL_TEXTURE29_ARB 0x84DD
#define GL_TEXTURE30_ARB 0x84DE
#define GL_TEXTURE31_ARB 0x84DF
#define GL_ACTIVE_TEXTURE_ARB 0x84E0
#define GL_CLIENT_ACTIVE_TEXTURE_ARB 0x84E1
#define GL_MAX_TEXTURE_UNITS_ARB 0x84E2
#define GL_MAX_VERTEX_UNITS_ARB 0x86A4
#define GL_ACTIVE_VERTEX_UNITS_ARB 0x86A5
#define GL_WEIGHT_SUM_UNITY_ARB 0x86A6
#define GL_VERTEX_BLEND_ARB 0x86A7
#define GL_CURRENT_WEIGHT_ARB 0x86A8
#define GL_WEIGHT_ARRAY_TYPE_ARB 0x86A9
#define GL_WEIGHT_ARRAY_STRIDE_ARB 0x86AA
#define GL_WEIGHT_ARRAY_SIZE_ARB 0x86AB
#define GL_WEIGHT_ARRAY_POINTER_ARB 0x86AC
#define GL_WEIGHT_ARRAY_ARB 0x86AD
#define GL_MODELVIEW0_ARB 0x1700
#define GL_MODELVIEW1_ARB 0x850A
#define GL_MODELVIEW2_ARB 0x8722
#define GL_MODELVIEW3_ARB 0x8723
#define GL_MODELVIEW4_ARB 0x8724
#define GL_MODELVIEW5_ARB 0x8725
#define GL_MODELVIEW6_ARB 0x8726
#define GL_MODELVIEW7_ARB 0x8727
#define GL_MODELVIEW8_ARB 0x8728
#define GL_MODELVIEW9_ARB 0x8729
#define GL_MODELVIEW10_ARB 0x872A
#define GL_MODELVIEW11_ARB 0x872B
#define GL_MODELVIEW12_ARB 0x872C
#define GL_MODELVIEW13_ARB 0x872D
#define GL_MODELVIEW14_ARB 0x872E
#define GL_MODELVIEW15_ARB 0x872F
#define GL_MODELVIEW16_ARB 0x8730
#define GL_MODELVIEW17_ARB 0x8731
#define GL_MODELVIEW18_ARB 0x8732
#define GL_MODELVIEW19_ARB 0x8733
#define GL_MODELVIEW20_ARB 0x8734
#define GL_MODELVIEW21_ARB 0x8735
#define GL_MODELVIEW22_ARB 0x8736
#define GL_MODELVIEW23_ARB 0x8737
#define GL_MODELVIEW24_ARB 0x8738
#define GL_MODELVIEW25_ARB 0x8739
#define GL_MODELVIEW26_ARB 0x873A
#define GL_MODELVIEW27_ARB 0x873B
#define GL_MODELVIEW28_ARB 0x873C
#define GL_MODELVIEW29_ARB 0x873D
#define GL_MODELVIEW30_ARB 0x873E
#define GL_MODELVIEW31_ARB 0x873F
#define GL_BUMP_ROT_MATRIX_ATI 0x8775
#define GL_BUMP_ROT_MATRIX_SIZE_ATI 0x8776
#define GL_BUMP_NUM_TEX_UNITS_ATI 0x8777
#define GL_BUMP_TEX_UNITS_ATI 0x8778
#define GL_DUDV_ATI 0x8779
#define GL_DU8DV8_ATI 0x877A
#define GL_BUMP_ENVMAP_ATI 0x877B
#define GL_BUMP_TARGET_ATI 0x877C
#define GL_PN_TRIANGLES_ATI 0x87F0
#define GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI 0x87F1
#define GL_PN_TRIANGLES_POINT_MODE_ATI 0x87F2
#define GL_PN_TRIANGLES_NORMAL_MODE_ATI 0x87F3
#define GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI 0x87F4
#define GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI 0x87F5
#define GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI 0x87F6
#define GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI 0x87F7
#define GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI 0x87F8
#define GL_STENCIL_BACK_FUNC_ATI 0x8800
#define GL_STENCIL_BACK_FAIL_ATI 0x8801
#define GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI 0x8802
#define GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI 0x8803
#define GL_COMPRESSED_RGB_3DC_ATI 0x8837
#define GL_MAX_VERTEX_STREAMS_ATI 0x876B
#define GL_VERTEX_SOURCE_ATI 0x876C
#define GL_VERTEX_STREAM0_ATI 0x876D
#define GL_VERTEX_STREAM1_ATI 0x876E
#define GL_VERTEX_STREAM2_ATI 0x876F
#define GL_VERTEX_STREAM3_ATI 0x8770
#define GL_VERTEX_STREAM4_ATI 0x8771
#define GL_VERTEX_STREAM5_ATI 0x8772
#define GL_VERTEX_STREAM6_ATI 0x8773
#define GL_VERTEX_STREAM7_ATI 0x8774
#define GL_TEXTURE_POINT_MODE_ATIX 0x60b0
#define GL_TEXTURE_POINT_ONE_COORD_ATIX 0x60b1
#define GL_TEXTURE_POINT_SPRITE_ATIX 0x60b2
#define GL_POINT_SPRITE_CULL_MODE_ATIX 0x60b3
#define GL_POINT_SPRITE_CULL_CENTER_ATIX 0x60b4
#define GL_POINT_SPRITE_CULL_CLIP_ATIX 0x60b5
#define GL_MODULATE_ADD_ATIX 0x8744
#define GL_MODULATE_SIGNED_ADD_ATIX 0x8745
#define GL_MODULATE_SUBTRACT_ATIX 0x8746
#define GL_SECONDARY_COLOR_ATIX 0x8747
#define GL_TEXTURE_OUTPUT_RGB_ATIX 0x8748
#define GL_TEXTURE_OUTPUT_ALPHA_ATIX 0x8749
#define GL_OUTPUT_POINT_SIZE_ATIX 0x610E
#define GL_CG_VERTEX_SHADER_EXT 0x890E
#define GL_CG_FRAGMENT_SHADER_EXT 0x890F
#define GL_DEPTH_BOUNDS_TEST_EXT 0x8890
#define GL_DEPTH_BOUNDS_EXT 0x8891
#define GL_FOG_COORDINATE_SOURCE_EXT 0x8450
#define GL_FOG_COORDINATE_EXT 0x8451
#define GL_FRAGMENT_DEPTH_EXT 0x8452
#define GL_CURRENT_FOG_COORDINATE_EXT 0x8453
#define GL_FOG_COORDINATE_ARRAY_TYPE_EXT 0x8454
#define GL_FOG_COORDINATE_ARRAY_STRIDE_EXT 0x8455
#define GL_FOG_COORDINATE_ARRAY_POINTER_EXT 0x8456
#define GL_FOG_COORDINATE_ARRAY_EXT 0x8457
#define GL_PIXEL_PACK_BUFFER_EXT 0x88EB
#define GL_PIXEL_UNPACK_BUFFER_EXT 0x88EC
#define GL_PIXEL_PACK_BUFFER_BINDING_EXT 0x88ED
#define GL_PIXEL_UNPACK_BUFFER_BINDING_EXT 0x88EF
#define GL_COLOR_SUM_EXT 0x8458
#define GL_CURRENT_SECONDARY_COLOR_EXT 0x8459
#define GL_SECONDARY_COLOR_ARRAY_SIZE_EXT 0x845A
#define GL_SECONDARY_COLOR_ARRAY_TYPE_EXT 0x845B
#define GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT 0x845C
#define GL_SECONDARY_COLOR_ARRAY_POINTER_EXT 0x845D
#define GL_SECONDARY_COLOR_ARRAY_EXT 0x845E
#define GL_NORMAL_MAP_EXT 0x8511
#define GL_REFLECTION_MAP_EXT 0x8512
#define GL_TEXTURE_CUBE_MAP_EXT 0x8513
#define GL_TEXTURE_BINDING_CUBE_MAP_EXT 0x8514
#define GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT 0x8515
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT 0x8516
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT 0x8517
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT 0x8518
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT 0x8519
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT 0x851A
#define GL_PROXY_TEXTURE_CUBE_MAP_EXT 0x851B
#define GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT 0x851C
#define GL_CLAMP_TO_EDGE_EXT 0x812F
#define GL_TEXTURE_RECTANGLE_EXT 0x84F5
#define GL_TEXTURE_BINDING_RECTANGLE_EXT 0x84F6
#define GL_PROXY_TEXTURE_RECTANGLE_EXT 0x84F7
#define GL_MAX_RECTANGLE_TEXTURE_SIZE_EXT 0x84F8
#define GL_VERTEX_SHADER_EXT 0x8780
#define GL_VERTEX_SHADER_BINDING_EXT 0x8781
#define GL_OP_INDEX_EXT 0x8782
#define GL_OP_NEGATE_EXT 0x8783
#define GL_OP_DOT3_EXT 0x8784
#define GL_OP_DOT4_EXT 0x8785
#define GL_OP_MUL_EXT 0x8786
#define GL_OP_ADD_EXT 0x8787
#define GL_OP_MADD_EXT 0x8788
#define GL_OP_FRAC_EXT 0x8789
#define GL_OP_MAX_EXT 0x878A
#define GL_OP_MIN_EXT 0x878B
#define GL_OP_SET_GE_EXT 0x878C
#define GL_OP_SET_LT_EXT 0x878D
#define GL_OP_CLAMP_EXT 0x878E
#define GL_OP_FLOOR_EXT 0x878F
#define GL_OP_ROUND_EXT 0x8790
#define GL_OP_EXP_BASE_2_EXT 0x8791
#define GL_OP_LOG_BASE_2_EXT 0x8792
#define GL_OP_POWER_EXT 0x8793
#define GL_OP_RECIP_EXT 0x8794
#define GL_OP_RECIP_SQRT_EXT 0x8795
#define GL_OP_SUB_EXT 0x8796
#define GL_OP_CROSS_PRODUCT_EXT 0x8797
#define GL_OP_MULTIPLY_MATRIX_EXT 0x8798
#define GL_OP_MOV_EXT 0x8799
#define GL_OUTPUT_VERTEX_EXT 0x879A
#define GL_OUTPUT_COLOR0_EXT 0x879B
#define GL_OUTPUT_COLOR1_EXT 0x879C
#define GL_OUTPUT_TEXTURE_COORD0_EXT 0x879D
#define GL_OUTPUT_TEXTURE_COORD1_EXT 0x879E
#define GL_OUTPUT_TEXTURE_COORD2_EXT 0x879F
#define GL_OUTPUT_TEXTURE_COORD3_EXT 0x87A0
#define GL_OUTPUT_TEXTURE_COORD4_EXT 0x87A1
#define GL_OUTPUT_TEXTURE_COORD5_EXT 0x87A2
#define GL_OUTPUT_TEXTURE_COORD6_EXT 0x87A3
#define GL_OUTPUT_TEXTURE_COORD7_EXT 0x87A4
#define GL_OUTPUT_TEXTURE_COORD8_EXT 0x87A5
#define GL_OUTPUT_TEXTURE_COORD9_EXT 0x87A6
#define GL_OUTPUT_TEXTURE_COORD10_EXT 0x87A7
#define GL_OUTPUT_TEXTURE_COORD11_EXT 0x87A8
#define GL_OUTPUT_TEXTURE_COORD12_EXT 0x87A9
#define GL_OUTPUT_TEXTURE_COORD13_EXT 0x87AA
#define GL_OUTPUT_TEXTURE_COORD14_EXT 0x87AB
#define GL_OUTPUT_TEXTURE_COORD15_EXT 0x87AC
#define GL_OUTPUT_TEXTURE_COORD16_EXT 0x87AD
#define GL_OUTPUT_TEXTURE_COORD17_EXT 0x87AE
#define GL_OUTPUT_TEXTURE_COORD18_EXT 0x87AF
#define GL_OUTPUT_TEXTURE_COORD19_EXT 0x87B0
#define GL_OUTPUT_TEXTURE_COORD20_EXT 0x87B1
#define GL_OUTPUT_TEXTURE_COORD21_EXT 0x87B2
#define GL_OUTPUT_TEXTURE_COORD22_EXT 0x87B3
#define GL_OUTPUT_TEXTURE_COORD23_EXT 0x87B4
#define GL_OUTPUT_TEXTURE_COORD24_EXT 0x87B5
#define GL_OUTPUT_TEXTURE_COORD25_EXT 0x87B6
#define GL_OUTPUT_TEXTURE_COORD26_EXT 0x87B7
#define GL_OUTPUT_TEXTURE_COORD27_EXT 0x87B8
#define GL_OUTPUT_TEXTURE_COORD28_EXT 0x87B9
#define GL_OUTPUT_TEXTURE_COORD29_EXT 0x87BA
#define GL_OUTPUT_TEXTURE_COORD30_EXT 0x87BB
#define GL_OUTPUT_TEXTURE_COORD31_EXT 0x87BC
#define GL_OUTPUT_FOG_EXT 0x87BD
#define GL_SCALAR_EXT 0x87BE
#define GL_VECTOR_EXT 0x87BF
#define GL_MATRIX_EXT 0x87C0
#define GL_VARIANT_EXT 0x87C1
#define GL_INVARIANT_EXT 0x87C2
#define GL_LOCAL_CONSTANT_EXT 0x87C3
#define GL_LOCAL_EXT 0x87C4
#define GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT 0x87C5
#define GL_MAX_VERTEX_SHADER_VARIANTS_EXT 0x87C6
#define GL_MAX_VERTEX_SHADER_INVARIANTS_EXT 0x87C7
#define GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT 0x87C8
#define GL_MAX_VERTEX_SHADER_LOCALS_EXT 0x87C9
#define GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT 0x87CA
#define GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT 0x87CB
#define GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT 0x87CC
#define GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT 0x87CD
#define GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT 0x87CE
#define GL_VERTEX_SHADER_INSTRUCTIONS_EXT 0x87CF
#define GL_VERTEX_SHADER_VARIANTS_EXT 0x87D0
#define GL_VERTEX_SHADER_INVARIANTS_EXT 0x87D1
#define GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT 0x87D2
#define GL_VERTEX_SHADER_LOCALS_EXT 0x87D3
#define GL_VERTEX_SHADER_OPTIMIZED_EXT 0x87D4
#define GL_X_EXT 0x87D5
#define GL_Y_EXT 0x87D6
#define GL_Z_EXT 0x87D7
#define GL_W_EXT 0x87D8
#define GL_NEGATIVE_X_EXT 0x87D9
#define GL_NEGATIVE_Y_EXT 0x87DA
#define GL_NEGATIVE_Z_EXT 0x87DB
#define GL_NEGATIVE_W_EXT 0x87DC
#define GL_ZERO_EXT 0x87DD
#define GL_ONE_EXT 0x87DE
#define GL_NEGATIVE_ONE_EXT 0x87DF
#define GL_NORMALIZED_RANGE_EXT 0x87E0
#define GL_FULL_RANGE_EXT 0x87E1
#define GL_CURRENT_VERTEX_EXT 0x87E2
#define GL_MVP_MATRIX_EXT 0x87E3
#define GL_VARIANT_VALUE_EXT 0x87E4
#define GL_VARIANT_DATATYPE_EXT 0x87E5
#define GL_VARIANT_ARRAY_STRIDE_EXT 0x87E6
#define GL_VARIANT_ARRAY_TYPE_EXT 0x87E7
#define GL_VARIANT_ARRAY_EXT 0x87E8
#define GL_VARIANT_ARRAY_POINTER_EXT 0x87E9
#define GL_INVARIANT_VALUE_EXT 0x87EA
#define GL_INVARIANT_DATATYPE_EXT 0x87EB
#define GL_LOCAL_CONSTANT_VALUE_EXT 0x87EC
#define GL_LOCAL_CONSTANT_DATATYPE_EXT 0x87ED
#define GL_KTX_FRONT_REGION 0x0
#define GL_KTX_BACK_REGION 0x1
#define GL_KTX_Z_REGION 0x2
#define GL_KTX_STENCIL_REGION 0x3
#define GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV 0x88F4
#define GL_MAX_PROGRAM_CALL_DEPTH_NV 0x88F5
#define GL_MAX_PROGRAM_IF_DEPTH_NV 0x88F6
#define GL_MAX_PROGRAM_LOOP_DEPTH_NV 0x88F7
#define GL_MAX_PROGRAM_LOOP_COUNT_NV 0x88F8
#define GL_ACCUM 0x0100
#define GL_LOAD 0x0101
#define GL_RETURN 0x0102
#define GL_MULT 0x0103
#define GL_ADD 0x0104
#define GL_NEVER 0x0200
#define GL_LESS 0x0201
#define GL_EQUAL 0x0202
#define GL_LEQUAL 0x0203
#define GL_GREATER 0x0204
#define GL_NOTEQUAL 0x0205
#define GL_GEQUAL 0x0206
#define GL_ALWAYS 0x0207
#define GL_CURRENT_BIT 0x00000001
#define GL_POINT_BIT 0x00000002
#define GL_LINE_BIT 0x00000004
#define GL_POLYGON_BIT 0x00000008
#define GL_POLYGON_STIPPLE_BIT 0x00000010
#define GL_PIXEL_MODE_BIT 0x00000020
#define GL_LIGHTING_BIT 0x00000040
#define GL_FOG_BIT 0x00000080
#define GL_DEPTH_BUFFER_BIT 0x00000100
#define GL_ACCUM_BUFFER_BIT 0x00000200
#define GL_STENCIL_BUFFER_BIT 0x00000400
#define GL_VIEWPORT_BIT 0x00000800
#define GL_TRANSFORM_BIT 0x00001000
#define GL_ENABLE_BIT 0x00002000
#define GL_COLOR_BUFFER_BIT 0x00004000
#define GL_HINT_BIT 0x00008000
#define GL_EVAL_BIT 0x00010000
#define GL_LIST_BIT 0x00020000
#define GL_TEXTURE_BIT 0x00040000
#define GL_SCISSOR_BIT 0x00080000
#define GL_ALL_ATTRIB_BITS 0x000fffff
#define GL_POINTS 0x0000
#define GL_LINES 0x0001
#define GL_LINE_LOOP 0x0002
#define GL_LINE_STRIP 0x0003
#define GL_TRIANGLES 0x0004
#define GL_TRIANGLE_STRIP 0x0005
#define GL_TRIANGLE_FAN 0x0006
#define GL_QUADS 0x0007
#define GL_QUAD_STRIP 0x0008
#define GL_POLYGON 0x0009
#define GL_ZERO 0
#define GL_ONE 1
#define GL_SRC_COLOR 0x0300
#define GL_ONE_MINUS_SRC_COLOR 0x0301
#define GL_SRC_ALPHA 0x0302
#define GL_ONE_MINUS_SRC_ALPHA 0x0303
#define GL_DST_ALPHA 0x0304
#define GL_ONE_MINUS_DST_ALPHA 0x0305
#define GL_DST_COLOR 0x0306
#define GL_ONE_MINUS_DST_COLOR 0x0307
#define GL_SRC_ALPHA_SATURATE 0x0308
#define GL_TRUE 1
#define GL_FALSE 0
#define GL_CLIP_PLANE0 0x3000
#define GL_CLIP_PLANE1 0x3001
#define GL_CLIP_PLANE2 0x3002
#define GL_CLIP_PLANE3 0x3003
#define GL_CLIP_PLANE4 0x3004
#define GL_CLIP_PLANE5 0x3005
#define GL_BYTE 0x1400
#define GL_UNSIGNED_BYTE 0x1401
#define GL_SHORT 0x1402
#define GL_UNSIGNED_SHORT 0x1403
#define GL_INT 0x1404
#define GL_UNSIGNED_INT 0x1405
#define GL_FLOAT 0x1406
#define GL_2_BYTES 0x1407
#define GL_3_BYTES 0x1408
#define GL_4_BYTES 0x1409
#define GL_DOUBLE 0x140A
#define GL_NONE 0
#define GL_FRONT_LEFT 0x0400
#define GL_FRONT_RIGHT 0x0401
#define GL_BACK_LEFT 0x0402
#define GL_BACK_RIGHT 0x0403
#define GL_FRONT 0x0404
#define GL_BACK 0x0405
#define GL_LEFT 0x0406
#define GL_RIGHT 0x0407
#define GL_FRONT_AND_BACK 0x0408
#define GL_AUX0 0x0409
#define GL_AUX1 0x040A
#define GL_AUX2 0x040B
#define GL_AUX3 0x040C
#define GL_NO_ERROR 0
#define GL_INVALID_ENUM 0x0500
#define GL_INVALID_VALUE 0x0501
#define GL_INVALID_OPERATION 0x0502
#define GL_STACK_OVERFLOW 0x0503
#define GL_STACK_UNDERFLOW 0x0504
#define GL_OUT_OF_MEMORY 0x0505
#define GL_2D 0x0600
#define GL_3D 0x0601
#define GL_3D_COLOR 0x0602
#define GL_3D_COLOR_TEXTURE 0x0603
#define GL_4D_COLOR_TEXTURE 0x0604
#define GL_PASS_THROUGH_TOKEN 0x0700
#define GL_POINT_TOKEN 0x0701
#define GL_LINE_TOKEN 0x0702
#define GL_POLYGON_TOKEN 0x0703
#define GL_BITMAP_TOKEN 0x0704
#define GL_DRAW_PIXEL_TOKEN 0x0705
#define GL_COPY_PIXEL_TOKEN 0x0706
#define GL_LINE_RESET_TOKEN 0x0707
#define GL_EXP 0x0800
#define GL_EXP2 0x0801
#define GL_CW 0x0900
#define GL_CCW 0x0901
#define GL_COEFF 0x0A00
#define GL_ORDER 0x0A01
#define GL_DOMAIN 0x0A02
#define GL_CURRENT_COLOR 0x0B00
#define GL_CURRENT_INDEX 0x0B01
#define GL_CURRENT_NORMAL 0x0B02
#define GL_CURRENT_TEXTURE_COORDS 0x0B03
#define GL_CURRENT_RASTER_COLOR 0x0B04
#define GL_CURRENT_RASTER_INDEX 0x0B05
#define GL_CURRENT_RASTER_TEXTURE_COORDS 0x0B06
#define GL_CURRENT_RASTER_POSITION 0x0B07
#define GL_CURRENT_RASTER_POSITION_VALID 0x0B08
#define GL_CURRENT_RASTER_DISTANCE 0x0B09
#define GL_POINT_SMOOTH 0x0B10
#define GL_POINT_SIZE 0x0B11
#define GL_POINT_SIZE_RANGE 0x0B12
#define GL_POINT_SIZE_GRANULARITY 0x0B13
#define GL_LINE_SMOOTH 0x0B20
#define GL_LINE_WIDTH 0x0B21
#define GL_LINE_WIDTH_RANGE 0x0B22
#define GL_LINE_WIDTH_GRANULARITY 0x0B23
#define GL_LINE_STIPPLE 0x0B24
#define GL_LINE_STIPPLE_PATTERN 0x0B25
#define GL_LINE_STIPPLE_REPEAT 0x0B26
#define GL_LIST_MODE 0x0B30
#define GL_MAX_LIST_NESTING 0x0B31
#define GL_LIST_BASE 0x0B32
#define GL_LIST_INDEX 0x0B33
#define GL_POLYGON_MODE 0x0B40
#define GL_POLYGON_SMOOTH 0x0B41
#define GL_POLYGON_STIPPLE 0x0B42
#define GL_EDGE_FLAG 0x0B43
#define GL_CULL_FACE 0x0B44
#define GL_CULL_FACE_MODE 0x0B45
#define GL_FRONT_FACE 0x0B46
#define GL_LIGHTING 0x0B50
#define GL_LIGHT_MODEL_LOCAL_VIEWER 0x0B51
#define GL_LIGHT_MODEL_TWO_SIDE 0x0B52
#define GL_LIGHT_MODEL_AMBIENT 0x0B53
#define GL_SHADE_MODEL 0x0B54
#define GL_COLOR_MATERIAL_FACE 0x0B55
#define GL_COLOR_MATERIAL_PARAMETER 0x0B56
#define GL_COLOR_MATERIAL 0x0B57
#define GL_FOG 0x0B60
#define GL_FOG_INDEX 0x0B61
#define GL_FOG_DENSITY 0x0B62
#define GL_FOG_START 0x0B63
#define GL_FOG_END 0x0B64
#define GL_FOG_MODE 0x0B65
#define GL_FOG_COLOR 0x0B66
#define GL_DEPTH_RANGE 0x0B70
#define GL_DEPTH_TEST 0x0B71
#define GL_DEPTH_WRITEMASK 0x0B72
#define GL_DEPTH_CLEAR_VALUE 0x0B73
#define GL_DEPTH_FUNC 0x0B74
#define GL_ACCUM_CLEAR_VALUE 0x0B80
#define GL_STENCIL_TEST 0x0B90
#define GL_STENCIL_CLEAR_VALUE 0x0B91
#define GL_STENCIL_FUNC 0x0B92
#define GL_STENCIL_VALUE_MASK 0x0B93
#define GL_STENCIL_FAIL 0x0B94
#define GL_STENCIL_PASS_DEPTH_FAIL 0x0B95
#define GL_STENCIL_PASS_DEPTH_PASS 0x0B96
#define GL_STENCIL_REF 0x0B97
#define GL_STENCIL_WRITEMASK 0x0B98
#define GL_MATRIX_MODE 0x0BA0
#define GL_NORMALIZE 0x0BA1
#define GL_VIEWPORT 0x0BA2
#define GL_MODELVIEW_STACK_DEPTH 0x0BA3
#define GL_PROJECTION_STACK_DEPTH 0x0BA4
#define GL_TEXTURE_STACK_DEPTH 0x0BA5
#define GL_MODELVIEW_MATRIX 0x0BA6
#define GL_PROJECTION_MATRIX 0x0BA7
#define GL_TEXTURE_MATRIX 0x0BA8
#define GL_ATTRIB_STACK_DEPTH 0x0BB0
#define GL_CLIENT_ATTRIB_STACK_DEPTH 0x0BB1
#define GL_ALPHA_TEST 0x0BC0
#define GL_ALPHA_TEST_FUNC 0x0BC1
#define GL_ALPHA_TEST_REF 0x0BC2
#define GL_DITHER 0x0BD0
#define GL_BLEND_DST 0x0BE0
#define GL_BLEND_SRC 0x0BE1
#define GL_BLEND 0x0BE2
#define GL_LOGIC_OP_MODE 0x0BF0
#define GL_INDEX_LOGIC_OP 0x0BF1
#define GL_COLOR_LOGIC_OP 0x0BF2
#define GL_AUX_BUFFERS 0x0C00
#define GL_DRAW_BUFFER 0x0C01
#define GL_READ_BUFFER 0x0C02
#define GL_SCISSOR_BOX 0x0C10
#define GL_SCISSOR_TEST 0x0C11
#define GL_INDEX_CLEAR_VALUE 0x0C20
#define GL_INDEX_WRITEMASK 0x0C21
#define GL_COLOR_CLEAR_VALUE 0x0C22
#define GL_COLOR_WRITEMASK 0x0C23
#define GL_INDEX_MODE 0x0C30
#define GL_RGBA_MODE 0x0C31
#define GL_DOUBLEBUFFER 0x0C32
#define GL_STEREO 0x0C33
#define GL_RENDER_MODE 0x0C40
#define GL_PERSPECTIVE_CORRECTION_HINT 0x0C50
#define GL_POINT_SMOOTH_HINT 0x0C51
#define GL_LINE_SMOOTH_HINT 0x0C52
#define GL_POLYGON_SMOOTH_HINT 0x0C53
#define GL_FOG_HINT 0x0C54
#define GL_TEXTURE_GEN_S 0x0C60
#define GL_TEXTURE_GEN_T 0x0C61
#define GL_TEXTURE_GEN_R 0x0C62
#define GL_TEXTURE_GEN_Q 0x0C63
#define GL_PIXEL_MAP_I_TO_I 0x0C70
#define GL_PIXEL_MAP_S_TO_S 0x0C71
#define GL_PIXEL_MAP_I_TO_R 0x0C72
#define GL_PIXEL_MAP_I_TO_G 0x0C73
#define GL_PIXEL_MAP_I_TO_B 0x0C74
#define GL_PIXEL_MAP_I_TO_A 0x0C75
#define GL_PIXEL_MAP_R_TO_R 0x0C76
#define GL_PIXEL_MAP_G_TO_G 0x0C77
#define GL_PIXEL_MAP_B_TO_B 0x0C78
#define GL_PIXEL_MAP_A_TO_A 0x0C79
#define GL_PIXEL_MAP_I_TO_I_SIZE 0x0CB0
#define GL_PIXEL_MAP_S_TO_S_SIZE 0x0CB1
#define GL_PIXEL_MAP_I_TO_R_SIZE 0x0CB2
#define GL_PIXEL_MAP_I_TO_G_SIZE 0x0CB3
#define GL_PIXEL_MAP_I_TO_B_SIZE 0x0CB4
#define GL_PIXEL_MAP_I_TO_A_SIZE 0x0CB5
#define GL_PIXEL_MAP_R_TO_R_SIZE 0x0CB6
#define GL_PIXEL_MAP_G_TO_G_SIZE 0x0CB7
#define GL_PIXEL_MAP_B_TO_B_SIZE 0x0CB8
#define GL_PIXEL_MAP_A_TO_A_SIZE 0x0CB9
#define GL_UNPACK_SWAP_BYTES 0x0CF0
#define GL_UNPACK_LSB_FIRST 0x0CF1
#define GL_UNPACK_ROW_LENGTH 0x0CF2
#define GL_UNPACK_SKIP_ROWS 0x0CF3
#define GL_UNPACK_SKIP_PIXELS 0x0CF4
#define GL_UNPACK_ALIGNMENT 0x0CF5
#define GL_PACK_SWAP_BYTES 0x0D00
#define GL_PACK_LSB_FIRST 0x0D01
#define GL_PACK_ROW_LENGTH 0x0D02
#define GL_PACK_SKIP_ROWS 0x0D03
#define GL_PACK_SKIP_PIXELS 0x0D04
#define GL_PACK_ALIGNMENT 0x0D05
#define GL_MAP_COLOR 0x0D10
#define GL_MAP_STENCIL 0x0D11
#define GL_INDEX_SHIFT 0x0D12
#define GL_INDEX_OFFSET 0x0D13
#define GL_RED_SCALE 0x0D14
#define GL_RED_BIAS 0x0D15
#define GL_ZOOM_X 0x0D16
#define GL_ZOOM_Y 0x0D17
#define GL_GREEN_SCALE 0x0D18
#define GL_GREEN_BIAS 0x0D19
#define GL_BLUE_SCALE 0x0D1A
#define GL_BLUE_BIAS 0x0D1B
#define GL_ALPHA_SCALE 0x0D1C
#define GL_ALPHA_BIAS 0x0D1D
#define GL_DEPTH_SCALE 0x0D1E
#define GL_DEPTH_BIAS 0x0D1F
#define GL_MAX_EVAL_ORDER 0x0D30
#define GL_MAX_LIGHTS 0x0D31
#define GL_MAX_CLIP_PLANES 0x0D32
#define GL_MAX_TEXTURE_SIZE 0x0D33
#define GL_MAX_PIXEL_MAP_TABLE 0x0D34
#define GL_MAX_ATTRIB_STACK_DEPTH 0x0D35
#define GL_MAX_MODELVIEW_STACK_DEPTH 0x0D36
#define GL_MAX_NAME_STACK_DEPTH 0x0D37
#define GL_MAX_PROJECTION_STACK_DEPTH 0x0D38
#define GL_MAX_TEXTURE_STACK_DEPTH 0x0D39
#define GL_MAX_VIEWPORT_DIMS 0x0D3A
#define GL_MAX_CLIENT_ATTRIB_STACK_DEPTH 0x0D3B
#define GL_SUBPIXEL_BITS 0x0D50
#define GL_INDEX_BITS 0x0D51
#define GL_RED_BITS 0x0D52
#define GL_GREEN_BITS 0x0D53
#define GL_BLUE_BITS 0x0D54
#define GL_ALPHA_BITS 0x0D55
#define GL_DEPTH_BITS 0x0D56
#define GL_STENCIL_BITS 0x0D57
#define GL_ACCUM_RED_BITS 0x0D58
#define GL_ACCUM_GREEN_BITS 0x0D59
#define GL_ACCUM_BLUE_BITS 0x0D5A
#define GL_ACCUM_ALPHA_BITS 0x0D5B
#define GL_NAME_STACK_DEPTH 0x0D70
#define GL_AUTO_NORMAL 0x0D80
#define GL_MAP1_COLOR_4 0x0D90
#define GL_MAP1_INDEX 0x0D91
#define GL_MAP1_NORMAL 0x0D92
#define GL_MAP1_TEXTURE_COORD_1 0x0D93
#define GL_MAP1_TEXTURE_COORD_2 0x0D94
#define GL_MAP1_TEXTURE_COORD_3 0x0D95
#define GL_MAP1_TEXTURE_COORD_4 0x0D96
#define GL_MAP1_VERTEX_3 0x0D97
#define GL_MAP1_VERTEX_4 0x0D98
#define GL_MAP2_COLOR_4 0x0DB0
#define GL_MAP2_INDEX 0x0DB1
#define GL_MAP2_NORMAL 0x0DB2
#define GL_MAP2_TEXTURE_COORD_1 0x0DB3
#define GL_MAP2_TEXTURE_COORD_2 0x0DB4
#define GL_MAP2_TEXTURE_COORD_3 0x0DB5
#define GL_MAP2_TEXTURE_COORD_4 0x0DB6
#define GL_MAP2_VERTEX_3 0x0DB7
#define GL_MAP2_VERTEX_4 0x0DB8
#define GL_MAP1_GRID_DOMAIN 0x0DD0
#define GL_MAP1_GRID_SEGMENTS 0x0DD1
#define GL_MAP2_GRID_DOMAIN 0x0DD2
#define GL_MAP2_GRID_SEGMENTS 0x0DD3
#define GL_TEXTURE_1D 0x0DE0
#define GL_TEXTURE_2D 0x0DE1
#define GL_FEEDBACK_BUFFER_POINTER 0x0DF0
#define GL_FEEDBACK_BUFFER_SIZE 0x0DF1
#define GL_FEEDBACK_BUFFER_TYPE 0x0DF2
#define GL_SELECTION_BUFFER_POINTER 0x0DF3
#define GL_SELECTION_BUFFER_SIZE 0x0DF4
#define GL_TEXTURE_WIDTH 0x1000
#define GL_TEXTURE_HEIGHT 0x1001
#define GL_TEXTURE_INTERNAL_FORMAT 0x1003
#define GL_TEXTURE_BORDER_COLOR 0x1004
#define GL_TEXTURE_BORDER 0x1005
#define GL_DONT_CARE 0x1100
#define GL_FASTEST 0x1101
#define GL_NICEST 0x1102
#define GL_LIGHT0 0x4000
#define GL_LIGHT1 0x4001
#define GL_LIGHT2 0x4002
#define GL_LIGHT3 0x4003
#define GL_LIGHT4 0x4004
#define GL_LIGHT5 0x4005
#define GL_LIGHT6 0x4006
#define GL_LIGHT7 0x4007
#define GL_AMBIENT 0x1200
#define GL_DIFFUSE 0x1201
#define GL_SPECULAR 0x1202
#define GL_POSITION 0x1203
#define GL_SPOT_DIRECTION 0x1204
#define GL_SPOT_EXPONENT 0x1205
#define GL_SPOT_CUTOFF 0x1206
#define GL_CONSTANT_ATTENUATION 0x1207
#define GL_LINEAR_ATTENUATION 0x1208
#define GL_QUADRATIC_ATTENUATION 0x1209
#define GL_COMPILE 0x1300
#define GL_COMPILE_AND_EXECUTE 0x1301
#define GL_CLEAR 0x1500
#define GL_AND 0x1501
#define GL_AND_REVERSE 0x1502
#define GL_COPY 0x1503
#define GL_AND_INVERTED 0x1504
#define GL_NOOP 0x1505
#define GL_XOR 0x1506
#define GL_OR 0x1507
#define GL_NOR 0x1508
#define GL_EQUIV 0x1509
#define GL_INVERT 0x150A
#define GL_OR_REVERSE 0x150B
#define GL_COPY_INVERTED 0x150C
#define GL_OR_INVERTED 0x150D
#define GL_NAND 0x150E
#define GL_SET 0x150F
#define GL_EMISSION 0x1600
#define GL_SHININESS 0x1601
#define GL_AMBIENT_AND_DIFFUSE 0x1602
#define GL_COLOR_INDEXES 0x1603
#define GL_MODELVIEW 0x1700
#define GL_PROJECTION 0x1701
#define GL_TEXTURE 0x1702
#define GL_COLOR 0x1800
#define GL_DEPTH 0x1801
#define GL_STENCIL 0x1802
#define GL_COLOR_INDEX 0x1900
#define GL_STENCIL_INDEX 0x1901
#define GL_DEPTH_COMPONENT 0x1902
#define GL_RED 0x1903
#define GL_GREEN 0x1904
#define GL_BLUE 0x1905
#define GL_ALPHA 0x1906
#define GL_RGB 0x1907
#define GL_RGBA 0x1908
#define GL_LUMINANCE 0x1909
#define GL_LUMINANCE_ALPHA 0x190A
#define GL_BITMAP 0x1A00
#define GL_POINT 0x1B00
#define GL_LINE 0x1B01
#define GL_FILL 0x1B02
#define GL_RENDER 0x1C00
#define GL_FEEDBACK 0x1C01
#define GL_SELECT 0x1C02
#define GL_FLAT 0x1D00
#define GL_SMOOTH 0x1D01
#define GL_KEEP 0x1E00
#define GL_REPLACE 0x1E01
#define GL_INCR 0x1E02
#define GL_DECR 0x1E03
#define GL_VENDOR 0x1F00
#define GL_RENDERER 0x1F01
#define GL_VERSION 0x1F02
#define GL_EXTENSIONS 0x1F03
#define GL_S 0x2000
#define GL_T 0x2001
#define GL_R 0x2002
#define GL_Q 0x2003
#define GL_MODULATE 0x2100
#define GL_DECAL 0x2101
#define GL_TEXTURE_ENV_MODE 0x2200
#define GL_TEXTURE_ENV_COLOR 0x2201
#define GL_TEXTURE_ENV 0x2300
#define GL_EYE_LINEAR 0x2400
#define GL_OBJECT_LINEAR 0x2401
#define GL_SPHERE_MAP 0x2402
#define GL_TEXTURE_GEN_MODE 0x2500
#define GL_OBJECT_PLANE 0x2501
#define GL_EYE_PLANE 0x2502
#define GL_NEAREST 0x2600
#define GL_LINEAR 0x2601
#define GL_NEAREST_MIPMAP_NEAREST 0x2700
#define GL_LINEAR_MIPMAP_NEAREST 0x2701
#define GL_NEAREST_MIPMAP_LINEAR 0x2702
#define GL_LINEAR_MIPMAP_LINEAR 0x2703
#define GL_TEXTURE_MAG_FILTER 0x2800
#define GL_TEXTURE_MIN_FILTER 0x2801
#define GL_TEXTURE_WRAP_S 0x2802
#define GL_TEXTURE_WRAP_T 0x2803
#define GL_CLAMP 0x2900
#define GL_REPEAT 0x2901
#define GL_CLIENT_PIXEL_STORE_BIT 0x00000001
#define GL_CLIENT_VERTEX_ARRAY_BIT 0x00000002
#define GL_CLIENT_ALL_ATTRIB_BITS 0xffffffff
#define GL_POLYGON_OFFSET_FACTOR 0x8038
#define GL_POLYGON_OFFSET_UNITS 0x2A00
#define GL_POLYGON_OFFSET_POINT 0x2A01
#define GL_POLYGON_OFFSET_LINE 0x2A02
#define GL_POLYGON_OFFSET_FILL 0x8037
#define GL_ALPHA4 0x803B
#define GL_ALPHA8 0x803C
#define GL_ALPHA12 0x803D
#define GL_ALPHA16 0x803E
#define GL_LUMINANCE4 0x803F
#define GL_LUMINANCE8 0x8040
#define GL_LUMINANCE12 0x8041
#define GL_LUMINANCE16 0x8042
#define GL_LUMINANCE4_ALPHA4 0x8043
#define GL_LUMINANCE6_ALPHA2 0x8044
#define GL_LUMINANCE8_ALPHA8 0x8045
#define GL_LUMINANCE12_ALPHA4 0x8046
#define GL_LUMINANCE12_ALPHA12 0x8047
#define GL_LUMINANCE16_ALPHA16 0x8048
#define GL_INTENSITY 0x8049
#define GL_INTENSITY4 0x804A
#define GL_INTENSITY8 0x804B
#define GL_INTENSITY12 0x804C
#define GL_INTENSITY16 0x804D
#define GL_R3_G3_B2 0x2A10
#define GL_RGB4 0x804F
#define GL_RGB5 0x8050
#define GL_RGB8 0x8051
#define GL_RGB10 0x8052
#define GL_RGB12 0x8053
#define GL_RGB16 0x8054
#define GL_RGBA2 0x8055
#define GL_RGBA4 0x8056
#define GL_RGB5_A1 0x8057
#define GL_RGBA8 0x8058
#define GL_RGB10_A2 0x8059
#define GL_RGBA12 0x805A
#define GL_RGBA16 0x805B
#define GL_TEXTURE_RED_SIZE 0x805C
#define GL_TEXTURE_GREEN_SIZE 0x805D
#define GL_TEXTURE_BLUE_SIZE 0x805E
#define GL_TEXTURE_ALPHA_SIZE 0x805F
#define GL_TEXTURE_LUMINANCE_SIZE 0x8060
#define GL_TEXTURE_INTENSITY_SIZE 0x8061
#define GL_PROXY_TEXTURE_1D 0x8063
#define GL_PROXY_TEXTURE_2D 0x8064
#define GL_TEXTURE_PRIORITY 0x8066
#define GL_TEXTURE_RESIDENT 0x8067
#define GL_TEXTURE_BINDING_1D 0x8068
#define GL_TEXTURE_BINDING_2D 0x8069
#define GL_VERTEX_ARRAY 0x8074
#define GL_NORMAL_ARRAY 0x8075
#define GL_COLOR_ARRAY 0x8076
#define GL_INDEX_ARRAY 0x8077
#define GL_TEXTURE_COORD_ARRAY 0x8078
#define GL_EDGE_FLAG_ARRAY 0x8079
#define GL_VERTEX_ARRAY_SIZE 0x807A
#define GL_VERTEX_ARRAY_TYPE 0x807B
#define GL_VERTEX_ARRAY_STRIDE 0x807C
#define GL_NORMAL_ARRAY_TYPE 0x807E
#define GL_NORMAL_ARRAY_STRIDE 0x807F
#define GL_COLOR_ARRAY_SIZE 0x8081
#define GL_COLOR_ARRAY_TYPE 0x8082
#define GL_COLOR_ARRAY_STRIDE 0x8083
#define GL_INDEX_ARRAY_TYPE 0x8085
#define GL_INDEX_ARRAY_STRIDE 0x8086
#define GL_TEXTURE_COORD_ARRAY_SIZE 0x8088
#define GL_TEXTURE_COORD_ARRAY_TYPE 0x8089
#define GL_TEXTURE_COORD_ARRAY_STRIDE 0x808A
#define GL_EDGE_FLAG_ARRAY_STRIDE 0x808C
#define GL_VERTEX_ARRAY_POINTER 0x808E
#define GL_NORMAL_ARRAY_POINTER 0x808F
#define GL_COLOR_ARRAY_POINTER 0x8090
#define GL_INDEX_ARRAY_POINTER 0x8091
#define GL_TEXTURE_COORD_ARRAY_POINTER 0x8092
#define GL_EDGE_FLAG_ARRAY_POINTER 0x8093
#define GL_V2F 0x2A20
#define GL_V3F 0x2A21
#define GL_C4UB_V2F 0x2A22
#define GL_C4UB_V3F 0x2A23
#define GL_C3F_V3F 0x2A24
#define GL_N3F_V3F 0x2A25
#define GL_C4F_N3F_V3F 0x2A26
#define GL_T2F_V3F 0x2A27
#define GL_T4F_V4F 0x2A28
#define GL_T2F_C4UB_V3F 0x2A29
#define GL_T2F_C3F_V3F 0x2A2A
#define GL_T2F_N3F_V3F 0x2A2B
#define GL_T2F_C4F_N3F_V3F 0x2A2C
#define GL_T4F_C4F_N3F_V4F 0x2A2D
#define GL_LOGIC_OP 0x0BF1
#define GL_TEXTURE_COMPONENTS 0x1003
#define GL_COLOR_INDEX1_EXT 0x80E2
#define GL_COLOR_INDEX2_EXT 0x80E3
#define GL_COLOR_INDEX4_EXT 0x80E4
#define GL_COLOR_INDEX8_EXT 0x80E5
#define GL_COLOR_INDEX12_EXT 0x80E6
#define GL_COLOR_INDEX16_EXT 0x80E7
#define GL_UNSIGNED_BYTE_3_3_2 0x8032
#define GL_UNSIGNED_SHORT_4_4_4_4 0x8033
#define GL_UNSIGNED_SHORT_5_5_5_1 0x8034
#define GL_UNSIGNED_INT_8_8_8_8 0x8035
#define GL_UNSIGNED_INT_10_10_10_2 0x8036
#define GL_RESCALE_NORMAL 0x803A
#define GL_UNSIGNED_BYTE_2_3_3_REV 0x8362
#define GL_UNSIGNED_SHORT_5_6_5 0x8363
#define GL_UNSIGNED_SHORT_5_6_5_REV 0x8364
#define GL_UNSIGNED_SHORT_4_4_4_4_REV 0x8365
#define GL_UNSIGNED_SHORT_1_5_5_5_REV 0x8366
#define GL_UNSIGNED_INT_8_8_8_8_REV 0x8367
#define GL_UNSIGNED_INT_2_10_10_10_REV 0x8368
#define GL_BGR 0x80E0
#define GL_BGRA 0x80E1
#define GL_MAX_ELEMENTS_VERTICES 0x80E8
#define GL_MAX_ELEMENTS_INDICES 0x80E9
#define GL_CLAMP_TO_EDGE 0x812F
#define GL_TEXTURE_MIN_LOD 0x813A
#define GL_TEXTURE_MAX_LOD 0x813B
#define GL_TEXTURE_BASE_LEVEL 0x813C
#define GL_TEXTURE_MAX_LEVEL 0x813D
#define GL_LIGHT_MODEL_COLOR_CONTROL 0x81F8
#define GL_SINGLE_COLOR 0x81F9
#define GL_SEPARATE_SPECULAR_COLOR 0x81FA
#define GL_SMOOTH_POINT_SIZE_RANGE 0x0B12
#define GL_SMOOTH_POINT_SIZE_GRANULARITY 0x0B13
#define GL_SMOOTH_LINE_WIDTH_RANGE 0x0B22
#define GL_SMOOTH_LINE_WIDTH_GRANULARITY 0x0B23
#define GL_ALIASED_POINT_SIZE_RANGE 0x846D
#define GL_ALIASED_LINE_WIDTH_RANGE 0x846E
#define GL_PACK_SKIP_IMAGES 0x806B
#define GL_PACK_IMAGE_HEIGHT 0x806C
#define GL_UNPACK_SKIP_IMAGES 0x806D
#define GL_UNPACK_IMAGE_HEIGHT 0x806E
#define GL_TEXTURE_3D 0x806F
#define GL_PROXY_TEXTURE_3D 0x8070
#define GL_TEXTURE_DEPTH 0x8071
#define GL_TEXTURE_WRAP_R 0x8072
#define GL_MAX_3D_TEXTURE_SIZE 0x8073
#define GL_TEXTURE_BINDING_3D 0x806A
#define GL_TEXTURE0 0x84C0
#define GL_TEXTURE1 0x84C1
#define GL_TEXTURE2 0x84C2
#define GL_TEXTURE3 0x84C3
#define GL_TEXTURE4 0x84C4
#define GL_TEXTURE5 0x84C5
#define GL_TEXTURE6 0x84C6
#define GL_TEXTURE7 0x84C7
#define GL_TEXTURE8 0x84C8
#define GL_TEXTURE9 0x84C9
#define GL_TEXTURE10 0x84CA
#define GL_TEXTURE11 0x84CB
#define GL_TEXTURE12 0x84CC
#define GL_TEXTURE13 0x84CD
#define GL_TEXTURE14 0x84CE
#define GL_TEXTURE15 0x84CF
#define GL_TEXTURE16 0x84D0
#define GL_TEXTURE17 0x84D1
#define GL_TEXTURE18 0x84D2
#define GL_TEXTURE19 0x84D3
#define GL_TEXTURE20 0x84D4
#define GL_TEXTURE21 0x84D5
#define GL_TEXTURE22 0x84D6
#define GL_TEXTURE23 0x84D7
#define GL_TEXTURE24 0x84D8
#define GL_TEXTURE25 0x84D9
#define GL_TEXTURE26 0x84DA
#define GL_TEXTURE27 0x84DB
#define GL_TEXTURE28 0x84DC
#define GL_TEXTURE29 0x84DD
#define GL_TEXTURE30 0x84DE
#define GL_TEXTURE31 0x84DF
#define GL_ACTIVE_TEXTURE 0x84E0
#define GL_CLIENT_ACTIVE_TEXTURE 0x84E1
#define GL_MAX_TEXTURE_UNITS 0x84E2
#define GL_NORMAL_MAP 0x8511
#define GL_REFLECTION_MAP 0x8512
#define GL_TEXTURE_CUBE_MAP 0x8513
#define GL_TEXTURE_BINDING_CUBE_MAP 0x8514
#define GL_TEXTURE_CUBE_MAP_POSITIVE_X 0x8515
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_X 0x8516
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Y 0x8517
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Y 0x8518
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Z 0x8519
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Z 0x851A
#define GL_PROXY_TEXTURE_CUBE_MAP 0x851B
#define GL_MAX_CUBE_MAP_TEXTURE_SIZE 0x851C
#define GL_COMPRESSED_ALPHA 0x84E9
#define GL_COMPRESSED_LUMINANCE 0x84EA
#define GL_COMPRESSED_LUMINANCE_ALPHA 0x84EB
#define GL_COMPRESSED_INTENSITY 0x84EC
#define GL_COMPRESSED_RGB 0x84ED
#define GL_COMPRESSED_RGBA 0x84EE
#define GL_TEXTURE_COMPRESSION_HINT 0x84EF
#define GL_TEXTURE_COMPRESSED_IMAGE_SIZE 0x86A0
#define GL_TEXTURE_COMPRESSED 0x86A1
#define GL_NUM_COMPRESSED_TEXTURE_FORMATS 0x86A2
#define GL_COMPRESSED_TEXTURE_FORMATS 0x86A3
#define GL_MULTISAMPLE 0x809D
#define GL_SAMPLE_ALPHA_TO_COVERAGE 0x809E
#define GL_SAMPLE_ALPHA_TO_ONE 0x809F
#define GL_SAMPLE_COVERAGE 0x80A0
#define GL_SAMPLE_BUFFERS 0x80A8
#define GL_SAMPLES 0x80A9
#define GL_SAMPLE_COVERAGE_VALUE 0x80AA
#define GL_SAMPLE_COVERAGE_INVERT 0x80AB
#define GL_MULTISAMPLE_BIT 0x20000000
#define GL_TRANSPOSE_MODELVIEW_MATRIX 0x84E3
#define GL_TRANSPOSE_PROJECTION_MATRIX 0x84E4
#define GL_TRANSPOSE_TEXTURE_MATRIX 0x84E5
#define GL_TRANSPOSE_COLOR_MATRIX 0x84E6
#define GL_COMBINE 0x8570
#define GL_COMBINE_RGB 0x8571
#define GL_COMBINE_ALPHA 0x8572
#define GL_SOURCE0_RGB 0x8580
#define GL_SOURCE1_RGB 0x8581
#define GL_SOURCE2_RGB 0x8582
#define GL_SOURCE0_ALPHA 0x8588
#define GL_SOURCE1_ALPHA 0x8589
#define GL_SOURCE2_ALPHA 0x858A
#define GL_OPERAND0_RGB 0x8590
#define GL_OPERAND1_RGB 0x8591
#define GL_OPERAND2_RGB 0x8592
#define GL_OPERAND0_ALPHA 0x8598
#define GL_OPERAND1_ALPHA 0x8599
#define GL_OPERAND2_ALPHA 0x859A
#define GL_RGB_SCALE 0x8573
#define GL_ADD_SIGNED 0x8574
#define GL_INTERPOLATE 0x8575
#define GL_SUBTRACT 0x84E7
#define GL_CONSTANT 0x8576
#define GL_PRIMARY_COLOR 0x8577
#define GL_PREVIOUS 0x8578
#define GL_DOT3_RGB 0x86AE
#define GL_DOT3_RGBA 0x86AF
#define GL_CLAMP_TO_BORDER 0x812D
#define GL_GENERATE_MIPMAP 0x8191
#define GL_GENERATE_MIPMAP_HINT 0x8192
#define GL_DEPTH_COMPONENT16 0x81A5
#define GL_DEPTH_COMPONENT24 0x81A6
#define GL_DEPTH_COMPONENT32 0x81A7
#define GL_TEXTURE_DEPTH_SIZE 0x884A
#define GL_DEPTH_TEXTURE_MODE 0x884B
#define GL_TEXTURE_COMPARE_MODE 0x884C
#define GL_TEXTURE_COMPARE_FUNC 0x884D
#define GL_COMPARE_R_TO_TEXTURE 0x884E
#define GL_FOG_COORDINATE_SOURCE 0x8450
#define GL_FOG_COORDINATE 0x8451
#define GL_FRAGMENT_DEPTH 0x8452
#define GL_CURRENT_FOG_COORDINATE 0x8453
#define GL_FOG_COORDINATE_ARRAY_TYPE 0x8454
#define GL_FOG_COORDINATE_ARRAY_STRIDE 0x8455
#define GL_FOG_COORDINATE_ARRAY_POINTER 0x8456
#define GL_FOG_COORDINATE_ARRAY 0x8457
#define GL_POINT_SIZE_MIN 0x8126
#define GL_POINT_SIZE_MAX 0x8127
#define GL_POINT_FADE_THRESHOLD_SIZE 0x8128
#define GL_POINT_DISTANCE_ATTENUATION 0x8129
#define GL_COLOR_SUM 0x8458
#define GL_CURRENT_SECONDARY_COLOR 0x8459
#define GL_SECONDARY_COLOR_ARRAY_SIZE 0x845A
#define GL_SECONDARY_COLOR_ARRAY_TYPE 0x845B
#define GL_SECONDARY_COLOR_ARRAY_STRIDE 0x845C
#define GL_SECONDARY_COLOR_ARRAY_POINTER 0x845D
#define GL_SECONDARY_COLOR_ARRAY 0x845E
#define GL_BLEND_DST_RGB 0x80C8
#define GL_BLEND_SRC_RGB 0x80C9
#define GL_BLEND_DST_ALPHA 0x80CA
#define GL_BLEND_SRC_ALPHA 0x80CB
#define GL_INCR_WRAP 0x8507
#define GL_DECR_WRAP 0x8508
#define GL_TEXTURE_FILTER_CONTROL 0x8500
#define GL_TEXTURE_LOD_BIAS 0x8501
#define GL_MAX_TEXTURE_LOD_BIAS 0x84FD
#define GL_MIRRORED_REPEAT 0x8370
#define GL_BUFFER_SIZE 0x8764
#define GL_BUFFER_USAGE 0x8765
#define GL_QUERY_COUNTER_BITS 0x8864
#define GL_CURRENT_QUERY 0x8865
#define GL_QUERY_RESULT 0x8866
#define GL_QUERY_RESULT_AVAILABLE 0x8867
#define GL_ARRAY_BUFFER 0x8892
#define GL_ELEMENT_ARRAY_BUFFER 0x8893
#define GL_ARRAY_BUFFER_BINDING 0x8894
#define GL_ELEMENT_ARRAY_BUFFER_BINDING 0x8895
#define GL_VERTEX_ARRAY_BUFFER_BINDING 0x8896
#define GL_NORMAL_ARRAY_BUFFER_BINDING 0x8897
#define GL_COLOR_ARRAY_BUFFER_BINDING 0x8898
#define GL_INDEX_ARRAY_BUFFER_BINDING 0x8899
#define GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING 0x889A
#define GL_EDGE_FLAG_ARRAY_BUFFER_BINDING 0x889B
#define GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING 0x889C
#define GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING 0x889D
#define GL_WEIGHT_ARRAY_BUFFER_BINDING 0x889E
#define GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING 0x889F
#define GL_READ_ONLY 0x88B8
#define GL_WRITE_ONLY 0x88B9
#define GL_READ_WRITE 0x88BA
#define GL_BUFFER_ACCESS 0x88BB
#define GL_BUFFER_MAPPED 0x88BC
#define GL_BUFFER_MAP_POINTER 0x88BD
#define GL_STREAM_DRAW 0x88E0
#define GL_STREAM_READ 0x88E1
#define GL_STREAM_COPY 0x88E2
#define GL_STATIC_DRAW 0x88E4
#define GL_STATIC_READ 0x88E5
#define GL_STATIC_COPY 0x88E6
#define GL_DYNAMIC_DRAW 0x88E8
#define GL_DYNAMIC_READ 0x88E9
#define GL_DYNAMIC_COPY 0x88EA
#define GL_SAMPLES_PASSED 0x8914
#define GL_FOG_COORD_SRC GL_FOG_COORDINATE_SOURCE
#define GL_FOG_COORD GL_FOG_COORDINATE
#define GL_CURRENT_FOG_COORD GL_CURRENT_FOG_COORDINATE
#define GL_FOG_COORD_ARRAY_TYPE GL_FOG_COORDINATE_ARRAY_TYPE
#define GL_FOG_COORD_ARRAY_STRIDE GL_FOG_COORDINATE_ARRAY_STRIDE
#define GL_FOG_COORD_ARRAY_POINTER GL_FOG_COORDINATE_ARRAY_POINTER
#define GL_FOG_COORD_ARRAY GL_FOG_COORDINATE_ARRAY
#define GL_FOG_COORD_ARRAY_BUFFER_BINDING GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING
#define GL_SRC0_RGB GL_SOURCE0_RGB
#define GL_SRC1_RGB GL_SOURCE1_RGB
#define GL_SRC2_RGB GL_SOURCE2_RGB
#define GL_SRC0_ALPHA GL_SOURCE0_ALPHA
#define GL_SRC1_ALPHA GL_SOURCE1_ALPHA
#define GL_SRC2_ALPHA GL_SOURCE2_ALPHA
#define GL_BLEND_EQUATION_RGB GL_BLEND_EQUATION
#define GL_VERTEX_ATTRIB_ARRAY_ENABLED 0x8622
#define GL_VERTEX_ATTRIB_ARRAY_SIZE 0x8623
#define GL_VERTEX_ATTRIB_ARRAY_STRIDE 0x8624
#define GL_VERTEX_ATTRIB_ARRAY_TYPE 0x8625
#define GL_CURRENT_VERTEX_ATTRIB 0x8626
#define GL_VERTEX_PROGRAM_POINT_SIZE 0x8642
#define GL_VERTEX_PROGRAM_TWO_SIDE 0x8643
#define GL_VERTEX_ATTRIB_ARRAY_POINTER 0x8645
#define GL_STENCIL_BACK_FUNC 0x8800
#define GL_STENCIL_BACK_FAIL 0x8801
#define GL_STENCIL_BACK_PASS_DEPTH_FAIL 0x8802
#define GL_STENCIL_BACK_PASS_DEPTH_PASS 0x8803
#define GL_MAX_DRAW_BUFFERS 0x8824
#define GL_DRAW_BUFFER0 0x8825
#define GL_DRAW_BUFFER1 0x8826
#define GL_DRAW_BUFFER2 0x8827
#define GL_DRAW_BUFFER3 0x8828
#define GL_DRAW_BUFFER4 0x8829
#define GL_DRAW_BUFFER5 0x882A
#define GL_DRAW_BUFFER6 0x882B
#define GL_DRAW_BUFFER7 0x882C
#define GL_DRAW_BUFFER8 0x882D
#define GL_DRAW_BUFFER9 0x882E
#define GL_DRAW_BUFFER10 0x882F
#define GL_DRAW_BUFFER11 0x8830
#define GL_DRAW_BUFFER12 0x8831
#define GL_DRAW_BUFFER13 0x8832
#define GL_DRAW_BUFFER14 0x8833
#define GL_DRAW_BUFFER15 0x8834
#define GL_BLEND_EQUATION_ALPHA 0x883D
#define GL_POINT_SPRITE 0x8861
#define GL_COORD_REPLACE 0x8862
#define GL_MAX_VERTEX_ATTRIBS 0x8869
#define GL_VERTEX_ATTRIB_ARRAY_NORMALIZED 0x886A
#define GL_MAX_TEXTURE_COORDS 0x8871
#define GL_MAX_TEXTURE_IMAGE_UNITS 0x8872
#define GL_FRAGMENT_SHADER 0x8B30
#define GL_VERTEX_SHADER 0x8B31
#define GL_MAX_FRAGMENT_UNIFORM_COMPONENTS 0x8B49
#define GL_MAX_VERTEX_UNIFORM_COMPONENTS 0x8B4A
#define GL_MAX_VARYING_FLOATS 0x8B4B
#define GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS 0x8B4C
#define GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS 0x8B4D
#define GL_SHADER_TYPE 0x8B4F
#define GL_FLOAT_VEC2 0x8B50
#define GL_FLOAT_VEC3 0x8B51
#define GL_FLOAT_VEC4 0x8B52
#define GL_INT_VEC2 0x8B53
#define GL_INT_VEC3 0x8B54
#define GL_INT_VEC4 0x8B55
#define GL_BOOL 0x8B56
#define GL_BOOL_VEC2 0x8B57
#define GL_BOOL_VEC3 0x8B58
#define GL_BOOL_VEC4 0x8B59
#define GL_FLOAT_MAT2 0x8B5A
#define GL_FLOAT_MAT3 0x8B5B
#define GL_FLOAT_MAT4 0x8B5C
#define GL_SAMPLER_1D 0x8B5D
#define GL_SAMPLER_2D 0x8B5E
#define GL_SAMPLER_3D 0x8B5F
#define GL_SAMPLER_CUBE 0x8B60
#define GL_SAMPLER_1D_SHADOW 0x8B61
#define GL_SAMPLER_2D_SHADOW 0x8B62
#define GL_DELETE_STATUS 0x8B80
#define GL_COMPILE_STATUS 0x8B81
#define GL_LINK_STATUS 0x8B82
#define GL_VALIDATE_STATUS 0x8B83
#define GL_INFO_LOG_LENGTH 0x8B84
#define GL_ATTACHED_SHADERS 0x8B85
#define GL_ACTIVE_UNIFORMS 0x8B86
#define GL_ACTIVE_UNIFORM_MAX_LENGTH 0x8B87
#define GL_SHADER_SOURCE_LENGTH 0x8B88
#define GL_ACTIVE_ATTRIBUTES 0x8B89
#define GL_ACTIVE_ATTRIBUTE_MAX_LENGTH 0x8B8A
#define GL_FRAGMENT_SHADER_DERIVATIVE_HINT 0x8B8B
#define GL_SHADING_LANGUAGE_VERSION 0x8B8C
#define GL_CURRENT_PROGRAM 0x8B8D
#define GL_POINT_SPRITE_COORD_ORIGIN 0x8CA0
#define GL_LOWER_LEFT 0x8CA1
#define GL_UPPER_LEFT 0x8CA2
#define GL_STENCIL_BACK_REF 0x8CA3
#define GL_STENCIL_BACK_VALUE_MASK 0x8CA4
#define GL_STENCIL_BACK_WRITEMASK 0x8CA5
#define GL_CURRENT_RASTER_SECONDARY_COLOR 0x845F
#define GL_PIXEL_PACK_BUFFER 0x88EB
#define GL_PIXEL_UNPACK_BUFFER 0x88EC
#define GL_PIXEL_PACK_BUFFER_BINDING 0x88ED
#define GL_PIXEL_UNPACK_BUFFER_BINDING 0x88EF
#define GL_SRGB 0x8C40
#define GL_SRGB8 0x8C41
#define GL_SRGB_ALPHA 0x8C42
#define GL_SRGB8_ALPHA8 0x8C43
#define GL_SLUMINANCE_ALPHA 0x8C44
#define GL_SLUMINANCE8_ALPHA8 0x8C45
#define GL_SLUMINANCE 0x8C46
#define GL_SLUMINANCE8 0x8C47
#define GL_COMPRESSED_SRGB 0x8C48
#define GL_COMPRESSED_SRGB_ALPHA 0x8C49
#define GL_COMPRESSED_SLUMINANCE 0x8C4A
#define GL_COMPRESSED_SLUMINANCE_ALPHA 0x8C4B


static int gl_constant_array[] = {
GL_CONSTANT_COLOR,
GL_ONE_MINUS_CONSTANT_COLOR,
GL_CONSTANT_ALPHA,
GL_ONE_MINUS_CONSTANT_ALPHA,
GL_BLEND_COLOR,
GL_FUNC_ADD,
GL_MIN,
GL_MAX,
GL_BLEND_EQUATION,
GL_FUNC_SUBTRACT,
GL_FUNC_REVERSE_SUBTRACT,
GL_CONVOLUTION_1D,
GL_CONVOLUTION_2D,
GL_SEPARABLE_2D,
GL_CONVOLUTION_BORDER_MODE,
GL_CONVOLUTION_FILTER_SCALE,
GL_CONVOLUTION_FILTER_BIAS,
GL_REDUCE,
GL_CONVOLUTION_FORMAT,
GL_CONVOLUTION_WIDTH,
GL_CONVOLUTION_HEIGHT,
GL_MAX_CONVOLUTION_WIDTH,
GL_MAX_CONVOLUTION_HEIGHT,
GL_POST_CONVOLUTION_RED_SCALE,
GL_POST_CONVOLUTION_GREEN_SCALE,
GL_POST_CONVOLUTION_BLUE_SCALE,
GL_POST_CONVOLUTION_ALPHA_SCALE,
GL_POST_CONVOLUTION_RED_BIAS,
GL_POST_CONVOLUTION_GREEN_BIAS,
GL_POST_CONVOLUTION_BLUE_BIAS,
GL_POST_CONVOLUTION_ALPHA_BIAS,
GL_HISTOGRAM,
GL_PROXY_HISTOGRAM,
GL_HISTOGRAM_WIDTH,
GL_HISTOGRAM_FORMAT,
GL_HISTOGRAM_RED_SIZE,
GL_HISTOGRAM_GREEN_SIZE,
GL_HISTOGRAM_BLUE_SIZE,
GL_HISTOGRAM_ALPHA_SIZE,
GL_HISTOGRAM_LUMINANCE_SIZE,
GL_HISTOGRAM_SINK,
GL_MINMAX,
GL_MINMAX_FORMAT,
GL_MINMAX_SINK,
GL_TABLE_TOO_LARGE,
GL_COLOR_MATRIX,
GL_COLOR_MATRIX_STACK_DEPTH,
GL_MAX_COLOR_MATRIX_STACK_DEPTH,
GL_POST_COLOR_MATRIX_RED_SCALE,
GL_POST_COLOR_MATRIX_GREEN_SCALE,
GL_POST_COLOR_MATRIX_BLUE_SCALE,
GL_POST_COLOR_MATRIX_ALPHA_SCALE,
GL_POST_COLOR_MATRIX_RED_BIAS,
GL_POST_COLOR_MATRIX_GREEN_BIAS,
GL_POST_COLOR_MATRIX_BLUE_BIAS,
GL_POST_COLOR_MATRIX_ALPHA_BIAS,
GL_COLOR_TABLE,
GL_POST_CONVOLUTION_COLOR_TABLE,
GL_POST_COLOR_MATRIX_COLOR_TABLE,
GL_PROXY_COLOR_TABLE,
GL_PROXY_POST_CONVOLUTION_COLOR_TABLE,
GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE,
GL_COLOR_TABLE_SCALE,
GL_COLOR_TABLE_BIAS,
GL_COLOR_TABLE_FORMAT,
GL_COLOR_TABLE_WIDTH,
GL_COLOR_TABLE_RED_SIZE,
GL_COLOR_TABLE_GREEN_SIZE,
GL_COLOR_TABLE_BLUE_SIZE,
GL_COLOR_TABLE_ALPHA_SIZE,
GL_COLOR_TABLE_LUMINANCE_SIZE,
GL_COLOR_TABLE_INTENSITY_SIZE,
GL_IGNORE_BORDER,
GL_CONSTANT_BORDER,
GL_WRAP_BORDER,
GL_REPLICATE_BORDER,
GL_CONVOLUTION_BORDER_COLOR,
GL_MATRIX_PALETTE_ARB,
GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB,
GL_MAX_PALETTE_MATRICES_ARB,
GL_CURRENT_PALETTE_MATRIX_ARB,
GL_MATRIX_INDEX_ARRAY_ARB,
GL_CURRENT_MATRIX_INDEX_ARB,
GL_MATRIX_INDEX_ARRAY_SIZE_ARB,
GL_MATRIX_INDEX_ARRAY_TYPE_ARB,
GL_MATRIX_INDEX_ARRAY_STRIDE_ARB,
GL_MATRIX_INDEX_ARRAY_POINTER_ARB,
GL_TEXTURE0_ARB,
GL_TEXTURE1_ARB,
GL_TEXTURE2_ARB,
GL_TEXTURE3_ARB,
GL_TEXTURE4_ARB,
GL_TEXTURE5_ARB,
GL_TEXTURE6_ARB,
GL_TEXTURE7_ARB,
GL_TEXTURE8_ARB,
GL_TEXTURE9_ARB,
GL_TEXTURE10_ARB,
GL_TEXTURE11_ARB,
GL_TEXTURE12_ARB,
GL_TEXTURE13_ARB,
GL_TEXTURE14_ARB,
GL_TEXTURE15_ARB,
GL_TEXTURE16_ARB,
GL_TEXTURE17_ARB,
GL_TEXTURE18_ARB,
GL_TEXTURE19_ARB,
GL_TEXTURE20_ARB,
GL_TEXTURE21_ARB,
GL_TEXTURE22_ARB,
GL_TEXTURE23_ARB,
GL_TEXTURE24_ARB,
GL_TEXTURE25_ARB,
GL_TEXTURE26_ARB,
GL_TEXTURE27_ARB,
GL_TEXTURE28_ARB,
GL_TEXTURE29_ARB,
GL_TEXTURE30_ARB,
GL_TEXTURE31_ARB,
GL_ACTIVE_TEXTURE_ARB,
GL_CLIENT_ACTIVE_TEXTURE_ARB,
GL_MAX_TEXTURE_UNITS_ARB,
GL_MAX_VERTEX_UNITS_ARB,
GL_ACTIVE_VERTEX_UNITS_ARB,
GL_WEIGHT_SUM_UNITY_ARB,
GL_VERTEX_BLEND_ARB,
GL_CURRENT_WEIGHT_ARB,
GL_WEIGHT_ARRAY_TYPE_ARB,
GL_WEIGHT_ARRAY_STRIDE_ARB,
GL_WEIGHT_ARRAY_SIZE_ARB,
GL_WEIGHT_ARRAY_POINTER_ARB,
GL_WEIGHT_ARRAY_ARB,
GL_MODELVIEW0_ARB,
GL_MODELVIEW1_ARB,
GL_MODELVIEW2_ARB,
GL_MODELVIEW3_ARB,
GL_MODELVIEW4_ARB,
GL_MODELVIEW5_ARB,
GL_MODELVIEW6_ARB,
GL_MODELVIEW7_ARB,
GL_MODELVIEW8_ARB,
GL_MODELVIEW9_ARB,
GL_MODELVIEW10_ARB,
GL_MODELVIEW11_ARB,
GL_MODELVIEW12_ARB,
GL_MODELVIEW13_ARB,
GL_MODELVIEW14_ARB,
GL_MODELVIEW15_ARB,
GL_MODELVIEW16_ARB,
GL_MODELVIEW17_ARB,
GL_MODELVIEW18_ARB,
GL_MODELVIEW19_ARB,
GL_MODELVIEW20_ARB,
GL_MODELVIEW21_ARB,
GL_MODELVIEW22_ARB,
GL_MODELVIEW23_ARB,
GL_MODELVIEW24_ARB,
GL_MODELVIEW25_ARB,
GL_MODELVIEW26_ARB,
GL_MODELVIEW27_ARB,
GL_MODELVIEW28_ARB,
GL_MODELVIEW29_ARB,
GL_MODELVIEW30_ARB,
GL_MODELVIEW31_ARB,
GL_BUMP_ROT_MATRIX_ATI,
GL_BUMP_ROT_MATRIX_SIZE_ATI,
GL_BUMP_NUM_TEX_UNITS_ATI,
GL_BUMP_TEX_UNITS_ATI,
GL_DUDV_ATI,
GL_DU8DV8_ATI,
GL_BUMP_ENVMAP_ATI,
GL_BUMP_TARGET_ATI,
GL_PN_TRIANGLES_ATI,
GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI,
GL_PN_TRIANGLES_POINT_MODE_ATI,
GL_PN_TRIANGLES_NORMAL_MODE_ATI,
GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI,
GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI,
GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI,
GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI,
GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI,
GL_STENCIL_BACK_FUNC_ATI,
GL_STENCIL_BACK_FAIL_ATI,
GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI,
GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI,
GL_COMPRESSED_RGB_3DC_ATI,
GL_MAX_VERTEX_STREAMS_ATI,
GL_VERTEX_SOURCE_ATI,
GL_VERTEX_STREAM0_ATI,
GL_VERTEX_STREAM1_ATI,
GL_VERTEX_STREAM2_ATI,
GL_VERTEX_STREAM3_ATI,
GL_VERTEX_STREAM4_ATI,
GL_VERTEX_STREAM5_ATI,
GL_VERTEX_STREAM6_ATI,
GL_VERTEX_STREAM7_ATI,
GL_TEXTURE_POINT_MODE_ATIX,
GL_TEXTURE_POINT_ONE_COORD_ATIX,
GL_TEXTURE_POINT_SPRITE_ATIX,
GL_POINT_SPRITE_CULL_MODE_ATIX,
GL_POINT_SPRITE_CULL_CENTER_ATIX,
GL_POINT_SPRITE_CULL_CLIP_ATIX,
GL_MODULATE_ADD_ATIX,
GL_MODULATE_SIGNED_ADD_ATIX,
GL_MODULATE_SUBTRACT_ATIX,
GL_SECONDARY_COLOR_ATIX,
GL_TEXTURE_OUTPUT_RGB_ATIX,
GL_TEXTURE_OUTPUT_ALPHA_ATIX,
GL_OUTPUT_POINT_SIZE_ATIX,
GL_CG_VERTEX_SHADER_EXT,
GL_CG_FRAGMENT_SHADER_EXT,
GL_DEPTH_BOUNDS_TEST_EXT,
GL_DEPTH_BOUNDS_EXT,
GL_FOG_COORDINATE_SOURCE_EXT,
GL_FOG_COORDINATE_EXT,
GL_FRAGMENT_DEPTH_EXT,
GL_CURRENT_FOG_COORDINATE_EXT,
GL_FOG_COORDINATE_ARRAY_TYPE_EXT,
GL_FOG_COORDINATE_ARRAY_STRIDE_EXT,
GL_FOG_COORDINATE_ARRAY_POINTER_EXT,
GL_FOG_COORDINATE_ARRAY_EXT,
GL_PIXEL_PACK_BUFFER_EXT,
GL_PIXEL_UNPACK_BUFFER_EXT,
GL_PIXEL_PACK_BUFFER_BINDING_EXT,
GL_PIXEL_UNPACK_BUFFER_BINDING_EXT,
GL_COLOR_SUM_EXT,
GL_CURRENT_SECONDARY_COLOR_EXT,
GL_SECONDARY_COLOR_ARRAY_SIZE_EXT,
GL_SECONDARY_COLOR_ARRAY_TYPE_EXT,
GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT,
GL_SECONDARY_COLOR_ARRAY_POINTER_EXT,
GL_SECONDARY_COLOR_ARRAY_EXT,
GL_NORMAL_MAP_EXT,
GL_REFLECTION_MAP_EXT,
GL_TEXTURE_CUBE_MAP_EXT,
GL_TEXTURE_BINDING_CUBE_MAP_EXT,
GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT,
GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT,
GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT,
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT,
GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT,
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT,
GL_PROXY_TEXTURE_CUBE_MAP_EXT,
GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT,
GL_CLAMP_TO_EDGE_EXT,
GL_TEXTURE_RECTANGLE_EXT,
GL_TEXTURE_BINDING_RECTANGLE_EXT,
GL_PROXY_TEXTURE_RECTANGLE_EXT,
GL_MAX_RECTANGLE_TEXTURE_SIZE_EXT,
GL_VERTEX_SHADER_EXT,
GL_VERTEX_SHADER_BINDING_EXT,
GL_OP_INDEX_EXT,
GL_OP_NEGATE_EXT,
GL_OP_DOT3_EXT,
GL_OP_DOT4_EXT,
GL_OP_MUL_EXT,
GL_OP_ADD_EXT,
GL_OP_MADD_EXT,
GL_OP_FRAC_EXT,
GL_OP_MAX_EXT,
GL_OP_MIN_EXT,
GL_OP_SET_GE_EXT,
GL_OP_SET_LT_EXT,
GL_OP_CLAMP_EXT,
GL_OP_FLOOR_EXT,
GL_OP_ROUND_EXT,
GL_OP_EXP_BASE_2_EXT,
GL_OP_LOG_BASE_2_EXT,
GL_OP_POWER_EXT,
GL_OP_RECIP_EXT,
GL_OP_RECIP_SQRT_EXT,
GL_OP_SUB_EXT,
GL_OP_CROSS_PRODUCT_EXT,
GL_OP_MULTIPLY_MATRIX_EXT,
GL_OP_MOV_EXT,
GL_OUTPUT_VERTEX_EXT,
GL_OUTPUT_COLOR0_EXT,
GL_OUTPUT_COLOR1_EXT,
GL_OUTPUT_TEXTURE_COORD0_EXT,
GL_OUTPUT_TEXTURE_COORD1_EXT,
GL_OUTPUT_TEXTURE_COORD2_EXT,
GL_OUTPUT_TEXTURE_COORD3_EXT,
GL_OUTPUT_TEXTURE_COORD4_EXT,
GL_OUTPUT_TEXTURE_COORD5_EXT,
GL_OUTPUT_TEXTURE_COORD6_EXT,
GL_OUTPUT_TEXTURE_COORD7_EXT,
GL_OUTPUT_TEXTURE_COORD8_EXT,
GL_OUTPUT_TEXTURE_COORD9_EXT,
GL_OUTPUT_TEXTURE_COORD10_EXT,
GL_OUTPUT_TEXTURE_COORD11_EXT,
GL_OUTPUT_TEXTURE_COORD12_EXT,
GL_OUTPUT_TEXTURE_COORD13_EXT,
GL_OUTPUT_TEXTURE_COORD14_EXT,
GL_OUTPUT_TEXTURE_COORD15_EXT,
GL_OUTPUT_TEXTURE_COORD16_EXT,
GL_OUTPUT_TEXTURE_COORD17_EXT,
GL_OUTPUT_TEXTURE_COORD18_EXT,
GL_OUTPUT_TEXTURE_COORD19_EXT,
GL_OUTPUT_TEXTURE_COORD20_EXT,
GL_OUTPUT_TEXTURE_COORD21_EXT,
GL_OUTPUT_TEXTURE_COORD22_EXT,
GL_OUTPUT_TEXTURE_COORD23_EXT,
GL_OUTPUT_TEXTURE_COORD24_EXT,
GL_OUTPUT_TEXTURE_COORD25_EXT,
GL_OUTPUT_TEXTURE_COORD26_EXT,
GL_OUTPUT_TEXTURE_COORD27_EXT,
GL_OUTPUT_TEXTURE_COORD28_EXT,
GL_OUTPUT_TEXTURE_COORD29_EXT,
GL_OUTPUT_TEXTURE_COORD30_EXT,
GL_OUTPUT_TEXTURE_COORD31_EXT,
GL_OUTPUT_FOG_EXT,
GL_SCALAR_EXT,
GL_VECTOR_EXT,
GL_MATRIX_EXT,
GL_VARIANT_EXT,
GL_INVARIANT_EXT,
GL_LOCAL_CONSTANT_EXT,
GL_LOCAL_EXT,
GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT,
GL_MAX_VERTEX_SHADER_VARIANTS_EXT,
GL_MAX_VERTEX_SHADER_INVARIANTS_EXT,
GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT,
GL_MAX_VERTEX_SHADER_LOCALS_EXT,
GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT,
GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT,
GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT,
GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT,
GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT,
GL_VERTEX_SHADER_INSTRUCTIONS_EXT,
GL_VERTEX_SHADER_VARIANTS_EXT,
GL_VERTEX_SHADER_INVARIANTS_EXT,
GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT,
GL_VERTEX_SHADER_LOCALS_EXT,
GL_VERTEX_SHADER_OPTIMIZED_EXT,
GL_X_EXT,
GL_Y_EXT,
GL_Z_EXT,
GL_W_EXT,
GL_NEGATIVE_X_EXT,
GL_NEGATIVE_Y_EXT,
GL_NEGATIVE_Z_EXT,
GL_NEGATIVE_W_EXT,
GL_ZERO_EXT,
GL_ONE_EXT,
GL_NEGATIVE_ONE_EXT,
GL_NORMALIZED_RANGE_EXT,
GL_FULL_RANGE_EXT,
GL_CURRENT_VERTEX_EXT,
GL_MVP_MATRIX_EXT,
GL_VARIANT_VALUE_EXT,
GL_VARIANT_DATATYPE_EXT,
GL_VARIANT_ARRAY_STRIDE_EXT,
GL_VARIANT_ARRAY_TYPE_EXT,
GL_VARIANT_ARRAY_EXT,
GL_VARIANT_ARRAY_POINTER_EXT,
GL_INVARIANT_VALUE_EXT,
GL_INVARIANT_DATATYPE_EXT,
GL_LOCAL_CONSTANT_VALUE_EXT,
GL_LOCAL_CONSTANT_DATATYPE_EXT,
GL_KTX_FRONT_REGION,
GL_KTX_BACK_REGION,
GL_KTX_Z_REGION,
GL_KTX_STENCIL_REGION,
GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV,
GL_MAX_PROGRAM_CALL_DEPTH_NV,
GL_MAX_PROGRAM_IF_DEPTH_NV,
GL_MAX_PROGRAM_LOOP_DEPTH_NV,
GL_MAX_PROGRAM_LOOP_COUNT_NV,
GL_ACCUM,
GL_LOAD,
GL_RETURN,
GL_MULT,
GL_ADD,
GL_NEVER,
GL_LESS,
GL_EQUAL,
GL_LEQUAL,
GL_GREATER,
GL_NOTEQUAL,
GL_GEQUAL,
GL_ALWAYS,
GL_CURRENT_BIT,
GL_POINT_BIT,
GL_LINE_BIT,
GL_POLYGON_BIT,
GL_POLYGON_STIPPLE_BIT,
GL_PIXEL_MODE_BIT,
GL_LIGHTING_BIT,
GL_FOG_BIT,
GL_DEPTH_BUFFER_BIT,
GL_ACCUM_BUFFER_BIT,
GL_STENCIL_BUFFER_BIT,
GL_VIEWPORT_BIT,
GL_TRANSFORM_BIT,
GL_ENABLE_BIT,
GL_COLOR_BUFFER_BIT,
GL_HINT_BIT,
GL_EVAL_BIT,
GL_LIST_BIT,
GL_TEXTURE_BIT,
GL_SCISSOR_BIT,
GL_ALL_ATTRIB_BITS,
GL_POINTS,
GL_LINES,
GL_LINE_LOOP,
GL_LINE_STRIP,
GL_TRIANGLES,
GL_TRIANGLE_STRIP,
GL_TRIANGLE_FAN,
GL_QUADS,
GL_QUAD_STRIP,
GL_POLYGON,
GL_ZERO,
GL_ONE,
GL_SRC_COLOR,
GL_ONE_MINUS_SRC_COLOR,
GL_SRC_ALPHA,
GL_ONE_MINUS_SRC_ALPHA,
GL_DST_ALPHA,
GL_ONE_MINUS_DST_ALPHA,
GL_DST_COLOR,
GL_ONE_MINUS_DST_COLOR,
GL_SRC_ALPHA_SATURATE,
GL_TRUE,
GL_FALSE,
GL_CLIP_PLANE0,
GL_CLIP_PLANE1,
GL_CLIP_PLANE2,
GL_CLIP_PLANE3,
GL_CLIP_PLANE4,
GL_CLIP_PLANE5,
GL_BYTE,
GL_UNSIGNED_BYTE,
GL_SHORT,
GL_UNSIGNED_SHORT,
GL_INT,
GL_UNSIGNED_INT,
GL_FLOAT,
GL_2_BYTES,
GL_3_BYTES,
GL_4_BYTES,
GL_DOUBLE,
GL_NONE,
GL_FRONT_LEFT,
GL_FRONT_RIGHT,
GL_BACK_LEFT,
GL_BACK_RIGHT,
GL_FRONT,
GL_BACK,
GL_LEFT,
GL_RIGHT,
GL_FRONT_AND_BACK,
GL_AUX0,
GL_AUX1,
GL_AUX2,
GL_AUX3,
GL_NO_ERROR,
GL_INVALID_ENUM,
GL_INVALID_VALUE,
GL_INVALID_OPERATION,
GL_STACK_OVERFLOW,
GL_STACK_UNDERFLOW,
GL_OUT_OF_MEMORY,
GL_2D,
GL_3D,
GL_3D_COLOR,
GL_3D_COLOR_TEXTURE,
GL_4D_COLOR_TEXTURE,
GL_PASS_THROUGH_TOKEN,
GL_POINT_TOKEN,
GL_LINE_TOKEN,
GL_POLYGON_TOKEN,
GL_BITMAP_TOKEN,
GL_DRAW_PIXEL_TOKEN,
GL_COPY_PIXEL_TOKEN,
GL_LINE_RESET_TOKEN,
GL_EXP,
GL_EXP2,
GL_CW,
GL_CCW,
GL_COEFF,
GL_ORDER,
GL_DOMAIN,
GL_CURRENT_COLOR,
GL_CURRENT_INDEX,
GL_CURRENT_NORMAL,
GL_CURRENT_TEXTURE_COORDS,
GL_CURRENT_RASTER_COLOR,
GL_CURRENT_RASTER_INDEX,
GL_CURRENT_RASTER_TEXTURE_COORDS,
GL_CURRENT_RASTER_POSITION,
GL_CURRENT_RASTER_POSITION_VALID,
GL_CURRENT_RASTER_DISTANCE,
GL_POINT_SMOOTH,
GL_POINT_SIZE,
GL_POINT_SIZE_RANGE,
GL_POINT_SIZE_GRANULARITY,
GL_LINE_SMOOTH,
GL_LINE_WIDTH,
GL_LINE_WIDTH_RANGE,
GL_LINE_WIDTH_GRANULARITY,
GL_LINE_STIPPLE,
GL_LINE_STIPPLE_PATTERN,
GL_LINE_STIPPLE_REPEAT,
GL_LIST_MODE,
GL_MAX_LIST_NESTING,
GL_LIST_BASE,
GL_LIST_INDEX,
GL_POLYGON_MODE,
GL_POLYGON_SMOOTH,
GL_POLYGON_STIPPLE,
GL_EDGE_FLAG,
GL_CULL_FACE,
GL_CULL_FACE_MODE,
GL_FRONT_FACE,
GL_LIGHTING,
GL_LIGHT_MODEL_LOCAL_VIEWER,
GL_LIGHT_MODEL_TWO_SIDE,
GL_LIGHT_MODEL_AMBIENT,
GL_SHADE_MODEL,
GL_COLOR_MATERIAL_FACE,
GL_COLOR_MATERIAL_PARAMETER,
GL_COLOR_MATERIAL,
GL_FOG,
GL_FOG_INDEX,
GL_FOG_DENSITY,
GL_FOG_START,
GL_FOG_END,
GL_FOG_MODE,
GL_FOG_COLOR,
GL_DEPTH_RANGE,
GL_DEPTH_TEST,
GL_DEPTH_WRITEMASK,
GL_DEPTH_CLEAR_VALUE,
GL_DEPTH_FUNC,
GL_ACCUM_CLEAR_VALUE,
GL_STENCIL_TEST,
GL_STENCIL_CLEAR_VALUE,
GL_STENCIL_FUNC,
GL_STENCIL_VALUE_MASK,
GL_STENCIL_FAIL,
GL_STENCIL_PASS_DEPTH_FAIL,
GL_STENCIL_PASS_DEPTH_PASS,
GL_STENCIL_REF,
GL_STENCIL_WRITEMASK,
GL_MATRIX_MODE,
GL_NORMALIZE,
GL_VIEWPORT,
GL_MODELVIEW_STACK_DEPTH,
GL_PROJECTION_STACK_DEPTH,
GL_TEXTURE_STACK_DEPTH,
GL_MODELVIEW_MATRIX,
GL_PROJECTION_MATRIX,
GL_TEXTURE_MATRIX,
GL_ATTRIB_STACK_DEPTH,
GL_CLIENT_ATTRIB_STACK_DEPTH,
GL_ALPHA_TEST,
GL_ALPHA_TEST_FUNC,
GL_ALPHA_TEST_REF,
GL_DITHER,
GL_BLEND_DST,
GL_BLEND_SRC,
GL_BLEND,
GL_LOGIC_OP_MODE,
GL_INDEX_LOGIC_OP,
GL_COLOR_LOGIC_OP,
GL_AUX_BUFFERS,
GL_DRAW_BUFFER,
GL_READ_BUFFER,
GL_SCISSOR_BOX,
GL_SCISSOR_TEST,
GL_INDEX_CLEAR_VALUE,
GL_INDEX_WRITEMASK,
GL_COLOR_CLEAR_VALUE,
GL_COLOR_WRITEMASK,
GL_INDEX_MODE,
GL_RGBA_MODE,
GL_DOUBLEBUFFER,
GL_STEREO,
GL_RENDER_MODE,
GL_PERSPECTIVE_CORRECTION_HINT,
GL_POINT_SMOOTH_HINT,
GL_LINE_SMOOTH_HINT,
GL_POLYGON_SMOOTH_HINT,
GL_FOG_HINT,
GL_TEXTURE_GEN_S,
GL_TEXTURE_GEN_T,
GL_TEXTURE_GEN_R,
GL_TEXTURE_GEN_Q,
GL_PIXEL_MAP_I_TO_I,
GL_PIXEL_MAP_S_TO_S,
GL_PIXEL_MAP_I_TO_R,
GL_PIXEL_MAP_I_TO_G,
GL_PIXEL_MAP_I_TO_B,
GL_PIXEL_MAP_I_TO_A,
GL_PIXEL_MAP_R_TO_R,
GL_PIXEL_MAP_G_TO_G,
GL_PIXEL_MAP_B_TO_B,
GL_PIXEL_MAP_A_TO_A,
GL_PIXEL_MAP_I_TO_I_SIZE,
GL_PIXEL_MAP_S_TO_S_SIZE,
GL_PIXEL_MAP_I_TO_R_SIZE,
GL_PIXEL_MAP_I_TO_G_SIZE,
GL_PIXEL_MAP_I_TO_B_SIZE,
GL_PIXEL_MAP_I_TO_A_SIZE,
GL_PIXEL_MAP_R_TO_R_SIZE,
GL_PIXEL_MAP_G_TO_G_SIZE,
GL_PIXEL_MAP_B_TO_B_SIZE,
GL_PIXEL_MAP_A_TO_A_SIZE,
GL_UNPACK_SWAP_BYTES,
GL_UNPACK_LSB_FIRST,
GL_UNPACK_ROW_LENGTH,
GL_UNPACK_SKIP_ROWS,
GL_UNPACK_SKIP_PIXELS,
GL_UNPACK_ALIGNMENT,
GL_PACK_SWAP_BYTES,
GL_PACK_LSB_FIRST,
GL_PACK_ROW_LENGTH,
GL_PACK_SKIP_ROWS,
GL_PACK_SKIP_PIXELS,
GL_PACK_ALIGNMENT,
GL_MAP_COLOR,
GL_MAP_STENCIL,
GL_INDEX_SHIFT,
GL_INDEX_OFFSET,
GL_RED_SCALE,
GL_RED_BIAS,
GL_ZOOM_X,
GL_ZOOM_Y,
GL_GREEN_SCALE,
GL_GREEN_BIAS,
GL_BLUE_SCALE,
GL_BLUE_BIAS,
GL_ALPHA_SCALE,
GL_ALPHA_BIAS,
GL_DEPTH_SCALE,
GL_DEPTH_BIAS,
GL_MAX_EVAL_ORDER,
GL_MAX_LIGHTS,
GL_MAX_CLIP_PLANES,
GL_MAX_TEXTURE_SIZE,
GL_MAX_PIXEL_MAP_TABLE,
GL_MAX_ATTRIB_STACK_DEPTH,
GL_MAX_MODELVIEW_STACK_DEPTH,
GL_MAX_NAME_STACK_DEPTH,
GL_MAX_PROJECTION_STACK_DEPTH,
GL_MAX_TEXTURE_STACK_DEPTH,
GL_MAX_VIEWPORT_DIMS,
GL_MAX_CLIENT_ATTRIB_STACK_DEPTH,
GL_SUBPIXEL_BITS,
GL_INDEX_BITS,
GL_RED_BITS,
GL_GREEN_BITS,
GL_BLUE_BITS,
GL_ALPHA_BITS,
GL_DEPTH_BITS,
GL_STENCIL_BITS,
GL_ACCUM_RED_BITS,
GL_ACCUM_GREEN_BITS,
GL_ACCUM_BLUE_BITS,
GL_ACCUM_ALPHA_BITS,
GL_NAME_STACK_DEPTH,
GL_AUTO_NORMAL,
GL_MAP1_COLOR_4,
GL_MAP1_INDEX,
GL_MAP1_NORMAL,
GL_MAP1_TEXTURE_COORD_1,
GL_MAP1_TEXTURE_COORD_2,
GL_MAP1_TEXTURE_COORD_3,
GL_MAP1_TEXTURE_COORD_4,
GL_MAP1_VERTEX_3,
GL_MAP1_VERTEX_4,
GL_MAP2_COLOR_4,
GL_MAP2_INDEX,
GL_MAP2_NORMAL,
GL_MAP2_TEXTURE_COORD_1,
GL_MAP2_TEXTURE_COORD_2,
GL_MAP2_TEXTURE_COORD_3,
GL_MAP2_TEXTURE_COORD_4,
GL_MAP2_VERTEX_3,
GL_MAP2_VERTEX_4,
GL_MAP1_GRID_DOMAIN,
GL_MAP1_GRID_SEGMENTS,
GL_MAP2_GRID_DOMAIN,
GL_MAP2_GRID_SEGMENTS,
GL_TEXTURE_1D,
GL_TEXTURE_2D,
GL_FEEDBACK_BUFFER_POINTER,
GL_FEEDBACK_BUFFER_SIZE,
GL_FEEDBACK_BUFFER_TYPE,
GL_SELECTION_BUFFER_POINTER,
GL_SELECTION_BUFFER_SIZE,
GL_TEXTURE_WIDTH,
GL_TEXTURE_HEIGHT,
GL_TEXTURE_INTERNAL_FORMAT,
GL_TEXTURE_BORDER_COLOR,
GL_TEXTURE_BORDER,
GL_DONT_CARE,
GL_FASTEST,
GL_NICEST,
GL_LIGHT0,
GL_LIGHT1,
GL_LIGHT2,
GL_LIGHT3,
GL_LIGHT4,
GL_LIGHT5,
GL_LIGHT6,
GL_LIGHT7,
GL_AMBIENT,
GL_DIFFUSE,
GL_SPECULAR,
GL_POSITION,
GL_SPOT_DIRECTION,
GL_SPOT_EXPONENT,
GL_SPOT_CUTOFF,
GL_CONSTANT_ATTENUATION,
GL_LINEAR_ATTENUATION,
GL_QUADRATIC_ATTENUATION,
GL_COMPILE,
GL_COMPILE_AND_EXECUTE,
GL_CLEAR,
GL_AND,
GL_AND_REVERSE,
GL_COPY,
GL_AND_INVERTED,
GL_NOOP,
GL_XOR,
GL_OR,
GL_NOR,
GL_EQUIV,
GL_INVERT,
GL_OR_REVERSE,
GL_COPY_INVERTED,
GL_OR_INVERTED,
GL_NAND,
GL_SET,
GL_EMISSION,
GL_SHININESS,
GL_AMBIENT_AND_DIFFUSE,
GL_COLOR_INDEXES,
GL_MODELVIEW,
GL_PROJECTION,
GL_TEXTURE,
GL_COLOR,
GL_DEPTH,
GL_STENCIL,
GL_COLOR_INDEX,
GL_STENCIL_INDEX,
GL_DEPTH_COMPONENT,
GL_RED,
GL_GREEN,
GL_BLUE,
GL_ALPHA,
GL_RGB,
GL_RGBA,
GL_LUMINANCE,
GL_LUMINANCE_ALPHA,
GL_BITMAP,
GL_POINT,
GL_LINE,
GL_FILL,
GL_RENDER,
GL_FEEDBACK,
GL_SELECT,
GL_FLAT,
GL_SMOOTH,
GL_KEEP,
GL_REPLACE,
GL_INCR,
GL_DECR,
GL_VENDOR,
GL_RENDERER,
GL_VERSION,
GL_EXTENSIONS,
GL_S,
GL_T,
GL_R,
GL_Q,
GL_MODULATE,
GL_DECAL,
GL_TEXTURE_ENV_MODE,
GL_TEXTURE_ENV_COLOR,
GL_TEXTURE_ENV,
GL_EYE_LINEAR,
GL_OBJECT_LINEAR,
GL_SPHERE_MAP,
GL_TEXTURE_GEN_MODE,
GL_OBJECT_PLANE,
GL_EYE_PLANE,
GL_NEAREST,
GL_LINEAR,
GL_NEAREST_MIPMAP_NEAREST,
GL_LINEAR_MIPMAP_NEAREST,
GL_NEAREST_MIPMAP_LINEAR,
GL_LINEAR_MIPMAP_LINEAR,
GL_TEXTURE_MAG_FILTER,
GL_TEXTURE_MIN_FILTER,
GL_TEXTURE_WRAP_S,
GL_TEXTURE_WRAP_T,
GL_CLAMP,
GL_REPEAT,
GL_CLIENT_PIXEL_STORE_BIT,
GL_CLIENT_VERTEX_ARRAY_BIT,
GL_CLIENT_ALL_ATTRIB_BITS,
GL_POLYGON_OFFSET_FACTOR,
GL_POLYGON_OFFSET_UNITS,
GL_POLYGON_OFFSET_POINT,
GL_POLYGON_OFFSET_LINE,
GL_POLYGON_OFFSET_FILL,
GL_ALPHA4,
GL_ALPHA8,
GL_ALPHA12,
GL_ALPHA16,
GL_LUMINANCE4,
GL_LUMINANCE8,
GL_LUMINANCE12,
GL_LUMINANCE16,
GL_LUMINANCE4_ALPHA4,
GL_LUMINANCE6_ALPHA2,
GL_LUMINANCE8_ALPHA8,
GL_LUMINANCE12_ALPHA4,
GL_LUMINANCE12_ALPHA12,
GL_LUMINANCE16_ALPHA16,
GL_INTENSITY,
GL_INTENSITY4,
GL_INTENSITY8,
GL_INTENSITY12,
GL_INTENSITY16,
GL_R3_G3_B2,
GL_RGB4,
GL_RGB5,
GL_RGB8,
GL_RGB10,
GL_RGB12,
GL_RGB16,
GL_RGBA2,
GL_RGBA4,
GL_RGB5_A1,
GL_RGBA8,
GL_RGB10_A2,
GL_RGBA12,
GL_RGBA16,
GL_TEXTURE_RED_SIZE,
GL_TEXTURE_GREEN_SIZE,
GL_TEXTURE_BLUE_SIZE,
GL_TEXTURE_ALPHA_SIZE,
GL_TEXTURE_LUMINANCE_SIZE,
GL_TEXTURE_INTENSITY_SIZE,
GL_PROXY_TEXTURE_1D,
GL_PROXY_TEXTURE_2D,
GL_TEXTURE_PRIORITY,
GL_TEXTURE_RESIDENT,
GL_TEXTURE_BINDING_1D,
GL_TEXTURE_BINDING_2D,
GL_VERTEX_ARRAY,
GL_NORMAL_ARRAY,
GL_COLOR_ARRAY,
GL_INDEX_ARRAY,
GL_TEXTURE_COORD_ARRAY,
GL_EDGE_FLAG_ARRAY,
GL_VERTEX_ARRAY_SIZE,
GL_VERTEX_ARRAY_TYPE,
GL_VERTEX_ARRAY_STRIDE,
GL_NORMAL_ARRAY_TYPE,
GL_NORMAL_ARRAY_STRIDE,
GL_COLOR_ARRAY_SIZE,
GL_COLOR_ARRAY_TYPE,
GL_COLOR_ARRAY_STRIDE,
GL_INDEX_ARRAY_TYPE,
GL_INDEX_ARRAY_STRIDE,
GL_TEXTURE_COORD_ARRAY_SIZE,
GL_TEXTURE_COORD_ARRAY_TYPE,
GL_TEXTURE_COORD_ARRAY_STRIDE,
GL_EDGE_FLAG_ARRAY_STRIDE,
GL_VERTEX_ARRAY_POINTER,
GL_NORMAL_ARRAY_POINTER,
GL_COLOR_ARRAY_POINTER,
GL_INDEX_ARRAY_POINTER,
GL_TEXTURE_COORD_ARRAY_POINTER,
GL_EDGE_FLAG_ARRAY_POINTER,
GL_V2F,
GL_V3F,
GL_C4UB_V2F,
GL_C4UB_V3F,
GL_C3F_V3F,
GL_N3F_V3F,
GL_C4F_N3F_V3F,
GL_T2F_V3F,
GL_T4F_V4F,
GL_T2F_C4UB_V3F,
GL_T2F_C3F_V3F,
GL_T2F_N3F_V3F,
GL_T2F_C4F_N3F_V3F,
GL_T4F_C4F_N3F_V4F,
GL_LOGIC_OP,
GL_TEXTURE_COMPONENTS,
GL_COLOR_INDEX1_EXT,
GL_COLOR_INDEX2_EXT,
GL_COLOR_INDEX4_EXT,
GL_COLOR_INDEX8_EXT,
GL_COLOR_INDEX12_EXT,
GL_COLOR_INDEX16_EXT,
GL_UNSIGNED_BYTE_3_3_2,
GL_UNSIGNED_SHORT_4_4_4_4,
GL_UNSIGNED_SHORT_5_5_5_1,
GL_UNSIGNED_INT_8_8_8_8,
GL_UNSIGNED_INT_10_10_10_2,
GL_RESCALE_NORMAL,
GL_UNSIGNED_BYTE_2_3_3_REV,
GL_UNSIGNED_SHORT_5_6_5,
GL_UNSIGNED_SHORT_5_6_5_REV,
GL_UNSIGNED_SHORT_4_4_4_4_REV,
GL_UNSIGNED_SHORT_1_5_5_5_REV,
GL_UNSIGNED_INT_8_8_8_8_REV,
GL_UNSIGNED_INT_2_10_10_10_REV,
GL_BGR,
GL_BGRA,
GL_MAX_ELEMENTS_VERTICES,
GL_MAX_ELEMENTS_INDICES,
GL_CLAMP_TO_EDGE,
GL_TEXTURE_MIN_LOD,
GL_TEXTURE_MAX_LOD,
GL_TEXTURE_BASE_LEVEL,
GL_TEXTURE_MAX_LEVEL,
GL_LIGHT_MODEL_COLOR_CONTROL,
GL_SINGLE_COLOR,
GL_SEPARATE_SPECULAR_COLOR,
GL_SMOOTH_POINT_SIZE_RANGE,
GL_SMOOTH_POINT_SIZE_GRANULARITY,
GL_SMOOTH_LINE_WIDTH_RANGE,
GL_SMOOTH_LINE_WIDTH_GRANULARITY,
GL_ALIASED_POINT_SIZE_RANGE,
GL_ALIASED_LINE_WIDTH_RANGE,
GL_PACK_SKIP_IMAGES,
GL_PACK_IMAGE_HEIGHT,
GL_UNPACK_SKIP_IMAGES,
GL_UNPACK_IMAGE_HEIGHT,
GL_TEXTURE_3D,
GL_PROXY_TEXTURE_3D,
GL_TEXTURE_DEPTH,
GL_TEXTURE_WRAP_R,
GL_MAX_3D_TEXTURE_SIZE,
GL_TEXTURE_BINDING_3D,
GL_TEXTURE0,
GL_TEXTURE1,
GL_TEXTURE2,
GL_TEXTURE3,
GL_TEXTURE4,
GL_TEXTURE5,
GL_TEXTURE6,
GL_TEXTURE7,
GL_TEXTURE8,
GL_TEXTURE9,
GL_TEXTURE10,
GL_TEXTURE11,
GL_TEXTURE12,
GL_TEXTURE13,
GL_TEXTURE14,
GL_TEXTURE15,
GL_TEXTURE16,
GL_TEXTURE17,
GL_TEXTURE18,
GL_TEXTURE19,
GL_TEXTURE20,
GL_TEXTURE21,
GL_TEXTURE22,
GL_TEXTURE23,
GL_TEXTURE24,
GL_TEXTURE25,
GL_TEXTURE26,
GL_TEXTURE27,
GL_TEXTURE28,
GL_TEXTURE29,
GL_TEXTURE30,
GL_TEXTURE31,
GL_ACTIVE_TEXTURE,
GL_CLIENT_ACTIVE_TEXTURE,
GL_MAX_TEXTURE_UNITS,
GL_NORMAL_MAP,
GL_REFLECTION_MAP,
GL_TEXTURE_CUBE_MAP,
GL_TEXTURE_BINDING_CUBE_MAP,
GL_TEXTURE_CUBE_MAP_POSITIVE_X,
GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
GL_TEXTURE_CUBE_MAP_POSITIVE_Y,
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,
GL_TEXTURE_CUBE_MAP_POSITIVE_Z,
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z,
GL_PROXY_TEXTURE_CUBE_MAP,
GL_MAX_CUBE_MAP_TEXTURE_SIZE,
GL_COMPRESSED_ALPHA,
GL_COMPRESSED_LUMINANCE,
GL_COMPRESSED_LUMINANCE_ALPHA,
GL_COMPRESSED_INTENSITY,
GL_COMPRESSED_RGB,
GL_COMPRESSED_RGBA,
GL_TEXTURE_COMPRESSION_HINT,
GL_TEXTURE_COMPRESSED_IMAGE_SIZE,
GL_TEXTURE_COMPRESSED,
GL_NUM_COMPRESSED_TEXTURE_FORMATS,
GL_COMPRESSED_TEXTURE_FORMATS,
GL_MULTISAMPLE,
GL_SAMPLE_ALPHA_TO_COVERAGE,
GL_SAMPLE_ALPHA_TO_ONE,
GL_SAMPLE_COVERAGE,
GL_SAMPLE_BUFFERS,
GL_SAMPLES,
GL_SAMPLE_COVERAGE_VALUE,
GL_SAMPLE_COVERAGE_INVERT,
GL_MULTISAMPLE_BIT,
GL_TRANSPOSE_MODELVIEW_MATRIX,
GL_TRANSPOSE_PROJECTION_MATRIX,
GL_TRANSPOSE_TEXTURE_MATRIX,
GL_TRANSPOSE_COLOR_MATRIX,
GL_COMBINE,
GL_COMBINE_RGB,
GL_COMBINE_ALPHA,
GL_SOURCE0_RGB,
GL_SOURCE1_RGB,
GL_SOURCE2_RGB,
GL_SOURCE0_ALPHA,
GL_SOURCE1_ALPHA,
GL_SOURCE2_ALPHA,
GL_OPERAND0_RGB,
GL_OPERAND1_RGB,
GL_OPERAND2_RGB,
GL_OPERAND0_ALPHA,
GL_OPERAND1_ALPHA,
GL_OPERAND2_ALPHA,
GL_RGB_SCALE,
GL_ADD_SIGNED,
GL_INTERPOLATE,
GL_SUBTRACT,
GL_CONSTANT,
GL_PRIMARY_COLOR,
GL_PREVIOUS,
GL_DOT3_RGB,
GL_DOT3_RGBA,
GL_CLAMP_TO_BORDER,
GL_GENERATE_MIPMAP,
GL_GENERATE_MIPMAP_HINT,
GL_DEPTH_COMPONENT16,
GL_DEPTH_COMPONENT24,
GL_DEPTH_COMPONENT32,
GL_TEXTURE_DEPTH_SIZE,
GL_DEPTH_TEXTURE_MODE,
GL_TEXTURE_COMPARE_MODE,
GL_TEXTURE_COMPARE_FUNC,
GL_COMPARE_R_TO_TEXTURE,
GL_FOG_COORDINATE_SOURCE,
GL_FOG_COORDINATE,
GL_FRAGMENT_DEPTH,
GL_CURRENT_FOG_COORDINATE,
GL_FOG_COORDINATE_ARRAY_TYPE,
GL_FOG_COORDINATE_ARRAY_STRIDE,
GL_FOG_COORDINATE_ARRAY_POINTER,
GL_FOG_COORDINATE_ARRAY,
GL_POINT_SIZE_MIN,
GL_POINT_SIZE_MAX,
GL_POINT_FADE_THRESHOLD_SIZE,
GL_POINT_DISTANCE_ATTENUATION,
GL_COLOR_SUM,
GL_CURRENT_SECONDARY_COLOR,
GL_SECONDARY_COLOR_ARRAY_SIZE,
GL_SECONDARY_COLOR_ARRAY_TYPE,
GL_SECONDARY_COLOR_ARRAY_STRIDE,
GL_SECONDARY_COLOR_ARRAY_POINTER,
GL_SECONDARY_COLOR_ARRAY,
GL_BLEND_DST_RGB,
GL_BLEND_SRC_RGB,
GL_BLEND_DST_ALPHA,
GL_BLEND_SRC_ALPHA,
GL_INCR_WRAP,
GL_DECR_WRAP,
GL_TEXTURE_FILTER_CONTROL,
GL_TEXTURE_LOD_BIAS,
GL_MAX_TEXTURE_LOD_BIAS,
GL_MIRRORED_REPEAT,
GL_BUFFER_SIZE,
GL_BUFFER_USAGE,
GL_QUERY_COUNTER_BITS,
GL_CURRENT_QUERY,
GL_QUERY_RESULT,
GL_QUERY_RESULT_AVAILABLE,
GL_ARRAY_BUFFER,
GL_ELEMENT_ARRAY_BUFFER,
GL_ARRAY_BUFFER_BINDING,
GL_ELEMENT_ARRAY_BUFFER_BINDING,
GL_VERTEX_ARRAY_BUFFER_BINDING,
GL_NORMAL_ARRAY_BUFFER_BINDING,
GL_COLOR_ARRAY_BUFFER_BINDING,
GL_INDEX_ARRAY_BUFFER_BINDING,
GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING,
GL_EDGE_FLAG_ARRAY_BUFFER_BINDING,
GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING,
GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING,
GL_WEIGHT_ARRAY_BUFFER_BINDING,
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING,
GL_READ_ONLY,
GL_WRITE_ONLY,
GL_READ_WRITE,
GL_BUFFER_ACCESS,
GL_BUFFER_MAPPED,
GL_BUFFER_MAP_POINTER,
GL_STREAM_DRAW,
GL_STREAM_READ,
GL_STREAM_COPY,
GL_STATIC_DRAW,
GL_STATIC_READ,
GL_STATIC_COPY,
GL_DYNAMIC_DRAW,
GL_DYNAMIC_READ,
GL_DYNAMIC_COPY,
GL_SAMPLES_PASSED,
GL_FOG_COORD_SRC,
GL_FOG_COORD,
GL_CURRENT_FOG_COORD,
GL_FOG_COORD_ARRAY_TYPE,
GL_FOG_COORD_ARRAY_STRIDE,
GL_FOG_COORD_ARRAY_POINTER,
GL_FOG_COORD_ARRAY,
GL_FOG_COORD_ARRAY_BUFFER_BINDING,
GL_SRC0_RGB,
GL_SRC1_RGB,
GL_SRC2_RGB,
GL_SRC0_ALPHA,
GL_SRC1_ALPHA,
GL_SRC2_ALPHA,
GL_BLEND_EQUATION_RGB,
GL_VERTEX_ATTRIB_ARRAY_ENABLED,
GL_VERTEX_ATTRIB_ARRAY_SIZE,
GL_VERTEX_ATTRIB_ARRAY_STRIDE,
GL_VERTEX_ATTRIB_ARRAY_TYPE,
GL_CURRENT_VERTEX_ATTRIB,
GL_VERTEX_PROGRAM_POINT_SIZE,
GL_VERTEX_PROGRAM_TWO_SIDE,
GL_VERTEX_ATTRIB_ARRAY_POINTER,
GL_STENCIL_BACK_FUNC,
GL_STENCIL_BACK_FAIL,
GL_STENCIL_BACK_PASS_DEPTH_FAIL,
GL_STENCIL_BACK_PASS_DEPTH_PASS,
GL_MAX_DRAW_BUFFERS,
GL_DRAW_BUFFER0,
GL_DRAW_BUFFER1,
GL_DRAW_BUFFER2,
GL_DRAW_BUFFER3,
GL_DRAW_BUFFER4,
GL_DRAW_BUFFER5,
GL_DRAW_BUFFER6,
GL_DRAW_BUFFER7,
GL_DRAW_BUFFER8,
GL_DRAW_BUFFER9,
GL_DRAW_BUFFER10,
GL_DRAW_BUFFER11,
GL_DRAW_BUFFER12,
GL_DRAW_BUFFER13,
GL_DRAW_BUFFER14,
GL_DRAW_BUFFER15,
GL_BLEND_EQUATION_ALPHA,
GL_POINT_SPRITE,
GL_COORD_REPLACE,
GL_MAX_VERTEX_ATTRIBS,
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED,
GL_MAX_TEXTURE_COORDS,
GL_MAX_TEXTURE_IMAGE_UNITS,
GL_FRAGMENT_SHADER,
GL_VERTEX_SHADER,
GL_MAX_FRAGMENT_UNIFORM_COMPONENTS,
GL_MAX_VERTEX_UNIFORM_COMPONENTS,
GL_MAX_VARYING_FLOATS,
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS,
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS,
GL_SHADER_TYPE,
GL_FLOAT_VEC2,
GL_FLOAT_VEC3,
GL_FLOAT_VEC4,
GL_INT_VEC2,
GL_INT_VEC3,
GL_INT_VEC4,
GL_BOOL,
GL_BOOL_VEC2,
GL_BOOL_VEC3,
GL_BOOL_VEC4,
GL_FLOAT_MAT2,
GL_FLOAT_MAT3,
GL_FLOAT_MAT4,
GL_SAMPLER_1D,
GL_SAMPLER_2D,
GL_SAMPLER_3D,
GL_SAMPLER_CUBE,
GL_SAMPLER_1D_SHADOW,
GL_SAMPLER_2D_SHADOW,
GL_DELETE_STATUS,
GL_COMPILE_STATUS,
GL_LINK_STATUS,
GL_VALIDATE_STATUS,
GL_INFO_LOG_LENGTH,
GL_ATTACHED_SHADERS,
GL_ACTIVE_UNIFORMS,
GL_ACTIVE_UNIFORM_MAX_LENGTH,
GL_SHADER_SOURCE_LENGTH,
GL_ACTIVE_ATTRIBUTES,
GL_ACTIVE_ATTRIBUTE_MAX_LENGTH,
GL_FRAGMENT_SHADER_DERIVATIVE_HINT,
GL_SHADING_LANGUAGE_VERSION,
GL_CURRENT_PROGRAM,
GL_POINT_SPRITE_COORD_ORIGIN,
GL_LOWER_LEFT,
GL_UPPER_LEFT,
GL_STENCIL_BACK_REF,
GL_STENCIL_BACK_VALUE_MASK,
GL_STENCIL_BACK_WRITEMASK,
GL_CURRENT_RASTER_SECONDARY_COLOR,
GL_PIXEL_PACK_BUFFER,
GL_PIXEL_UNPACK_BUFFER,
GL_PIXEL_PACK_BUFFER_BINDING,
GL_PIXEL_UNPACK_BUFFER_BINDING,
GL_SRGB,
GL_SRGB8,
GL_SRGB_ALPHA,
GL_SRGB8_ALPHA8,
GL_SLUMINANCE_ALPHA,
GL_SLUMINANCE8_ALPHA8,
GL_SLUMINANCE,
GL_SLUMINANCE8,
GL_COMPRESSED_SRGB,
GL_COMPRESSED_SRGB_ALPHA,
GL_COMPRESSED_SLUMINANCE,
GL_COMPRESSED_SLUMINANCE_ALPHA,

};

static GLenum get_glenum(value v)
{
	GLenum ret = 0;
    int i = Int_val(v);
	if(i >= 0 && i< sizeof(gl_constant_array))
	{
		ret = gl_constant_array[i];
	}
	else
	FailWith("Enumeration out of bounds");
	return ret;
} 

value glstub_glenum_of_int(value v)
{
	CAMLparam1(v);
	int lv = Int_val(v);
	int ret = -1;
	int i = 0;
	for(i=0; i< sizeof(gl_constant_array); i++ )
	{
		if(gl_constant_array[i] == lv)ret = i;
	}
	if(ret == -1)failwith("Unknown glEnum");
	CAMLreturn (Val_int(ret));
}

value glstub_int_of_glenum(value v)
{
		 CAMLparam1(v);
		 GLenum ret = get_glenum(v);
		 CAMLreturn(Val_int(ret));
}


/* glColorTable */
typedef void APIENTRY (*pstub_glColorTable)(GLenum,GLenum,GLsizei,GLenum,GLenum,GLvoid*);
static pstub_glColorTable stub_glColorTable = NULL;
static int loaded_glColorTable = 0;

value glstub_glColorTable(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLsizei lv2 =  Int_val(v2);
	GLenum lv3 =  get_glenum(v3);
	GLenum lv4 =  get_glenum(v4);
	GLvoid* lv5 =  Data_bigarray_val(v5);

	if(stub_glColorTable){(*stub_glColorTable)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glColorTable) { 
			loaded_glColorTable = 1;
			init_lib(); 
			stub_glColorTable=(pstub_glColorTable)get_proc_address("glColorTable");
			if(stub_glColorTable){(*stub_glColorTable)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glColorTable"); 
			} 
		} else { 
			FailWith("glColorTable not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glColorTable_byte(value * argv, int n)
{
	return glstub_glColorTable(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glColorSubTable */
typedef void APIENTRY (*pstub_glColorSubTable)(GLenum,GLsizei,GLsizei,GLenum,GLenum,GLvoid*);
static pstub_glColorSubTable stub_glColorSubTable = NULL;
static int loaded_glColorSubTable = 0;

value glstub_glColorSubTable(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizei lv1 =  Int_val(v1);
	GLsizei lv2 =  Int_val(v2);
	GLenum lv3 =  get_glenum(v3);
	GLenum lv4 =  get_glenum(v4);
	GLvoid* lv5 =  Data_bigarray_val(v5);

	if(stub_glColorSubTable){(*stub_glColorSubTable)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glColorSubTable) { 
			loaded_glColorSubTable = 1;
			init_lib(); 
			stub_glColorSubTable=(pstub_glColorSubTable)get_proc_address("glColorSubTable");
			if(stub_glColorSubTable){(*stub_glColorSubTable)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glColorSubTable"); 
			} 
		} else { 
			FailWith("glColorSubTable not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glColorSubTable_byte(value * argv, int n)
{
	return glstub_glColorSubTable(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glColorTableParameteriv */
typedef void APIENTRY (*pstub_glColorTableParameteriv)(GLenum,GLenum,GLint*);
static pstub_glColorTableParameteriv stub_glColorTableParameteriv = NULL;
static int loaded_glColorTableParameteriv = 0;

value glstub_glColorTableParameteriv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glColorTableParameteriv){(*stub_glColorTableParameteriv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glColorTableParameteriv) { 
			loaded_glColorTableParameteriv = 1;
			init_lib(); 
			stub_glColorTableParameteriv=(pstub_glColorTableParameteriv)get_proc_address("glColorTableParameteriv");
			if(stub_glColorTableParameteriv){(*stub_glColorTableParameteriv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glColorTableParameteriv"); 
			} 
		} else { 
			FailWith("glColorTableParameteriv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColorTableParameterfv */
typedef void APIENTRY (*pstub_glColorTableParameterfv)(GLenum,GLenum,GLfloat*);
static pstub_glColorTableParameterfv stub_glColorTableParameterfv = NULL;
static int loaded_glColorTableParameterfv = 0;

value glstub_glColorTableParameterfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glColorTableParameterfv){(*stub_glColorTableParameterfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glColorTableParameterfv) { 
			loaded_glColorTableParameterfv = 1;
			init_lib(); 
			stub_glColorTableParameterfv=(pstub_glColorTableParameterfv)get_proc_address("glColorTableParameterfv");
			if(stub_glColorTableParameterfv){(*stub_glColorTableParameterfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glColorTableParameterfv"); 
			} 
		} else { 
			FailWith("glColorTableParameterfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glCopyColorSubTable */
typedef void APIENTRY (*pstub_glCopyColorSubTable)(GLenum,GLsizei,GLint,GLint,GLsizei);
static pstub_glCopyColorSubTable stub_glCopyColorSubTable = NULL;
static int loaded_glCopyColorSubTable = 0;

value glstub_glCopyColorSubTable(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizei lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLsizei lv4 =  Int_val(v4);

	if(stub_glCopyColorSubTable){(*stub_glCopyColorSubTable)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glCopyColorSubTable) { 
			loaded_glCopyColorSubTable = 1;
			init_lib(); 
			stub_glCopyColorSubTable=(pstub_glCopyColorSubTable)get_proc_address("glCopyColorSubTable");
			if(stub_glCopyColorSubTable){(*stub_glCopyColorSubTable)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glCopyColorSubTable"); 
			} 
		} else { 
			FailWith("glCopyColorSubTable not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glCopyColorTable */
typedef void APIENTRY (*pstub_glCopyColorTable)(GLenum,GLenum,GLint,GLint,GLsizei);
static pstub_glCopyColorTable stub_glCopyColorTable = NULL;
static int loaded_glCopyColorTable = 0;

value glstub_glCopyColorTable(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLsizei lv4 =  Int_val(v4);

	if(stub_glCopyColorTable){(*stub_glCopyColorTable)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glCopyColorTable) { 
			loaded_glCopyColorTable = 1;
			init_lib(); 
			stub_glCopyColorTable=(pstub_glCopyColorTable)get_proc_address("glCopyColorTable");
			if(stub_glCopyColorTable){(*stub_glCopyColorTable)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glCopyColorTable"); 
			} 
		} else { 
			FailWith("glCopyColorTable not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetColorTable */
typedef void APIENTRY (*pstub_glGetColorTable)(GLenum,GLenum,GLenum,GLvoid*);
static pstub_glGetColorTable stub_glGetColorTable = NULL;
static int loaded_glGetColorTable = 0;

value glstub_glGetColorTable(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLenum lv2 =  get_glenum(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);

	if(stub_glGetColorTable){(*stub_glGetColorTable)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glGetColorTable) { 
			loaded_glGetColorTable = 1;
			init_lib(); 
			stub_glGetColorTable=(pstub_glGetColorTable)get_proc_address("glGetColorTable");
			if(stub_glGetColorTable){(*stub_glGetColorTable)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glGetColorTable"); 
			} 
		} else { 
			FailWith("glGetColorTable not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetColorTableParameterfv */
typedef void APIENTRY (*pstub_glGetColorTableParameterfv)(GLenum,GLenum,GLfloat*);
static pstub_glGetColorTableParameterfv stub_glGetColorTableParameterfv = NULL;
static int loaded_glGetColorTableParameterfv = 0;

value glstub_glGetColorTableParameterfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetColorTableParameterfv){(*stub_glGetColorTableParameterfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetColorTableParameterfv) { 
			loaded_glGetColorTableParameterfv = 1;
			init_lib(); 
			stub_glGetColorTableParameterfv=(pstub_glGetColorTableParameterfv)get_proc_address("glGetColorTableParameterfv");
			if(stub_glGetColorTableParameterfv){(*stub_glGetColorTableParameterfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetColorTableParameterfv"); 
			} 
		} else { 
			FailWith("glGetColorTableParameterfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetColorTableParameteriv */
typedef void APIENTRY (*pstub_glGetColorTableParameteriv)(GLenum,GLenum,GLint*);
static pstub_glGetColorTableParameteriv stub_glGetColorTableParameteriv = NULL;
static int loaded_glGetColorTableParameteriv = 0;

value glstub_glGetColorTableParameteriv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetColorTableParameteriv){(*stub_glGetColorTableParameteriv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetColorTableParameteriv) { 
			loaded_glGetColorTableParameteriv = 1;
			init_lib(); 
			stub_glGetColorTableParameteriv=(pstub_glGetColorTableParameteriv)get_proc_address("glGetColorTableParameteriv");
			if(stub_glGetColorTableParameteriv){(*stub_glGetColorTableParameteriv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetColorTableParameteriv"); 
			} 
		} else { 
			FailWith("glGetColorTableParameteriv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glHistogram */
typedef void APIENTRY (*pstub_glHistogram)(GLenum,GLsizei,GLenum,GLboolean);
static pstub_glHistogram stub_glHistogram = NULL;
static int loaded_glHistogram = 0;

value glstub_glHistogram(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizei lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLboolean lv3 =  Bool_val(v3);

	if(stub_glHistogram){(*stub_glHistogram)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glHistogram) { 
			loaded_glHistogram = 1;
			init_lib(); 
			stub_glHistogram=(pstub_glHistogram)get_proc_address("glHistogram");
			if(stub_glHistogram){(*stub_glHistogram)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glHistogram"); 
			} 
		} else { 
			FailWith("glHistogram not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glResetHistogram */
typedef void APIENTRY (*pstub_glResetHistogram)(GLenum);
static pstub_glResetHistogram stub_glResetHistogram = NULL;
static int loaded_glResetHistogram = 0;

value glstub_glResetHistogram(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glResetHistogram){(*stub_glResetHistogram)(lv0);}
	else { 
		if(!loaded_glResetHistogram) { 
			loaded_glResetHistogram = 1;
			init_lib(); 
			stub_glResetHistogram=(pstub_glResetHistogram)get_proc_address("glResetHistogram");
			if(stub_glResetHistogram){(*stub_glResetHistogram)(lv0);}
			else { 
				FailWith("Cannot load glResetHistogram"); 
			} 
		} else { 
			FailWith("glResetHistogram not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetHistogram */
typedef void APIENTRY (*pstub_glGetHistogram)(GLenum,GLboolean,GLenum,GLenum,GLvoid*);
static pstub_glGetHistogram stub_glGetHistogram = NULL;
static int loaded_glGetHistogram = 0;

value glstub_glGetHistogram(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLboolean lv1 =  Bool_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLenum lv3 =  get_glenum(v3);
	GLvoid* lv4 =  Data_bigarray_val(v4);

	if(stub_glGetHistogram){(*stub_glGetHistogram)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glGetHistogram) { 
			loaded_glGetHistogram = 1;
			init_lib(); 
			stub_glGetHistogram=(pstub_glGetHistogram)get_proc_address("glGetHistogram");
			if(stub_glGetHistogram){(*stub_glGetHistogram)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glGetHistogram"); 
			} 
		} else { 
			FailWith("glGetHistogram not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetHistogramParameterfv */
typedef void APIENTRY (*pstub_glGetHistogramParameterfv)(GLenum,GLenum,GLfloat*);
static pstub_glGetHistogramParameterfv stub_glGetHistogramParameterfv = NULL;
static int loaded_glGetHistogramParameterfv = 0;

value glstub_glGetHistogramParameterfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetHistogramParameterfv){(*stub_glGetHistogramParameterfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetHistogramParameterfv) { 
			loaded_glGetHistogramParameterfv = 1;
			init_lib(); 
			stub_glGetHistogramParameterfv=(pstub_glGetHistogramParameterfv)get_proc_address("glGetHistogramParameterfv");
			if(stub_glGetHistogramParameterfv){(*stub_glGetHistogramParameterfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetHistogramParameterfv"); 
			} 
		} else { 
			FailWith("glGetHistogramParameterfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetHistogramParameteriv */
typedef void APIENTRY (*pstub_glGetHistogramParameteriv)(GLenum,GLenum,GLint*);
static pstub_glGetHistogramParameteriv stub_glGetHistogramParameteriv = NULL;
static int loaded_glGetHistogramParameteriv = 0;

value glstub_glGetHistogramParameteriv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetHistogramParameteriv){(*stub_glGetHistogramParameteriv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetHistogramParameteriv) { 
			loaded_glGetHistogramParameteriv = 1;
			init_lib(); 
			stub_glGetHistogramParameteriv=(pstub_glGetHistogramParameteriv)get_proc_address("glGetHistogramParameteriv");
			if(stub_glGetHistogramParameteriv){(*stub_glGetHistogramParameteriv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetHistogramParameteriv"); 
			} 
		} else { 
			FailWith("glGetHistogramParameteriv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMinmax */
typedef void APIENTRY (*pstub_glMinmax)(GLenum,GLenum,GLboolean);
static pstub_glMinmax stub_glMinmax = NULL;
static int loaded_glMinmax = 0;

value glstub_glMinmax(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLboolean lv2 =  Bool_val(v2);

	if(stub_glMinmax){(*stub_glMinmax)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMinmax) { 
			loaded_glMinmax = 1;
			init_lib(); 
			stub_glMinmax=(pstub_glMinmax)get_proc_address("glMinmax");
			if(stub_glMinmax){(*stub_glMinmax)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMinmax"); 
			} 
		} else { 
			FailWith("glMinmax not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glResetMinmax */
typedef void APIENTRY (*pstub_glResetMinmax)(GLenum);
static pstub_glResetMinmax stub_glResetMinmax = NULL;
static int loaded_glResetMinmax = 0;

value glstub_glResetMinmax(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glResetMinmax){(*stub_glResetMinmax)(lv0);}
	else { 
		if(!loaded_glResetMinmax) { 
			loaded_glResetMinmax = 1;
			init_lib(); 
			stub_glResetMinmax=(pstub_glResetMinmax)get_proc_address("glResetMinmax");
			if(stub_glResetMinmax){(*stub_glResetMinmax)(lv0);}
			else { 
				FailWith("Cannot load glResetMinmax"); 
			} 
		} else { 
			FailWith("glResetMinmax not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetMinmaxParameterfv */
typedef void APIENTRY (*pstub_glGetMinmaxParameterfv)(GLenum,GLenum,GLfloat*);
static pstub_glGetMinmaxParameterfv stub_glGetMinmaxParameterfv = NULL;
static int loaded_glGetMinmaxParameterfv = 0;

value glstub_glGetMinmaxParameterfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetMinmaxParameterfv){(*stub_glGetMinmaxParameterfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetMinmaxParameterfv) { 
			loaded_glGetMinmaxParameterfv = 1;
			init_lib(); 
			stub_glGetMinmaxParameterfv=(pstub_glGetMinmaxParameterfv)get_proc_address("glGetMinmaxParameterfv");
			if(stub_glGetMinmaxParameterfv){(*stub_glGetMinmaxParameterfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetMinmaxParameterfv"); 
			} 
		} else { 
			FailWith("glGetMinmaxParameterfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetMinmaxParameteriv */
typedef void APIENTRY (*pstub_glGetMinmaxParameteriv)(GLenum,GLenum,GLint*);
static pstub_glGetMinmaxParameteriv stub_glGetMinmaxParameteriv = NULL;
static int loaded_glGetMinmaxParameteriv = 0;

value glstub_glGetMinmaxParameteriv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetMinmaxParameteriv){(*stub_glGetMinmaxParameteriv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetMinmaxParameteriv) { 
			loaded_glGetMinmaxParameteriv = 1;
			init_lib(); 
			stub_glGetMinmaxParameteriv=(pstub_glGetMinmaxParameteriv)get_proc_address("glGetMinmaxParameteriv");
			if(stub_glGetMinmaxParameteriv){(*stub_glGetMinmaxParameteriv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetMinmaxParameteriv"); 
			} 
		} else { 
			FailWith("glGetMinmaxParameteriv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glConvolutionFilter1D */
typedef void APIENTRY (*pstub_glConvolutionFilter1D)(GLenum,GLenum,GLsizei,GLenum,GLenum,GLvoid*);
static pstub_glConvolutionFilter1D stub_glConvolutionFilter1D = NULL;
static int loaded_glConvolutionFilter1D = 0;

value glstub_glConvolutionFilter1D(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLsizei lv2 =  Int_val(v2);
	GLenum lv3 =  get_glenum(v3);
	GLenum lv4 =  get_glenum(v4);
	GLvoid* lv5 =  Data_bigarray_val(v5);

	if(stub_glConvolutionFilter1D){(*stub_glConvolutionFilter1D)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glConvolutionFilter1D) { 
			loaded_glConvolutionFilter1D = 1;
			init_lib(); 
			stub_glConvolutionFilter1D=(pstub_glConvolutionFilter1D)get_proc_address("glConvolutionFilter1D");
			if(stub_glConvolutionFilter1D){(*stub_glConvolutionFilter1D)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glConvolutionFilter1D"); 
			} 
		} else { 
			FailWith("glConvolutionFilter1D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glConvolutionFilter1D_byte(value * argv, int n)
{
	return glstub_glConvolutionFilter1D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glConvolutionFilter2D */
typedef void APIENTRY (*pstub_glConvolutionFilter2D)(GLenum,GLenum,GLsizei,GLsizei,GLenum,GLenum,GLvoid*);
static pstub_glConvolutionFilter2D stub_glConvolutionFilter2D = NULL;
static int loaded_glConvolutionFilter2D = 0;

value glstub_glConvolutionFilter2D(value v0, value v1, value v2, value v3, value v4, value v5, value v6)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam2(v5,v6);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLsizei lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);
	GLenum lv4 =  get_glenum(v4);
	GLenum lv5 =  get_glenum(v5);
	GLvoid* lv6 =  Data_bigarray_val(v6);

	if(stub_glConvolutionFilter2D){(*stub_glConvolutionFilter2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
	else { 
		if(!loaded_glConvolutionFilter2D) { 
			loaded_glConvolutionFilter2D = 1;
			init_lib(); 
			stub_glConvolutionFilter2D=(pstub_glConvolutionFilter2D)get_proc_address("glConvolutionFilter2D");
			if(stub_glConvolutionFilter2D){(*stub_glConvolutionFilter2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
			else { 
				FailWith("Cannot load glConvolutionFilter2D"); 
			} 
		} else { 
			FailWith("glConvolutionFilter2D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glConvolutionFilter2D_byte(value * argv, int n)
{
	return glstub_glConvolutionFilter2D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]);
}

/* glConvolutionParameterf */
typedef void APIENTRY (*pstub_glConvolutionParameterf)(GLenum,GLenum,GLfloat);
static pstub_glConvolutionParameterf stub_glConvolutionParameterf = NULL;
static int loaded_glConvolutionParameterf = 0;

value glstub_glConvolutionParameterf(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glConvolutionParameterf){(*stub_glConvolutionParameterf)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glConvolutionParameterf) { 
			loaded_glConvolutionParameterf = 1;
			init_lib(); 
			stub_glConvolutionParameterf=(pstub_glConvolutionParameterf)get_proc_address("glConvolutionParameterf");
			if(stub_glConvolutionParameterf){(*stub_glConvolutionParameterf)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glConvolutionParameterf"); 
			} 
		} else { 
			FailWith("glConvolutionParameterf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glConvolutionParameterfv */
typedef void APIENTRY (*pstub_glConvolutionParameterfv)(GLenum,GLenum,GLfloat*);
static pstub_glConvolutionParameterfv stub_glConvolutionParameterfv = NULL;
static int loaded_glConvolutionParameterfv = 0;

value glstub_glConvolutionParameterfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glConvolutionParameterfv){(*stub_glConvolutionParameterfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glConvolutionParameterfv) { 
			loaded_glConvolutionParameterfv = 1;
			init_lib(); 
			stub_glConvolutionParameterfv=(pstub_glConvolutionParameterfv)get_proc_address("glConvolutionParameterfv");
			if(stub_glConvolutionParameterfv){(*stub_glConvolutionParameterfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glConvolutionParameterfv"); 
			} 
		} else { 
			FailWith("glConvolutionParameterfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glConvolutionParameteri */
typedef void APIENTRY (*pstub_glConvolutionParameteri)(GLenum,GLenum,GLint);
static pstub_glConvolutionParameteri stub_glConvolutionParameteri = NULL;
static int loaded_glConvolutionParameteri = 0;

value glstub_glConvolutionParameteri(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glConvolutionParameteri){(*stub_glConvolutionParameteri)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glConvolutionParameteri) { 
			loaded_glConvolutionParameteri = 1;
			init_lib(); 
			stub_glConvolutionParameteri=(pstub_glConvolutionParameteri)get_proc_address("glConvolutionParameteri");
			if(stub_glConvolutionParameteri){(*stub_glConvolutionParameteri)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glConvolutionParameteri"); 
			} 
		} else { 
			FailWith("glConvolutionParameteri not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glConvolutionParameteriv */
typedef void APIENTRY (*pstub_glConvolutionParameteriv)(GLenum,GLenum,GLint*);
static pstub_glConvolutionParameteriv stub_glConvolutionParameteriv = NULL;
static int loaded_glConvolutionParameteriv = 0;

value glstub_glConvolutionParameteriv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glConvolutionParameteriv){(*stub_glConvolutionParameteriv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glConvolutionParameteriv) { 
			loaded_glConvolutionParameteriv = 1;
			init_lib(); 
			stub_glConvolutionParameteriv=(pstub_glConvolutionParameteriv)get_proc_address("glConvolutionParameteriv");
			if(stub_glConvolutionParameteriv){(*stub_glConvolutionParameteriv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glConvolutionParameteriv"); 
			} 
		} else { 
			FailWith("glConvolutionParameteriv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glCopyConvolutionFilter1D */
typedef void APIENTRY (*pstub_glCopyConvolutionFilter1D)(GLenum,GLenum,GLint,GLint,GLsizei);
static pstub_glCopyConvolutionFilter1D stub_glCopyConvolutionFilter1D = NULL;
static int loaded_glCopyConvolutionFilter1D = 0;

value glstub_glCopyConvolutionFilter1D(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLsizei lv4 =  Int_val(v4);

	if(stub_glCopyConvolutionFilter1D){(*stub_glCopyConvolutionFilter1D)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glCopyConvolutionFilter1D) { 
			loaded_glCopyConvolutionFilter1D = 1;
			init_lib(); 
			stub_glCopyConvolutionFilter1D=(pstub_glCopyConvolutionFilter1D)get_proc_address("glCopyConvolutionFilter1D");
			if(stub_glCopyConvolutionFilter1D){(*stub_glCopyConvolutionFilter1D)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glCopyConvolutionFilter1D"); 
			} 
		} else { 
			FailWith("glCopyConvolutionFilter1D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glCopyConvolutionFilter2D */
typedef void APIENTRY (*pstub_glCopyConvolutionFilter2D)(GLenum,GLenum,GLint,GLint,GLsizei,GLsizei);
static pstub_glCopyConvolutionFilter2D stub_glCopyConvolutionFilter2D = NULL;
static int loaded_glCopyConvolutionFilter2D = 0;

value glstub_glCopyConvolutionFilter2D(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLsizei lv4 =  Int_val(v4);
	GLsizei lv5 =  Int_val(v5);

	if(stub_glCopyConvolutionFilter2D){(*stub_glCopyConvolutionFilter2D)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glCopyConvolutionFilter2D) { 
			loaded_glCopyConvolutionFilter2D = 1;
			init_lib(); 
			stub_glCopyConvolutionFilter2D=(pstub_glCopyConvolutionFilter2D)get_proc_address("glCopyConvolutionFilter2D");
			if(stub_glCopyConvolutionFilter2D){(*stub_glCopyConvolutionFilter2D)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glCopyConvolutionFilter2D"); 
			} 
		} else { 
			FailWith("glCopyConvolutionFilter2D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glCopyConvolutionFilter2D_byte(value * argv, int n)
{
	return glstub_glCopyConvolutionFilter2D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glGetConvolutionFilter */
typedef void APIENTRY (*pstub_glGetConvolutionFilter)(GLenum,GLenum,GLenum,GLvoid*);
static pstub_glGetConvolutionFilter stub_glGetConvolutionFilter = NULL;
static int loaded_glGetConvolutionFilter = 0;

value glstub_glGetConvolutionFilter(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLenum lv2 =  get_glenum(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);

	if(stub_glGetConvolutionFilter){(*stub_glGetConvolutionFilter)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glGetConvolutionFilter) { 
			loaded_glGetConvolutionFilter = 1;
			init_lib(); 
			stub_glGetConvolutionFilter=(pstub_glGetConvolutionFilter)get_proc_address("glGetConvolutionFilter");
			if(stub_glGetConvolutionFilter){(*stub_glGetConvolutionFilter)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glGetConvolutionFilter"); 
			} 
		} else { 
			FailWith("glGetConvolutionFilter not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetConvolutionParameterfv */
typedef void APIENTRY (*pstub_glGetConvolutionParameterfv)(GLenum,GLenum,GLfloat*);
static pstub_glGetConvolutionParameterfv stub_glGetConvolutionParameterfv = NULL;
static int loaded_glGetConvolutionParameterfv = 0;

value glstub_glGetConvolutionParameterfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetConvolutionParameterfv){(*stub_glGetConvolutionParameterfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetConvolutionParameterfv) { 
			loaded_glGetConvolutionParameterfv = 1;
			init_lib(); 
			stub_glGetConvolutionParameterfv=(pstub_glGetConvolutionParameterfv)get_proc_address("glGetConvolutionParameterfv");
			if(stub_glGetConvolutionParameterfv){(*stub_glGetConvolutionParameterfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetConvolutionParameterfv"); 
			} 
		} else { 
			FailWith("glGetConvolutionParameterfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetConvolutionParameteriv */
typedef void APIENTRY (*pstub_glGetConvolutionParameteriv)(GLenum,GLenum,GLint*);
static pstub_glGetConvolutionParameteriv stub_glGetConvolutionParameteriv = NULL;
static int loaded_glGetConvolutionParameteriv = 0;

value glstub_glGetConvolutionParameteriv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetConvolutionParameteriv){(*stub_glGetConvolutionParameteriv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetConvolutionParameteriv) { 
			loaded_glGetConvolutionParameteriv = 1;
			init_lib(); 
			stub_glGetConvolutionParameteriv=(pstub_glGetConvolutionParameteriv)get_proc_address("glGetConvolutionParameteriv");
			if(stub_glGetConvolutionParameteriv){(*stub_glGetConvolutionParameteriv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetConvolutionParameteriv"); 
			} 
		} else { 
			FailWith("glGetConvolutionParameteriv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSeparableFilter2D */
typedef void APIENTRY (*pstub_glSeparableFilter2D)(GLenum,GLenum,GLsizei,GLsizei,GLenum,GLenum,GLvoid*,GLvoid*);
static pstub_glSeparableFilter2D stub_glSeparableFilter2D = NULL;
static int loaded_glSeparableFilter2D = 0;

value glstub_glSeparableFilter2D(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam3(v5,v6,v7);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLsizei lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);
	GLenum lv4 =  get_glenum(v4);
	GLenum lv5 =  get_glenum(v5);
	GLvoid* lv6 =  Data_bigarray_val(v6);
	GLvoid* lv7 =  Data_bigarray_val(v7);

	if(stub_glSeparableFilter2D){(*stub_glSeparableFilter2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7);}
	else { 
		if(!loaded_glSeparableFilter2D) { 
			loaded_glSeparableFilter2D = 1;
			init_lib(); 
			stub_glSeparableFilter2D=(pstub_glSeparableFilter2D)get_proc_address("glSeparableFilter2D");
			if(stub_glSeparableFilter2D){(*stub_glSeparableFilter2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7);}
			else { 
				FailWith("Cannot load glSeparableFilter2D"); 
			} 
		} else { 
			FailWith("glSeparableFilter2D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glSeparableFilter2D_byte(value * argv, int n)
{
	return glstub_glSeparableFilter2D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7]);
}

/* glGetSeparableFilter */
typedef void APIENTRY (*pstub_glGetSeparableFilter)(GLenum,GLenum,GLenum,GLvoid*,GLvoid*,GLvoid*);
static pstub_glGetSeparableFilter stub_glGetSeparableFilter = NULL;
static int loaded_glGetSeparableFilter = 0;

value glstub_glGetSeparableFilter(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLenum lv2 =  get_glenum(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);
	GLvoid* lv4 =  Data_bigarray_val(v4);
	GLvoid* lv5 =  Data_bigarray_val(v5);

	if(stub_glGetSeparableFilter){(*stub_glGetSeparableFilter)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glGetSeparableFilter) { 
			loaded_glGetSeparableFilter = 1;
			init_lib(); 
			stub_glGetSeparableFilter=(pstub_glGetSeparableFilter)get_proc_address("glGetSeparableFilter");
			if(stub_glGetSeparableFilter){(*stub_glGetSeparableFilter)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glGetSeparableFilter"); 
			} 
		} else { 
			FailWith("glGetSeparableFilter not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glGetSeparableFilter_byte(value * argv, int n)
{
	return glstub_glGetSeparableFilter(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glGetMinmax */
typedef void APIENTRY (*pstub_glGetMinmax)(GLenum,GLboolean,GLenum,GLenum,GLvoid*);
static pstub_glGetMinmax stub_glGetMinmax = NULL;
static int loaded_glGetMinmax = 0;

value glstub_glGetMinmax(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLboolean lv1 =  Bool_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLenum lv3 =  get_glenum(v3);
	GLvoid* lv4 =  Data_bigarray_val(v4);

	if(stub_glGetMinmax){(*stub_glGetMinmax)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glGetMinmax) { 
			loaded_glGetMinmax = 1;
			init_lib(); 
			stub_glGetMinmax=(pstub_glGetMinmax)get_proc_address("glGetMinmax");
			if(stub_glGetMinmax){(*stub_glGetMinmax)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glGetMinmax"); 
			} 
		} else { 
			FailWith("glGetMinmax not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glCurrentPaletteMatrixARB */
typedef void APIENTRY (*pstub_glCurrentPaletteMatrixARB)(GLint);
static pstub_glCurrentPaletteMatrixARB stub_glCurrentPaletteMatrixARB = NULL;
static int loaded_glCurrentPaletteMatrixARB = 0;

value glstub_glCurrentPaletteMatrixARB(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);

	if(stub_glCurrentPaletteMatrixARB){(*stub_glCurrentPaletteMatrixARB)(lv0);}
	else { 
		if(!loaded_glCurrentPaletteMatrixARB) { 
			loaded_glCurrentPaletteMatrixARB = 1;
			init_lib(); 
			stub_glCurrentPaletteMatrixARB=(pstub_glCurrentPaletteMatrixARB)get_proc_address("glCurrentPaletteMatrixARB");
			if(stub_glCurrentPaletteMatrixARB){(*stub_glCurrentPaletteMatrixARB)(lv0);}
			else { 
				FailWith("Cannot load glCurrentPaletteMatrixARB"); 
			} 
		} else { 
			FailWith("glCurrentPaletteMatrixARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMatrixIndexPointerARB */
typedef void APIENTRY (*pstub_glMatrixIndexPointerARB)(GLint,GLenum,GLsizei,GLvoid*);
static pstub_glMatrixIndexPointerARB stub_glMatrixIndexPointerARB = NULL;
static int loaded_glMatrixIndexPointerARB = 0;

value glstub_glMatrixIndexPointerARB(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLsizei lv2 =  Int_val(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);

	if(stub_glMatrixIndexPointerARB){(*stub_glMatrixIndexPointerARB)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glMatrixIndexPointerARB) { 
			loaded_glMatrixIndexPointerARB = 1;
			init_lib(); 
			stub_glMatrixIndexPointerARB=(pstub_glMatrixIndexPointerARB)get_proc_address("glMatrixIndexPointerARB");
			if(stub_glMatrixIndexPointerARB){(*stub_glMatrixIndexPointerARB)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glMatrixIndexPointerARB"); 
			} 
		} else { 
			FailWith("glMatrixIndexPointerARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMatrixIndexubvARB */
typedef void APIENTRY (*pstub_glMatrixIndexubvARB)(GLint,GLubyte*);
static pstub_glMatrixIndexubvARB stub_glMatrixIndexubvARB = NULL;
static int loaded_glMatrixIndexubvARB = 0;

value glstub_glMatrixIndexubvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLubyte* lv1 =  Data_bigarray_val(v1);

	if(stub_glMatrixIndexubvARB){(*stub_glMatrixIndexubvARB)(lv0, lv1);}
	else { 
		if(!loaded_glMatrixIndexubvARB) { 
			loaded_glMatrixIndexubvARB = 1;
			init_lib(); 
			stub_glMatrixIndexubvARB=(pstub_glMatrixIndexubvARB)get_proc_address("glMatrixIndexubvARB");
			if(stub_glMatrixIndexubvARB){(*stub_glMatrixIndexubvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMatrixIndexubvARB"); 
			} 
		} else { 
			FailWith("glMatrixIndexubvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMatrixIndexusvARB */
typedef void APIENTRY (*pstub_glMatrixIndexusvARB)(GLint,GLushort*);
static pstub_glMatrixIndexusvARB stub_glMatrixIndexusvARB = NULL;
static int loaded_glMatrixIndexusvARB = 0;

value glstub_glMatrixIndexusvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLushort* lv1 =  Data_bigarray_val(v1);

	if(stub_glMatrixIndexusvARB){(*stub_glMatrixIndexusvARB)(lv0, lv1);}
	else { 
		if(!loaded_glMatrixIndexusvARB) { 
			loaded_glMatrixIndexusvARB = 1;
			init_lib(); 
			stub_glMatrixIndexusvARB=(pstub_glMatrixIndexusvARB)get_proc_address("glMatrixIndexusvARB");
			if(stub_glMatrixIndexusvARB){(*stub_glMatrixIndexusvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMatrixIndexusvARB"); 
			} 
		} else { 
			FailWith("glMatrixIndexusvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMatrixIndexuivARB */
typedef void APIENTRY (*pstub_glMatrixIndexuivARB)(GLint,GLuint*);
static pstub_glMatrixIndexuivARB stub_glMatrixIndexuivARB = NULL;
static int loaded_glMatrixIndexuivARB = 0;

value glstub_glMatrixIndexuivARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);

	if(stub_glMatrixIndexuivARB){(*stub_glMatrixIndexuivARB)(lv0, lv1);}
	else { 
		if(!loaded_glMatrixIndexuivARB) { 
			loaded_glMatrixIndexuivARB = 1;
			init_lib(); 
			stub_glMatrixIndexuivARB=(pstub_glMatrixIndexuivARB)get_proc_address("glMatrixIndexuivARB");
			if(stub_glMatrixIndexuivARB){(*stub_glMatrixIndexuivARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMatrixIndexuivARB"); 
			} 
		} else { 
			FailWith("glMatrixIndexuivARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glActiveTextureARB */
typedef void APIENTRY (*pstub_glActiveTextureARB)(GLenum);
static pstub_glActiveTextureARB stub_glActiveTextureARB = NULL;
static int loaded_glActiveTextureARB = 0;

value glstub_glActiveTextureARB(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glActiveTextureARB){(*stub_glActiveTextureARB)(lv0);}
	else { 
		if(!loaded_glActiveTextureARB) { 
			loaded_glActiveTextureARB = 1;
			init_lib(); 
			stub_glActiveTextureARB=(pstub_glActiveTextureARB)get_proc_address("glActiveTextureARB");
			if(stub_glActiveTextureARB){(*stub_glActiveTextureARB)(lv0);}
			else { 
				FailWith("Cannot load glActiveTextureARB"); 
			} 
		} else { 
			FailWith("glActiveTextureARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glClientActiveTextureARB */
typedef void APIENTRY (*pstub_glClientActiveTextureARB)(GLenum);
static pstub_glClientActiveTextureARB stub_glClientActiveTextureARB = NULL;
static int loaded_glClientActiveTextureARB = 0;

value glstub_glClientActiveTextureARB(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glClientActiveTextureARB){(*stub_glClientActiveTextureARB)(lv0);}
	else { 
		if(!loaded_glClientActiveTextureARB) { 
			loaded_glClientActiveTextureARB = 1;
			init_lib(); 
			stub_glClientActiveTextureARB=(pstub_glClientActiveTextureARB)get_proc_address("glClientActiveTextureARB");
			if(stub_glClientActiveTextureARB){(*stub_glClientActiveTextureARB)(lv0);}
			else { 
				FailWith("Cannot load glClientActiveTextureARB"); 
			} 
		} else { 
			FailWith("glClientActiveTextureARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1dARB */
typedef void APIENTRY (*pstub_glMultiTexCoord1dARB)(GLenum,GLdouble);
static pstub_glMultiTexCoord1dARB stub_glMultiTexCoord1dARB = NULL;
static int loaded_glMultiTexCoord1dARB = 0;

value glstub_glMultiTexCoord1dARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);

	if(stub_glMultiTexCoord1dARB){(*stub_glMultiTexCoord1dARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1dARB) { 
			loaded_glMultiTexCoord1dARB = 1;
			init_lib(); 
			stub_glMultiTexCoord1dARB=(pstub_glMultiTexCoord1dARB)get_proc_address("glMultiTexCoord1dARB");
			if(stub_glMultiTexCoord1dARB){(*stub_glMultiTexCoord1dARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1dARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1dARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1dvARB */
typedef void APIENTRY (*pstub_glMultiTexCoord1dvARB)(GLenum,GLdouble*);
static pstub_glMultiTexCoord1dvARB stub_glMultiTexCoord1dvARB = NULL;
static int loaded_glMultiTexCoord1dvARB = 0;

value glstub_glMultiTexCoord1dvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord1dvARB){(*stub_glMultiTexCoord1dvARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1dvARB) { 
			loaded_glMultiTexCoord1dvARB = 1;
			init_lib(); 
			stub_glMultiTexCoord1dvARB=(pstub_glMultiTexCoord1dvARB)get_proc_address("glMultiTexCoord1dvARB");
			if(stub_glMultiTexCoord1dvARB){(*stub_glMultiTexCoord1dvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1dvARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1dvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1fARB */
typedef void APIENTRY (*pstub_glMultiTexCoord1fARB)(GLenum,GLfloat);
static pstub_glMultiTexCoord1fARB stub_glMultiTexCoord1fARB = NULL;
static int loaded_glMultiTexCoord1fARB = 0;

value glstub_glMultiTexCoord1fARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glMultiTexCoord1fARB){(*stub_glMultiTexCoord1fARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1fARB) { 
			loaded_glMultiTexCoord1fARB = 1;
			init_lib(); 
			stub_glMultiTexCoord1fARB=(pstub_glMultiTexCoord1fARB)get_proc_address("glMultiTexCoord1fARB");
			if(stub_glMultiTexCoord1fARB){(*stub_glMultiTexCoord1fARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1fARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1fARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1fvARB */
typedef void APIENTRY (*pstub_glMultiTexCoord1fvARB)(GLenum,GLfloat*);
static pstub_glMultiTexCoord1fvARB stub_glMultiTexCoord1fvARB = NULL;
static int loaded_glMultiTexCoord1fvARB = 0;

value glstub_glMultiTexCoord1fvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord1fvARB){(*stub_glMultiTexCoord1fvARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1fvARB) { 
			loaded_glMultiTexCoord1fvARB = 1;
			init_lib(); 
			stub_glMultiTexCoord1fvARB=(pstub_glMultiTexCoord1fvARB)get_proc_address("glMultiTexCoord1fvARB");
			if(stub_glMultiTexCoord1fvARB){(*stub_glMultiTexCoord1fvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1fvARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1fvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1iARB */
typedef void APIENTRY (*pstub_glMultiTexCoord1iARB)(GLenum,GLint);
static pstub_glMultiTexCoord1iARB stub_glMultiTexCoord1iARB = NULL;
static int loaded_glMultiTexCoord1iARB = 0;

value glstub_glMultiTexCoord1iARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glMultiTexCoord1iARB){(*stub_glMultiTexCoord1iARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1iARB) { 
			loaded_glMultiTexCoord1iARB = 1;
			init_lib(); 
			stub_glMultiTexCoord1iARB=(pstub_glMultiTexCoord1iARB)get_proc_address("glMultiTexCoord1iARB");
			if(stub_glMultiTexCoord1iARB){(*stub_glMultiTexCoord1iARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1iARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1iARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1ivARB */
typedef void APIENTRY (*pstub_glMultiTexCoord1ivARB)(GLenum,GLint*);
static pstub_glMultiTexCoord1ivARB stub_glMultiTexCoord1ivARB = NULL;
static int loaded_glMultiTexCoord1ivARB = 0;

value glstub_glMultiTexCoord1ivARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord1ivARB){(*stub_glMultiTexCoord1ivARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1ivARB) { 
			loaded_glMultiTexCoord1ivARB = 1;
			init_lib(); 
			stub_glMultiTexCoord1ivARB=(pstub_glMultiTexCoord1ivARB)get_proc_address("glMultiTexCoord1ivARB");
			if(stub_glMultiTexCoord1ivARB){(*stub_glMultiTexCoord1ivARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1ivARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1ivARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1sARB */
typedef void APIENTRY (*pstub_glMultiTexCoord1sARB)(GLenum,GLshort);
static pstub_glMultiTexCoord1sARB stub_glMultiTexCoord1sARB = NULL;
static int loaded_glMultiTexCoord1sARB = 0;

value glstub_glMultiTexCoord1sARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort lv1 =  Int_val(v1);

	if(stub_glMultiTexCoord1sARB){(*stub_glMultiTexCoord1sARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1sARB) { 
			loaded_glMultiTexCoord1sARB = 1;
			init_lib(); 
			stub_glMultiTexCoord1sARB=(pstub_glMultiTexCoord1sARB)get_proc_address("glMultiTexCoord1sARB");
			if(stub_glMultiTexCoord1sARB){(*stub_glMultiTexCoord1sARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1sARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1sARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1svARB */
typedef void APIENTRY (*pstub_glMultiTexCoord1svARB)(GLenum,GLshort*);
static pstub_glMultiTexCoord1svARB stub_glMultiTexCoord1svARB = NULL;
static int loaded_glMultiTexCoord1svARB = 0;

value glstub_glMultiTexCoord1svARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord1svARB){(*stub_glMultiTexCoord1svARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1svARB) { 
			loaded_glMultiTexCoord1svARB = 1;
			init_lib(); 
			stub_glMultiTexCoord1svARB=(pstub_glMultiTexCoord1svARB)get_proc_address("glMultiTexCoord1svARB");
			if(stub_glMultiTexCoord1svARB){(*stub_glMultiTexCoord1svARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1svARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1svARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2dARB */
typedef void APIENTRY (*pstub_glMultiTexCoord2dARB)(GLenum,GLdouble,GLdouble);
static pstub_glMultiTexCoord2dARB stub_glMultiTexCoord2dARB = NULL;
static int loaded_glMultiTexCoord2dARB = 0;

value glstub_glMultiTexCoord2dARB(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glMultiTexCoord2dARB){(*stub_glMultiTexCoord2dARB)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMultiTexCoord2dARB) { 
			loaded_glMultiTexCoord2dARB = 1;
			init_lib(); 
			stub_glMultiTexCoord2dARB=(pstub_glMultiTexCoord2dARB)get_proc_address("glMultiTexCoord2dARB");
			if(stub_glMultiTexCoord2dARB){(*stub_glMultiTexCoord2dARB)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMultiTexCoord2dARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2dARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2dvARB */
typedef void APIENTRY (*pstub_glMultiTexCoord2dvARB)(GLenum,GLdouble*);
static pstub_glMultiTexCoord2dvARB stub_glMultiTexCoord2dvARB = NULL;
static int loaded_glMultiTexCoord2dvARB = 0;

value glstub_glMultiTexCoord2dvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord2dvARB){(*stub_glMultiTexCoord2dvARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord2dvARB) { 
			loaded_glMultiTexCoord2dvARB = 1;
			init_lib(); 
			stub_glMultiTexCoord2dvARB=(pstub_glMultiTexCoord2dvARB)get_proc_address("glMultiTexCoord2dvARB");
			if(stub_glMultiTexCoord2dvARB){(*stub_glMultiTexCoord2dvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord2dvARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2dvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2fARB */
typedef void APIENTRY (*pstub_glMultiTexCoord2fARB)(GLenum,GLfloat,GLfloat);
static pstub_glMultiTexCoord2fARB stub_glMultiTexCoord2fARB = NULL;
static int loaded_glMultiTexCoord2fARB = 0;

value glstub_glMultiTexCoord2fARB(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glMultiTexCoord2fARB){(*stub_glMultiTexCoord2fARB)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMultiTexCoord2fARB) { 
			loaded_glMultiTexCoord2fARB = 1;
			init_lib(); 
			stub_glMultiTexCoord2fARB=(pstub_glMultiTexCoord2fARB)get_proc_address("glMultiTexCoord2fARB");
			if(stub_glMultiTexCoord2fARB){(*stub_glMultiTexCoord2fARB)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMultiTexCoord2fARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2fARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2fvARB */
typedef void APIENTRY (*pstub_glMultiTexCoord2fvARB)(GLenum,GLfloat*);
static pstub_glMultiTexCoord2fvARB stub_glMultiTexCoord2fvARB = NULL;
static int loaded_glMultiTexCoord2fvARB = 0;

value glstub_glMultiTexCoord2fvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord2fvARB){(*stub_glMultiTexCoord2fvARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord2fvARB) { 
			loaded_glMultiTexCoord2fvARB = 1;
			init_lib(); 
			stub_glMultiTexCoord2fvARB=(pstub_glMultiTexCoord2fvARB)get_proc_address("glMultiTexCoord2fvARB");
			if(stub_glMultiTexCoord2fvARB){(*stub_glMultiTexCoord2fvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord2fvARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2fvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2iARB */
typedef void APIENTRY (*pstub_glMultiTexCoord2iARB)(GLenum,GLint,GLint);
static pstub_glMultiTexCoord2iARB stub_glMultiTexCoord2iARB = NULL;
static int loaded_glMultiTexCoord2iARB = 0;

value glstub_glMultiTexCoord2iARB(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glMultiTexCoord2iARB){(*stub_glMultiTexCoord2iARB)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMultiTexCoord2iARB) { 
			loaded_glMultiTexCoord2iARB = 1;
			init_lib(); 
			stub_glMultiTexCoord2iARB=(pstub_glMultiTexCoord2iARB)get_proc_address("glMultiTexCoord2iARB");
			if(stub_glMultiTexCoord2iARB){(*stub_glMultiTexCoord2iARB)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMultiTexCoord2iARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2iARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2ivARB */
typedef void APIENTRY (*pstub_glMultiTexCoord2ivARB)(GLenum,GLint*);
static pstub_glMultiTexCoord2ivARB stub_glMultiTexCoord2ivARB = NULL;
static int loaded_glMultiTexCoord2ivARB = 0;

value glstub_glMultiTexCoord2ivARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord2ivARB){(*stub_glMultiTexCoord2ivARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord2ivARB) { 
			loaded_glMultiTexCoord2ivARB = 1;
			init_lib(); 
			stub_glMultiTexCoord2ivARB=(pstub_glMultiTexCoord2ivARB)get_proc_address("glMultiTexCoord2ivARB");
			if(stub_glMultiTexCoord2ivARB){(*stub_glMultiTexCoord2ivARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord2ivARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2ivARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2sARB */
typedef void APIENTRY (*pstub_glMultiTexCoord2sARB)(GLenum,GLshort,GLshort);
static pstub_glMultiTexCoord2sARB stub_glMultiTexCoord2sARB = NULL;
static int loaded_glMultiTexCoord2sARB = 0;

value glstub_glMultiTexCoord2sARB(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);

	if(stub_glMultiTexCoord2sARB){(*stub_glMultiTexCoord2sARB)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMultiTexCoord2sARB) { 
			loaded_glMultiTexCoord2sARB = 1;
			init_lib(); 
			stub_glMultiTexCoord2sARB=(pstub_glMultiTexCoord2sARB)get_proc_address("glMultiTexCoord2sARB");
			if(stub_glMultiTexCoord2sARB){(*stub_glMultiTexCoord2sARB)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMultiTexCoord2sARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2sARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2svARB */
typedef void APIENTRY (*pstub_glMultiTexCoord2svARB)(GLenum,GLshort*);
static pstub_glMultiTexCoord2svARB stub_glMultiTexCoord2svARB = NULL;
static int loaded_glMultiTexCoord2svARB = 0;

value glstub_glMultiTexCoord2svARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord2svARB){(*stub_glMultiTexCoord2svARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord2svARB) { 
			loaded_glMultiTexCoord2svARB = 1;
			init_lib(); 
			stub_glMultiTexCoord2svARB=(pstub_glMultiTexCoord2svARB)get_proc_address("glMultiTexCoord2svARB");
			if(stub_glMultiTexCoord2svARB){(*stub_glMultiTexCoord2svARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord2svARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2svARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3dARB */
typedef void APIENTRY (*pstub_glMultiTexCoord3dARB)(GLenum,GLdouble,GLdouble,GLdouble);
static pstub_glMultiTexCoord3dARB stub_glMultiTexCoord3dARB = NULL;
static int loaded_glMultiTexCoord3dARB = 0;

value glstub_glMultiTexCoord3dARB(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);

	if(stub_glMultiTexCoord3dARB){(*stub_glMultiTexCoord3dARB)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glMultiTexCoord3dARB) { 
			loaded_glMultiTexCoord3dARB = 1;
			init_lib(); 
			stub_glMultiTexCoord3dARB=(pstub_glMultiTexCoord3dARB)get_proc_address("glMultiTexCoord3dARB");
			if(stub_glMultiTexCoord3dARB){(*stub_glMultiTexCoord3dARB)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glMultiTexCoord3dARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3dARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3dvARB */
typedef void APIENTRY (*pstub_glMultiTexCoord3dvARB)(GLenum,GLdouble*);
static pstub_glMultiTexCoord3dvARB stub_glMultiTexCoord3dvARB = NULL;
static int loaded_glMultiTexCoord3dvARB = 0;

value glstub_glMultiTexCoord3dvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord3dvARB){(*stub_glMultiTexCoord3dvARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord3dvARB) { 
			loaded_glMultiTexCoord3dvARB = 1;
			init_lib(); 
			stub_glMultiTexCoord3dvARB=(pstub_glMultiTexCoord3dvARB)get_proc_address("glMultiTexCoord3dvARB");
			if(stub_glMultiTexCoord3dvARB){(*stub_glMultiTexCoord3dvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord3dvARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3dvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3fARB */
typedef void APIENTRY (*pstub_glMultiTexCoord3fARB)(GLenum,GLfloat,GLfloat,GLfloat);
static pstub_glMultiTexCoord3fARB stub_glMultiTexCoord3fARB = NULL;
static int loaded_glMultiTexCoord3fARB = 0;

value glstub_glMultiTexCoord3fARB(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);

	if(stub_glMultiTexCoord3fARB){(*stub_glMultiTexCoord3fARB)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glMultiTexCoord3fARB) { 
			loaded_glMultiTexCoord3fARB = 1;
			init_lib(); 
			stub_glMultiTexCoord3fARB=(pstub_glMultiTexCoord3fARB)get_proc_address("glMultiTexCoord3fARB");
			if(stub_glMultiTexCoord3fARB){(*stub_glMultiTexCoord3fARB)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glMultiTexCoord3fARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3fARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3fvARB */
typedef void APIENTRY (*pstub_glMultiTexCoord3fvARB)(GLenum,GLfloat*);
static pstub_glMultiTexCoord3fvARB stub_glMultiTexCoord3fvARB = NULL;
static int loaded_glMultiTexCoord3fvARB = 0;

value glstub_glMultiTexCoord3fvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord3fvARB){(*stub_glMultiTexCoord3fvARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord3fvARB) { 
			loaded_glMultiTexCoord3fvARB = 1;
			init_lib(); 
			stub_glMultiTexCoord3fvARB=(pstub_glMultiTexCoord3fvARB)get_proc_address("glMultiTexCoord3fvARB");
			if(stub_glMultiTexCoord3fvARB){(*stub_glMultiTexCoord3fvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord3fvARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3fvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3iARB */
typedef void APIENTRY (*pstub_glMultiTexCoord3iARB)(GLenum,GLint,GLint,GLint);
static pstub_glMultiTexCoord3iARB stub_glMultiTexCoord3iARB = NULL;
static int loaded_glMultiTexCoord3iARB = 0;

value glstub_glMultiTexCoord3iARB(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);

	if(stub_glMultiTexCoord3iARB){(*stub_glMultiTexCoord3iARB)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glMultiTexCoord3iARB) { 
			loaded_glMultiTexCoord3iARB = 1;
			init_lib(); 
			stub_glMultiTexCoord3iARB=(pstub_glMultiTexCoord3iARB)get_proc_address("glMultiTexCoord3iARB");
			if(stub_glMultiTexCoord3iARB){(*stub_glMultiTexCoord3iARB)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glMultiTexCoord3iARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3iARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3ivARB */
typedef void APIENTRY (*pstub_glMultiTexCoord3ivARB)(GLenum,GLint*);
static pstub_glMultiTexCoord3ivARB stub_glMultiTexCoord3ivARB = NULL;
static int loaded_glMultiTexCoord3ivARB = 0;

value glstub_glMultiTexCoord3ivARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord3ivARB){(*stub_glMultiTexCoord3ivARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord3ivARB) { 
			loaded_glMultiTexCoord3ivARB = 1;
			init_lib(); 
			stub_glMultiTexCoord3ivARB=(pstub_glMultiTexCoord3ivARB)get_proc_address("glMultiTexCoord3ivARB");
			if(stub_glMultiTexCoord3ivARB){(*stub_glMultiTexCoord3ivARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord3ivARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3ivARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3sARB */
typedef void APIENTRY (*pstub_glMultiTexCoord3sARB)(GLenum,GLshort,GLshort,GLshort);
static pstub_glMultiTexCoord3sARB stub_glMultiTexCoord3sARB = NULL;
static int loaded_glMultiTexCoord3sARB = 0;

value glstub_glMultiTexCoord3sARB(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);

	if(stub_glMultiTexCoord3sARB){(*stub_glMultiTexCoord3sARB)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glMultiTexCoord3sARB) { 
			loaded_glMultiTexCoord3sARB = 1;
			init_lib(); 
			stub_glMultiTexCoord3sARB=(pstub_glMultiTexCoord3sARB)get_proc_address("glMultiTexCoord3sARB");
			if(stub_glMultiTexCoord3sARB){(*stub_glMultiTexCoord3sARB)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glMultiTexCoord3sARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3sARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3svARB */
typedef void APIENTRY (*pstub_glMultiTexCoord3svARB)(GLenum,GLshort*);
static pstub_glMultiTexCoord3svARB stub_glMultiTexCoord3svARB = NULL;
static int loaded_glMultiTexCoord3svARB = 0;

value glstub_glMultiTexCoord3svARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord3svARB){(*stub_glMultiTexCoord3svARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord3svARB) { 
			loaded_glMultiTexCoord3svARB = 1;
			init_lib(); 
			stub_glMultiTexCoord3svARB=(pstub_glMultiTexCoord3svARB)get_proc_address("glMultiTexCoord3svARB");
			if(stub_glMultiTexCoord3svARB){(*stub_glMultiTexCoord3svARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord3svARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3svARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4dARB */
typedef void APIENTRY (*pstub_glMultiTexCoord4dARB)(GLenum,GLdouble,GLdouble,GLdouble,GLdouble);
static pstub_glMultiTexCoord4dARB stub_glMultiTexCoord4dARB = NULL;
static int loaded_glMultiTexCoord4dARB = 0;

value glstub_glMultiTexCoord4dARB(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);
	GLdouble lv4 =  Double_val(v4);

	if(stub_glMultiTexCoord4dARB){(*stub_glMultiTexCoord4dARB)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glMultiTexCoord4dARB) { 
			loaded_glMultiTexCoord4dARB = 1;
			init_lib(); 
			stub_glMultiTexCoord4dARB=(pstub_glMultiTexCoord4dARB)get_proc_address("glMultiTexCoord4dARB");
			if(stub_glMultiTexCoord4dARB){(*stub_glMultiTexCoord4dARB)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glMultiTexCoord4dARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4dARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4dvARB */
typedef void APIENTRY (*pstub_glMultiTexCoord4dvARB)(GLenum,GLdouble*);
static pstub_glMultiTexCoord4dvARB stub_glMultiTexCoord4dvARB = NULL;
static int loaded_glMultiTexCoord4dvARB = 0;

value glstub_glMultiTexCoord4dvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord4dvARB){(*stub_glMultiTexCoord4dvARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord4dvARB) { 
			loaded_glMultiTexCoord4dvARB = 1;
			init_lib(); 
			stub_glMultiTexCoord4dvARB=(pstub_glMultiTexCoord4dvARB)get_proc_address("glMultiTexCoord4dvARB");
			if(stub_glMultiTexCoord4dvARB){(*stub_glMultiTexCoord4dvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord4dvARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4dvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4fARB */
typedef void APIENTRY (*pstub_glMultiTexCoord4fARB)(GLenum,GLfloat,GLfloat,GLfloat,GLfloat);
static pstub_glMultiTexCoord4fARB stub_glMultiTexCoord4fARB = NULL;
static int loaded_glMultiTexCoord4fARB = 0;

value glstub_glMultiTexCoord4fARB(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);
	GLfloat lv4 =  Double_val(v4);

	if(stub_glMultiTexCoord4fARB){(*stub_glMultiTexCoord4fARB)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glMultiTexCoord4fARB) { 
			loaded_glMultiTexCoord4fARB = 1;
			init_lib(); 
			stub_glMultiTexCoord4fARB=(pstub_glMultiTexCoord4fARB)get_proc_address("glMultiTexCoord4fARB");
			if(stub_glMultiTexCoord4fARB){(*stub_glMultiTexCoord4fARB)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glMultiTexCoord4fARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4fARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4fvARB */
typedef void APIENTRY (*pstub_glMultiTexCoord4fvARB)(GLenum,GLfloat*);
static pstub_glMultiTexCoord4fvARB stub_glMultiTexCoord4fvARB = NULL;
static int loaded_glMultiTexCoord4fvARB = 0;

value glstub_glMultiTexCoord4fvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord4fvARB){(*stub_glMultiTexCoord4fvARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord4fvARB) { 
			loaded_glMultiTexCoord4fvARB = 1;
			init_lib(); 
			stub_glMultiTexCoord4fvARB=(pstub_glMultiTexCoord4fvARB)get_proc_address("glMultiTexCoord4fvARB");
			if(stub_glMultiTexCoord4fvARB){(*stub_glMultiTexCoord4fvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord4fvARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4fvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4iARB */
typedef void APIENTRY (*pstub_glMultiTexCoord4iARB)(GLenum,GLint,GLint,GLint,GLint);
static pstub_glMultiTexCoord4iARB stub_glMultiTexCoord4iARB = NULL;
static int loaded_glMultiTexCoord4iARB = 0;

value glstub_glMultiTexCoord4iARB(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);

	if(stub_glMultiTexCoord4iARB){(*stub_glMultiTexCoord4iARB)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glMultiTexCoord4iARB) { 
			loaded_glMultiTexCoord4iARB = 1;
			init_lib(); 
			stub_glMultiTexCoord4iARB=(pstub_glMultiTexCoord4iARB)get_proc_address("glMultiTexCoord4iARB");
			if(stub_glMultiTexCoord4iARB){(*stub_glMultiTexCoord4iARB)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glMultiTexCoord4iARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4iARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4ivARB */
typedef void APIENTRY (*pstub_glMultiTexCoord4ivARB)(GLenum,GLint*);
static pstub_glMultiTexCoord4ivARB stub_glMultiTexCoord4ivARB = NULL;
static int loaded_glMultiTexCoord4ivARB = 0;

value glstub_glMultiTexCoord4ivARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord4ivARB){(*stub_glMultiTexCoord4ivARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord4ivARB) { 
			loaded_glMultiTexCoord4ivARB = 1;
			init_lib(); 
			stub_glMultiTexCoord4ivARB=(pstub_glMultiTexCoord4ivARB)get_proc_address("glMultiTexCoord4ivARB");
			if(stub_glMultiTexCoord4ivARB){(*stub_glMultiTexCoord4ivARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord4ivARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4ivARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4sARB */
typedef void APIENTRY (*pstub_glMultiTexCoord4sARB)(GLenum,GLshort,GLshort,GLshort,GLshort);
static pstub_glMultiTexCoord4sARB stub_glMultiTexCoord4sARB = NULL;
static int loaded_glMultiTexCoord4sARB = 0;

value glstub_glMultiTexCoord4sARB(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);
	GLshort lv4 =  Int_val(v4);

	if(stub_glMultiTexCoord4sARB){(*stub_glMultiTexCoord4sARB)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glMultiTexCoord4sARB) { 
			loaded_glMultiTexCoord4sARB = 1;
			init_lib(); 
			stub_glMultiTexCoord4sARB=(pstub_glMultiTexCoord4sARB)get_proc_address("glMultiTexCoord4sARB");
			if(stub_glMultiTexCoord4sARB){(*stub_glMultiTexCoord4sARB)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glMultiTexCoord4sARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4sARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4svARB */
typedef void APIENTRY (*pstub_glMultiTexCoord4svARB)(GLenum,GLshort*);
static pstub_glMultiTexCoord4svARB stub_glMultiTexCoord4svARB = NULL;
static int loaded_glMultiTexCoord4svARB = 0;

value glstub_glMultiTexCoord4svARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord4svARB){(*stub_glMultiTexCoord4svARB)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord4svARB) { 
			loaded_glMultiTexCoord4svARB = 1;
			init_lib(); 
			stub_glMultiTexCoord4svARB=(pstub_glMultiTexCoord4svARB)get_proc_address("glMultiTexCoord4svARB");
			if(stub_glMultiTexCoord4svARB){(*stub_glMultiTexCoord4svARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord4svARB"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4svARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWeightbvARB */
typedef void APIENTRY (*pstub_glWeightbvARB)(GLint,GLbyte*);
static pstub_glWeightbvARB stub_glWeightbvARB = NULL;
static int loaded_glWeightbvARB = 0;

value glstub_glWeightbvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLbyte* lv1 =  Data_bigarray_val(v1);

	if(stub_glWeightbvARB){(*stub_glWeightbvARB)(lv0, lv1);}
	else { 
		if(!loaded_glWeightbvARB) { 
			loaded_glWeightbvARB = 1;
			init_lib(); 
			stub_glWeightbvARB=(pstub_glWeightbvARB)get_proc_address("glWeightbvARB");
			if(stub_glWeightbvARB){(*stub_glWeightbvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glWeightbvARB"); 
			} 
		} else { 
			FailWith("glWeightbvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWeightsvARB */
typedef void APIENTRY (*pstub_glWeightsvARB)(GLint,GLshort*);
static pstub_glWeightsvARB stub_glWeightsvARB = NULL;
static int loaded_glWeightsvARB = 0;

value glstub_glWeightsvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glWeightsvARB){(*stub_glWeightsvARB)(lv0, lv1);}
	else { 
		if(!loaded_glWeightsvARB) { 
			loaded_glWeightsvARB = 1;
			init_lib(); 
			stub_glWeightsvARB=(pstub_glWeightsvARB)get_proc_address("glWeightsvARB");
			if(stub_glWeightsvARB){(*stub_glWeightsvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glWeightsvARB"); 
			} 
		} else { 
			FailWith("glWeightsvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWeightivARB */
typedef void APIENTRY (*pstub_glWeightivARB)(GLint,GLint*);
static pstub_glWeightivARB stub_glWeightivARB = NULL;
static int loaded_glWeightivARB = 0;

value glstub_glWeightivARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glWeightivARB){(*stub_glWeightivARB)(lv0, lv1);}
	else { 
		if(!loaded_glWeightivARB) { 
			loaded_glWeightivARB = 1;
			init_lib(); 
			stub_glWeightivARB=(pstub_glWeightivARB)get_proc_address("glWeightivARB");
			if(stub_glWeightivARB){(*stub_glWeightivARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glWeightivARB"); 
			} 
		} else { 
			FailWith("glWeightivARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWeightfvARB */
typedef void APIENTRY (*pstub_glWeightfvARB)(GLint,GLfloat*);
static pstub_glWeightfvARB stub_glWeightfvARB = NULL;
static int loaded_glWeightfvARB = 0;

value glstub_glWeightfvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glWeightfvARB){(*stub_glWeightfvARB)(lv0, lv1);}
	else { 
		if(!loaded_glWeightfvARB) { 
			loaded_glWeightfvARB = 1;
			init_lib(); 
			stub_glWeightfvARB=(pstub_glWeightfvARB)get_proc_address("glWeightfvARB");
			if(stub_glWeightfvARB){(*stub_glWeightfvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glWeightfvARB"); 
			} 
		} else { 
			FailWith("glWeightfvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWeightdvARB */
typedef void APIENTRY (*pstub_glWeightdvARB)(GLint,GLdouble*);
static pstub_glWeightdvARB stub_glWeightdvARB = NULL;
static int loaded_glWeightdvARB = 0;

value glstub_glWeightdvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glWeightdvARB){(*stub_glWeightdvARB)(lv0, lv1);}
	else { 
		if(!loaded_glWeightdvARB) { 
			loaded_glWeightdvARB = 1;
			init_lib(); 
			stub_glWeightdvARB=(pstub_glWeightdvARB)get_proc_address("glWeightdvARB");
			if(stub_glWeightdvARB){(*stub_glWeightdvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glWeightdvARB"); 
			} 
		} else { 
			FailWith("glWeightdvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWeightubvARB */
typedef void APIENTRY (*pstub_glWeightubvARB)(GLint,GLubyte*);
static pstub_glWeightubvARB stub_glWeightubvARB = NULL;
static int loaded_glWeightubvARB = 0;

value glstub_glWeightubvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLubyte* lv1 =  Data_bigarray_val(v1);

	if(stub_glWeightubvARB){(*stub_glWeightubvARB)(lv0, lv1);}
	else { 
		if(!loaded_glWeightubvARB) { 
			loaded_glWeightubvARB = 1;
			init_lib(); 
			stub_glWeightubvARB=(pstub_glWeightubvARB)get_proc_address("glWeightubvARB");
			if(stub_glWeightubvARB){(*stub_glWeightubvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glWeightubvARB"); 
			} 
		} else { 
			FailWith("glWeightubvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWeightusvARB */
typedef void APIENTRY (*pstub_glWeightusvARB)(GLint,GLushort*);
static pstub_glWeightusvARB stub_glWeightusvARB = NULL;
static int loaded_glWeightusvARB = 0;

value glstub_glWeightusvARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLushort* lv1 =  Data_bigarray_val(v1);

	if(stub_glWeightusvARB){(*stub_glWeightusvARB)(lv0, lv1);}
	else { 
		if(!loaded_glWeightusvARB) { 
			loaded_glWeightusvARB = 1;
			init_lib(); 
			stub_glWeightusvARB=(pstub_glWeightusvARB)get_proc_address("glWeightusvARB");
			if(stub_glWeightusvARB){(*stub_glWeightusvARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glWeightusvARB"); 
			} 
		} else { 
			FailWith("glWeightusvARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWeightuivARB */
typedef void APIENTRY (*pstub_glWeightuivARB)(GLint,GLuint*);
static pstub_glWeightuivARB stub_glWeightuivARB = NULL;
static int loaded_glWeightuivARB = 0;

value glstub_glWeightuivARB(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);

	if(stub_glWeightuivARB){(*stub_glWeightuivARB)(lv0, lv1);}
	else { 
		if(!loaded_glWeightuivARB) { 
			loaded_glWeightuivARB = 1;
			init_lib(); 
			stub_glWeightuivARB=(pstub_glWeightuivARB)get_proc_address("glWeightuivARB");
			if(stub_glWeightuivARB){(*stub_glWeightuivARB)(lv0, lv1);}
			else { 
				FailWith("Cannot load glWeightuivARB"); 
			} 
		} else { 
			FailWith("glWeightuivARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWeightPointerARB */
typedef void APIENTRY (*pstub_glWeightPointerARB)(GLint,GLenum,GLsizei,GLvoid*);
static pstub_glWeightPointerARB stub_glWeightPointerARB = NULL;
static int loaded_glWeightPointerARB = 0;

value glstub_glWeightPointerARB(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLsizei lv2 =  Int_val(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);

	if(stub_glWeightPointerARB){(*stub_glWeightPointerARB)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glWeightPointerARB) { 
			loaded_glWeightPointerARB = 1;
			init_lib(); 
			stub_glWeightPointerARB=(pstub_glWeightPointerARB)get_proc_address("glWeightPointerARB");
			if(stub_glWeightPointerARB){(*stub_glWeightPointerARB)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glWeightPointerARB"); 
			} 
		} else { 
			FailWith("glWeightPointerARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexBlendARB */
typedef void APIENTRY (*pstub_glVertexBlendARB)(GLint);
static pstub_glVertexBlendARB stub_glVertexBlendARB = NULL;
static int loaded_glVertexBlendARB = 0;

value glstub_glVertexBlendARB(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);

	if(stub_glVertexBlendARB){(*stub_glVertexBlendARB)(lv0);}
	else { 
		if(!loaded_glVertexBlendARB) { 
			loaded_glVertexBlendARB = 1;
			init_lib(); 
			stub_glVertexBlendARB=(pstub_glVertexBlendARB)get_proc_address("glVertexBlendARB");
			if(stub_glVertexBlendARB){(*stub_glVertexBlendARB)(lv0);}
			else { 
				FailWith("Cannot load glVertexBlendARB"); 
			} 
		} else { 
			FailWith("glVertexBlendARB not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexBumpParameterivATI */
typedef void APIENTRY (*pstub_glTexBumpParameterivATI)(GLenum,GLint*);
static pstub_glTexBumpParameterivATI stub_glTexBumpParameterivATI = NULL;
static int loaded_glTexBumpParameterivATI = 0;

value glstub_glTexBumpParameterivATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glTexBumpParameterivATI){(*stub_glTexBumpParameterivATI)(lv0, lv1);}
	else { 
		if(!loaded_glTexBumpParameterivATI) { 
			loaded_glTexBumpParameterivATI = 1;
			init_lib(); 
			stub_glTexBumpParameterivATI=(pstub_glTexBumpParameterivATI)get_proc_address("glTexBumpParameterivATI");
			if(stub_glTexBumpParameterivATI){(*stub_glTexBumpParameterivATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glTexBumpParameterivATI"); 
			} 
		} else { 
			FailWith("glTexBumpParameterivATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexBumpParameterfvATI */
typedef void APIENTRY (*pstub_glTexBumpParameterfvATI)(GLenum,GLfloat*);
static pstub_glTexBumpParameterfvATI stub_glTexBumpParameterfvATI = NULL;
static int loaded_glTexBumpParameterfvATI = 0;

value glstub_glTexBumpParameterfvATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glTexBumpParameterfvATI){(*stub_glTexBumpParameterfvATI)(lv0, lv1);}
	else { 
		if(!loaded_glTexBumpParameterfvATI) { 
			loaded_glTexBumpParameterfvATI = 1;
			init_lib(); 
			stub_glTexBumpParameterfvATI=(pstub_glTexBumpParameterfvATI)get_proc_address("glTexBumpParameterfvATI");
			if(stub_glTexBumpParameterfvATI){(*stub_glTexBumpParameterfvATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glTexBumpParameterfvATI"); 
			} 
		} else { 
			FailWith("glTexBumpParameterfvATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetTexBumpParameterivATI */
typedef void APIENTRY (*pstub_glGetTexBumpParameterivATI)(GLenum,GLint*);
static pstub_glGetTexBumpParameterivATI stub_glGetTexBumpParameterivATI = NULL;
static int loaded_glGetTexBumpParameterivATI = 0;

value glstub_glGetTexBumpParameterivATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glGetTexBumpParameterivATI){(*stub_glGetTexBumpParameterivATI)(lv0, lv1);}
	else { 
		if(!loaded_glGetTexBumpParameterivATI) { 
			loaded_glGetTexBumpParameterivATI = 1;
			init_lib(); 
			stub_glGetTexBumpParameterivATI=(pstub_glGetTexBumpParameterivATI)get_proc_address("glGetTexBumpParameterivATI");
			if(stub_glGetTexBumpParameterivATI){(*stub_glGetTexBumpParameterivATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGetTexBumpParameterivATI"); 
			} 
		} else { 
			FailWith("glGetTexBumpParameterivATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetTexBumpParameterfvATI */
typedef void APIENTRY (*pstub_glGetTexBumpParameterfvATI)(GLenum,GLfloat*);
static pstub_glGetTexBumpParameterfvATI stub_glGetTexBumpParameterfvATI = NULL;
static int loaded_glGetTexBumpParameterfvATI = 0;

value glstub_glGetTexBumpParameterfvATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glGetTexBumpParameterfvATI){(*stub_glGetTexBumpParameterfvATI)(lv0, lv1);}
	else { 
		if(!loaded_glGetTexBumpParameterfvATI) { 
			loaded_glGetTexBumpParameterfvATI = 1;
			init_lib(); 
			stub_glGetTexBumpParameterfvATI=(pstub_glGetTexBumpParameterfvATI)get_proc_address("glGetTexBumpParameterfvATI");
			if(stub_glGetTexBumpParameterfvATI){(*stub_glGetTexBumpParameterfvATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGetTexBumpParameterfvATI"); 
			} 
		} else { 
			FailWith("glGetTexBumpParameterfvATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMapObjectBufferATI */
typedef void* APIENTRY (*pstub_glMapObjectBufferATI)(GLuint);
static pstub_glMapObjectBufferATI stub_glMapObjectBufferATI = NULL;
static int loaded_glMapObjectBufferATI = 0;

value glstub_glMapObjectBufferATI(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	void* ret;
	if(stub_glMapObjectBufferATI){ret = (*stub_glMapObjectBufferATI)(lv0);}
	else { 
		if(!loaded_glMapObjectBufferATI) { 
			loaded_glMapObjectBufferATI = 1;
			init_lib(); 
			stub_glMapObjectBufferATI=(pstub_glMapObjectBufferATI)get_proc_address("glMapObjectBufferATI");
			if(stub_glMapObjectBufferATI){ret = (*stub_glMapObjectBufferATI)(lv0);}
			else { 
				FailWith("Cannot load glMapObjectBufferATI"); 
			} 
		} else { 
			FailWith("glMapObjectBufferATI not available"); 
		}
	}
	result = (value)(ret);
	CAMLreturn (result);
}

/* glUnmapObjectBufferATI */
typedef void APIENTRY (*pstub_glUnmapObjectBufferATI)(GLuint);
static pstub_glUnmapObjectBufferATI stub_glUnmapObjectBufferATI = NULL;
static int loaded_glUnmapObjectBufferATI = 0;

value glstub_glUnmapObjectBufferATI(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glUnmapObjectBufferATI){(*stub_glUnmapObjectBufferATI)(lv0);}
	else { 
		if(!loaded_glUnmapObjectBufferATI) { 
			loaded_glUnmapObjectBufferATI = 1;
			init_lib(); 
			stub_glUnmapObjectBufferATI=(pstub_glUnmapObjectBufferATI)get_proc_address("glUnmapObjectBufferATI");
			if(stub_glUnmapObjectBufferATI){(*stub_glUnmapObjectBufferATI)(lv0);}
			else { 
				FailWith("Cannot load glUnmapObjectBufferATI"); 
			} 
		} else { 
			FailWith("glUnmapObjectBufferATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPNTrianglesiATI */
typedef void APIENTRY (*pstub_glPNTrianglesiATI)(GLenum,GLint);
static pstub_glPNTrianglesiATI stub_glPNTrianglesiATI = NULL;
static int loaded_glPNTrianglesiATI = 0;

value glstub_glPNTrianglesiATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glPNTrianglesiATI){(*stub_glPNTrianglesiATI)(lv0, lv1);}
	else { 
		if(!loaded_glPNTrianglesiATI) { 
			loaded_glPNTrianglesiATI = 1;
			init_lib(); 
			stub_glPNTrianglesiATI=(pstub_glPNTrianglesiATI)get_proc_address("glPNTrianglesiATI");
			if(stub_glPNTrianglesiATI){(*stub_glPNTrianglesiATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glPNTrianglesiATI"); 
			} 
		} else { 
			FailWith("glPNTrianglesiATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPNTrianglesfATI */
typedef void APIENTRY (*pstub_glPNTrianglesfATI)(GLenum,GLfloat);
static pstub_glPNTrianglesfATI stub_glPNTrianglesfATI = NULL;
static int loaded_glPNTrianglesfATI = 0;

value glstub_glPNTrianglesfATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glPNTrianglesfATI){(*stub_glPNTrianglesfATI)(lv0, lv1);}
	else { 
		if(!loaded_glPNTrianglesfATI) { 
			loaded_glPNTrianglesfATI = 1;
			init_lib(); 
			stub_glPNTrianglesfATI=(pstub_glPNTrianglesfATI)get_proc_address("glPNTrianglesfATI");
			if(stub_glPNTrianglesfATI){(*stub_glPNTrianglesfATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glPNTrianglesfATI"); 
			} 
		} else { 
			FailWith("glPNTrianglesfATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glStencilOpSeparateATI */
typedef void APIENTRY (*pstub_glStencilOpSeparateATI)(GLenum,GLenum,GLenum,GLenum);
static pstub_glStencilOpSeparateATI stub_glStencilOpSeparateATI = NULL;
static int loaded_glStencilOpSeparateATI = 0;

value glstub_glStencilOpSeparateATI(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLenum lv2 =  get_glenum(v2);
	GLenum lv3 =  get_glenum(v3);

	if(stub_glStencilOpSeparateATI){(*stub_glStencilOpSeparateATI)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glStencilOpSeparateATI) { 
			loaded_glStencilOpSeparateATI = 1;
			init_lib(); 
			stub_glStencilOpSeparateATI=(pstub_glStencilOpSeparateATI)get_proc_address("glStencilOpSeparateATI");
			if(stub_glStencilOpSeparateATI){(*stub_glStencilOpSeparateATI)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glStencilOpSeparateATI"); 
			} 
		} else { 
			FailWith("glStencilOpSeparateATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glStencilFuncSeparateATI */
typedef void APIENTRY (*pstub_glStencilFuncSeparateATI)(GLenum,GLenum,GLint,GLuint);
static pstub_glStencilFuncSeparateATI stub_glStencilFuncSeparateATI = NULL;
static int loaded_glStencilFuncSeparateATI = 0;

value glstub_glStencilFuncSeparateATI(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint lv2 =  Int_val(v2);
	GLuint lv3 =  Int_val(v3);

	if(stub_glStencilFuncSeparateATI){(*stub_glStencilFuncSeparateATI)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glStencilFuncSeparateATI) { 
			loaded_glStencilFuncSeparateATI = 1;
			init_lib(); 
			stub_glStencilFuncSeparateATI=(pstub_glStencilFuncSeparateATI)get_proc_address("glStencilFuncSeparateATI");
			if(stub_glStencilFuncSeparateATI){(*stub_glStencilFuncSeparateATI)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glStencilFuncSeparateATI"); 
			} 
		} else { 
			FailWith("glStencilFuncSeparateATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glClientActiveVertexStreamATI */
typedef void APIENTRY (*pstub_glClientActiveVertexStreamATI)(GLenum);
static pstub_glClientActiveVertexStreamATI stub_glClientActiveVertexStreamATI = NULL;
static int loaded_glClientActiveVertexStreamATI = 0;

value glstub_glClientActiveVertexStreamATI(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glClientActiveVertexStreamATI){(*stub_glClientActiveVertexStreamATI)(lv0);}
	else { 
		if(!loaded_glClientActiveVertexStreamATI) { 
			loaded_glClientActiveVertexStreamATI = 1;
			init_lib(); 
			stub_glClientActiveVertexStreamATI=(pstub_glClientActiveVertexStreamATI)get_proc_address("glClientActiveVertexStreamATI");
			if(stub_glClientActiveVertexStreamATI){(*stub_glClientActiveVertexStreamATI)(lv0);}
			else { 
				FailWith("Cannot load glClientActiveVertexStreamATI"); 
			} 
		} else { 
			FailWith("glClientActiveVertexStreamATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexBlendEnviATI */
typedef void APIENTRY (*pstub_glVertexBlendEnviATI)(GLenum,GLint);
static pstub_glVertexBlendEnviATI stub_glVertexBlendEnviATI = NULL;
static int loaded_glVertexBlendEnviATI = 0;

value glstub_glVertexBlendEnviATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glVertexBlendEnviATI){(*stub_glVertexBlendEnviATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexBlendEnviATI) { 
			loaded_glVertexBlendEnviATI = 1;
			init_lib(); 
			stub_glVertexBlendEnviATI=(pstub_glVertexBlendEnviATI)get_proc_address("glVertexBlendEnviATI");
			if(stub_glVertexBlendEnviATI){(*stub_glVertexBlendEnviATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexBlendEnviATI"); 
			} 
		} else { 
			FailWith("glVertexBlendEnviATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexBlendEnvfATI */
typedef void APIENTRY (*pstub_glVertexBlendEnvfATI)(GLenum,GLfloat);
static pstub_glVertexBlendEnvfATI stub_glVertexBlendEnvfATI = NULL;
static int loaded_glVertexBlendEnvfATI = 0;

value glstub_glVertexBlendEnvfATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glVertexBlendEnvfATI){(*stub_glVertexBlendEnvfATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexBlendEnvfATI) { 
			loaded_glVertexBlendEnvfATI = 1;
			init_lib(); 
			stub_glVertexBlendEnvfATI=(pstub_glVertexBlendEnvfATI)get_proc_address("glVertexBlendEnvfATI");
			if(stub_glVertexBlendEnvfATI){(*stub_glVertexBlendEnvfATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexBlendEnvfATI"); 
			} 
		} else { 
			FailWith("glVertexBlendEnvfATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream2sATI */
typedef void APIENTRY (*pstub_glVertexStream2sATI)(GLenum,GLshort,GLshort);
static pstub_glVertexStream2sATI stub_glVertexStream2sATI = NULL;
static int loaded_glVertexStream2sATI = 0;

value glstub_glVertexStream2sATI(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);

	if(stub_glVertexStream2sATI){(*stub_glVertexStream2sATI)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glVertexStream2sATI) { 
			loaded_glVertexStream2sATI = 1;
			init_lib(); 
			stub_glVertexStream2sATI=(pstub_glVertexStream2sATI)get_proc_address("glVertexStream2sATI");
			if(stub_glVertexStream2sATI){(*stub_glVertexStream2sATI)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glVertexStream2sATI"); 
			} 
		} else { 
			FailWith("glVertexStream2sATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream2svATI */
typedef void APIENTRY (*pstub_glVertexStream2svATI)(GLenum,GLshort*);
static pstub_glVertexStream2svATI stub_glVertexStream2svATI = NULL;
static int loaded_glVertexStream2svATI = 0;

value glstub_glVertexStream2svATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexStream2svATI){(*stub_glVertexStream2svATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexStream2svATI) { 
			loaded_glVertexStream2svATI = 1;
			init_lib(); 
			stub_glVertexStream2svATI=(pstub_glVertexStream2svATI)get_proc_address("glVertexStream2svATI");
			if(stub_glVertexStream2svATI){(*stub_glVertexStream2svATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexStream2svATI"); 
			} 
		} else { 
			FailWith("glVertexStream2svATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream2iATI */
typedef void APIENTRY (*pstub_glVertexStream2iATI)(GLenum,GLint,GLint);
static pstub_glVertexStream2iATI stub_glVertexStream2iATI = NULL;
static int loaded_glVertexStream2iATI = 0;

value glstub_glVertexStream2iATI(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glVertexStream2iATI){(*stub_glVertexStream2iATI)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glVertexStream2iATI) { 
			loaded_glVertexStream2iATI = 1;
			init_lib(); 
			stub_glVertexStream2iATI=(pstub_glVertexStream2iATI)get_proc_address("glVertexStream2iATI");
			if(stub_glVertexStream2iATI){(*stub_glVertexStream2iATI)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glVertexStream2iATI"); 
			} 
		} else { 
			FailWith("glVertexStream2iATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream2ivATI */
typedef void APIENTRY (*pstub_glVertexStream2ivATI)(GLenum,GLint*);
static pstub_glVertexStream2ivATI stub_glVertexStream2ivATI = NULL;
static int loaded_glVertexStream2ivATI = 0;

value glstub_glVertexStream2ivATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexStream2ivATI){(*stub_glVertexStream2ivATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexStream2ivATI) { 
			loaded_glVertexStream2ivATI = 1;
			init_lib(); 
			stub_glVertexStream2ivATI=(pstub_glVertexStream2ivATI)get_proc_address("glVertexStream2ivATI");
			if(stub_glVertexStream2ivATI){(*stub_glVertexStream2ivATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexStream2ivATI"); 
			} 
		} else { 
			FailWith("glVertexStream2ivATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream2fATI */
typedef void APIENTRY (*pstub_glVertexStream2fATI)(GLenum,GLfloat,GLfloat);
static pstub_glVertexStream2fATI stub_glVertexStream2fATI = NULL;
static int loaded_glVertexStream2fATI = 0;

value glstub_glVertexStream2fATI(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glVertexStream2fATI){(*stub_glVertexStream2fATI)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glVertexStream2fATI) { 
			loaded_glVertexStream2fATI = 1;
			init_lib(); 
			stub_glVertexStream2fATI=(pstub_glVertexStream2fATI)get_proc_address("glVertexStream2fATI");
			if(stub_glVertexStream2fATI){(*stub_glVertexStream2fATI)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glVertexStream2fATI"); 
			} 
		} else { 
			FailWith("glVertexStream2fATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream2fvATI */
typedef void APIENTRY (*pstub_glVertexStream2fvATI)(GLenum,GLfloat*);
static pstub_glVertexStream2fvATI stub_glVertexStream2fvATI = NULL;
static int loaded_glVertexStream2fvATI = 0;

value glstub_glVertexStream2fvATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexStream2fvATI){(*stub_glVertexStream2fvATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexStream2fvATI) { 
			loaded_glVertexStream2fvATI = 1;
			init_lib(); 
			stub_glVertexStream2fvATI=(pstub_glVertexStream2fvATI)get_proc_address("glVertexStream2fvATI");
			if(stub_glVertexStream2fvATI){(*stub_glVertexStream2fvATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexStream2fvATI"); 
			} 
		} else { 
			FailWith("glVertexStream2fvATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream2dATI */
typedef void APIENTRY (*pstub_glVertexStream2dATI)(GLenum,GLdouble,GLdouble);
static pstub_glVertexStream2dATI stub_glVertexStream2dATI = NULL;
static int loaded_glVertexStream2dATI = 0;

value glstub_glVertexStream2dATI(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glVertexStream2dATI){(*stub_glVertexStream2dATI)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glVertexStream2dATI) { 
			loaded_glVertexStream2dATI = 1;
			init_lib(); 
			stub_glVertexStream2dATI=(pstub_glVertexStream2dATI)get_proc_address("glVertexStream2dATI");
			if(stub_glVertexStream2dATI){(*stub_glVertexStream2dATI)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glVertexStream2dATI"); 
			} 
		} else { 
			FailWith("glVertexStream2dATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream2dvATI */
typedef void APIENTRY (*pstub_glVertexStream2dvATI)(GLenum,GLdouble*);
static pstub_glVertexStream2dvATI stub_glVertexStream2dvATI = NULL;
static int loaded_glVertexStream2dvATI = 0;

value glstub_glVertexStream2dvATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexStream2dvATI){(*stub_glVertexStream2dvATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexStream2dvATI) { 
			loaded_glVertexStream2dvATI = 1;
			init_lib(); 
			stub_glVertexStream2dvATI=(pstub_glVertexStream2dvATI)get_proc_address("glVertexStream2dvATI");
			if(stub_glVertexStream2dvATI){(*stub_glVertexStream2dvATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexStream2dvATI"); 
			} 
		} else { 
			FailWith("glVertexStream2dvATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream3sATI */
typedef void APIENTRY (*pstub_glVertexStream3sATI)(GLenum,GLshort,GLshort,GLshort);
static pstub_glVertexStream3sATI stub_glVertexStream3sATI = NULL;
static int loaded_glVertexStream3sATI = 0;

value glstub_glVertexStream3sATI(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);

	if(stub_glVertexStream3sATI){(*stub_glVertexStream3sATI)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glVertexStream3sATI) { 
			loaded_glVertexStream3sATI = 1;
			init_lib(); 
			stub_glVertexStream3sATI=(pstub_glVertexStream3sATI)get_proc_address("glVertexStream3sATI");
			if(stub_glVertexStream3sATI){(*stub_glVertexStream3sATI)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glVertexStream3sATI"); 
			} 
		} else { 
			FailWith("glVertexStream3sATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream3svATI */
typedef void APIENTRY (*pstub_glVertexStream3svATI)(GLenum,GLshort*);
static pstub_glVertexStream3svATI stub_glVertexStream3svATI = NULL;
static int loaded_glVertexStream3svATI = 0;

value glstub_glVertexStream3svATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexStream3svATI){(*stub_glVertexStream3svATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexStream3svATI) { 
			loaded_glVertexStream3svATI = 1;
			init_lib(); 
			stub_glVertexStream3svATI=(pstub_glVertexStream3svATI)get_proc_address("glVertexStream3svATI");
			if(stub_glVertexStream3svATI){(*stub_glVertexStream3svATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexStream3svATI"); 
			} 
		} else { 
			FailWith("glVertexStream3svATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream3iATI */
typedef void APIENTRY (*pstub_glVertexStream3iATI)(GLenum,GLint,GLint,GLint);
static pstub_glVertexStream3iATI stub_glVertexStream3iATI = NULL;
static int loaded_glVertexStream3iATI = 0;

value glstub_glVertexStream3iATI(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);

	if(stub_glVertexStream3iATI){(*stub_glVertexStream3iATI)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glVertexStream3iATI) { 
			loaded_glVertexStream3iATI = 1;
			init_lib(); 
			stub_glVertexStream3iATI=(pstub_glVertexStream3iATI)get_proc_address("glVertexStream3iATI");
			if(stub_glVertexStream3iATI){(*stub_glVertexStream3iATI)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glVertexStream3iATI"); 
			} 
		} else { 
			FailWith("glVertexStream3iATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream3ivATI */
typedef void APIENTRY (*pstub_glVertexStream3ivATI)(GLenum,GLint*);
static pstub_glVertexStream3ivATI stub_glVertexStream3ivATI = NULL;
static int loaded_glVertexStream3ivATI = 0;

value glstub_glVertexStream3ivATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexStream3ivATI){(*stub_glVertexStream3ivATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexStream3ivATI) { 
			loaded_glVertexStream3ivATI = 1;
			init_lib(); 
			stub_glVertexStream3ivATI=(pstub_glVertexStream3ivATI)get_proc_address("glVertexStream3ivATI");
			if(stub_glVertexStream3ivATI){(*stub_glVertexStream3ivATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexStream3ivATI"); 
			} 
		} else { 
			FailWith("glVertexStream3ivATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream3fATI */
typedef void APIENTRY (*pstub_glVertexStream3fATI)(GLenum,GLfloat,GLfloat,GLfloat);
static pstub_glVertexStream3fATI stub_glVertexStream3fATI = NULL;
static int loaded_glVertexStream3fATI = 0;

value glstub_glVertexStream3fATI(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);

	if(stub_glVertexStream3fATI){(*stub_glVertexStream3fATI)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glVertexStream3fATI) { 
			loaded_glVertexStream3fATI = 1;
			init_lib(); 
			stub_glVertexStream3fATI=(pstub_glVertexStream3fATI)get_proc_address("glVertexStream3fATI");
			if(stub_glVertexStream3fATI){(*stub_glVertexStream3fATI)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glVertexStream3fATI"); 
			} 
		} else { 
			FailWith("glVertexStream3fATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream3fvATI */
typedef void APIENTRY (*pstub_glVertexStream3fvATI)(GLenum,GLfloat*);
static pstub_glVertexStream3fvATI stub_glVertexStream3fvATI = NULL;
static int loaded_glVertexStream3fvATI = 0;

value glstub_glVertexStream3fvATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexStream3fvATI){(*stub_glVertexStream3fvATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexStream3fvATI) { 
			loaded_glVertexStream3fvATI = 1;
			init_lib(); 
			stub_glVertexStream3fvATI=(pstub_glVertexStream3fvATI)get_proc_address("glVertexStream3fvATI");
			if(stub_glVertexStream3fvATI){(*stub_glVertexStream3fvATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexStream3fvATI"); 
			} 
		} else { 
			FailWith("glVertexStream3fvATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream3dATI */
typedef void APIENTRY (*pstub_glVertexStream3dATI)(GLenum,GLdouble,GLdouble,GLdouble);
static pstub_glVertexStream3dATI stub_glVertexStream3dATI = NULL;
static int loaded_glVertexStream3dATI = 0;

value glstub_glVertexStream3dATI(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);

	if(stub_glVertexStream3dATI){(*stub_glVertexStream3dATI)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glVertexStream3dATI) { 
			loaded_glVertexStream3dATI = 1;
			init_lib(); 
			stub_glVertexStream3dATI=(pstub_glVertexStream3dATI)get_proc_address("glVertexStream3dATI");
			if(stub_glVertexStream3dATI){(*stub_glVertexStream3dATI)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glVertexStream3dATI"); 
			} 
		} else { 
			FailWith("glVertexStream3dATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream3dvATI */
typedef void APIENTRY (*pstub_glVertexStream3dvATI)(GLenum,GLdouble*);
static pstub_glVertexStream3dvATI stub_glVertexStream3dvATI = NULL;
static int loaded_glVertexStream3dvATI = 0;

value glstub_glVertexStream3dvATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexStream3dvATI){(*stub_glVertexStream3dvATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexStream3dvATI) { 
			loaded_glVertexStream3dvATI = 1;
			init_lib(); 
			stub_glVertexStream3dvATI=(pstub_glVertexStream3dvATI)get_proc_address("glVertexStream3dvATI");
			if(stub_glVertexStream3dvATI){(*stub_glVertexStream3dvATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexStream3dvATI"); 
			} 
		} else { 
			FailWith("glVertexStream3dvATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream4sATI */
typedef void APIENTRY (*pstub_glVertexStream4sATI)(GLenum,GLshort,GLshort,GLshort,GLshort);
static pstub_glVertexStream4sATI stub_glVertexStream4sATI = NULL;
static int loaded_glVertexStream4sATI = 0;

value glstub_glVertexStream4sATI(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);
	GLshort lv4 =  Int_val(v4);

	if(stub_glVertexStream4sATI){(*stub_glVertexStream4sATI)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glVertexStream4sATI) { 
			loaded_glVertexStream4sATI = 1;
			init_lib(); 
			stub_glVertexStream4sATI=(pstub_glVertexStream4sATI)get_proc_address("glVertexStream4sATI");
			if(stub_glVertexStream4sATI){(*stub_glVertexStream4sATI)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glVertexStream4sATI"); 
			} 
		} else { 
			FailWith("glVertexStream4sATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream4svATI */
typedef void APIENTRY (*pstub_glVertexStream4svATI)(GLenum,GLshort*);
static pstub_glVertexStream4svATI stub_glVertexStream4svATI = NULL;
static int loaded_glVertexStream4svATI = 0;

value glstub_glVertexStream4svATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexStream4svATI){(*stub_glVertexStream4svATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexStream4svATI) { 
			loaded_glVertexStream4svATI = 1;
			init_lib(); 
			stub_glVertexStream4svATI=(pstub_glVertexStream4svATI)get_proc_address("glVertexStream4svATI");
			if(stub_glVertexStream4svATI){(*stub_glVertexStream4svATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexStream4svATI"); 
			} 
		} else { 
			FailWith("glVertexStream4svATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream4iATI */
typedef void APIENTRY (*pstub_glVertexStream4iATI)(GLenum,GLint,GLint,GLint,GLint);
static pstub_glVertexStream4iATI stub_glVertexStream4iATI = NULL;
static int loaded_glVertexStream4iATI = 0;

value glstub_glVertexStream4iATI(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);

	if(stub_glVertexStream4iATI){(*stub_glVertexStream4iATI)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glVertexStream4iATI) { 
			loaded_glVertexStream4iATI = 1;
			init_lib(); 
			stub_glVertexStream4iATI=(pstub_glVertexStream4iATI)get_proc_address("glVertexStream4iATI");
			if(stub_glVertexStream4iATI){(*stub_glVertexStream4iATI)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glVertexStream4iATI"); 
			} 
		} else { 
			FailWith("glVertexStream4iATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream4ivATI */
typedef void APIENTRY (*pstub_glVertexStream4ivATI)(GLenum,GLint*);
static pstub_glVertexStream4ivATI stub_glVertexStream4ivATI = NULL;
static int loaded_glVertexStream4ivATI = 0;

value glstub_glVertexStream4ivATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexStream4ivATI){(*stub_glVertexStream4ivATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexStream4ivATI) { 
			loaded_glVertexStream4ivATI = 1;
			init_lib(); 
			stub_glVertexStream4ivATI=(pstub_glVertexStream4ivATI)get_proc_address("glVertexStream4ivATI");
			if(stub_glVertexStream4ivATI){(*stub_glVertexStream4ivATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexStream4ivATI"); 
			} 
		} else { 
			FailWith("glVertexStream4ivATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream4fATI */
typedef void APIENTRY (*pstub_glVertexStream4fATI)(GLenum,GLfloat,GLfloat,GLfloat,GLfloat);
static pstub_glVertexStream4fATI stub_glVertexStream4fATI = NULL;
static int loaded_glVertexStream4fATI = 0;

value glstub_glVertexStream4fATI(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);
	GLfloat lv4 =  Double_val(v4);

	if(stub_glVertexStream4fATI){(*stub_glVertexStream4fATI)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glVertexStream4fATI) { 
			loaded_glVertexStream4fATI = 1;
			init_lib(); 
			stub_glVertexStream4fATI=(pstub_glVertexStream4fATI)get_proc_address("glVertexStream4fATI");
			if(stub_glVertexStream4fATI){(*stub_glVertexStream4fATI)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glVertexStream4fATI"); 
			} 
		} else { 
			FailWith("glVertexStream4fATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream4fvATI */
typedef void APIENTRY (*pstub_glVertexStream4fvATI)(GLenum,GLfloat*);
static pstub_glVertexStream4fvATI stub_glVertexStream4fvATI = NULL;
static int loaded_glVertexStream4fvATI = 0;

value glstub_glVertexStream4fvATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexStream4fvATI){(*stub_glVertexStream4fvATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexStream4fvATI) { 
			loaded_glVertexStream4fvATI = 1;
			init_lib(); 
			stub_glVertexStream4fvATI=(pstub_glVertexStream4fvATI)get_proc_address("glVertexStream4fvATI");
			if(stub_glVertexStream4fvATI){(*stub_glVertexStream4fvATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexStream4fvATI"); 
			} 
		} else { 
			FailWith("glVertexStream4fvATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream4dATI */
typedef void APIENTRY (*pstub_glVertexStream4dATI)(GLenum,GLdouble,GLdouble,GLdouble,GLdouble);
static pstub_glVertexStream4dATI stub_glVertexStream4dATI = NULL;
static int loaded_glVertexStream4dATI = 0;

value glstub_glVertexStream4dATI(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);
	GLdouble lv4 =  Double_val(v4);

	if(stub_glVertexStream4dATI){(*stub_glVertexStream4dATI)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glVertexStream4dATI) { 
			loaded_glVertexStream4dATI = 1;
			init_lib(); 
			stub_glVertexStream4dATI=(pstub_glVertexStream4dATI)get_proc_address("glVertexStream4dATI");
			if(stub_glVertexStream4dATI){(*stub_glVertexStream4dATI)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glVertexStream4dATI"); 
			} 
		} else { 
			FailWith("glVertexStream4dATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexStream4dvATI */
typedef void APIENTRY (*pstub_glVertexStream4dvATI)(GLenum,GLdouble*);
static pstub_glVertexStream4dvATI stub_glVertexStream4dvATI = NULL;
static int loaded_glVertexStream4dvATI = 0;

value glstub_glVertexStream4dvATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexStream4dvATI){(*stub_glVertexStream4dvATI)(lv0, lv1);}
	else { 
		if(!loaded_glVertexStream4dvATI) { 
			loaded_glVertexStream4dvATI = 1;
			init_lib(); 
			stub_glVertexStream4dvATI=(pstub_glVertexStream4dvATI)get_proc_address("glVertexStream4dvATI");
			if(stub_glVertexStream4dvATI){(*stub_glVertexStream4dvATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexStream4dvATI"); 
			} 
		} else { 
			FailWith("glVertexStream4dvATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormalStream3bATI */
typedef void APIENTRY (*pstub_glNormalStream3bATI)(GLenum,GLbyte,GLbyte,GLbyte);
static pstub_glNormalStream3bATI stub_glNormalStream3bATI = NULL;
static int loaded_glNormalStream3bATI = 0;

value glstub_glNormalStream3bATI(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLbyte lv1 =  Int_val(v1);
	GLbyte lv2 =  Int_val(v2);
	GLbyte lv3 =  Int_val(v3);

	if(stub_glNormalStream3bATI){(*stub_glNormalStream3bATI)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glNormalStream3bATI) { 
			loaded_glNormalStream3bATI = 1;
			init_lib(); 
			stub_glNormalStream3bATI=(pstub_glNormalStream3bATI)get_proc_address("glNormalStream3bATI");
			if(stub_glNormalStream3bATI){(*stub_glNormalStream3bATI)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glNormalStream3bATI"); 
			} 
		} else { 
			FailWith("glNormalStream3bATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormalStream3bvATI */
typedef void APIENTRY (*pstub_glNormalStream3bvATI)(GLenum,GLbyte*);
static pstub_glNormalStream3bvATI stub_glNormalStream3bvATI = NULL;
static int loaded_glNormalStream3bvATI = 0;

value glstub_glNormalStream3bvATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLbyte* lv1 =  Data_bigarray_val(v1);

	if(stub_glNormalStream3bvATI){(*stub_glNormalStream3bvATI)(lv0, lv1);}
	else { 
		if(!loaded_glNormalStream3bvATI) { 
			loaded_glNormalStream3bvATI = 1;
			init_lib(); 
			stub_glNormalStream3bvATI=(pstub_glNormalStream3bvATI)get_proc_address("glNormalStream3bvATI");
			if(stub_glNormalStream3bvATI){(*stub_glNormalStream3bvATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glNormalStream3bvATI"); 
			} 
		} else { 
			FailWith("glNormalStream3bvATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormalStream3sATI */
typedef void APIENTRY (*pstub_glNormalStream3sATI)(GLenum,GLshort,GLshort,GLshort);
static pstub_glNormalStream3sATI stub_glNormalStream3sATI = NULL;
static int loaded_glNormalStream3sATI = 0;

value glstub_glNormalStream3sATI(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);

	if(stub_glNormalStream3sATI){(*stub_glNormalStream3sATI)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glNormalStream3sATI) { 
			loaded_glNormalStream3sATI = 1;
			init_lib(); 
			stub_glNormalStream3sATI=(pstub_glNormalStream3sATI)get_proc_address("glNormalStream3sATI");
			if(stub_glNormalStream3sATI){(*stub_glNormalStream3sATI)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glNormalStream3sATI"); 
			} 
		} else { 
			FailWith("glNormalStream3sATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormalStream3svATI */
typedef void APIENTRY (*pstub_glNormalStream3svATI)(GLenum,GLshort*);
static pstub_glNormalStream3svATI stub_glNormalStream3svATI = NULL;
static int loaded_glNormalStream3svATI = 0;

value glstub_glNormalStream3svATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glNormalStream3svATI){(*stub_glNormalStream3svATI)(lv0, lv1);}
	else { 
		if(!loaded_glNormalStream3svATI) { 
			loaded_glNormalStream3svATI = 1;
			init_lib(); 
			stub_glNormalStream3svATI=(pstub_glNormalStream3svATI)get_proc_address("glNormalStream3svATI");
			if(stub_glNormalStream3svATI){(*stub_glNormalStream3svATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glNormalStream3svATI"); 
			} 
		} else { 
			FailWith("glNormalStream3svATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormalStream3iATI */
typedef void APIENTRY (*pstub_glNormalStream3iATI)(GLenum,GLint,GLint,GLint);
static pstub_glNormalStream3iATI stub_glNormalStream3iATI = NULL;
static int loaded_glNormalStream3iATI = 0;

value glstub_glNormalStream3iATI(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);

	if(stub_glNormalStream3iATI){(*stub_glNormalStream3iATI)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glNormalStream3iATI) { 
			loaded_glNormalStream3iATI = 1;
			init_lib(); 
			stub_glNormalStream3iATI=(pstub_glNormalStream3iATI)get_proc_address("glNormalStream3iATI");
			if(stub_glNormalStream3iATI){(*stub_glNormalStream3iATI)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glNormalStream3iATI"); 
			} 
		} else { 
			FailWith("glNormalStream3iATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormalStream3ivATI */
typedef void APIENTRY (*pstub_glNormalStream3ivATI)(GLenum,GLint*);
static pstub_glNormalStream3ivATI stub_glNormalStream3ivATI = NULL;
static int loaded_glNormalStream3ivATI = 0;

value glstub_glNormalStream3ivATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glNormalStream3ivATI){(*stub_glNormalStream3ivATI)(lv0, lv1);}
	else { 
		if(!loaded_glNormalStream3ivATI) { 
			loaded_glNormalStream3ivATI = 1;
			init_lib(); 
			stub_glNormalStream3ivATI=(pstub_glNormalStream3ivATI)get_proc_address("glNormalStream3ivATI");
			if(stub_glNormalStream3ivATI){(*stub_glNormalStream3ivATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glNormalStream3ivATI"); 
			} 
		} else { 
			FailWith("glNormalStream3ivATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormalStream3fATI */
typedef void APIENTRY (*pstub_glNormalStream3fATI)(GLenum,GLfloat,GLfloat,GLfloat);
static pstub_glNormalStream3fATI stub_glNormalStream3fATI = NULL;
static int loaded_glNormalStream3fATI = 0;

value glstub_glNormalStream3fATI(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);

	if(stub_glNormalStream3fATI){(*stub_glNormalStream3fATI)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glNormalStream3fATI) { 
			loaded_glNormalStream3fATI = 1;
			init_lib(); 
			stub_glNormalStream3fATI=(pstub_glNormalStream3fATI)get_proc_address("glNormalStream3fATI");
			if(stub_glNormalStream3fATI){(*stub_glNormalStream3fATI)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glNormalStream3fATI"); 
			} 
		} else { 
			FailWith("glNormalStream3fATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormalStream3fvATI */
typedef void APIENTRY (*pstub_glNormalStream3fvATI)(GLenum,GLfloat*);
static pstub_glNormalStream3fvATI stub_glNormalStream3fvATI = NULL;
static int loaded_glNormalStream3fvATI = 0;

value glstub_glNormalStream3fvATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glNormalStream3fvATI){(*stub_glNormalStream3fvATI)(lv0, lv1);}
	else { 
		if(!loaded_glNormalStream3fvATI) { 
			loaded_glNormalStream3fvATI = 1;
			init_lib(); 
			stub_glNormalStream3fvATI=(pstub_glNormalStream3fvATI)get_proc_address("glNormalStream3fvATI");
			if(stub_glNormalStream3fvATI){(*stub_glNormalStream3fvATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glNormalStream3fvATI"); 
			} 
		} else { 
			FailWith("glNormalStream3fvATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormalStream3dATI */
typedef void APIENTRY (*pstub_glNormalStream3dATI)(GLenum,GLdouble,GLdouble,GLdouble);
static pstub_glNormalStream3dATI stub_glNormalStream3dATI = NULL;
static int loaded_glNormalStream3dATI = 0;

value glstub_glNormalStream3dATI(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);

	if(stub_glNormalStream3dATI){(*stub_glNormalStream3dATI)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glNormalStream3dATI) { 
			loaded_glNormalStream3dATI = 1;
			init_lib(); 
			stub_glNormalStream3dATI=(pstub_glNormalStream3dATI)get_proc_address("glNormalStream3dATI");
			if(stub_glNormalStream3dATI){(*stub_glNormalStream3dATI)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glNormalStream3dATI"); 
			} 
		} else { 
			FailWith("glNormalStream3dATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormalStream3dvATI */
typedef void APIENTRY (*pstub_glNormalStream3dvATI)(GLenum,GLdouble*);
static pstub_glNormalStream3dvATI stub_glNormalStream3dvATI = NULL;
static int loaded_glNormalStream3dvATI = 0;

value glstub_glNormalStream3dvATI(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glNormalStream3dvATI){(*stub_glNormalStream3dvATI)(lv0, lv1);}
	else { 
		if(!loaded_glNormalStream3dvATI) { 
			loaded_glNormalStream3dvATI = 1;
			init_lib(); 
			stub_glNormalStream3dvATI=(pstub_glNormalStream3dvATI)get_proc_address("glNormalStream3dvATI");
			if(stub_glNormalStream3dvATI){(*stub_glNormalStream3dvATI)(lv0, lv1);}
			else { 
				FailWith("Cannot load glNormalStream3dvATI"); 
			} 
		} else { 
			FailWith("glNormalStream3dvATI not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDepthBoundsEXT */
typedef void APIENTRY (*pstub_glDepthBoundsEXT)(GLclampd,GLclampd);
static pstub_glDepthBoundsEXT stub_glDepthBoundsEXT = NULL;
static int loaded_glDepthBoundsEXT = 0;

value glstub_glDepthBoundsEXT(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLclampd lv0 =  Double_val(v0);
	GLclampd lv1 =  Double_val(v1);

	if(stub_glDepthBoundsEXT){(*stub_glDepthBoundsEXT)(lv0, lv1);}
	else { 
		if(!loaded_glDepthBoundsEXT) { 
			loaded_glDepthBoundsEXT = 1;
			init_lib(); 
			stub_glDepthBoundsEXT=(pstub_glDepthBoundsEXT)get_proc_address("glDepthBoundsEXT");
			if(stub_glDepthBoundsEXT){(*stub_glDepthBoundsEXT)(lv0, lv1);}
			else { 
				FailWith("Cannot load glDepthBoundsEXT"); 
			} 
		} else { 
			FailWith("glDepthBoundsEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDrawRangeElementsEXT */
typedef void APIENTRY (*pstub_glDrawRangeElementsEXT)(GLenum,GLuint,GLuint,GLsizei,GLenum,GLvoid*);
static pstub_glDrawRangeElementsEXT stub_glDrawRangeElementsEXT = NULL;
static int loaded_glDrawRangeElementsEXT = 0;

value glstub_glDrawRangeElementsEXT(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLuint lv1 =  Int_val(v1);
	GLuint lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);
	GLenum lv4 =  get_glenum(v4);
	GLvoid* lv5 =  Data_bigarray_val(v5);

	if(stub_glDrawRangeElementsEXT){(*stub_glDrawRangeElementsEXT)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glDrawRangeElementsEXT) { 
			loaded_glDrawRangeElementsEXT = 1;
			init_lib(); 
			stub_glDrawRangeElementsEXT=(pstub_glDrawRangeElementsEXT)get_proc_address("glDrawRangeElementsEXT");
			if(stub_glDrawRangeElementsEXT){(*stub_glDrawRangeElementsEXT)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glDrawRangeElementsEXT"); 
			} 
		} else { 
			FailWith("glDrawRangeElementsEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glDrawRangeElementsEXT_byte(value * argv, int n)
{
	return glstub_glDrawRangeElementsEXT(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glFogCoordfEXT */
typedef void APIENTRY (*pstub_glFogCoordfEXT)(GLfloat);
static pstub_glFogCoordfEXT stub_glFogCoordfEXT = NULL;
static int loaded_glFogCoordfEXT = 0;

value glstub_glFogCoordfEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);

	if(stub_glFogCoordfEXT){(*stub_glFogCoordfEXT)(lv0);}
	else { 
		if(!loaded_glFogCoordfEXT) { 
			loaded_glFogCoordfEXT = 1;
			init_lib(); 
			stub_glFogCoordfEXT=(pstub_glFogCoordfEXT)get_proc_address("glFogCoordfEXT");
			if(stub_glFogCoordfEXT){(*stub_glFogCoordfEXT)(lv0);}
			else { 
				FailWith("Cannot load glFogCoordfEXT"); 
			} 
		} else { 
			FailWith("glFogCoordfEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFogCoordfvEXT */
typedef void APIENTRY (*pstub_glFogCoordfvEXT)(GLfloat*);
static pstub_glFogCoordfvEXT stub_glFogCoordfvEXT = NULL;
static int loaded_glFogCoordfvEXT = 0;

value glstub_glFogCoordfvEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glFogCoordfvEXT){(*stub_glFogCoordfvEXT)(lv0);}
	else { 
		if(!loaded_glFogCoordfvEXT) { 
			loaded_glFogCoordfvEXT = 1;
			init_lib(); 
			stub_glFogCoordfvEXT=(pstub_glFogCoordfvEXT)get_proc_address("glFogCoordfvEXT");
			if(stub_glFogCoordfvEXT){(*stub_glFogCoordfvEXT)(lv0);}
			else { 
				FailWith("Cannot load glFogCoordfvEXT"); 
			} 
		} else { 
			FailWith("glFogCoordfvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFogCoorddEXT */
typedef void APIENTRY (*pstub_glFogCoorddEXT)(GLdouble);
static pstub_glFogCoorddEXT stub_glFogCoorddEXT = NULL;
static int loaded_glFogCoorddEXT = 0;

value glstub_glFogCoorddEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);

	if(stub_glFogCoorddEXT){(*stub_glFogCoorddEXT)(lv0);}
	else { 
		if(!loaded_glFogCoorddEXT) { 
			loaded_glFogCoorddEXT = 1;
			init_lib(); 
			stub_glFogCoorddEXT=(pstub_glFogCoorddEXT)get_proc_address("glFogCoorddEXT");
			if(stub_glFogCoorddEXT){(*stub_glFogCoorddEXT)(lv0);}
			else { 
				FailWith("Cannot load glFogCoorddEXT"); 
			} 
		} else { 
			FailWith("glFogCoorddEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFogCoorddvEXT */
typedef void APIENTRY (*pstub_glFogCoorddvEXT)(GLdouble*);
static pstub_glFogCoorddvEXT stub_glFogCoorddvEXT = NULL;
static int loaded_glFogCoorddvEXT = 0;

value glstub_glFogCoorddvEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glFogCoorddvEXT){(*stub_glFogCoorddvEXT)(lv0);}
	else { 
		if(!loaded_glFogCoorddvEXT) { 
			loaded_glFogCoorddvEXT = 1;
			init_lib(); 
			stub_glFogCoorddvEXT=(pstub_glFogCoorddvEXT)get_proc_address("glFogCoorddvEXT");
			if(stub_glFogCoorddvEXT){(*stub_glFogCoorddvEXT)(lv0);}
			else { 
				FailWith("Cannot load glFogCoorddvEXT"); 
			} 
		} else { 
			FailWith("glFogCoorddvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFogCoordPointerEXT */
typedef void APIENTRY (*pstub_glFogCoordPointerEXT)(GLenum,GLsizei,GLvoid*);
static pstub_glFogCoordPointerEXT stub_glFogCoordPointerEXT = NULL;
static int loaded_glFogCoordPointerEXT = 0;

value glstub_glFogCoordPointerEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizei lv1 =  Int_val(v1);
	GLvoid* lv2 =  Data_bigarray_val(v2);

	if(stub_glFogCoordPointerEXT){(*stub_glFogCoordPointerEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glFogCoordPointerEXT) { 
			loaded_glFogCoordPointerEXT = 1;
			init_lib(); 
			stub_glFogCoordPointerEXT=(pstub_glFogCoordPointerEXT)get_proc_address("glFogCoordPointerEXT");
			if(stub_glFogCoordPointerEXT){(*stub_glFogCoordPointerEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glFogCoordPointerEXT"); 
			} 
		} else { 
			FailWith("glFogCoordPointerEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3bEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3bEXT)(GLbyte,GLbyte,GLbyte);
static pstub_glSecondaryColor3bEXT stub_glSecondaryColor3bEXT = NULL;
static int loaded_glSecondaryColor3bEXT = 0;

value glstub_glSecondaryColor3bEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLbyte lv0 =  Int_val(v0);
	GLbyte lv1 =  Int_val(v1);
	GLbyte lv2 =  Int_val(v2);

	if(stub_glSecondaryColor3bEXT){(*stub_glSecondaryColor3bEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3bEXT) { 
			loaded_glSecondaryColor3bEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3bEXT=(pstub_glSecondaryColor3bEXT)get_proc_address("glSecondaryColor3bEXT");
			if(stub_glSecondaryColor3bEXT){(*stub_glSecondaryColor3bEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3bEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3bEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3bvEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3bvEXT)(GLbyte*);
static pstub_glSecondaryColor3bvEXT stub_glSecondaryColor3bvEXT = NULL;
static int loaded_glSecondaryColor3bvEXT = 0;

value glstub_glSecondaryColor3bvEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLbyte* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3bvEXT){(*stub_glSecondaryColor3bvEXT)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3bvEXT) { 
			loaded_glSecondaryColor3bvEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3bvEXT=(pstub_glSecondaryColor3bvEXT)get_proc_address("glSecondaryColor3bvEXT");
			if(stub_glSecondaryColor3bvEXT){(*stub_glSecondaryColor3bvEXT)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3bvEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3bvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3dEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3dEXT)(GLdouble,GLdouble,GLdouble);
static pstub_glSecondaryColor3dEXT stub_glSecondaryColor3dEXT = NULL;
static int loaded_glSecondaryColor3dEXT = 0;

value glstub_glSecondaryColor3dEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glSecondaryColor3dEXT){(*stub_glSecondaryColor3dEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3dEXT) { 
			loaded_glSecondaryColor3dEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3dEXT=(pstub_glSecondaryColor3dEXT)get_proc_address("glSecondaryColor3dEXT");
			if(stub_glSecondaryColor3dEXT){(*stub_glSecondaryColor3dEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3dEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3dEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3dvEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3dvEXT)(GLdouble*);
static pstub_glSecondaryColor3dvEXT stub_glSecondaryColor3dvEXT = NULL;
static int loaded_glSecondaryColor3dvEXT = 0;

value glstub_glSecondaryColor3dvEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3dvEXT){(*stub_glSecondaryColor3dvEXT)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3dvEXT) { 
			loaded_glSecondaryColor3dvEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3dvEXT=(pstub_glSecondaryColor3dvEXT)get_proc_address("glSecondaryColor3dvEXT");
			if(stub_glSecondaryColor3dvEXT){(*stub_glSecondaryColor3dvEXT)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3dvEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3dvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3fEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3fEXT)(GLfloat,GLfloat,GLfloat);
static pstub_glSecondaryColor3fEXT stub_glSecondaryColor3fEXT = NULL;
static int loaded_glSecondaryColor3fEXT = 0;

value glstub_glSecondaryColor3fEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glSecondaryColor3fEXT){(*stub_glSecondaryColor3fEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3fEXT) { 
			loaded_glSecondaryColor3fEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3fEXT=(pstub_glSecondaryColor3fEXT)get_proc_address("glSecondaryColor3fEXT");
			if(stub_glSecondaryColor3fEXT){(*stub_glSecondaryColor3fEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3fEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3fEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3fvEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3fvEXT)(GLfloat*);
static pstub_glSecondaryColor3fvEXT stub_glSecondaryColor3fvEXT = NULL;
static int loaded_glSecondaryColor3fvEXT = 0;

value glstub_glSecondaryColor3fvEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3fvEXT){(*stub_glSecondaryColor3fvEXT)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3fvEXT) { 
			loaded_glSecondaryColor3fvEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3fvEXT=(pstub_glSecondaryColor3fvEXT)get_proc_address("glSecondaryColor3fvEXT");
			if(stub_glSecondaryColor3fvEXT){(*stub_glSecondaryColor3fvEXT)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3fvEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3fvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3iEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3iEXT)(GLint,GLint,GLint);
static pstub_glSecondaryColor3iEXT stub_glSecondaryColor3iEXT = NULL;
static int loaded_glSecondaryColor3iEXT = 0;

value glstub_glSecondaryColor3iEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glSecondaryColor3iEXT){(*stub_glSecondaryColor3iEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3iEXT) { 
			loaded_glSecondaryColor3iEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3iEXT=(pstub_glSecondaryColor3iEXT)get_proc_address("glSecondaryColor3iEXT");
			if(stub_glSecondaryColor3iEXT){(*stub_glSecondaryColor3iEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3iEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3iEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3ivEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3ivEXT)(GLint*);
static pstub_glSecondaryColor3ivEXT stub_glSecondaryColor3ivEXT = NULL;
static int loaded_glSecondaryColor3ivEXT = 0;

value glstub_glSecondaryColor3ivEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3ivEXT){(*stub_glSecondaryColor3ivEXT)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3ivEXT) { 
			loaded_glSecondaryColor3ivEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3ivEXT=(pstub_glSecondaryColor3ivEXT)get_proc_address("glSecondaryColor3ivEXT");
			if(stub_glSecondaryColor3ivEXT){(*stub_glSecondaryColor3ivEXT)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3ivEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3ivEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3sEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3sEXT)(GLshort,GLshort,GLshort);
static pstub_glSecondaryColor3sEXT stub_glSecondaryColor3sEXT = NULL;
static int loaded_glSecondaryColor3sEXT = 0;

value glstub_glSecondaryColor3sEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);

	if(stub_glSecondaryColor3sEXT){(*stub_glSecondaryColor3sEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3sEXT) { 
			loaded_glSecondaryColor3sEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3sEXT=(pstub_glSecondaryColor3sEXT)get_proc_address("glSecondaryColor3sEXT");
			if(stub_glSecondaryColor3sEXT){(*stub_glSecondaryColor3sEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3sEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3sEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3svEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3svEXT)(GLshort*);
static pstub_glSecondaryColor3svEXT stub_glSecondaryColor3svEXT = NULL;
static int loaded_glSecondaryColor3svEXT = 0;

value glstub_glSecondaryColor3svEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3svEXT){(*stub_glSecondaryColor3svEXT)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3svEXT) { 
			loaded_glSecondaryColor3svEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3svEXT=(pstub_glSecondaryColor3svEXT)get_proc_address("glSecondaryColor3svEXT");
			if(stub_glSecondaryColor3svEXT){(*stub_glSecondaryColor3svEXT)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3svEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3svEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3ubEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3ubEXT)(GLubyte,GLubyte,GLubyte);
static pstub_glSecondaryColor3ubEXT stub_glSecondaryColor3ubEXT = NULL;
static int loaded_glSecondaryColor3ubEXT = 0;

value glstub_glSecondaryColor3ubEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLubyte lv0 =  Int_val(v0);
	GLubyte lv1 =  Int_val(v1);
	GLubyte lv2 =  Int_val(v2);

	if(stub_glSecondaryColor3ubEXT){(*stub_glSecondaryColor3ubEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3ubEXT) { 
			loaded_glSecondaryColor3ubEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3ubEXT=(pstub_glSecondaryColor3ubEXT)get_proc_address("glSecondaryColor3ubEXT");
			if(stub_glSecondaryColor3ubEXT){(*stub_glSecondaryColor3ubEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3ubEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3ubEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3ubvEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3ubvEXT)(GLubyte*);
static pstub_glSecondaryColor3ubvEXT stub_glSecondaryColor3ubvEXT = NULL;
static int loaded_glSecondaryColor3ubvEXT = 0;

value glstub_glSecondaryColor3ubvEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLubyte* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3ubvEXT){(*stub_glSecondaryColor3ubvEXT)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3ubvEXT) { 
			loaded_glSecondaryColor3ubvEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3ubvEXT=(pstub_glSecondaryColor3ubvEXT)get_proc_address("glSecondaryColor3ubvEXT");
			if(stub_glSecondaryColor3ubvEXT){(*stub_glSecondaryColor3ubvEXT)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3ubvEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3ubvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3uiEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3uiEXT)(GLuint,GLuint,GLuint);
static pstub_glSecondaryColor3uiEXT stub_glSecondaryColor3uiEXT = NULL;
static int loaded_glSecondaryColor3uiEXT = 0;

value glstub_glSecondaryColor3uiEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint lv1 =  Int_val(v1);
	GLuint lv2 =  Int_val(v2);

	if(stub_glSecondaryColor3uiEXT){(*stub_glSecondaryColor3uiEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3uiEXT) { 
			loaded_glSecondaryColor3uiEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3uiEXT=(pstub_glSecondaryColor3uiEXT)get_proc_address("glSecondaryColor3uiEXT");
			if(stub_glSecondaryColor3uiEXT){(*stub_glSecondaryColor3uiEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3uiEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3uiEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3uivEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3uivEXT)(GLuint*);
static pstub_glSecondaryColor3uivEXT stub_glSecondaryColor3uivEXT = NULL;
static int loaded_glSecondaryColor3uivEXT = 0;

value glstub_glSecondaryColor3uivEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3uivEXT){(*stub_glSecondaryColor3uivEXT)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3uivEXT) { 
			loaded_glSecondaryColor3uivEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3uivEXT=(pstub_glSecondaryColor3uivEXT)get_proc_address("glSecondaryColor3uivEXT");
			if(stub_glSecondaryColor3uivEXT){(*stub_glSecondaryColor3uivEXT)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3uivEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3uivEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3usEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3usEXT)(GLushort,GLushort,GLushort);
static pstub_glSecondaryColor3usEXT stub_glSecondaryColor3usEXT = NULL;
static int loaded_glSecondaryColor3usEXT = 0;

value glstub_glSecondaryColor3usEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLushort lv0 =  Int_val(v0);
	GLushort lv1 =  Int_val(v1);
	GLushort lv2 =  Int_val(v2);

	if(stub_glSecondaryColor3usEXT){(*stub_glSecondaryColor3usEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3usEXT) { 
			loaded_glSecondaryColor3usEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3usEXT=(pstub_glSecondaryColor3usEXT)get_proc_address("glSecondaryColor3usEXT");
			if(stub_glSecondaryColor3usEXT){(*stub_glSecondaryColor3usEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3usEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3usEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3usvEXT */
typedef void APIENTRY (*pstub_glSecondaryColor3usvEXT)(GLushort*);
static pstub_glSecondaryColor3usvEXT stub_glSecondaryColor3usvEXT = NULL;
static int loaded_glSecondaryColor3usvEXT = 0;

value glstub_glSecondaryColor3usvEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLushort* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3usvEXT){(*stub_glSecondaryColor3usvEXT)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3usvEXT) { 
			loaded_glSecondaryColor3usvEXT = 1;
			init_lib(); 
			stub_glSecondaryColor3usvEXT=(pstub_glSecondaryColor3usvEXT)get_proc_address("glSecondaryColor3usvEXT");
			if(stub_glSecondaryColor3usvEXT){(*stub_glSecondaryColor3usvEXT)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3usvEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColor3usvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColorPointerEXT */
typedef void APIENTRY (*pstub_glSecondaryColorPointerEXT)(GLint,GLenum,GLsizei,GLvoid*);
static pstub_glSecondaryColorPointerEXT stub_glSecondaryColorPointerEXT = NULL;
static int loaded_glSecondaryColorPointerEXT = 0;

value glstub_glSecondaryColorPointerEXT(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLsizei lv2 =  Int_val(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);

	if(stub_glSecondaryColorPointerEXT){(*stub_glSecondaryColorPointerEXT)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glSecondaryColorPointerEXT) { 
			loaded_glSecondaryColorPointerEXT = 1;
			init_lib(); 
			stub_glSecondaryColorPointerEXT=(pstub_glSecondaryColorPointerEXT)get_proc_address("glSecondaryColorPointerEXT");
			if(stub_glSecondaryColorPointerEXT){(*stub_glSecondaryColorPointerEXT)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glSecondaryColorPointerEXT"); 
			} 
		} else { 
			FailWith("glSecondaryColorPointerEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBeginVertexShaderEXT */
typedef void APIENTRY (*pstub_glBeginVertexShaderEXT)(GLvoid);
static pstub_glBeginVertexShaderEXT stub_glBeginVertexShaderEXT = NULL;
static int loaded_glBeginVertexShaderEXT = 0;

value glstub_glBeginVertexShaderEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);

	if(stub_glBeginVertexShaderEXT){(*stub_glBeginVertexShaderEXT)();}
	else { 
		if(!loaded_glBeginVertexShaderEXT) { 
			loaded_glBeginVertexShaderEXT = 1;
			init_lib(); 
			stub_glBeginVertexShaderEXT=(pstub_glBeginVertexShaderEXT)get_proc_address("glBeginVertexShaderEXT");
			if(stub_glBeginVertexShaderEXT){(*stub_glBeginVertexShaderEXT)();}
			else { 
				FailWith("Cannot load glBeginVertexShaderEXT"); 
			} 
		} else { 
			FailWith("glBeginVertexShaderEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEndVertexShaderEXT */
typedef void APIENTRY (*pstub_glEndVertexShaderEXT)(GLvoid);
static pstub_glEndVertexShaderEXT stub_glEndVertexShaderEXT = NULL;
static int loaded_glEndVertexShaderEXT = 0;

value glstub_glEndVertexShaderEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);

	if(stub_glEndVertexShaderEXT){(*stub_glEndVertexShaderEXT)();}
	else { 
		if(!loaded_glEndVertexShaderEXT) { 
			loaded_glEndVertexShaderEXT = 1;
			init_lib(); 
			stub_glEndVertexShaderEXT=(pstub_glEndVertexShaderEXT)get_proc_address("glEndVertexShaderEXT");
			if(stub_glEndVertexShaderEXT){(*stub_glEndVertexShaderEXT)();}
			else { 
				FailWith("Cannot load glEndVertexShaderEXT"); 
			} 
		} else { 
			FailWith("glEndVertexShaderEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBindVertexShaderEXT */
typedef void APIENTRY (*pstub_glBindVertexShaderEXT)(GLuint);
static pstub_glBindVertexShaderEXT stub_glBindVertexShaderEXT = NULL;
static int loaded_glBindVertexShaderEXT = 0;

value glstub_glBindVertexShaderEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glBindVertexShaderEXT){(*stub_glBindVertexShaderEXT)(lv0);}
	else { 
		if(!loaded_glBindVertexShaderEXT) { 
			loaded_glBindVertexShaderEXT = 1;
			init_lib(); 
			stub_glBindVertexShaderEXT=(pstub_glBindVertexShaderEXT)get_proc_address("glBindVertexShaderEXT");
			if(stub_glBindVertexShaderEXT){(*stub_glBindVertexShaderEXT)(lv0);}
			else { 
				FailWith("Cannot load glBindVertexShaderEXT"); 
			} 
		} else { 
			FailWith("glBindVertexShaderEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGenVertexShadersEXT */
typedef GLuint APIENTRY (*pstub_glGenVertexShadersEXT)(GLuint);
static pstub_glGenVertexShadersEXT stub_glGenVertexShadersEXT = NULL;
static int loaded_glGenVertexShadersEXT = 0;

value glstub_glGenVertexShadersEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint ret;
	if(stub_glGenVertexShadersEXT){ret = (*stub_glGenVertexShadersEXT)(lv0);}
	else { 
		if(!loaded_glGenVertexShadersEXT) { 
			loaded_glGenVertexShadersEXT = 1;
			init_lib(); 
			stub_glGenVertexShadersEXT=(pstub_glGenVertexShadersEXT)get_proc_address("glGenVertexShadersEXT");
			if(stub_glGenVertexShadersEXT){ret = (*stub_glGenVertexShadersEXT)(lv0);}
			else { 
				FailWith("Cannot load glGenVertexShadersEXT"); 
			} 
		} else { 
			FailWith("glGenVertexShadersEXT not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glDeleteVertexShaderEXT */
typedef void APIENTRY (*pstub_glDeleteVertexShaderEXT)(GLuint);
static pstub_glDeleteVertexShaderEXT stub_glDeleteVertexShaderEXT = NULL;
static int loaded_glDeleteVertexShaderEXT = 0;

value glstub_glDeleteVertexShaderEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glDeleteVertexShaderEXT){(*stub_glDeleteVertexShaderEXT)(lv0);}
	else { 
		if(!loaded_glDeleteVertexShaderEXT) { 
			loaded_glDeleteVertexShaderEXT = 1;
			init_lib(); 
			stub_glDeleteVertexShaderEXT=(pstub_glDeleteVertexShaderEXT)get_proc_address("glDeleteVertexShaderEXT");
			if(stub_glDeleteVertexShaderEXT){(*stub_glDeleteVertexShaderEXT)(lv0);}
			else { 
				FailWith("Cannot load glDeleteVertexShaderEXT"); 
			} 
		} else { 
			FailWith("glDeleteVertexShaderEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glShaderOp1EXT */
typedef void APIENTRY (*pstub_glShaderOp1EXT)(GLenum,GLuint,GLuint);
static pstub_glShaderOp1EXT stub_glShaderOp1EXT = NULL;
static int loaded_glShaderOp1EXT = 0;

value glstub_glShaderOp1EXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLuint lv1 =  Int_val(v1);
	GLuint lv2 =  Int_val(v2);

	if(stub_glShaderOp1EXT){(*stub_glShaderOp1EXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glShaderOp1EXT) { 
			loaded_glShaderOp1EXT = 1;
			init_lib(); 
			stub_glShaderOp1EXT=(pstub_glShaderOp1EXT)get_proc_address("glShaderOp1EXT");
			if(stub_glShaderOp1EXT){(*stub_glShaderOp1EXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glShaderOp1EXT"); 
			} 
		} else { 
			FailWith("glShaderOp1EXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glShaderOp2EXT */
typedef void APIENTRY (*pstub_glShaderOp2EXT)(GLenum,GLuint,GLuint,GLuint);
static pstub_glShaderOp2EXT stub_glShaderOp2EXT = NULL;
static int loaded_glShaderOp2EXT = 0;

value glstub_glShaderOp2EXT(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLuint lv1 =  Int_val(v1);
	GLuint lv2 =  Int_val(v2);
	GLuint lv3 =  Int_val(v3);

	if(stub_glShaderOp2EXT){(*stub_glShaderOp2EXT)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glShaderOp2EXT) { 
			loaded_glShaderOp2EXT = 1;
			init_lib(); 
			stub_glShaderOp2EXT=(pstub_glShaderOp2EXT)get_proc_address("glShaderOp2EXT");
			if(stub_glShaderOp2EXT){(*stub_glShaderOp2EXT)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glShaderOp2EXT"); 
			} 
		} else { 
			FailWith("glShaderOp2EXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glShaderOp3EXT */
typedef void APIENTRY (*pstub_glShaderOp3EXT)(GLenum,GLuint,GLuint,GLuint,GLuint);
static pstub_glShaderOp3EXT stub_glShaderOp3EXT = NULL;
static int loaded_glShaderOp3EXT = 0;

value glstub_glShaderOp3EXT(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLuint lv1 =  Int_val(v1);
	GLuint lv2 =  Int_val(v2);
	GLuint lv3 =  Int_val(v3);
	GLuint lv4 =  Int_val(v4);

	if(stub_glShaderOp3EXT){(*stub_glShaderOp3EXT)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glShaderOp3EXT) { 
			loaded_glShaderOp3EXT = 1;
			init_lib(); 
			stub_glShaderOp3EXT=(pstub_glShaderOp3EXT)get_proc_address("glShaderOp3EXT");
			if(stub_glShaderOp3EXT){(*stub_glShaderOp3EXT)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glShaderOp3EXT"); 
			} 
		} else { 
			FailWith("glShaderOp3EXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSwizzleEXT */
typedef void APIENTRY (*pstub_glSwizzleEXT)(GLuint,GLuint,GLenum,GLenum,GLenum,GLenum);
static pstub_glSwizzleEXT stub_glSwizzleEXT = NULL;
static int loaded_glSwizzleEXT = 0;

value glstub_glSwizzleEXT(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLenum lv3 =  get_glenum(v3);
	GLenum lv4 =  get_glenum(v4);
	GLenum lv5 =  get_glenum(v5);

	if(stub_glSwizzleEXT){(*stub_glSwizzleEXT)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glSwizzleEXT) { 
			loaded_glSwizzleEXT = 1;
			init_lib(); 
			stub_glSwizzleEXT=(pstub_glSwizzleEXT)get_proc_address("glSwizzleEXT");
			if(stub_glSwizzleEXT){(*stub_glSwizzleEXT)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glSwizzleEXT"); 
			} 
		} else { 
			FailWith("glSwizzleEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glSwizzleEXT_byte(value * argv, int n)
{
	return glstub_glSwizzleEXT(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glWriteMaskEXT */
typedef void APIENTRY (*pstub_glWriteMaskEXT)(GLuint,GLuint,GLenum,GLenum,GLenum,GLenum);
static pstub_glWriteMaskEXT stub_glWriteMaskEXT = NULL;
static int loaded_glWriteMaskEXT = 0;

value glstub_glWriteMaskEXT(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLenum lv3 =  get_glenum(v3);
	GLenum lv4 =  get_glenum(v4);
	GLenum lv5 =  get_glenum(v5);

	if(stub_glWriteMaskEXT){(*stub_glWriteMaskEXT)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glWriteMaskEXT) { 
			loaded_glWriteMaskEXT = 1;
			init_lib(); 
			stub_glWriteMaskEXT=(pstub_glWriteMaskEXT)get_proc_address("glWriteMaskEXT");
			if(stub_glWriteMaskEXT){(*stub_glWriteMaskEXT)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glWriteMaskEXT"); 
			} 
		} else { 
			FailWith("glWriteMaskEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glWriteMaskEXT_byte(value * argv, int n)
{
	return glstub_glWriteMaskEXT(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glInsertComponentEXT */
typedef void APIENTRY (*pstub_glInsertComponentEXT)(GLuint,GLuint,GLuint);
static pstub_glInsertComponentEXT stub_glInsertComponentEXT = NULL;
static int loaded_glInsertComponentEXT = 0;

value glstub_glInsertComponentEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint lv1 =  Int_val(v1);
	GLuint lv2 =  Int_val(v2);

	if(stub_glInsertComponentEXT){(*stub_glInsertComponentEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glInsertComponentEXT) { 
			loaded_glInsertComponentEXT = 1;
			init_lib(); 
			stub_glInsertComponentEXT=(pstub_glInsertComponentEXT)get_proc_address("glInsertComponentEXT");
			if(stub_glInsertComponentEXT){(*stub_glInsertComponentEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glInsertComponentEXT"); 
			} 
		} else { 
			FailWith("glInsertComponentEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glExtractComponentEXT */
typedef void APIENTRY (*pstub_glExtractComponentEXT)(GLuint,GLuint,GLuint);
static pstub_glExtractComponentEXT stub_glExtractComponentEXT = NULL;
static int loaded_glExtractComponentEXT = 0;

value glstub_glExtractComponentEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint lv1 =  Int_val(v1);
	GLuint lv2 =  Int_val(v2);

	if(stub_glExtractComponentEXT){(*stub_glExtractComponentEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glExtractComponentEXT) { 
			loaded_glExtractComponentEXT = 1;
			init_lib(); 
			stub_glExtractComponentEXT=(pstub_glExtractComponentEXT)get_proc_address("glExtractComponentEXT");
			if(stub_glExtractComponentEXT){(*stub_glExtractComponentEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glExtractComponentEXT"); 
			} 
		} else { 
			FailWith("glExtractComponentEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGenSymbolsEXT */
typedef GLuint APIENTRY (*pstub_glGenSymbolsEXT)(GLenum,GLenum,GLenum,GLuint);
static pstub_glGenSymbolsEXT stub_glGenSymbolsEXT = NULL;
static int loaded_glGenSymbolsEXT = 0;

value glstub_glGenSymbolsEXT(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLenum lv2 =  get_glenum(v2);
	GLuint lv3 =  Int_val(v3);
	GLuint ret;
	if(stub_glGenSymbolsEXT){ret = (*stub_glGenSymbolsEXT)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glGenSymbolsEXT) { 
			loaded_glGenSymbolsEXT = 1;
			init_lib(); 
			stub_glGenSymbolsEXT=(pstub_glGenSymbolsEXT)get_proc_address("glGenSymbolsEXT");
			if(stub_glGenSymbolsEXT){ret = (*stub_glGenSymbolsEXT)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glGenSymbolsEXT"); 
			} 
		} else { 
			FailWith("glGenSymbolsEXT not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glSetInvariantEXT */
typedef void APIENTRY (*pstub_glSetInvariantEXT)(GLuint,GLenum,GLvoid*);
static pstub_glSetInvariantEXT stub_glSetInvariantEXT = NULL;
static int loaded_glSetInvariantEXT = 0;

value glstub_glSetInvariantEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLvoid* lv2 =  Data_bigarray_val(v2);

	if(stub_glSetInvariantEXT){(*stub_glSetInvariantEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSetInvariantEXT) { 
			loaded_glSetInvariantEXT = 1;
			init_lib(); 
			stub_glSetInvariantEXT=(pstub_glSetInvariantEXT)get_proc_address("glSetInvariantEXT");
			if(stub_glSetInvariantEXT){(*stub_glSetInvariantEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSetInvariantEXT"); 
			} 
		} else { 
			FailWith("glSetInvariantEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSetLocalConstantEXT */
typedef void APIENTRY (*pstub_glSetLocalConstantEXT)(GLuint,GLenum,GLvoid*);
static pstub_glSetLocalConstantEXT stub_glSetLocalConstantEXT = NULL;
static int loaded_glSetLocalConstantEXT = 0;

value glstub_glSetLocalConstantEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLvoid* lv2 =  Data_bigarray_val(v2);

	if(stub_glSetLocalConstantEXT){(*stub_glSetLocalConstantEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSetLocalConstantEXT) { 
			loaded_glSetLocalConstantEXT = 1;
			init_lib(); 
			stub_glSetLocalConstantEXT=(pstub_glSetLocalConstantEXT)get_proc_address("glSetLocalConstantEXT");
			if(stub_glSetLocalConstantEXT){(*stub_glSetLocalConstantEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSetLocalConstantEXT"); 
			} 
		} else { 
			FailWith("glSetLocalConstantEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVariantbvEXT */
typedef void APIENTRY (*pstub_glVariantbvEXT)(GLuint,GLbyte*);
static pstub_glVariantbvEXT stub_glVariantbvEXT = NULL;
static int loaded_glVariantbvEXT = 0;

value glstub_glVariantbvEXT(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLbyte* lv1 =  Data_bigarray_val(v1);

	if(stub_glVariantbvEXT){(*stub_glVariantbvEXT)(lv0, lv1);}
	else { 
		if(!loaded_glVariantbvEXT) { 
			loaded_glVariantbvEXT = 1;
			init_lib(); 
			stub_glVariantbvEXT=(pstub_glVariantbvEXT)get_proc_address("glVariantbvEXT");
			if(stub_glVariantbvEXT){(*stub_glVariantbvEXT)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVariantbvEXT"); 
			} 
		} else { 
			FailWith("glVariantbvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVariantsvEXT */
typedef void APIENTRY (*pstub_glVariantsvEXT)(GLuint,GLshort*);
static pstub_glVariantsvEXT stub_glVariantsvEXT = NULL;
static int loaded_glVariantsvEXT = 0;

value glstub_glVariantsvEXT(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glVariantsvEXT){(*stub_glVariantsvEXT)(lv0, lv1);}
	else { 
		if(!loaded_glVariantsvEXT) { 
			loaded_glVariantsvEXT = 1;
			init_lib(); 
			stub_glVariantsvEXT=(pstub_glVariantsvEXT)get_proc_address("glVariantsvEXT");
			if(stub_glVariantsvEXT){(*stub_glVariantsvEXT)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVariantsvEXT"); 
			} 
		} else { 
			FailWith("glVariantsvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVariantivEXT */
typedef void APIENTRY (*pstub_glVariantivEXT)(GLuint,GLint*);
static pstub_glVariantivEXT stub_glVariantivEXT = NULL;
static int loaded_glVariantivEXT = 0;

value glstub_glVariantivEXT(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glVariantivEXT){(*stub_glVariantivEXT)(lv0, lv1);}
	else { 
		if(!loaded_glVariantivEXT) { 
			loaded_glVariantivEXT = 1;
			init_lib(); 
			stub_glVariantivEXT=(pstub_glVariantivEXT)get_proc_address("glVariantivEXT");
			if(stub_glVariantivEXT){(*stub_glVariantivEXT)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVariantivEXT"); 
			} 
		} else { 
			FailWith("glVariantivEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVariantfvEXT */
typedef void APIENTRY (*pstub_glVariantfvEXT)(GLuint,GLfloat*);
static pstub_glVariantfvEXT stub_glVariantfvEXT = NULL;
static int loaded_glVariantfvEXT = 0;

value glstub_glVariantfvEXT(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glVariantfvEXT){(*stub_glVariantfvEXT)(lv0, lv1);}
	else { 
		if(!loaded_glVariantfvEXT) { 
			loaded_glVariantfvEXT = 1;
			init_lib(); 
			stub_glVariantfvEXT=(pstub_glVariantfvEXT)get_proc_address("glVariantfvEXT");
			if(stub_glVariantfvEXT){(*stub_glVariantfvEXT)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVariantfvEXT"); 
			} 
		} else { 
			FailWith("glVariantfvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVariantdvEXT */
typedef void APIENTRY (*pstub_glVariantdvEXT)(GLuint,GLdouble*);
static pstub_glVariantdvEXT stub_glVariantdvEXT = NULL;
static int loaded_glVariantdvEXT = 0;

value glstub_glVariantdvEXT(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glVariantdvEXT){(*stub_glVariantdvEXT)(lv0, lv1);}
	else { 
		if(!loaded_glVariantdvEXT) { 
			loaded_glVariantdvEXT = 1;
			init_lib(); 
			stub_glVariantdvEXT=(pstub_glVariantdvEXT)get_proc_address("glVariantdvEXT");
			if(stub_glVariantdvEXT){(*stub_glVariantdvEXT)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVariantdvEXT"); 
			} 
		} else { 
			FailWith("glVariantdvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVariantubvEXT */
typedef void APIENTRY (*pstub_glVariantubvEXT)(GLuint,GLubyte*);
static pstub_glVariantubvEXT stub_glVariantubvEXT = NULL;
static int loaded_glVariantubvEXT = 0;

value glstub_glVariantubvEXT(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLubyte* lv1 =  Data_bigarray_val(v1);

	if(stub_glVariantubvEXT){(*stub_glVariantubvEXT)(lv0, lv1);}
	else { 
		if(!loaded_glVariantubvEXT) { 
			loaded_glVariantubvEXT = 1;
			init_lib(); 
			stub_glVariantubvEXT=(pstub_glVariantubvEXT)get_proc_address("glVariantubvEXT");
			if(stub_glVariantubvEXT){(*stub_glVariantubvEXT)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVariantubvEXT"); 
			} 
		} else { 
			FailWith("glVariantubvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVariantusvEXT */
typedef void APIENTRY (*pstub_glVariantusvEXT)(GLuint,GLushort*);
static pstub_glVariantusvEXT stub_glVariantusvEXT = NULL;
static int loaded_glVariantusvEXT = 0;

value glstub_glVariantusvEXT(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLushort* lv1 =  Data_bigarray_val(v1);

	if(stub_glVariantusvEXT){(*stub_glVariantusvEXT)(lv0, lv1);}
	else { 
		if(!loaded_glVariantusvEXT) { 
			loaded_glVariantusvEXT = 1;
			init_lib(); 
			stub_glVariantusvEXT=(pstub_glVariantusvEXT)get_proc_address("glVariantusvEXT");
			if(stub_glVariantusvEXT){(*stub_glVariantusvEXT)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVariantusvEXT"); 
			} 
		} else { 
			FailWith("glVariantusvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVariantuivEXT */
typedef void APIENTRY (*pstub_glVariantuivEXT)(GLuint,GLuint*);
static pstub_glVariantuivEXT stub_glVariantuivEXT = NULL;
static int loaded_glVariantuivEXT = 0;

value glstub_glVariantuivEXT(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);

	if(stub_glVariantuivEXT){(*stub_glVariantuivEXT)(lv0, lv1);}
	else { 
		if(!loaded_glVariantuivEXT) { 
			loaded_glVariantuivEXT = 1;
			init_lib(); 
			stub_glVariantuivEXT=(pstub_glVariantuivEXT)get_proc_address("glVariantuivEXT");
			if(stub_glVariantuivEXT){(*stub_glVariantuivEXT)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVariantuivEXT"); 
			} 
		} else { 
			FailWith("glVariantuivEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVariantPointerEXT */
typedef void APIENTRY (*pstub_glVariantPointerEXT)(GLuint,GLenum,GLuint,GLvoid*);
static pstub_glVariantPointerEXT stub_glVariantPointerEXT = NULL;
static int loaded_glVariantPointerEXT = 0;

value glstub_glVariantPointerEXT(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLuint lv2 =  Int_val(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);

	if(stub_glVariantPointerEXT){(*stub_glVariantPointerEXT)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glVariantPointerEXT) { 
			loaded_glVariantPointerEXT = 1;
			init_lib(); 
			stub_glVariantPointerEXT=(pstub_glVariantPointerEXT)get_proc_address("glVariantPointerEXT");
			if(stub_glVariantPointerEXT){(*stub_glVariantPointerEXT)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glVariantPointerEXT"); 
			} 
		} else { 
			FailWith("glVariantPointerEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEnableVariantClientStateEXT */
typedef void APIENTRY (*pstub_glEnableVariantClientStateEXT)(GLuint);
static pstub_glEnableVariantClientStateEXT stub_glEnableVariantClientStateEXT = NULL;
static int loaded_glEnableVariantClientStateEXT = 0;

value glstub_glEnableVariantClientStateEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glEnableVariantClientStateEXT){(*stub_glEnableVariantClientStateEXT)(lv0);}
	else { 
		if(!loaded_glEnableVariantClientStateEXT) { 
			loaded_glEnableVariantClientStateEXT = 1;
			init_lib(); 
			stub_glEnableVariantClientStateEXT=(pstub_glEnableVariantClientStateEXT)get_proc_address("glEnableVariantClientStateEXT");
			if(stub_glEnableVariantClientStateEXT){(*stub_glEnableVariantClientStateEXT)(lv0);}
			else { 
				FailWith("Cannot load glEnableVariantClientStateEXT"); 
			} 
		} else { 
			FailWith("glEnableVariantClientStateEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDisableVariantClientStateEXT */
typedef void APIENTRY (*pstub_glDisableVariantClientStateEXT)(GLuint);
static pstub_glDisableVariantClientStateEXT stub_glDisableVariantClientStateEXT = NULL;
static int loaded_glDisableVariantClientStateEXT = 0;

value glstub_glDisableVariantClientStateEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glDisableVariantClientStateEXT){(*stub_glDisableVariantClientStateEXT)(lv0);}
	else { 
		if(!loaded_glDisableVariantClientStateEXT) { 
			loaded_glDisableVariantClientStateEXT = 1;
			init_lib(); 
			stub_glDisableVariantClientStateEXT=(pstub_glDisableVariantClientStateEXT)get_proc_address("glDisableVariantClientStateEXT");
			if(stub_glDisableVariantClientStateEXT){(*stub_glDisableVariantClientStateEXT)(lv0);}
			else { 
				FailWith("Cannot load glDisableVariantClientStateEXT"); 
			} 
		} else { 
			FailWith("glDisableVariantClientStateEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBindLightParameterEXT */
typedef GLuint APIENTRY (*pstub_glBindLightParameterEXT)(GLenum,GLenum);
static pstub_glBindLightParameterEXT stub_glBindLightParameterEXT = NULL;
static int loaded_glBindLightParameterEXT = 0;

value glstub_glBindLightParameterEXT(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLuint ret;
	if(stub_glBindLightParameterEXT){ret = (*stub_glBindLightParameterEXT)(lv0, lv1);}
	else { 
		if(!loaded_glBindLightParameterEXT) { 
			loaded_glBindLightParameterEXT = 1;
			init_lib(); 
			stub_glBindLightParameterEXT=(pstub_glBindLightParameterEXT)get_proc_address("glBindLightParameterEXT");
			if(stub_glBindLightParameterEXT){ret = (*stub_glBindLightParameterEXT)(lv0, lv1);}
			else { 
				FailWith("Cannot load glBindLightParameterEXT"); 
			} 
		} else { 
			FailWith("glBindLightParameterEXT not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glBindMaterialParameterEXT */
typedef GLuint APIENTRY (*pstub_glBindMaterialParameterEXT)(GLenum,GLenum);
static pstub_glBindMaterialParameterEXT stub_glBindMaterialParameterEXT = NULL;
static int loaded_glBindMaterialParameterEXT = 0;

value glstub_glBindMaterialParameterEXT(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLuint ret;
	if(stub_glBindMaterialParameterEXT){ret = (*stub_glBindMaterialParameterEXT)(lv0, lv1);}
	else { 
		if(!loaded_glBindMaterialParameterEXT) { 
			loaded_glBindMaterialParameterEXT = 1;
			init_lib(); 
			stub_glBindMaterialParameterEXT=(pstub_glBindMaterialParameterEXT)get_proc_address("glBindMaterialParameterEXT");
			if(stub_glBindMaterialParameterEXT){ret = (*stub_glBindMaterialParameterEXT)(lv0, lv1);}
			else { 
				FailWith("Cannot load glBindMaterialParameterEXT"); 
			} 
		} else { 
			FailWith("glBindMaterialParameterEXT not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glBindTexGenParameterEXT */
typedef GLuint APIENTRY (*pstub_glBindTexGenParameterEXT)(GLenum,GLenum,GLenum);
static pstub_glBindTexGenParameterEXT stub_glBindTexGenParameterEXT = NULL;
static int loaded_glBindTexGenParameterEXT = 0;

value glstub_glBindTexGenParameterEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLenum lv2 =  get_glenum(v2);
	GLuint ret;
	if(stub_glBindTexGenParameterEXT){ret = (*stub_glBindTexGenParameterEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glBindTexGenParameterEXT) { 
			loaded_glBindTexGenParameterEXT = 1;
			init_lib(); 
			stub_glBindTexGenParameterEXT=(pstub_glBindTexGenParameterEXT)get_proc_address("glBindTexGenParameterEXT");
			if(stub_glBindTexGenParameterEXT){ret = (*stub_glBindTexGenParameterEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glBindTexGenParameterEXT"); 
			} 
		} else { 
			FailWith("glBindTexGenParameterEXT not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glBindTextureUnitParameterEXT */
typedef GLuint APIENTRY (*pstub_glBindTextureUnitParameterEXT)(GLenum,GLenum);
static pstub_glBindTextureUnitParameterEXT stub_glBindTextureUnitParameterEXT = NULL;
static int loaded_glBindTextureUnitParameterEXT = 0;

value glstub_glBindTextureUnitParameterEXT(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLuint ret;
	if(stub_glBindTextureUnitParameterEXT){ret = (*stub_glBindTextureUnitParameterEXT)(lv0, lv1);}
	else { 
		if(!loaded_glBindTextureUnitParameterEXT) { 
			loaded_glBindTextureUnitParameterEXT = 1;
			init_lib(); 
			stub_glBindTextureUnitParameterEXT=(pstub_glBindTextureUnitParameterEXT)get_proc_address("glBindTextureUnitParameterEXT");
			if(stub_glBindTextureUnitParameterEXT){ret = (*stub_glBindTextureUnitParameterEXT)(lv0, lv1);}
			else { 
				FailWith("Cannot load glBindTextureUnitParameterEXT"); 
			} 
		} else { 
			FailWith("glBindTextureUnitParameterEXT not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glBindParameterEXT */
typedef GLuint APIENTRY (*pstub_glBindParameterEXT)(GLenum);
static pstub_glBindParameterEXT stub_glBindParameterEXT = NULL;
static int loaded_glBindParameterEXT = 0;

value glstub_glBindParameterEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLuint ret;
	if(stub_glBindParameterEXT){ret = (*stub_glBindParameterEXT)(lv0);}
	else { 
		if(!loaded_glBindParameterEXT) { 
			loaded_glBindParameterEXT = 1;
			init_lib(); 
			stub_glBindParameterEXT=(pstub_glBindParameterEXT)get_proc_address("glBindParameterEXT");
			if(stub_glBindParameterEXT){ret = (*stub_glBindParameterEXT)(lv0);}
			else { 
				FailWith("Cannot load glBindParameterEXT"); 
			} 
		} else { 
			FailWith("glBindParameterEXT not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glIsVariantEnabledEXT */
typedef GLboolean APIENTRY (*pstub_glIsVariantEnabledEXT)(GLuint,GLenum);
static pstub_glIsVariantEnabledEXT stub_glIsVariantEnabledEXT = NULL;
static int loaded_glIsVariantEnabledEXT = 0;

value glstub_glIsVariantEnabledEXT(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLboolean ret;
	if(stub_glIsVariantEnabledEXT){ret = (*stub_glIsVariantEnabledEXT)(lv0, lv1);}
	else { 
		if(!loaded_glIsVariantEnabledEXT) { 
			loaded_glIsVariantEnabledEXT = 1;
			init_lib(); 
			stub_glIsVariantEnabledEXT=(pstub_glIsVariantEnabledEXT)get_proc_address("glIsVariantEnabledEXT");
			if(stub_glIsVariantEnabledEXT){ret = (*stub_glIsVariantEnabledEXT)(lv0, lv1);}
			else { 
				FailWith("Cannot load glIsVariantEnabledEXT"); 
			} 
		} else { 
			FailWith("glIsVariantEnabledEXT not available"); 
		}
	}
	result = Bool_val(ret);
	CAMLreturn (result);
}

/* glGetVariantBooleanvEXT */
typedef void APIENTRY (*pstub_glGetVariantBooleanvEXT)(GLuint,GLenum,GLboolean*);
static pstub_glGetVariantBooleanvEXT stub_glGetVariantBooleanvEXT = NULL;
static int loaded_glGetVariantBooleanvEXT = 0;

value glstub_glGetVariantBooleanvEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLboolean* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetVariantBooleanvEXT){(*stub_glGetVariantBooleanvEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetVariantBooleanvEXT) { 
			loaded_glGetVariantBooleanvEXT = 1;
			init_lib(); 
			stub_glGetVariantBooleanvEXT=(pstub_glGetVariantBooleanvEXT)get_proc_address("glGetVariantBooleanvEXT");
			if(stub_glGetVariantBooleanvEXT){(*stub_glGetVariantBooleanvEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetVariantBooleanvEXT"); 
			} 
		} else { 
			FailWith("glGetVariantBooleanvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetVariantIntegervEXT */
typedef void APIENTRY (*pstub_glGetVariantIntegervEXT)(GLuint,GLenum,GLint*);
static pstub_glGetVariantIntegervEXT stub_glGetVariantIntegervEXT = NULL;
static int loaded_glGetVariantIntegervEXT = 0;

value glstub_glGetVariantIntegervEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetVariantIntegervEXT){(*stub_glGetVariantIntegervEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetVariantIntegervEXT) { 
			loaded_glGetVariantIntegervEXT = 1;
			init_lib(); 
			stub_glGetVariantIntegervEXT=(pstub_glGetVariantIntegervEXT)get_proc_address("glGetVariantIntegervEXT");
			if(stub_glGetVariantIntegervEXT){(*stub_glGetVariantIntegervEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetVariantIntegervEXT"); 
			} 
		} else { 
			FailWith("glGetVariantIntegervEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetVariantFloatvEXT */
typedef void APIENTRY (*pstub_glGetVariantFloatvEXT)(GLuint,GLenum,GLfloat*);
static pstub_glGetVariantFloatvEXT stub_glGetVariantFloatvEXT = NULL;
static int loaded_glGetVariantFloatvEXT = 0;

value glstub_glGetVariantFloatvEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetVariantFloatvEXT){(*stub_glGetVariantFloatvEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetVariantFloatvEXT) { 
			loaded_glGetVariantFloatvEXT = 1;
			init_lib(); 
			stub_glGetVariantFloatvEXT=(pstub_glGetVariantFloatvEXT)get_proc_address("glGetVariantFloatvEXT");
			if(stub_glGetVariantFloatvEXT){(*stub_glGetVariantFloatvEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetVariantFloatvEXT"); 
			} 
		} else { 
			FailWith("glGetVariantFloatvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetVariantPointervEXT */
typedef void APIENTRY (*pstub_glGetVariantPointervEXT)(GLuint,GLenum,GLvoid**);
static pstub_glGetVariantPointervEXT stub_glGetVariantPointervEXT = NULL;
static int loaded_glGetVariantPointervEXT = 0;

value glstub_glGetVariantPointervEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLvoid** lv2 =  Data_bigarray_val(v2);

	if(stub_glGetVariantPointervEXT){(*stub_glGetVariantPointervEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetVariantPointervEXT) { 
			loaded_glGetVariantPointervEXT = 1;
			init_lib(); 
			stub_glGetVariantPointervEXT=(pstub_glGetVariantPointervEXT)get_proc_address("glGetVariantPointervEXT");
			if(stub_glGetVariantPointervEXT){(*stub_glGetVariantPointervEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetVariantPointervEXT"); 
			} 
		} else { 
			FailWith("glGetVariantPointervEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetInvariantBooleanvEXT */
typedef void APIENTRY (*pstub_glGetInvariantBooleanvEXT)(GLuint,GLenum,GLboolean*);
static pstub_glGetInvariantBooleanvEXT stub_glGetInvariantBooleanvEXT = NULL;
static int loaded_glGetInvariantBooleanvEXT = 0;

value glstub_glGetInvariantBooleanvEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLboolean* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetInvariantBooleanvEXT){(*stub_glGetInvariantBooleanvEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetInvariantBooleanvEXT) { 
			loaded_glGetInvariantBooleanvEXT = 1;
			init_lib(); 
			stub_glGetInvariantBooleanvEXT=(pstub_glGetInvariantBooleanvEXT)get_proc_address("glGetInvariantBooleanvEXT");
			if(stub_glGetInvariantBooleanvEXT){(*stub_glGetInvariantBooleanvEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetInvariantBooleanvEXT"); 
			} 
		} else { 
			FailWith("glGetInvariantBooleanvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetInvariantIntegervEXT */
typedef void APIENTRY (*pstub_glGetInvariantIntegervEXT)(GLuint,GLenum,GLint*);
static pstub_glGetInvariantIntegervEXT stub_glGetInvariantIntegervEXT = NULL;
static int loaded_glGetInvariantIntegervEXT = 0;

value glstub_glGetInvariantIntegervEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetInvariantIntegervEXT){(*stub_glGetInvariantIntegervEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetInvariantIntegervEXT) { 
			loaded_glGetInvariantIntegervEXT = 1;
			init_lib(); 
			stub_glGetInvariantIntegervEXT=(pstub_glGetInvariantIntegervEXT)get_proc_address("glGetInvariantIntegervEXT");
			if(stub_glGetInvariantIntegervEXT){(*stub_glGetInvariantIntegervEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetInvariantIntegervEXT"); 
			} 
		} else { 
			FailWith("glGetInvariantIntegervEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetInvariantFloatvEXT */
typedef void APIENTRY (*pstub_glGetInvariantFloatvEXT)(GLuint,GLenum,GLfloat*);
static pstub_glGetInvariantFloatvEXT stub_glGetInvariantFloatvEXT = NULL;
static int loaded_glGetInvariantFloatvEXT = 0;

value glstub_glGetInvariantFloatvEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetInvariantFloatvEXT){(*stub_glGetInvariantFloatvEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetInvariantFloatvEXT) { 
			loaded_glGetInvariantFloatvEXT = 1;
			init_lib(); 
			stub_glGetInvariantFloatvEXT=(pstub_glGetInvariantFloatvEXT)get_proc_address("glGetInvariantFloatvEXT");
			if(stub_glGetInvariantFloatvEXT){(*stub_glGetInvariantFloatvEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetInvariantFloatvEXT"); 
			} 
		} else { 
			FailWith("glGetInvariantFloatvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetLocalConstantBooleanvEXT */
typedef void APIENTRY (*pstub_glGetLocalConstantBooleanvEXT)(GLuint,GLenum,GLboolean*);
static pstub_glGetLocalConstantBooleanvEXT stub_glGetLocalConstantBooleanvEXT = NULL;
static int loaded_glGetLocalConstantBooleanvEXT = 0;

value glstub_glGetLocalConstantBooleanvEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLboolean* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetLocalConstantBooleanvEXT){(*stub_glGetLocalConstantBooleanvEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetLocalConstantBooleanvEXT) { 
			loaded_glGetLocalConstantBooleanvEXT = 1;
			init_lib(); 
			stub_glGetLocalConstantBooleanvEXT=(pstub_glGetLocalConstantBooleanvEXT)get_proc_address("glGetLocalConstantBooleanvEXT");
			if(stub_glGetLocalConstantBooleanvEXT){(*stub_glGetLocalConstantBooleanvEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetLocalConstantBooleanvEXT"); 
			} 
		} else { 
			FailWith("glGetLocalConstantBooleanvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetLocalConstantIntegervEXT */
typedef void APIENTRY (*pstub_glGetLocalConstantIntegervEXT)(GLuint,GLenum,GLint*);
static pstub_glGetLocalConstantIntegervEXT stub_glGetLocalConstantIntegervEXT = NULL;
static int loaded_glGetLocalConstantIntegervEXT = 0;

value glstub_glGetLocalConstantIntegervEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetLocalConstantIntegervEXT){(*stub_glGetLocalConstantIntegervEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetLocalConstantIntegervEXT) { 
			loaded_glGetLocalConstantIntegervEXT = 1;
			init_lib(); 
			stub_glGetLocalConstantIntegervEXT=(pstub_glGetLocalConstantIntegervEXT)get_proc_address("glGetLocalConstantIntegervEXT");
			if(stub_glGetLocalConstantIntegervEXT){(*stub_glGetLocalConstantIntegervEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetLocalConstantIntegervEXT"); 
			} 
		} else { 
			FailWith("glGetLocalConstantIntegervEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetLocalConstantFloatvEXT */
typedef void APIENTRY (*pstub_glGetLocalConstantFloatvEXT)(GLuint,GLenum,GLfloat*);
static pstub_glGetLocalConstantFloatvEXT stub_glGetLocalConstantFloatvEXT = NULL;
static int loaded_glGetLocalConstantFloatvEXT = 0;

value glstub_glGetLocalConstantFloatvEXT(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetLocalConstantFloatvEXT){(*stub_glGetLocalConstantFloatvEXT)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetLocalConstantFloatvEXT) { 
			loaded_glGetLocalConstantFloatvEXT = 1;
			init_lib(); 
			stub_glGetLocalConstantFloatvEXT=(pstub_glGetLocalConstantFloatvEXT)get_proc_address("glGetLocalConstantFloatvEXT");
			if(stub_glGetLocalConstantFloatvEXT){(*stub_glGetLocalConstantFloatvEXT)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetLocalConstantFloatvEXT"); 
			} 
		} else { 
			FailWith("glGetLocalConstantFloatvEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBufferRegionEnabledEXT */
typedef GLuint APIENTRY (*pstub_glBufferRegionEnabledEXT)(GLvoid);
static pstub_glBufferRegionEnabledEXT stub_glBufferRegionEnabledEXT = NULL;
static int loaded_glBufferRegionEnabledEXT = 0;

value glstub_glBufferRegionEnabledEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint ret;
	if(stub_glBufferRegionEnabledEXT){ret = (*stub_glBufferRegionEnabledEXT)();}
	else { 
		if(!loaded_glBufferRegionEnabledEXT) { 
			loaded_glBufferRegionEnabledEXT = 1;
			init_lib(); 
			stub_glBufferRegionEnabledEXT=(pstub_glBufferRegionEnabledEXT)get_proc_address("glBufferRegionEnabledEXT");
			if(stub_glBufferRegionEnabledEXT){ret = (*stub_glBufferRegionEnabledEXT)();}
			else { 
				FailWith("Cannot load glBufferRegionEnabledEXT"); 
			} 
		} else { 
			FailWith("glBufferRegionEnabledEXT not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glNewBufferRegionEXT */
typedef GLuint APIENTRY (*pstub_glNewBufferRegionEXT)(GLenum);
static pstub_glNewBufferRegionEXT stub_glNewBufferRegionEXT = NULL;
static int loaded_glNewBufferRegionEXT = 0;

value glstub_glNewBufferRegionEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLuint ret;
	if(stub_glNewBufferRegionEXT){ret = (*stub_glNewBufferRegionEXT)(lv0);}
	else { 
		if(!loaded_glNewBufferRegionEXT) { 
			loaded_glNewBufferRegionEXT = 1;
			init_lib(); 
			stub_glNewBufferRegionEXT=(pstub_glNewBufferRegionEXT)get_proc_address("glNewBufferRegionEXT");
			if(stub_glNewBufferRegionEXT){ret = (*stub_glNewBufferRegionEXT)(lv0);}
			else { 
				FailWith("Cannot load glNewBufferRegionEXT"); 
			} 
		} else { 
			FailWith("glNewBufferRegionEXT not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glDeleteBufferRegionEXT */
typedef void APIENTRY (*pstub_glDeleteBufferRegionEXT)(GLenum);
static pstub_glDeleteBufferRegionEXT stub_glDeleteBufferRegionEXT = NULL;
static int loaded_glDeleteBufferRegionEXT = 0;

value glstub_glDeleteBufferRegionEXT(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glDeleteBufferRegionEXT){(*stub_glDeleteBufferRegionEXT)(lv0);}
	else { 
		if(!loaded_glDeleteBufferRegionEXT) { 
			loaded_glDeleteBufferRegionEXT = 1;
			init_lib(); 
			stub_glDeleteBufferRegionEXT=(pstub_glDeleteBufferRegionEXT)get_proc_address("glDeleteBufferRegionEXT");
			if(stub_glDeleteBufferRegionEXT){(*stub_glDeleteBufferRegionEXT)(lv0);}
			else { 
				FailWith("Cannot load glDeleteBufferRegionEXT"); 
			} 
		} else { 
			FailWith("glDeleteBufferRegionEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glReadBufferRegionEXT */
typedef void APIENTRY (*pstub_glReadBufferRegionEXT)(GLuint,GLint,GLint,GLsizei,GLsizei);
static pstub_glReadBufferRegionEXT stub_glReadBufferRegionEXT = NULL;
static int loaded_glReadBufferRegionEXT = 0;

value glstub_glReadBufferRegionEXT(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);
	GLsizei lv4 =  Int_val(v4);

	if(stub_glReadBufferRegionEXT){(*stub_glReadBufferRegionEXT)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glReadBufferRegionEXT) { 
			loaded_glReadBufferRegionEXT = 1;
			init_lib(); 
			stub_glReadBufferRegionEXT=(pstub_glReadBufferRegionEXT)get_proc_address("glReadBufferRegionEXT");
			if(stub_glReadBufferRegionEXT){(*stub_glReadBufferRegionEXT)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glReadBufferRegionEXT"); 
			} 
		} else { 
			FailWith("glReadBufferRegionEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDrawBufferRegionEXT */
typedef void APIENTRY (*pstub_glDrawBufferRegionEXT)(GLuint,GLint,GLint,GLsizei,GLsizei,GLint,GLint);
static pstub_glDrawBufferRegionEXT stub_glDrawBufferRegionEXT = NULL;
static int loaded_glDrawBufferRegionEXT = 0;

value glstub_glDrawBufferRegionEXT(value v0, value v1, value v2, value v3, value v4, value v5, value v6)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam2(v5,v6);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);
	GLsizei lv4 =  Int_val(v4);
	GLint lv5 =  Int_val(v5);
	GLint lv6 =  Int_val(v6);

	if(stub_glDrawBufferRegionEXT){(*stub_glDrawBufferRegionEXT)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
	else { 
		if(!loaded_glDrawBufferRegionEXT) { 
			loaded_glDrawBufferRegionEXT = 1;
			init_lib(); 
			stub_glDrawBufferRegionEXT=(pstub_glDrawBufferRegionEXT)get_proc_address("glDrawBufferRegionEXT");
			if(stub_glDrawBufferRegionEXT){(*stub_glDrawBufferRegionEXT)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
			else { 
				FailWith("Cannot load glDrawBufferRegionEXT"); 
			} 
		} else { 
			FailWith("glDrawBufferRegionEXT not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glDrawBufferRegionEXT_byte(value * argv, int n)
{
	return glstub_glDrawBufferRegionEXT(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]);
}

/* glAccum */
typedef void APIENTRY (*pstub_glAccum)(GLenum,GLfloat);
static pstub_glAccum stub_glAccum = NULL;
static int loaded_glAccum = 0;

value glstub_glAccum(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glAccum){(*stub_glAccum)(lv0, lv1);}
	else { 
		if(!loaded_glAccum) { 
			loaded_glAccum = 1;
			init_lib(); 
			stub_glAccum=(pstub_glAccum)get_proc_address("glAccum");
			if(stub_glAccum){(*stub_glAccum)(lv0, lv1);}
			else { 
				FailWith("Cannot load glAccum"); 
			} 
		} else { 
			FailWith("glAccum not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glAlphaFunc */
typedef void APIENTRY (*pstub_glAlphaFunc)(GLenum,GLclampf);
static pstub_glAlphaFunc stub_glAlphaFunc = NULL;
static int loaded_glAlphaFunc = 0;

value glstub_glAlphaFunc(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLclampf lv1 =  Double_val(v1);

	if(stub_glAlphaFunc){(*stub_glAlphaFunc)(lv0, lv1);}
	else { 
		if(!loaded_glAlphaFunc) { 
			loaded_glAlphaFunc = 1;
			init_lib(); 
			stub_glAlphaFunc=(pstub_glAlphaFunc)get_proc_address("glAlphaFunc");
			if(stub_glAlphaFunc){(*stub_glAlphaFunc)(lv0, lv1);}
			else { 
				FailWith("Cannot load glAlphaFunc"); 
			} 
		} else { 
			FailWith("glAlphaFunc not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glAreTexturesResident */
typedef GLboolean APIENTRY (*pstub_glAreTexturesResident)(GLsizei,GLuint*,GLboolean*);
static pstub_glAreTexturesResident stub_glAreTexturesResident = NULL;
static int loaded_glAreTexturesResident = 0;

value glstub_glAreTexturesResident(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);
	GLboolean* lv2 =  Data_bigarray_val(v2);
	GLboolean ret;
	if(stub_glAreTexturesResident){ret = (*stub_glAreTexturesResident)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glAreTexturesResident) { 
			loaded_glAreTexturesResident = 1;
			init_lib(); 
			stub_glAreTexturesResident=(pstub_glAreTexturesResident)get_proc_address("glAreTexturesResident");
			if(stub_glAreTexturesResident){ret = (*stub_glAreTexturesResident)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glAreTexturesResident"); 
			} 
		} else { 
			FailWith("glAreTexturesResident not available"); 
		}
	}
	result = Bool_val(ret);
	CAMLreturn (result);
}

/* glArrayElement */
typedef void APIENTRY (*pstub_glArrayElement)(GLint);
static pstub_glArrayElement stub_glArrayElement = NULL;
static int loaded_glArrayElement = 0;

value glstub_glArrayElement(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);

	if(stub_glArrayElement){(*stub_glArrayElement)(lv0);}
	else { 
		if(!loaded_glArrayElement) { 
			loaded_glArrayElement = 1;
			init_lib(); 
			stub_glArrayElement=(pstub_glArrayElement)get_proc_address("glArrayElement");
			if(stub_glArrayElement){(*stub_glArrayElement)(lv0);}
			else { 
				FailWith("Cannot load glArrayElement"); 
			} 
		} else { 
			FailWith("glArrayElement not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBegin */
typedef void APIENTRY (*pstub_glBegin)(GLenum);
static pstub_glBegin stub_glBegin = NULL;
static int loaded_glBegin = 0;

value glstub_glBegin(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glBegin){(*stub_glBegin)(lv0);}
	else { 
		if(!loaded_glBegin) { 
			loaded_glBegin = 1;
			init_lib(); 
			stub_glBegin=(pstub_glBegin)get_proc_address("glBegin");
			if(stub_glBegin){(*stub_glBegin)(lv0);}
			else { 
				FailWith("Cannot load glBegin"); 
			} 
		} else { 
			FailWith("glBegin not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBindTexture */
typedef void APIENTRY (*pstub_glBindTexture)(GLenum,GLuint);
static pstub_glBindTexture stub_glBindTexture = NULL;
static int loaded_glBindTexture = 0;

value glstub_glBindTexture(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLuint lv1 =  Int_val(v1);

	if(stub_glBindTexture){(*stub_glBindTexture)(lv0, lv1);}
	else { 
		if(!loaded_glBindTexture) { 
			loaded_glBindTexture = 1;
			init_lib(); 
			stub_glBindTexture=(pstub_glBindTexture)get_proc_address("glBindTexture");
			if(stub_glBindTexture){(*stub_glBindTexture)(lv0, lv1);}
			else { 
				FailWith("Cannot load glBindTexture"); 
			} 
		} else { 
			FailWith("glBindTexture not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBitmap */
typedef void APIENTRY (*pstub_glBitmap)(GLsizei,GLsizei,GLfloat,GLfloat,GLfloat,GLfloat,GLubyte*);
static pstub_glBitmap stub_glBitmap = NULL;
static int loaded_glBitmap = 0;

value glstub_glBitmap(value v0, value v1, value v2, value v3, value v4, value v5, value v6)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam2(v5,v6);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);
	GLfloat lv4 =  Double_val(v4);
	GLfloat lv5 =  Double_val(v5);
	GLubyte* lv6 =  Data_bigarray_val(v6);

	if(stub_glBitmap){(*stub_glBitmap)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
	else { 
		if(!loaded_glBitmap) { 
			loaded_glBitmap = 1;
			init_lib(); 
			stub_glBitmap=(pstub_glBitmap)get_proc_address("glBitmap");
			if(stub_glBitmap){(*stub_glBitmap)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
			else { 
				FailWith("Cannot load glBitmap"); 
			} 
		} else { 
			FailWith("glBitmap not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glBitmap_byte(value * argv, int n)
{
	return glstub_glBitmap(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]);
}

/* glBlendFunc */
typedef void APIENTRY (*pstub_glBlendFunc)(GLenum,GLenum);
static pstub_glBlendFunc stub_glBlendFunc = NULL;
static int loaded_glBlendFunc = 0;

value glstub_glBlendFunc(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);

	if(stub_glBlendFunc){(*stub_glBlendFunc)(lv0, lv1);}
	else { 
		if(!loaded_glBlendFunc) { 
			loaded_glBlendFunc = 1;
			init_lib(); 
			stub_glBlendFunc=(pstub_glBlendFunc)get_proc_address("glBlendFunc");
			if(stub_glBlendFunc){(*stub_glBlendFunc)(lv0, lv1);}
			else { 
				FailWith("Cannot load glBlendFunc"); 
			} 
		} else { 
			FailWith("glBlendFunc not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glCallList */
typedef void APIENTRY (*pstub_glCallList)(GLuint);
static pstub_glCallList stub_glCallList = NULL;
static int loaded_glCallList = 0;

value glstub_glCallList(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glCallList){(*stub_glCallList)(lv0);}
	else { 
		if(!loaded_glCallList) { 
			loaded_glCallList = 1;
			init_lib(); 
			stub_glCallList=(pstub_glCallList)get_proc_address("glCallList");
			if(stub_glCallList){(*stub_glCallList)(lv0);}
			else { 
				FailWith("Cannot load glCallList"); 
			} 
		} else { 
			FailWith("glCallList not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glCallLists */
typedef void APIENTRY (*pstub_glCallLists)(GLsizei,GLenum,GLvoid*);
static pstub_glCallLists stub_glCallLists = NULL;
static int loaded_glCallLists = 0;

value glstub_glCallLists(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLvoid* lv2 =  Data_bigarray_val(v2);

	if(stub_glCallLists){(*stub_glCallLists)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glCallLists) { 
			loaded_glCallLists = 1;
			init_lib(); 
			stub_glCallLists=(pstub_glCallLists)get_proc_address("glCallLists");
			if(stub_glCallLists){(*stub_glCallLists)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glCallLists"); 
			} 
		} else { 
			FailWith("glCallLists not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glClear */
typedef void APIENTRY (*pstub_glClear)(GLbitfield);
static pstub_glClear stub_glClear = NULL;
static int loaded_glClear = 0;

value glstub_glClear(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLbitfield lv0 =  get_glenum_list(v0);

	if(stub_glClear){(*stub_glClear)(lv0);}
	else { 
		if(!loaded_glClear) { 
			loaded_glClear = 1;
			init_lib(); 
			stub_glClear=(pstub_glClear)get_proc_address("glClear");
			if(stub_glClear){(*stub_glClear)(lv0);}
			else { 
				FailWith("Cannot load glClear"); 
			} 
		} else { 
			FailWith("glClear not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glClearAccum */
typedef void APIENTRY (*pstub_glClearAccum)(GLfloat,GLfloat,GLfloat,GLfloat);
static pstub_glClearAccum stub_glClearAccum = NULL;
static int loaded_glClearAccum = 0;

value glstub_glClearAccum(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);

	if(stub_glClearAccum){(*stub_glClearAccum)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glClearAccum) { 
			loaded_glClearAccum = 1;
			init_lib(); 
			stub_glClearAccum=(pstub_glClearAccum)get_proc_address("glClearAccum");
			if(stub_glClearAccum){(*stub_glClearAccum)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glClearAccum"); 
			} 
		} else { 
			FailWith("glClearAccum not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glClearColor */
typedef void APIENTRY (*pstub_glClearColor)(GLclampf,GLclampf,GLclampf,GLclampf);
static pstub_glClearColor stub_glClearColor = NULL;
static int loaded_glClearColor = 0;

value glstub_glClearColor(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLclampf lv0 =  Double_val(v0);
	GLclampf lv1 =  Double_val(v1);
	GLclampf lv2 =  Double_val(v2);
	GLclampf lv3 =  Double_val(v3);

	if(stub_glClearColor){(*stub_glClearColor)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glClearColor) { 
			loaded_glClearColor = 1;
			init_lib(); 
			stub_glClearColor=(pstub_glClearColor)get_proc_address("glClearColor");
			if(stub_glClearColor){(*stub_glClearColor)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glClearColor"); 
			} 
		} else { 
			FailWith("glClearColor not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glClearDepth */
typedef void APIENTRY (*pstub_glClearDepth)(GLclampd);
static pstub_glClearDepth stub_glClearDepth = NULL;
static int loaded_glClearDepth = 0;

value glstub_glClearDepth(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLclampd lv0 =  Double_val(v0);

	if(stub_glClearDepth){(*stub_glClearDepth)(lv0);}
	else { 
		if(!loaded_glClearDepth) { 
			loaded_glClearDepth = 1;
			init_lib(); 
			stub_glClearDepth=(pstub_glClearDepth)get_proc_address("glClearDepth");
			if(stub_glClearDepth){(*stub_glClearDepth)(lv0);}
			else { 
				FailWith("Cannot load glClearDepth"); 
			} 
		} else { 
			FailWith("glClearDepth not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glClearIndex */
typedef void APIENTRY (*pstub_glClearIndex)(GLfloat);
static pstub_glClearIndex stub_glClearIndex = NULL;
static int loaded_glClearIndex = 0;

value glstub_glClearIndex(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);

	if(stub_glClearIndex){(*stub_glClearIndex)(lv0);}
	else { 
		if(!loaded_glClearIndex) { 
			loaded_glClearIndex = 1;
			init_lib(); 
			stub_glClearIndex=(pstub_glClearIndex)get_proc_address("glClearIndex");
			if(stub_glClearIndex){(*stub_glClearIndex)(lv0);}
			else { 
				FailWith("Cannot load glClearIndex"); 
			} 
		} else { 
			FailWith("glClearIndex not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glClearStencil */
typedef void APIENTRY (*pstub_glClearStencil)(GLint);
static pstub_glClearStencil stub_glClearStencil = NULL;
static int loaded_glClearStencil = 0;

value glstub_glClearStencil(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);

	if(stub_glClearStencil){(*stub_glClearStencil)(lv0);}
	else { 
		if(!loaded_glClearStencil) { 
			loaded_glClearStencil = 1;
			init_lib(); 
			stub_glClearStencil=(pstub_glClearStencil)get_proc_address("glClearStencil");
			if(stub_glClearStencil){(*stub_glClearStencil)(lv0);}
			else { 
				FailWith("Cannot load glClearStencil"); 
			} 
		} else { 
			FailWith("glClearStencil not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glClipPlane */
typedef void APIENTRY (*pstub_glClipPlane)(GLenum,GLdouble*);
static pstub_glClipPlane stub_glClipPlane = NULL;
static int loaded_glClipPlane = 0;

value glstub_glClipPlane(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glClipPlane){(*stub_glClipPlane)(lv0, lv1);}
	else { 
		if(!loaded_glClipPlane) { 
			loaded_glClipPlane = 1;
			init_lib(); 
			stub_glClipPlane=(pstub_glClipPlane)get_proc_address("glClipPlane");
			if(stub_glClipPlane){(*stub_glClipPlane)(lv0, lv1);}
			else { 
				FailWith("Cannot load glClipPlane"); 
			} 
		} else { 
			FailWith("glClipPlane not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3b */
typedef void APIENTRY (*pstub_glColor3b)(GLbyte,GLbyte,GLbyte);
static pstub_glColor3b stub_glColor3b = NULL;
static int loaded_glColor3b = 0;

value glstub_glColor3b(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLbyte lv0 =  Int_val(v0);
	GLbyte lv1 =  Int_val(v1);
	GLbyte lv2 =  Int_val(v2);

	if(stub_glColor3b){(*stub_glColor3b)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glColor3b) { 
			loaded_glColor3b = 1;
			init_lib(); 
			stub_glColor3b=(pstub_glColor3b)get_proc_address("glColor3b");
			if(stub_glColor3b){(*stub_glColor3b)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glColor3b"); 
			} 
		} else { 
			FailWith("glColor3b not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3bv */
typedef void APIENTRY (*pstub_glColor3bv)(GLbyte*);
static pstub_glColor3bv stub_glColor3bv = NULL;
static int loaded_glColor3bv = 0;

value glstub_glColor3bv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLbyte* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor3bv){(*stub_glColor3bv)(lv0);}
	else { 
		if(!loaded_glColor3bv) { 
			loaded_glColor3bv = 1;
			init_lib(); 
			stub_glColor3bv=(pstub_glColor3bv)get_proc_address("glColor3bv");
			if(stub_glColor3bv){(*stub_glColor3bv)(lv0);}
			else { 
				FailWith("Cannot load glColor3bv"); 
			} 
		} else { 
			FailWith("glColor3bv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3d */
typedef void APIENTRY (*pstub_glColor3d)(GLdouble,GLdouble,GLdouble);
static pstub_glColor3d stub_glColor3d = NULL;
static int loaded_glColor3d = 0;

value glstub_glColor3d(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glColor3d){(*stub_glColor3d)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glColor3d) { 
			loaded_glColor3d = 1;
			init_lib(); 
			stub_glColor3d=(pstub_glColor3d)get_proc_address("glColor3d");
			if(stub_glColor3d){(*stub_glColor3d)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glColor3d"); 
			} 
		} else { 
			FailWith("glColor3d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3dv */
typedef void APIENTRY (*pstub_glColor3dv)(GLdouble*);
static pstub_glColor3dv stub_glColor3dv = NULL;
static int loaded_glColor3dv = 0;

value glstub_glColor3dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor3dv){(*stub_glColor3dv)(lv0);}
	else { 
		if(!loaded_glColor3dv) { 
			loaded_glColor3dv = 1;
			init_lib(); 
			stub_glColor3dv=(pstub_glColor3dv)get_proc_address("glColor3dv");
			if(stub_glColor3dv){(*stub_glColor3dv)(lv0);}
			else { 
				FailWith("Cannot load glColor3dv"); 
			} 
		} else { 
			FailWith("glColor3dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3f */
typedef void APIENTRY (*pstub_glColor3f)(GLfloat,GLfloat,GLfloat);
static pstub_glColor3f stub_glColor3f = NULL;
static int loaded_glColor3f = 0;

value glstub_glColor3f(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glColor3f){(*stub_glColor3f)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glColor3f) { 
			loaded_glColor3f = 1;
			init_lib(); 
			stub_glColor3f=(pstub_glColor3f)get_proc_address("glColor3f");
			if(stub_glColor3f){(*stub_glColor3f)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glColor3f"); 
			} 
		} else { 
			FailWith("glColor3f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3fv */
typedef void APIENTRY (*pstub_glColor3fv)(GLfloat*);
static pstub_glColor3fv stub_glColor3fv = NULL;
static int loaded_glColor3fv = 0;

value glstub_glColor3fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor3fv){(*stub_glColor3fv)(lv0);}
	else { 
		if(!loaded_glColor3fv) { 
			loaded_glColor3fv = 1;
			init_lib(); 
			stub_glColor3fv=(pstub_glColor3fv)get_proc_address("glColor3fv");
			if(stub_glColor3fv){(*stub_glColor3fv)(lv0);}
			else { 
				FailWith("Cannot load glColor3fv"); 
			} 
		} else { 
			FailWith("glColor3fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3i */
typedef void APIENTRY (*pstub_glColor3i)(GLint,GLint,GLint);
static pstub_glColor3i stub_glColor3i = NULL;
static int loaded_glColor3i = 0;

value glstub_glColor3i(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glColor3i){(*stub_glColor3i)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glColor3i) { 
			loaded_glColor3i = 1;
			init_lib(); 
			stub_glColor3i=(pstub_glColor3i)get_proc_address("glColor3i");
			if(stub_glColor3i){(*stub_glColor3i)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glColor3i"); 
			} 
		} else { 
			FailWith("glColor3i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3iv */
typedef void APIENTRY (*pstub_glColor3iv)(GLint*);
static pstub_glColor3iv stub_glColor3iv = NULL;
static int loaded_glColor3iv = 0;

value glstub_glColor3iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor3iv){(*stub_glColor3iv)(lv0);}
	else { 
		if(!loaded_glColor3iv) { 
			loaded_glColor3iv = 1;
			init_lib(); 
			stub_glColor3iv=(pstub_glColor3iv)get_proc_address("glColor3iv");
			if(stub_glColor3iv){(*stub_glColor3iv)(lv0);}
			else { 
				FailWith("Cannot load glColor3iv"); 
			} 
		} else { 
			FailWith("glColor3iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3s */
typedef void APIENTRY (*pstub_glColor3s)(GLshort,GLshort,GLshort);
static pstub_glColor3s stub_glColor3s = NULL;
static int loaded_glColor3s = 0;

value glstub_glColor3s(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);

	if(stub_glColor3s){(*stub_glColor3s)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glColor3s) { 
			loaded_glColor3s = 1;
			init_lib(); 
			stub_glColor3s=(pstub_glColor3s)get_proc_address("glColor3s");
			if(stub_glColor3s){(*stub_glColor3s)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glColor3s"); 
			} 
		} else { 
			FailWith("glColor3s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3sv */
typedef void APIENTRY (*pstub_glColor3sv)(GLshort*);
static pstub_glColor3sv stub_glColor3sv = NULL;
static int loaded_glColor3sv = 0;

value glstub_glColor3sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor3sv){(*stub_glColor3sv)(lv0);}
	else { 
		if(!loaded_glColor3sv) { 
			loaded_glColor3sv = 1;
			init_lib(); 
			stub_glColor3sv=(pstub_glColor3sv)get_proc_address("glColor3sv");
			if(stub_glColor3sv){(*stub_glColor3sv)(lv0);}
			else { 
				FailWith("Cannot load glColor3sv"); 
			} 
		} else { 
			FailWith("glColor3sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3ub */
typedef void APIENTRY (*pstub_glColor3ub)(GLubyte,GLubyte,GLubyte);
static pstub_glColor3ub stub_glColor3ub = NULL;
static int loaded_glColor3ub = 0;

value glstub_glColor3ub(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLubyte lv0 =  Int_val(v0);
	GLubyte lv1 =  Int_val(v1);
	GLubyte lv2 =  Int_val(v2);

	if(stub_glColor3ub){(*stub_glColor3ub)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glColor3ub) { 
			loaded_glColor3ub = 1;
			init_lib(); 
			stub_glColor3ub=(pstub_glColor3ub)get_proc_address("glColor3ub");
			if(stub_glColor3ub){(*stub_glColor3ub)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glColor3ub"); 
			} 
		} else { 
			FailWith("glColor3ub not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3ubv */
typedef void APIENTRY (*pstub_glColor3ubv)(GLubyte*);
static pstub_glColor3ubv stub_glColor3ubv = NULL;
static int loaded_glColor3ubv = 0;

value glstub_glColor3ubv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLubyte* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor3ubv){(*stub_glColor3ubv)(lv0);}
	else { 
		if(!loaded_glColor3ubv) { 
			loaded_glColor3ubv = 1;
			init_lib(); 
			stub_glColor3ubv=(pstub_glColor3ubv)get_proc_address("glColor3ubv");
			if(stub_glColor3ubv){(*stub_glColor3ubv)(lv0);}
			else { 
				FailWith("Cannot load glColor3ubv"); 
			} 
		} else { 
			FailWith("glColor3ubv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3ui */
typedef void APIENTRY (*pstub_glColor3ui)(GLuint,GLuint,GLuint);
static pstub_glColor3ui stub_glColor3ui = NULL;
static int loaded_glColor3ui = 0;

value glstub_glColor3ui(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint lv1 =  Int_val(v1);
	GLuint lv2 =  Int_val(v2);

	if(stub_glColor3ui){(*stub_glColor3ui)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glColor3ui) { 
			loaded_glColor3ui = 1;
			init_lib(); 
			stub_glColor3ui=(pstub_glColor3ui)get_proc_address("glColor3ui");
			if(stub_glColor3ui){(*stub_glColor3ui)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glColor3ui"); 
			} 
		} else { 
			FailWith("glColor3ui not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3uiv */
typedef void APIENTRY (*pstub_glColor3uiv)(GLuint*);
static pstub_glColor3uiv stub_glColor3uiv = NULL;
static int loaded_glColor3uiv = 0;

value glstub_glColor3uiv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor3uiv){(*stub_glColor3uiv)(lv0);}
	else { 
		if(!loaded_glColor3uiv) { 
			loaded_glColor3uiv = 1;
			init_lib(); 
			stub_glColor3uiv=(pstub_glColor3uiv)get_proc_address("glColor3uiv");
			if(stub_glColor3uiv){(*stub_glColor3uiv)(lv0);}
			else { 
				FailWith("Cannot load glColor3uiv"); 
			} 
		} else { 
			FailWith("glColor3uiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3us */
typedef void APIENTRY (*pstub_glColor3us)(GLushort,GLushort,GLushort);
static pstub_glColor3us stub_glColor3us = NULL;
static int loaded_glColor3us = 0;

value glstub_glColor3us(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLushort lv0 =  Int_val(v0);
	GLushort lv1 =  Int_val(v1);
	GLushort lv2 =  Int_val(v2);

	if(stub_glColor3us){(*stub_glColor3us)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glColor3us) { 
			loaded_glColor3us = 1;
			init_lib(); 
			stub_glColor3us=(pstub_glColor3us)get_proc_address("glColor3us");
			if(stub_glColor3us){(*stub_glColor3us)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glColor3us"); 
			} 
		} else { 
			FailWith("glColor3us not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor3usv */
typedef void APIENTRY (*pstub_glColor3usv)(GLushort*);
static pstub_glColor3usv stub_glColor3usv = NULL;
static int loaded_glColor3usv = 0;

value glstub_glColor3usv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLushort* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor3usv){(*stub_glColor3usv)(lv0);}
	else { 
		if(!loaded_glColor3usv) { 
			loaded_glColor3usv = 1;
			init_lib(); 
			stub_glColor3usv=(pstub_glColor3usv)get_proc_address("glColor3usv");
			if(stub_glColor3usv){(*stub_glColor3usv)(lv0);}
			else { 
				FailWith("Cannot load glColor3usv"); 
			} 
		} else { 
			FailWith("glColor3usv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4b */
typedef void APIENTRY (*pstub_glColor4b)(GLbyte,GLbyte,GLbyte,GLbyte);
static pstub_glColor4b stub_glColor4b = NULL;
static int loaded_glColor4b = 0;

value glstub_glColor4b(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLbyte lv0 =  Int_val(v0);
	GLbyte lv1 =  Int_val(v1);
	GLbyte lv2 =  Int_val(v2);
	GLbyte lv3 =  Int_val(v3);

	if(stub_glColor4b){(*stub_glColor4b)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glColor4b) { 
			loaded_glColor4b = 1;
			init_lib(); 
			stub_glColor4b=(pstub_glColor4b)get_proc_address("glColor4b");
			if(stub_glColor4b){(*stub_glColor4b)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glColor4b"); 
			} 
		} else { 
			FailWith("glColor4b not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4bv */
typedef void APIENTRY (*pstub_glColor4bv)(GLbyte*);
static pstub_glColor4bv stub_glColor4bv = NULL;
static int loaded_glColor4bv = 0;

value glstub_glColor4bv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLbyte* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor4bv){(*stub_glColor4bv)(lv0);}
	else { 
		if(!loaded_glColor4bv) { 
			loaded_glColor4bv = 1;
			init_lib(); 
			stub_glColor4bv=(pstub_glColor4bv)get_proc_address("glColor4bv");
			if(stub_glColor4bv){(*stub_glColor4bv)(lv0);}
			else { 
				FailWith("Cannot load glColor4bv"); 
			} 
		} else { 
			FailWith("glColor4bv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4d */
typedef void APIENTRY (*pstub_glColor4d)(GLdouble,GLdouble,GLdouble,GLdouble);
static pstub_glColor4d stub_glColor4d = NULL;
static int loaded_glColor4d = 0;

value glstub_glColor4d(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);

	if(stub_glColor4d){(*stub_glColor4d)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glColor4d) { 
			loaded_glColor4d = 1;
			init_lib(); 
			stub_glColor4d=(pstub_glColor4d)get_proc_address("glColor4d");
			if(stub_glColor4d){(*stub_glColor4d)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glColor4d"); 
			} 
		} else { 
			FailWith("glColor4d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4dv */
typedef void APIENTRY (*pstub_glColor4dv)(GLdouble*);
static pstub_glColor4dv stub_glColor4dv = NULL;
static int loaded_glColor4dv = 0;

value glstub_glColor4dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor4dv){(*stub_glColor4dv)(lv0);}
	else { 
		if(!loaded_glColor4dv) { 
			loaded_glColor4dv = 1;
			init_lib(); 
			stub_glColor4dv=(pstub_glColor4dv)get_proc_address("glColor4dv");
			if(stub_glColor4dv){(*stub_glColor4dv)(lv0);}
			else { 
				FailWith("Cannot load glColor4dv"); 
			} 
		} else { 
			FailWith("glColor4dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4f */
typedef void APIENTRY (*pstub_glColor4f)(GLfloat,GLfloat,GLfloat,GLfloat);
static pstub_glColor4f stub_glColor4f = NULL;
static int loaded_glColor4f = 0;

value glstub_glColor4f(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);

	if(stub_glColor4f){(*stub_glColor4f)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glColor4f) { 
			loaded_glColor4f = 1;
			init_lib(); 
			stub_glColor4f=(pstub_glColor4f)get_proc_address("glColor4f");
			if(stub_glColor4f){(*stub_glColor4f)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glColor4f"); 
			} 
		} else { 
			FailWith("glColor4f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4fv */
typedef void APIENTRY (*pstub_glColor4fv)(GLfloat*);
static pstub_glColor4fv stub_glColor4fv = NULL;
static int loaded_glColor4fv = 0;

value glstub_glColor4fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor4fv){(*stub_glColor4fv)(lv0);}
	else { 
		if(!loaded_glColor4fv) { 
			loaded_glColor4fv = 1;
			init_lib(); 
			stub_glColor4fv=(pstub_glColor4fv)get_proc_address("glColor4fv");
			if(stub_glColor4fv){(*stub_glColor4fv)(lv0);}
			else { 
				FailWith("Cannot load glColor4fv"); 
			} 
		} else { 
			FailWith("glColor4fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4i */
typedef void APIENTRY (*pstub_glColor4i)(GLint,GLint,GLint,GLint);
static pstub_glColor4i stub_glColor4i = NULL;
static int loaded_glColor4i = 0;

value glstub_glColor4i(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);

	if(stub_glColor4i){(*stub_glColor4i)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glColor4i) { 
			loaded_glColor4i = 1;
			init_lib(); 
			stub_glColor4i=(pstub_glColor4i)get_proc_address("glColor4i");
			if(stub_glColor4i){(*stub_glColor4i)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glColor4i"); 
			} 
		} else { 
			FailWith("glColor4i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4iv */
typedef void APIENTRY (*pstub_glColor4iv)(GLint*);
static pstub_glColor4iv stub_glColor4iv = NULL;
static int loaded_glColor4iv = 0;

value glstub_glColor4iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor4iv){(*stub_glColor4iv)(lv0);}
	else { 
		if(!loaded_glColor4iv) { 
			loaded_glColor4iv = 1;
			init_lib(); 
			stub_glColor4iv=(pstub_glColor4iv)get_proc_address("glColor4iv");
			if(stub_glColor4iv){(*stub_glColor4iv)(lv0);}
			else { 
				FailWith("Cannot load glColor4iv"); 
			} 
		} else { 
			FailWith("glColor4iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4s */
typedef void APIENTRY (*pstub_glColor4s)(GLshort,GLshort,GLshort,GLshort);
static pstub_glColor4s stub_glColor4s = NULL;
static int loaded_glColor4s = 0;

value glstub_glColor4s(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);

	if(stub_glColor4s){(*stub_glColor4s)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glColor4s) { 
			loaded_glColor4s = 1;
			init_lib(); 
			stub_glColor4s=(pstub_glColor4s)get_proc_address("glColor4s");
			if(stub_glColor4s){(*stub_glColor4s)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glColor4s"); 
			} 
		} else { 
			FailWith("glColor4s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4sv */
typedef void APIENTRY (*pstub_glColor4sv)(GLshort*);
static pstub_glColor4sv stub_glColor4sv = NULL;
static int loaded_glColor4sv = 0;

value glstub_glColor4sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor4sv){(*stub_glColor4sv)(lv0);}
	else { 
		if(!loaded_glColor4sv) { 
			loaded_glColor4sv = 1;
			init_lib(); 
			stub_glColor4sv=(pstub_glColor4sv)get_proc_address("glColor4sv");
			if(stub_glColor4sv){(*stub_glColor4sv)(lv0);}
			else { 
				FailWith("Cannot load glColor4sv"); 
			} 
		} else { 
			FailWith("glColor4sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4ub */
typedef void APIENTRY (*pstub_glColor4ub)(GLubyte,GLubyte,GLubyte,GLubyte);
static pstub_glColor4ub stub_glColor4ub = NULL;
static int loaded_glColor4ub = 0;

value glstub_glColor4ub(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLubyte lv0 =  Int_val(v0);
	GLubyte lv1 =  Int_val(v1);
	GLubyte lv2 =  Int_val(v2);
	GLubyte lv3 =  Int_val(v3);

	if(stub_glColor4ub){(*stub_glColor4ub)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glColor4ub) { 
			loaded_glColor4ub = 1;
			init_lib(); 
			stub_glColor4ub=(pstub_glColor4ub)get_proc_address("glColor4ub");
			if(stub_glColor4ub){(*stub_glColor4ub)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glColor4ub"); 
			} 
		} else { 
			FailWith("glColor4ub not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4ubv */
typedef void APIENTRY (*pstub_glColor4ubv)(GLubyte*);
static pstub_glColor4ubv stub_glColor4ubv = NULL;
static int loaded_glColor4ubv = 0;

value glstub_glColor4ubv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLubyte* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor4ubv){(*stub_glColor4ubv)(lv0);}
	else { 
		if(!loaded_glColor4ubv) { 
			loaded_glColor4ubv = 1;
			init_lib(); 
			stub_glColor4ubv=(pstub_glColor4ubv)get_proc_address("glColor4ubv");
			if(stub_glColor4ubv){(*stub_glColor4ubv)(lv0);}
			else { 
				FailWith("Cannot load glColor4ubv"); 
			} 
		} else { 
			FailWith("glColor4ubv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4ui */
typedef void APIENTRY (*pstub_glColor4ui)(GLuint,GLuint,GLuint,GLuint);
static pstub_glColor4ui stub_glColor4ui = NULL;
static int loaded_glColor4ui = 0;

value glstub_glColor4ui(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint lv1 =  Int_val(v1);
	GLuint lv2 =  Int_val(v2);
	GLuint lv3 =  Int_val(v3);

	if(stub_glColor4ui){(*stub_glColor4ui)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glColor4ui) { 
			loaded_glColor4ui = 1;
			init_lib(); 
			stub_glColor4ui=(pstub_glColor4ui)get_proc_address("glColor4ui");
			if(stub_glColor4ui){(*stub_glColor4ui)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glColor4ui"); 
			} 
		} else { 
			FailWith("glColor4ui not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4uiv */
typedef void APIENTRY (*pstub_glColor4uiv)(GLuint*);
static pstub_glColor4uiv stub_glColor4uiv = NULL;
static int loaded_glColor4uiv = 0;

value glstub_glColor4uiv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor4uiv){(*stub_glColor4uiv)(lv0);}
	else { 
		if(!loaded_glColor4uiv) { 
			loaded_glColor4uiv = 1;
			init_lib(); 
			stub_glColor4uiv=(pstub_glColor4uiv)get_proc_address("glColor4uiv");
			if(stub_glColor4uiv){(*stub_glColor4uiv)(lv0);}
			else { 
				FailWith("Cannot load glColor4uiv"); 
			} 
		} else { 
			FailWith("glColor4uiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4us */
typedef void APIENTRY (*pstub_glColor4us)(GLushort,GLushort,GLushort,GLushort);
static pstub_glColor4us stub_glColor4us = NULL;
static int loaded_glColor4us = 0;

value glstub_glColor4us(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLushort lv0 =  Int_val(v0);
	GLushort lv1 =  Int_val(v1);
	GLushort lv2 =  Int_val(v2);
	GLushort lv3 =  Int_val(v3);

	if(stub_glColor4us){(*stub_glColor4us)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glColor4us) { 
			loaded_glColor4us = 1;
			init_lib(); 
			stub_glColor4us=(pstub_glColor4us)get_proc_address("glColor4us");
			if(stub_glColor4us){(*stub_glColor4us)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glColor4us"); 
			} 
		} else { 
			FailWith("glColor4us not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColor4usv */
typedef void APIENTRY (*pstub_glColor4usv)(GLushort*);
static pstub_glColor4usv stub_glColor4usv = NULL;
static int loaded_glColor4usv = 0;

value glstub_glColor4usv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLushort* lv0 =  Data_bigarray_val(v0);

	if(stub_glColor4usv){(*stub_glColor4usv)(lv0);}
	else { 
		if(!loaded_glColor4usv) { 
			loaded_glColor4usv = 1;
			init_lib(); 
			stub_glColor4usv=(pstub_glColor4usv)get_proc_address("glColor4usv");
			if(stub_glColor4usv){(*stub_glColor4usv)(lv0);}
			else { 
				FailWith("Cannot load glColor4usv"); 
			} 
		} else { 
			FailWith("glColor4usv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColorMask */
typedef void APIENTRY (*pstub_glColorMask)(GLboolean,GLboolean,GLboolean,GLboolean);
static pstub_glColorMask stub_glColorMask = NULL;
static int loaded_glColorMask = 0;

value glstub_glColorMask(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLboolean lv0 =  Bool_val(v0);
	GLboolean lv1 =  Bool_val(v1);
	GLboolean lv2 =  Bool_val(v2);
	GLboolean lv3 =  Bool_val(v3);

	if(stub_glColorMask){(*stub_glColorMask)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glColorMask) { 
			loaded_glColorMask = 1;
			init_lib(); 
			stub_glColorMask=(pstub_glColorMask)get_proc_address("glColorMask");
			if(stub_glColorMask){(*stub_glColorMask)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glColorMask"); 
			} 
		} else { 
			FailWith("glColorMask not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColorMaterial */
typedef void APIENTRY (*pstub_glColorMaterial)(GLenum,GLenum);
static pstub_glColorMaterial stub_glColorMaterial = NULL;
static int loaded_glColorMaterial = 0;

value glstub_glColorMaterial(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);

	if(stub_glColorMaterial){(*stub_glColorMaterial)(lv0, lv1);}
	else { 
		if(!loaded_glColorMaterial) { 
			loaded_glColorMaterial = 1;
			init_lib(); 
			stub_glColorMaterial=(pstub_glColorMaterial)get_proc_address("glColorMaterial");
			if(stub_glColorMaterial){(*stub_glColorMaterial)(lv0, lv1);}
			else { 
				FailWith("Cannot load glColorMaterial"); 
			} 
		} else { 
			FailWith("glColorMaterial not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glColorPointer */
typedef void APIENTRY (*pstub_glColorPointer)(GLint,GLenum,GLsizei,GLvoid*);
static pstub_glColorPointer stub_glColorPointer = NULL;
static int loaded_glColorPointer = 0;

value glstub_glColorPointer(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLsizei lv2 =  Int_val(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);

	if(stub_glColorPointer){(*stub_glColorPointer)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glColorPointer) { 
			loaded_glColorPointer = 1;
			init_lib(); 
			stub_glColorPointer=(pstub_glColorPointer)get_proc_address("glColorPointer");
			if(stub_glColorPointer){(*stub_glColorPointer)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glColorPointer"); 
			} 
		} else { 
			FailWith("glColorPointer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glCopyPixels */
typedef void APIENTRY (*pstub_glCopyPixels)(GLint,GLint,GLsizei,GLsizei,GLenum);
static pstub_glCopyPixels stub_glCopyPixels = NULL;
static int loaded_glCopyPixels = 0;

value glstub_glCopyPixels(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLsizei lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);
	GLenum lv4 =  get_glenum(v4);

	if(stub_glCopyPixels){(*stub_glCopyPixels)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glCopyPixels) { 
			loaded_glCopyPixels = 1;
			init_lib(); 
			stub_glCopyPixels=(pstub_glCopyPixels)get_proc_address("glCopyPixels");
			if(stub_glCopyPixels){(*stub_glCopyPixels)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glCopyPixels"); 
			} 
		} else { 
			FailWith("glCopyPixels not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glCopyTexImage1D */
typedef void APIENTRY (*pstub_glCopyTexImage1D)(GLenum,GLint,GLenum,GLint,GLint,GLsizei,GLint);
static pstub_glCopyTexImage1D stub_glCopyTexImage1D = NULL;
static int loaded_glCopyTexImage1D = 0;

value glstub_glCopyTexImage1D(value v0, value v1, value v2, value v3, value v4, value v5, value v6)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam2(v5,v6);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);
	GLsizei lv5 =  Int_val(v5);
	GLint lv6 =  Int_val(v6);

	if(stub_glCopyTexImage1D){(*stub_glCopyTexImage1D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
	else { 
		if(!loaded_glCopyTexImage1D) { 
			loaded_glCopyTexImage1D = 1;
			init_lib(); 
			stub_glCopyTexImage1D=(pstub_glCopyTexImage1D)get_proc_address("glCopyTexImage1D");
			if(stub_glCopyTexImage1D){(*stub_glCopyTexImage1D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
			else { 
				FailWith("Cannot load glCopyTexImage1D"); 
			} 
		} else { 
			FailWith("glCopyTexImage1D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glCopyTexImage1D_byte(value * argv, int n)
{
	return glstub_glCopyTexImage1D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]);
}

/* glCopyTexImage2D */
typedef void APIENTRY (*pstub_glCopyTexImage2D)(GLenum,GLint,GLenum,GLint,GLint,GLsizei,GLsizei,GLint);
static pstub_glCopyTexImage2D stub_glCopyTexImage2D = NULL;
static int loaded_glCopyTexImage2D = 0;

value glstub_glCopyTexImage2D(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam3(v5,v6,v7);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);
	GLsizei lv5 =  Int_val(v5);
	GLsizei lv6 =  Int_val(v6);
	GLint lv7 =  Int_val(v7);

	if(stub_glCopyTexImage2D){(*stub_glCopyTexImage2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7);}
	else { 
		if(!loaded_glCopyTexImage2D) { 
			loaded_glCopyTexImage2D = 1;
			init_lib(); 
			stub_glCopyTexImage2D=(pstub_glCopyTexImage2D)get_proc_address("glCopyTexImage2D");
			if(stub_glCopyTexImage2D){(*stub_glCopyTexImage2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7);}
			else { 
				FailWith("Cannot load glCopyTexImage2D"); 
			} 
		} else { 
			FailWith("glCopyTexImage2D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glCopyTexImage2D_byte(value * argv, int n)
{
	return glstub_glCopyTexImage2D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7]);
}

/* glCopyTexSubImage1D */
typedef void APIENTRY (*pstub_glCopyTexSubImage1D)(GLenum,GLint,GLint,GLint,GLint,GLsizei);
static pstub_glCopyTexSubImage1D stub_glCopyTexSubImage1D = NULL;
static int loaded_glCopyTexSubImage1D = 0;

value glstub_glCopyTexSubImage1D(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);
	GLsizei lv5 =  Int_val(v5);

	if(stub_glCopyTexSubImage1D){(*stub_glCopyTexSubImage1D)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glCopyTexSubImage1D) { 
			loaded_glCopyTexSubImage1D = 1;
			init_lib(); 
			stub_glCopyTexSubImage1D=(pstub_glCopyTexSubImage1D)get_proc_address("glCopyTexSubImage1D");
			if(stub_glCopyTexSubImage1D){(*stub_glCopyTexSubImage1D)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glCopyTexSubImage1D"); 
			} 
		} else { 
			FailWith("glCopyTexSubImage1D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glCopyTexSubImage1D_byte(value * argv, int n)
{
	return glstub_glCopyTexSubImage1D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glCopyTexSubImage2D */
typedef void APIENTRY (*pstub_glCopyTexSubImage2D)(GLenum,GLint,GLint,GLint,GLint,GLint,GLsizei,GLsizei);
static pstub_glCopyTexSubImage2D stub_glCopyTexSubImage2D = NULL;
static int loaded_glCopyTexSubImage2D = 0;

value glstub_glCopyTexSubImage2D(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam3(v5,v6,v7);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);
	GLint lv5 =  Int_val(v5);
	GLsizei lv6 =  Int_val(v6);
	GLsizei lv7 =  Int_val(v7);

	if(stub_glCopyTexSubImage2D){(*stub_glCopyTexSubImage2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7);}
	else { 
		if(!loaded_glCopyTexSubImage2D) { 
			loaded_glCopyTexSubImage2D = 1;
			init_lib(); 
			stub_glCopyTexSubImage2D=(pstub_glCopyTexSubImage2D)get_proc_address("glCopyTexSubImage2D");
			if(stub_glCopyTexSubImage2D){(*stub_glCopyTexSubImage2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7);}
			else { 
				FailWith("Cannot load glCopyTexSubImage2D"); 
			} 
		} else { 
			FailWith("glCopyTexSubImage2D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glCopyTexSubImage2D_byte(value * argv, int n)
{
	return glstub_glCopyTexSubImage2D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7]);
}

/* glCullFace */
typedef void APIENTRY (*pstub_glCullFace)(GLenum);
static pstub_glCullFace stub_glCullFace = NULL;
static int loaded_glCullFace = 0;

value glstub_glCullFace(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glCullFace){(*stub_glCullFace)(lv0);}
	else { 
		if(!loaded_glCullFace) { 
			loaded_glCullFace = 1;
			init_lib(); 
			stub_glCullFace=(pstub_glCullFace)get_proc_address("glCullFace");
			if(stub_glCullFace){(*stub_glCullFace)(lv0);}
			else { 
				FailWith("Cannot load glCullFace"); 
			} 
		} else { 
			FailWith("glCullFace not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDeleteLists */
typedef void APIENTRY (*pstub_glDeleteLists)(GLuint,GLsizei);
static pstub_glDeleteLists stub_glDeleteLists = NULL;
static int loaded_glDeleteLists = 0;

value glstub_glDeleteLists(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);

	if(stub_glDeleteLists){(*stub_glDeleteLists)(lv0, lv1);}
	else { 
		if(!loaded_glDeleteLists) { 
			loaded_glDeleteLists = 1;
			init_lib(); 
			stub_glDeleteLists=(pstub_glDeleteLists)get_proc_address("glDeleteLists");
			if(stub_glDeleteLists){(*stub_glDeleteLists)(lv0, lv1);}
			else { 
				FailWith("Cannot load glDeleteLists"); 
			} 
		} else { 
			FailWith("glDeleteLists not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDeleteTextures */
typedef void APIENTRY (*pstub_glDeleteTextures)(GLsizei,GLuint*);
static pstub_glDeleteTextures stub_glDeleteTextures = NULL;
static int loaded_glDeleteTextures = 0;

value glstub_glDeleteTextures(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);

	if(stub_glDeleteTextures){(*stub_glDeleteTextures)(lv0, lv1);}
	else { 
		if(!loaded_glDeleteTextures) { 
			loaded_glDeleteTextures = 1;
			init_lib(); 
			stub_glDeleteTextures=(pstub_glDeleteTextures)get_proc_address("glDeleteTextures");
			if(stub_glDeleteTextures){(*stub_glDeleteTextures)(lv0, lv1);}
			else { 
				FailWith("Cannot load glDeleteTextures"); 
			} 
		} else { 
			FailWith("glDeleteTextures not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDepthFunc */
typedef void APIENTRY (*pstub_glDepthFunc)(GLenum);
static pstub_glDepthFunc stub_glDepthFunc = NULL;
static int loaded_glDepthFunc = 0;

value glstub_glDepthFunc(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glDepthFunc){(*stub_glDepthFunc)(lv0);}
	else { 
		if(!loaded_glDepthFunc) { 
			loaded_glDepthFunc = 1;
			init_lib(); 
			stub_glDepthFunc=(pstub_glDepthFunc)get_proc_address("glDepthFunc");
			if(stub_glDepthFunc){(*stub_glDepthFunc)(lv0);}
			else { 
				FailWith("Cannot load glDepthFunc"); 
			} 
		} else { 
			FailWith("glDepthFunc not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDepthMask */
typedef void APIENTRY (*pstub_glDepthMask)(GLboolean);
static pstub_glDepthMask stub_glDepthMask = NULL;
static int loaded_glDepthMask = 0;

value glstub_glDepthMask(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLboolean lv0 =  Bool_val(v0);

	if(stub_glDepthMask){(*stub_glDepthMask)(lv0);}
	else { 
		if(!loaded_glDepthMask) { 
			loaded_glDepthMask = 1;
			init_lib(); 
			stub_glDepthMask=(pstub_glDepthMask)get_proc_address("glDepthMask");
			if(stub_glDepthMask){(*stub_glDepthMask)(lv0);}
			else { 
				FailWith("Cannot load glDepthMask"); 
			} 
		} else { 
			FailWith("glDepthMask not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDepthRange */
typedef void APIENTRY (*pstub_glDepthRange)(GLclampd,GLclampd);
static pstub_glDepthRange stub_glDepthRange = NULL;
static int loaded_glDepthRange = 0;

value glstub_glDepthRange(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLclampd lv0 =  Double_val(v0);
	GLclampd lv1 =  Double_val(v1);

	if(stub_glDepthRange){(*stub_glDepthRange)(lv0, lv1);}
	else { 
		if(!loaded_glDepthRange) { 
			loaded_glDepthRange = 1;
			init_lib(); 
			stub_glDepthRange=(pstub_glDepthRange)get_proc_address("glDepthRange");
			if(stub_glDepthRange){(*stub_glDepthRange)(lv0, lv1);}
			else { 
				FailWith("Cannot load glDepthRange"); 
			} 
		} else { 
			FailWith("glDepthRange not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDisable */
typedef void APIENTRY (*pstub_glDisable)(GLenum);
static pstub_glDisable stub_glDisable = NULL;
static int loaded_glDisable = 0;

value glstub_glDisable(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glDisable){(*stub_glDisable)(lv0);}
	else { 
		if(!loaded_glDisable) { 
			loaded_glDisable = 1;
			init_lib(); 
			stub_glDisable=(pstub_glDisable)get_proc_address("glDisable");
			if(stub_glDisable){(*stub_glDisable)(lv0);}
			else { 
				FailWith("Cannot load glDisable"); 
			} 
		} else { 
			FailWith("glDisable not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDisableClientState */
typedef void APIENTRY (*pstub_glDisableClientState)(GLenum);
static pstub_glDisableClientState stub_glDisableClientState = NULL;
static int loaded_glDisableClientState = 0;

value glstub_glDisableClientState(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glDisableClientState){(*stub_glDisableClientState)(lv0);}
	else { 
		if(!loaded_glDisableClientState) { 
			loaded_glDisableClientState = 1;
			init_lib(); 
			stub_glDisableClientState=(pstub_glDisableClientState)get_proc_address("glDisableClientState");
			if(stub_glDisableClientState){(*stub_glDisableClientState)(lv0);}
			else { 
				FailWith("Cannot load glDisableClientState"); 
			} 
		} else { 
			FailWith("glDisableClientState not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDrawArrays */
typedef void APIENTRY (*pstub_glDrawArrays)(GLenum,GLint,GLsizei);
static pstub_glDrawArrays stub_glDrawArrays = NULL;
static int loaded_glDrawArrays = 0;

value glstub_glDrawArrays(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLsizei lv2 =  Int_val(v2);

	if(stub_glDrawArrays){(*stub_glDrawArrays)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glDrawArrays) { 
			loaded_glDrawArrays = 1;
			init_lib(); 
			stub_glDrawArrays=(pstub_glDrawArrays)get_proc_address("glDrawArrays");
			if(stub_glDrawArrays){(*stub_glDrawArrays)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glDrawArrays"); 
			} 
		} else { 
			FailWith("glDrawArrays not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDrawBuffer */
typedef void APIENTRY (*pstub_glDrawBuffer)(GLenum);
static pstub_glDrawBuffer stub_glDrawBuffer = NULL;
static int loaded_glDrawBuffer = 0;

value glstub_glDrawBuffer(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glDrawBuffer){(*stub_glDrawBuffer)(lv0);}
	else { 
		if(!loaded_glDrawBuffer) { 
			loaded_glDrawBuffer = 1;
			init_lib(); 
			stub_glDrawBuffer=(pstub_glDrawBuffer)get_proc_address("glDrawBuffer");
			if(stub_glDrawBuffer){(*stub_glDrawBuffer)(lv0);}
			else { 
				FailWith("Cannot load glDrawBuffer"); 
			} 
		} else { 
			FailWith("glDrawBuffer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDrawElements */
typedef void APIENTRY (*pstub_glDrawElements)(GLenum,GLsizei,GLenum,GLvoid*);
static pstub_glDrawElements stub_glDrawElements = NULL;
static int loaded_glDrawElements = 0;

value glstub_glDrawElements(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizei lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);

	if(stub_glDrawElements){(*stub_glDrawElements)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glDrawElements) { 
			loaded_glDrawElements = 1;
			init_lib(); 
			stub_glDrawElements=(pstub_glDrawElements)get_proc_address("glDrawElements");
			if(stub_glDrawElements){(*stub_glDrawElements)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glDrawElements"); 
			} 
		} else { 
			FailWith("glDrawElements not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDrawPixels */
typedef void APIENTRY (*pstub_glDrawPixels)(GLsizei,GLsizei,GLenum,GLenum,GLvoid*);
static pstub_glDrawPixels stub_glDrawPixels = NULL;
static int loaded_glDrawPixels = 0;

value glstub_glDrawPixels(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLenum lv3 =  get_glenum(v3);
	GLvoid* lv4 =  Data_bigarray_val(v4);

	if(stub_glDrawPixels){(*stub_glDrawPixels)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glDrawPixels) { 
			loaded_glDrawPixels = 1;
			init_lib(); 
			stub_glDrawPixels=(pstub_glDrawPixels)get_proc_address("glDrawPixels");
			if(stub_glDrawPixels){(*stub_glDrawPixels)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glDrawPixels"); 
			} 
		} else { 
			FailWith("glDrawPixels not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEdgeFlag */
typedef void APIENTRY (*pstub_glEdgeFlag)(GLboolean);
static pstub_glEdgeFlag stub_glEdgeFlag = NULL;
static int loaded_glEdgeFlag = 0;

value glstub_glEdgeFlag(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLboolean lv0 =  Bool_val(v0);

	if(stub_glEdgeFlag){(*stub_glEdgeFlag)(lv0);}
	else { 
		if(!loaded_glEdgeFlag) { 
			loaded_glEdgeFlag = 1;
			init_lib(); 
			stub_glEdgeFlag=(pstub_glEdgeFlag)get_proc_address("glEdgeFlag");
			if(stub_glEdgeFlag){(*stub_glEdgeFlag)(lv0);}
			else { 
				FailWith("Cannot load glEdgeFlag"); 
			} 
		} else { 
			FailWith("glEdgeFlag not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEdgeFlagPointer */
typedef void APIENTRY (*pstub_glEdgeFlagPointer)(GLsizei,GLvoid*);
static pstub_glEdgeFlagPointer stub_glEdgeFlagPointer = NULL;
static int loaded_glEdgeFlagPointer = 0;

value glstub_glEdgeFlagPointer(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLvoid* lv1 =  Data_bigarray_val(v1);

	if(stub_glEdgeFlagPointer){(*stub_glEdgeFlagPointer)(lv0, lv1);}
	else { 
		if(!loaded_glEdgeFlagPointer) { 
			loaded_glEdgeFlagPointer = 1;
			init_lib(); 
			stub_glEdgeFlagPointer=(pstub_glEdgeFlagPointer)get_proc_address("glEdgeFlagPointer");
			if(stub_glEdgeFlagPointer){(*stub_glEdgeFlagPointer)(lv0, lv1);}
			else { 
				FailWith("Cannot load glEdgeFlagPointer"); 
			} 
		} else { 
			FailWith("glEdgeFlagPointer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEdgeFlagv */
typedef void APIENTRY (*pstub_glEdgeFlagv)(GLboolean*);
static pstub_glEdgeFlagv stub_glEdgeFlagv = NULL;
static int loaded_glEdgeFlagv = 0;

value glstub_glEdgeFlagv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLboolean* lv0 =  Data_bigarray_val(v0);

	if(stub_glEdgeFlagv){(*stub_glEdgeFlagv)(lv0);}
	else { 
		if(!loaded_glEdgeFlagv) { 
			loaded_glEdgeFlagv = 1;
			init_lib(); 
			stub_glEdgeFlagv=(pstub_glEdgeFlagv)get_proc_address("glEdgeFlagv");
			if(stub_glEdgeFlagv){(*stub_glEdgeFlagv)(lv0);}
			else { 
				FailWith("Cannot load glEdgeFlagv"); 
			} 
		} else { 
			FailWith("glEdgeFlagv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEnable */
typedef void APIENTRY (*pstub_glEnable)(GLenum);
static pstub_glEnable stub_glEnable = NULL;
static int loaded_glEnable = 0;

value glstub_glEnable(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glEnable){(*stub_glEnable)(lv0);}
	else { 
		if(!loaded_glEnable) { 
			loaded_glEnable = 1;
			init_lib(); 
			stub_glEnable=(pstub_glEnable)get_proc_address("glEnable");
			if(stub_glEnable){(*stub_glEnable)(lv0);}
			else { 
				FailWith("Cannot load glEnable"); 
			} 
		} else { 
			FailWith("glEnable not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEnableClientState */
typedef void APIENTRY (*pstub_glEnableClientState)(GLenum);
static pstub_glEnableClientState stub_glEnableClientState = NULL;
static int loaded_glEnableClientState = 0;

value glstub_glEnableClientState(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glEnableClientState){(*stub_glEnableClientState)(lv0);}
	else { 
		if(!loaded_glEnableClientState) { 
			loaded_glEnableClientState = 1;
			init_lib(); 
			stub_glEnableClientState=(pstub_glEnableClientState)get_proc_address("glEnableClientState");
			if(stub_glEnableClientState){(*stub_glEnableClientState)(lv0);}
			else { 
				FailWith("Cannot load glEnableClientState"); 
			} 
		} else { 
			FailWith("glEnableClientState not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEnd */
typedef void APIENTRY (*pstub_glEnd)(GLvoid);
static pstub_glEnd stub_glEnd = NULL;
static int loaded_glEnd = 0;

value glstub_glEnd(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);

	if(stub_glEnd){(*stub_glEnd)();}
	else { 
		if(!loaded_glEnd) { 
			loaded_glEnd = 1;
			init_lib(); 
			stub_glEnd=(pstub_glEnd)get_proc_address("glEnd");
			if(stub_glEnd){(*stub_glEnd)();}
			else { 
				FailWith("Cannot load glEnd"); 
			} 
		} else { 
			FailWith("glEnd not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEndList */
typedef void APIENTRY (*pstub_glEndList)(GLvoid);
static pstub_glEndList stub_glEndList = NULL;
static int loaded_glEndList = 0;

value glstub_glEndList(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);

	if(stub_glEndList){(*stub_glEndList)();}
	else { 
		if(!loaded_glEndList) { 
			loaded_glEndList = 1;
			init_lib(); 
			stub_glEndList=(pstub_glEndList)get_proc_address("glEndList");
			if(stub_glEndList){(*stub_glEndList)();}
			else { 
				FailWith("Cannot load glEndList"); 
			} 
		} else { 
			FailWith("glEndList not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEvalCoord1d */
typedef void APIENTRY (*pstub_glEvalCoord1d)(GLdouble);
static pstub_glEvalCoord1d stub_glEvalCoord1d = NULL;
static int loaded_glEvalCoord1d = 0;

value glstub_glEvalCoord1d(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);

	if(stub_glEvalCoord1d){(*stub_glEvalCoord1d)(lv0);}
	else { 
		if(!loaded_glEvalCoord1d) { 
			loaded_glEvalCoord1d = 1;
			init_lib(); 
			stub_glEvalCoord1d=(pstub_glEvalCoord1d)get_proc_address("glEvalCoord1d");
			if(stub_glEvalCoord1d){(*stub_glEvalCoord1d)(lv0);}
			else { 
				FailWith("Cannot load glEvalCoord1d"); 
			} 
		} else { 
			FailWith("glEvalCoord1d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEvalCoord1dv */
typedef void APIENTRY (*pstub_glEvalCoord1dv)(GLdouble*);
static pstub_glEvalCoord1dv stub_glEvalCoord1dv = NULL;
static int loaded_glEvalCoord1dv = 0;

value glstub_glEvalCoord1dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glEvalCoord1dv){(*stub_glEvalCoord1dv)(lv0);}
	else { 
		if(!loaded_glEvalCoord1dv) { 
			loaded_glEvalCoord1dv = 1;
			init_lib(); 
			stub_glEvalCoord1dv=(pstub_glEvalCoord1dv)get_proc_address("glEvalCoord1dv");
			if(stub_glEvalCoord1dv){(*stub_glEvalCoord1dv)(lv0);}
			else { 
				FailWith("Cannot load glEvalCoord1dv"); 
			} 
		} else { 
			FailWith("glEvalCoord1dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEvalCoord1f */
typedef void APIENTRY (*pstub_glEvalCoord1f)(GLfloat);
static pstub_glEvalCoord1f stub_glEvalCoord1f = NULL;
static int loaded_glEvalCoord1f = 0;

value glstub_glEvalCoord1f(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);

	if(stub_glEvalCoord1f){(*stub_glEvalCoord1f)(lv0);}
	else { 
		if(!loaded_glEvalCoord1f) { 
			loaded_glEvalCoord1f = 1;
			init_lib(); 
			stub_glEvalCoord1f=(pstub_glEvalCoord1f)get_proc_address("glEvalCoord1f");
			if(stub_glEvalCoord1f){(*stub_glEvalCoord1f)(lv0);}
			else { 
				FailWith("Cannot load glEvalCoord1f"); 
			} 
		} else { 
			FailWith("glEvalCoord1f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEvalCoord1fv */
typedef void APIENTRY (*pstub_glEvalCoord1fv)(GLfloat*);
static pstub_glEvalCoord1fv stub_glEvalCoord1fv = NULL;
static int loaded_glEvalCoord1fv = 0;

value glstub_glEvalCoord1fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glEvalCoord1fv){(*stub_glEvalCoord1fv)(lv0);}
	else { 
		if(!loaded_glEvalCoord1fv) { 
			loaded_glEvalCoord1fv = 1;
			init_lib(); 
			stub_glEvalCoord1fv=(pstub_glEvalCoord1fv)get_proc_address("glEvalCoord1fv");
			if(stub_glEvalCoord1fv){(*stub_glEvalCoord1fv)(lv0);}
			else { 
				FailWith("Cannot load glEvalCoord1fv"); 
			} 
		} else { 
			FailWith("glEvalCoord1fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEvalCoord2d */
typedef void APIENTRY (*pstub_glEvalCoord2d)(GLdouble,GLdouble);
static pstub_glEvalCoord2d stub_glEvalCoord2d = NULL;
static int loaded_glEvalCoord2d = 0;

value glstub_glEvalCoord2d(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);

	if(stub_glEvalCoord2d){(*stub_glEvalCoord2d)(lv0, lv1);}
	else { 
		if(!loaded_glEvalCoord2d) { 
			loaded_glEvalCoord2d = 1;
			init_lib(); 
			stub_glEvalCoord2d=(pstub_glEvalCoord2d)get_proc_address("glEvalCoord2d");
			if(stub_glEvalCoord2d){(*stub_glEvalCoord2d)(lv0, lv1);}
			else { 
				FailWith("Cannot load glEvalCoord2d"); 
			} 
		} else { 
			FailWith("glEvalCoord2d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEvalCoord2dv */
typedef void APIENTRY (*pstub_glEvalCoord2dv)(GLdouble*);
static pstub_glEvalCoord2dv stub_glEvalCoord2dv = NULL;
static int loaded_glEvalCoord2dv = 0;

value glstub_glEvalCoord2dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glEvalCoord2dv){(*stub_glEvalCoord2dv)(lv0);}
	else { 
		if(!loaded_glEvalCoord2dv) { 
			loaded_glEvalCoord2dv = 1;
			init_lib(); 
			stub_glEvalCoord2dv=(pstub_glEvalCoord2dv)get_proc_address("glEvalCoord2dv");
			if(stub_glEvalCoord2dv){(*stub_glEvalCoord2dv)(lv0);}
			else { 
				FailWith("Cannot load glEvalCoord2dv"); 
			} 
		} else { 
			FailWith("glEvalCoord2dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEvalCoord2f */
typedef void APIENTRY (*pstub_glEvalCoord2f)(GLfloat,GLfloat);
static pstub_glEvalCoord2f stub_glEvalCoord2f = NULL;
static int loaded_glEvalCoord2f = 0;

value glstub_glEvalCoord2f(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glEvalCoord2f){(*stub_glEvalCoord2f)(lv0, lv1);}
	else { 
		if(!loaded_glEvalCoord2f) { 
			loaded_glEvalCoord2f = 1;
			init_lib(); 
			stub_glEvalCoord2f=(pstub_glEvalCoord2f)get_proc_address("glEvalCoord2f");
			if(stub_glEvalCoord2f){(*stub_glEvalCoord2f)(lv0, lv1);}
			else { 
				FailWith("Cannot load glEvalCoord2f"); 
			} 
		} else { 
			FailWith("glEvalCoord2f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEvalCoord2fv */
typedef void APIENTRY (*pstub_glEvalCoord2fv)(GLfloat*);
static pstub_glEvalCoord2fv stub_glEvalCoord2fv = NULL;
static int loaded_glEvalCoord2fv = 0;

value glstub_glEvalCoord2fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glEvalCoord2fv){(*stub_glEvalCoord2fv)(lv0);}
	else { 
		if(!loaded_glEvalCoord2fv) { 
			loaded_glEvalCoord2fv = 1;
			init_lib(); 
			stub_glEvalCoord2fv=(pstub_glEvalCoord2fv)get_proc_address("glEvalCoord2fv");
			if(stub_glEvalCoord2fv){(*stub_glEvalCoord2fv)(lv0);}
			else { 
				FailWith("Cannot load glEvalCoord2fv"); 
			} 
		} else { 
			FailWith("glEvalCoord2fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEvalMesh1 */
typedef void APIENTRY (*pstub_glEvalMesh1)(GLenum,GLint,GLint);
static pstub_glEvalMesh1 stub_glEvalMesh1 = NULL;
static int loaded_glEvalMesh1 = 0;

value glstub_glEvalMesh1(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glEvalMesh1){(*stub_glEvalMesh1)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glEvalMesh1) { 
			loaded_glEvalMesh1 = 1;
			init_lib(); 
			stub_glEvalMesh1=(pstub_glEvalMesh1)get_proc_address("glEvalMesh1");
			if(stub_glEvalMesh1){(*stub_glEvalMesh1)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glEvalMesh1"); 
			} 
		} else { 
			FailWith("glEvalMesh1 not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEvalMesh2 */
typedef void APIENTRY (*pstub_glEvalMesh2)(GLenum,GLint,GLint,GLint,GLint);
static pstub_glEvalMesh2 stub_glEvalMesh2 = NULL;
static int loaded_glEvalMesh2 = 0;

value glstub_glEvalMesh2(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);

	if(stub_glEvalMesh2){(*stub_glEvalMesh2)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glEvalMesh2) { 
			loaded_glEvalMesh2 = 1;
			init_lib(); 
			stub_glEvalMesh2=(pstub_glEvalMesh2)get_proc_address("glEvalMesh2");
			if(stub_glEvalMesh2){(*stub_glEvalMesh2)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glEvalMesh2"); 
			} 
		} else { 
			FailWith("glEvalMesh2 not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEvalPoint1 */
typedef void APIENTRY (*pstub_glEvalPoint1)(GLint);
static pstub_glEvalPoint1 stub_glEvalPoint1 = NULL;
static int loaded_glEvalPoint1 = 0;

value glstub_glEvalPoint1(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);

	if(stub_glEvalPoint1){(*stub_glEvalPoint1)(lv0);}
	else { 
		if(!loaded_glEvalPoint1) { 
			loaded_glEvalPoint1 = 1;
			init_lib(); 
			stub_glEvalPoint1=(pstub_glEvalPoint1)get_proc_address("glEvalPoint1");
			if(stub_glEvalPoint1){(*stub_glEvalPoint1)(lv0);}
			else { 
				FailWith("Cannot load glEvalPoint1"); 
			} 
		} else { 
			FailWith("glEvalPoint1 not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEvalPoint2 */
typedef void APIENTRY (*pstub_glEvalPoint2)(GLint,GLint);
static pstub_glEvalPoint2 stub_glEvalPoint2 = NULL;
static int loaded_glEvalPoint2 = 0;

value glstub_glEvalPoint2(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glEvalPoint2){(*stub_glEvalPoint2)(lv0, lv1);}
	else { 
		if(!loaded_glEvalPoint2) { 
			loaded_glEvalPoint2 = 1;
			init_lib(); 
			stub_glEvalPoint2=(pstub_glEvalPoint2)get_proc_address("glEvalPoint2");
			if(stub_glEvalPoint2){(*stub_glEvalPoint2)(lv0, lv1);}
			else { 
				FailWith("Cannot load glEvalPoint2"); 
			} 
		} else { 
			FailWith("glEvalPoint2 not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFeedbackBuffer */
typedef void APIENTRY (*pstub_glFeedbackBuffer)(GLsizei,GLenum,GLfloat*);
static pstub_glFeedbackBuffer stub_glFeedbackBuffer = NULL;
static int loaded_glFeedbackBuffer = 0;

value glstub_glFeedbackBuffer(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glFeedbackBuffer){(*stub_glFeedbackBuffer)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glFeedbackBuffer) { 
			loaded_glFeedbackBuffer = 1;
			init_lib(); 
			stub_glFeedbackBuffer=(pstub_glFeedbackBuffer)get_proc_address("glFeedbackBuffer");
			if(stub_glFeedbackBuffer){(*stub_glFeedbackBuffer)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glFeedbackBuffer"); 
			} 
		} else { 
			FailWith("glFeedbackBuffer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFinish */
typedef void APIENTRY (*pstub_glFinish)(GLvoid);
static pstub_glFinish stub_glFinish = NULL;
static int loaded_glFinish = 0;

value glstub_glFinish(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);

	if(stub_glFinish){(*stub_glFinish)();}
	else { 
		if(!loaded_glFinish) { 
			loaded_glFinish = 1;
			init_lib(); 
			stub_glFinish=(pstub_glFinish)get_proc_address("glFinish");
			if(stub_glFinish){(*stub_glFinish)();}
			else { 
				FailWith("Cannot load glFinish"); 
			} 
		} else { 
			FailWith("glFinish not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFlush */
typedef void APIENTRY (*pstub_glFlush)(GLvoid);
static pstub_glFlush stub_glFlush = NULL;
static int loaded_glFlush = 0;

value glstub_glFlush(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);

	if(stub_glFlush){(*stub_glFlush)();}
	else { 
		if(!loaded_glFlush) { 
			loaded_glFlush = 1;
			init_lib(); 
			stub_glFlush=(pstub_glFlush)get_proc_address("glFlush");
			if(stub_glFlush){(*stub_glFlush)();}
			else { 
				FailWith("Cannot load glFlush"); 
			} 
		} else { 
			FailWith("glFlush not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFogf */
typedef void APIENTRY (*pstub_glFogf)(GLenum,GLfloat);
static pstub_glFogf stub_glFogf = NULL;
static int loaded_glFogf = 0;

value glstub_glFogf(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glFogf){(*stub_glFogf)(lv0, lv1);}
	else { 
		if(!loaded_glFogf) { 
			loaded_glFogf = 1;
			init_lib(); 
			stub_glFogf=(pstub_glFogf)get_proc_address("glFogf");
			if(stub_glFogf){(*stub_glFogf)(lv0, lv1);}
			else { 
				FailWith("Cannot load glFogf"); 
			} 
		} else { 
			FailWith("glFogf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFogfv */
typedef void APIENTRY (*pstub_glFogfv)(GLenum,GLfloat*);
static pstub_glFogfv stub_glFogfv = NULL;
static int loaded_glFogfv = 0;

value glstub_glFogfv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glFogfv){(*stub_glFogfv)(lv0, lv1);}
	else { 
		if(!loaded_glFogfv) { 
			loaded_glFogfv = 1;
			init_lib(); 
			stub_glFogfv=(pstub_glFogfv)get_proc_address("glFogfv");
			if(stub_glFogfv){(*stub_glFogfv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glFogfv"); 
			} 
		} else { 
			FailWith("glFogfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFogi */
typedef void APIENTRY (*pstub_glFogi)(GLenum,GLint);
static pstub_glFogi stub_glFogi = NULL;
static int loaded_glFogi = 0;

value glstub_glFogi(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glFogi){(*stub_glFogi)(lv0, lv1);}
	else { 
		if(!loaded_glFogi) { 
			loaded_glFogi = 1;
			init_lib(); 
			stub_glFogi=(pstub_glFogi)get_proc_address("glFogi");
			if(stub_glFogi){(*stub_glFogi)(lv0, lv1);}
			else { 
				FailWith("Cannot load glFogi"); 
			} 
		} else { 
			FailWith("glFogi not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFogiv */
typedef void APIENTRY (*pstub_glFogiv)(GLenum,GLint*);
static pstub_glFogiv stub_glFogiv = NULL;
static int loaded_glFogiv = 0;

value glstub_glFogiv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glFogiv){(*stub_glFogiv)(lv0, lv1);}
	else { 
		if(!loaded_glFogiv) { 
			loaded_glFogiv = 1;
			init_lib(); 
			stub_glFogiv=(pstub_glFogiv)get_proc_address("glFogiv");
			if(stub_glFogiv){(*stub_glFogiv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glFogiv"); 
			} 
		} else { 
			FailWith("glFogiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFrontFace */
typedef void APIENTRY (*pstub_glFrontFace)(GLenum);
static pstub_glFrontFace stub_glFrontFace = NULL;
static int loaded_glFrontFace = 0;

value glstub_glFrontFace(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glFrontFace){(*stub_glFrontFace)(lv0);}
	else { 
		if(!loaded_glFrontFace) { 
			loaded_glFrontFace = 1;
			init_lib(); 
			stub_glFrontFace=(pstub_glFrontFace)get_proc_address("glFrontFace");
			if(stub_glFrontFace){(*stub_glFrontFace)(lv0);}
			else { 
				FailWith("Cannot load glFrontFace"); 
			} 
		} else { 
			FailWith("glFrontFace not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFrustum */
typedef void APIENTRY (*pstub_glFrustum)(GLdouble,GLdouble,GLdouble,GLdouble,GLdouble,GLdouble);
static pstub_glFrustum stub_glFrustum = NULL;
static int loaded_glFrustum = 0;

value glstub_glFrustum(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);
	GLdouble lv4 =  Double_val(v4);
	GLdouble lv5 =  Double_val(v5);

	if(stub_glFrustum){(*stub_glFrustum)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glFrustum) { 
			loaded_glFrustum = 1;
			init_lib(); 
			stub_glFrustum=(pstub_glFrustum)get_proc_address("glFrustum");
			if(stub_glFrustum){(*stub_glFrustum)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glFrustum"); 
			} 
		} else { 
			FailWith("glFrustum not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glFrustum_byte(value * argv, int n)
{
	return glstub_glFrustum(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glGenLists */
typedef GLuint APIENTRY (*pstub_glGenLists)(GLsizei);
static pstub_glGenLists stub_glGenLists = NULL;
static int loaded_glGenLists = 0;

value glstub_glGenLists(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLuint ret;
	if(stub_glGenLists){ret = (*stub_glGenLists)(lv0);}
	else { 
		if(!loaded_glGenLists) { 
			loaded_glGenLists = 1;
			init_lib(); 
			stub_glGenLists=(pstub_glGenLists)get_proc_address("glGenLists");
			if(stub_glGenLists){ret = (*stub_glGenLists)(lv0);}
			else { 
				FailWith("Cannot load glGenLists"); 
			} 
		} else { 
			FailWith("glGenLists not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glGenTextures */
typedef void APIENTRY (*pstub_glGenTextures)(GLsizei,GLuint*);
static pstub_glGenTextures stub_glGenTextures = NULL;
static int loaded_glGenTextures = 0;

value glstub_glGenTextures(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);

	if(stub_glGenTextures){(*stub_glGenTextures)(lv0, lv1);}
	else { 
		if(!loaded_glGenTextures) { 
			loaded_glGenTextures = 1;
			init_lib(); 
			stub_glGenTextures=(pstub_glGenTextures)get_proc_address("glGenTextures");
			if(stub_glGenTextures){(*stub_glGenTextures)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGenTextures"); 
			} 
		} else { 
			FailWith("glGenTextures not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetBooleanv */
typedef void APIENTRY (*pstub_glGetBooleanv)(GLenum,GLboolean*);
static pstub_glGetBooleanv stub_glGetBooleanv = NULL;
static int loaded_glGetBooleanv = 0;

value glstub_glGetBooleanv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLboolean* lv1 =  Data_bigarray_val(v1);

	if(stub_glGetBooleanv){(*stub_glGetBooleanv)(lv0, lv1);}
	else { 
		if(!loaded_glGetBooleanv) { 
			loaded_glGetBooleanv = 1;
			init_lib(); 
			stub_glGetBooleanv=(pstub_glGetBooleanv)get_proc_address("glGetBooleanv");
			if(stub_glGetBooleanv){(*stub_glGetBooleanv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGetBooleanv"); 
			} 
		} else { 
			FailWith("glGetBooleanv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetClipPlane */
typedef void APIENTRY (*pstub_glGetClipPlane)(GLenum,GLdouble*);
static pstub_glGetClipPlane stub_glGetClipPlane = NULL;
static int loaded_glGetClipPlane = 0;

value glstub_glGetClipPlane(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glGetClipPlane){(*stub_glGetClipPlane)(lv0, lv1);}
	else { 
		if(!loaded_glGetClipPlane) { 
			loaded_glGetClipPlane = 1;
			init_lib(); 
			stub_glGetClipPlane=(pstub_glGetClipPlane)get_proc_address("glGetClipPlane");
			if(stub_glGetClipPlane){(*stub_glGetClipPlane)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGetClipPlane"); 
			} 
		} else { 
			FailWith("glGetClipPlane not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetDoublev */
typedef void APIENTRY (*pstub_glGetDoublev)(GLenum,GLdouble*);
static pstub_glGetDoublev stub_glGetDoublev = NULL;
static int loaded_glGetDoublev = 0;

value glstub_glGetDoublev(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glGetDoublev){(*stub_glGetDoublev)(lv0, lv1);}
	else { 
		if(!loaded_glGetDoublev) { 
			loaded_glGetDoublev = 1;
			init_lib(); 
			stub_glGetDoublev=(pstub_glGetDoublev)get_proc_address("glGetDoublev");
			if(stub_glGetDoublev){(*stub_glGetDoublev)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGetDoublev"); 
			} 
		} else { 
			FailWith("glGetDoublev not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetError */
typedef GLenum APIENTRY (*pstub_glGetError)(GLvoid);
static pstub_glGetError stub_glGetError = NULL;
static int loaded_glGetError = 0;

value glstub_glGetError(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum ret;
	if(stub_glGetError){ret = (*stub_glGetError)();}
	else { 
		if(!loaded_glGetError) { 
			loaded_glGetError = 1;
			init_lib(); 
			stub_glGetError=(pstub_glGetError)get_proc_address("glGetError");
			if(stub_glGetError){ret = (*stub_glGetError)();}
			else { 
				FailWith("Cannot load glGetError"); 
			} 
		} else { 
			FailWith("glGetError not available"); 
		}
	}
	result = get_glenum(ret);
	CAMLreturn (result);
}

/* glGetFloatv */
typedef void APIENTRY (*pstub_glGetFloatv)(GLenum,GLfloat*);
static pstub_glGetFloatv stub_glGetFloatv = NULL;
static int loaded_glGetFloatv = 0;

value glstub_glGetFloatv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glGetFloatv){(*stub_glGetFloatv)(lv0, lv1);}
	else { 
		if(!loaded_glGetFloatv) { 
			loaded_glGetFloatv = 1;
			init_lib(); 
			stub_glGetFloatv=(pstub_glGetFloatv)get_proc_address("glGetFloatv");
			if(stub_glGetFloatv){(*stub_glGetFloatv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGetFloatv"); 
			} 
		} else { 
			FailWith("glGetFloatv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetIntegerv */
typedef void APIENTRY (*pstub_glGetIntegerv)(GLenum,GLint*);
static pstub_glGetIntegerv stub_glGetIntegerv = NULL;
static int loaded_glGetIntegerv = 0;

value glstub_glGetIntegerv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glGetIntegerv){(*stub_glGetIntegerv)(lv0, lv1);}
	else { 
		if(!loaded_glGetIntegerv) { 
			loaded_glGetIntegerv = 1;
			init_lib(); 
			stub_glGetIntegerv=(pstub_glGetIntegerv)get_proc_address("glGetIntegerv");
			if(stub_glGetIntegerv){(*stub_glGetIntegerv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGetIntegerv"); 
			} 
		} else { 
			FailWith("glGetIntegerv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetLightfv */
typedef void APIENTRY (*pstub_glGetLightfv)(GLenum,GLenum,GLfloat*);
static pstub_glGetLightfv stub_glGetLightfv = NULL;
static int loaded_glGetLightfv = 0;

value glstub_glGetLightfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetLightfv){(*stub_glGetLightfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetLightfv) { 
			loaded_glGetLightfv = 1;
			init_lib(); 
			stub_glGetLightfv=(pstub_glGetLightfv)get_proc_address("glGetLightfv");
			if(stub_glGetLightfv){(*stub_glGetLightfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetLightfv"); 
			} 
		} else { 
			FailWith("glGetLightfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetLightiv */
typedef void APIENTRY (*pstub_glGetLightiv)(GLenum,GLenum,GLint*);
static pstub_glGetLightiv stub_glGetLightiv = NULL;
static int loaded_glGetLightiv = 0;

value glstub_glGetLightiv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetLightiv){(*stub_glGetLightiv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetLightiv) { 
			loaded_glGetLightiv = 1;
			init_lib(); 
			stub_glGetLightiv=(pstub_glGetLightiv)get_proc_address("glGetLightiv");
			if(stub_glGetLightiv){(*stub_glGetLightiv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetLightiv"); 
			} 
		} else { 
			FailWith("glGetLightiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetMapdv */
typedef void APIENTRY (*pstub_glGetMapdv)(GLenum,GLenum,GLdouble*);
static pstub_glGetMapdv stub_glGetMapdv = NULL;
static int loaded_glGetMapdv = 0;

value glstub_glGetMapdv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLdouble* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetMapdv){(*stub_glGetMapdv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetMapdv) { 
			loaded_glGetMapdv = 1;
			init_lib(); 
			stub_glGetMapdv=(pstub_glGetMapdv)get_proc_address("glGetMapdv");
			if(stub_glGetMapdv){(*stub_glGetMapdv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetMapdv"); 
			} 
		} else { 
			FailWith("glGetMapdv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetMapfv */
typedef void APIENTRY (*pstub_glGetMapfv)(GLenum,GLenum,GLfloat*);
static pstub_glGetMapfv stub_glGetMapfv = NULL;
static int loaded_glGetMapfv = 0;

value glstub_glGetMapfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetMapfv){(*stub_glGetMapfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetMapfv) { 
			loaded_glGetMapfv = 1;
			init_lib(); 
			stub_glGetMapfv=(pstub_glGetMapfv)get_proc_address("glGetMapfv");
			if(stub_glGetMapfv){(*stub_glGetMapfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetMapfv"); 
			} 
		} else { 
			FailWith("glGetMapfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetMapiv */
typedef void APIENTRY (*pstub_glGetMapiv)(GLenum,GLenum,GLint*);
static pstub_glGetMapiv stub_glGetMapiv = NULL;
static int loaded_glGetMapiv = 0;

value glstub_glGetMapiv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetMapiv){(*stub_glGetMapiv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetMapiv) { 
			loaded_glGetMapiv = 1;
			init_lib(); 
			stub_glGetMapiv=(pstub_glGetMapiv)get_proc_address("glGetMapiv");
			if(stub_glGetMapiv){(*stub_glGetMapiv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetMapiv"); 
			} 
		} else { 
			FailWith("glGetMapiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetMaterialfv */
typedef void APIENTRY (*pstub_glGetMaterialfv)(GLenum,GLenum,GLfloat*);
static pstub_glGetMaterialfv stub_glGetMaterialfv = NULL;
static int loaded_glGetMaterialfv = 0;

value glstub_glGetMaterialfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetMaterialfv){(*stub_glGetMaterialfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetMaterialfv) { 
			loaded_glGetMaterialfv = 1;
			init_lib(); 
			stub_glGetMaterialfv=(pstub_glGetMaterialfv)get_proc_address("glGetMaterialfv");
			if(stub_glGetMaterialfv){(*stub_glGetMaterialfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetMaterialfv"); 
			} 
		} else { 
			FailWith("glGetMaterialfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetMaterialiv */
typedef void APIENTRY (*pstub_glGetMaterialiv)(GLenum,GLenum,GLint*);
static pstub_glGetMaterialiv stub_glGetMaterialiv = NULL;
static int loaded_glGetMaterialiv = 0;

value glstub_glGetMaterialiv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetMaterialiv){(*stub_glGetMaterialiv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetMaterialiv) { 
			loaded_glGetMaterialiv = 1;
			init_lib(); 
			stub_glGetMaterialiv=(pstub_glGetMaterialiv)get_proc_address("glGetMaterialiv");
			if(stub_glGetMaterialiv){(*stub_glGetMaterialiv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetMaterialiv"); 
			} 
		} else { 
			FailWith("glGetMaterialiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetPixelMapfv */
typedef void APIENTRY (*pstub_glGetPixelMapfv)(GLenum,GLfloat*);
static pstub_glGetPixelMapfv stub_glGetPixelMapfv = NULL;
static int loaded_glGetPixelMapfv = 0;

value glstub_glGetPixelMapfv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glGetPixelMapfv){(*stub_glGetPixelMapfv)(lv0, lv1);}
	else { 
		if(!loaded_glGetPixelMapfv) { 
			loaded_glGetPixelMapfv = 1;
			init_lib(); 
			stub_glGetPixelMapfv=(pstub_glGetPixelMapfv)get_proc_address("glGetPixelMapfv");
			if(stub_glGetPixelMapfv){(*stub_glGetPixelMapfv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGetPixelMapfv"); 
			} 
		} else { 
			FailWith("glGetPixelMapfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetPixelMapuiv */
typedef void APIENTRY (*pstub_glGetPixelMapuiv)(GLenum,GLuint*);
static pstub_glGetPixelMapuiv stub_glGetPixelMapuiv = NULL;
static int loaded_glGetPixelMapuiv = 0;

value glstub_glGetPixelMapuiv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);

	if(stub_glGetPixelMapuiv){(*stub_glGetPixelMapuiv)(lv0, lv1);}
	else { 
		if(!loaded_glGetPixelMapuiv) { 
			loaded_glGetPixelMapuiv = 1;
			init_lib(); 
			stub_glGetPixelMapuiv=(pstub_glGetPixelMapuiv)get_proc_address("glGetPixelMapuiv");
			if(stub_glGetPixelMapuiv){(*stub_glGetPixelMapuiv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGetPixelMapuiv"); 
			} 
		} else { 
			FailWith("glGetPixelMapuiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetPixelMapusv */
typedef void APIENTRY (*pstub_glGetPixelMapusv)(GLenum,GLushort*);
static pstub_glGetPixelMapusv stub_glGetPixelMapusv = NULL;
static int loaded_glGetPixelMapusv = 0;

value glstub_glGetPixelMapusv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLushort* lv1 =  Data_bigarray_val(v1);

	if(stub_glGetPixelMapusv){(*stub_glGetPixelMapusv)(lv0, lv1);}
	else { 
		if(!loaded_glGetPixelMapusv) { 
			loaded_glGetPixelMapusv = 1;
			init_lib(); 
			stub_glGetPixelMapusv=(pstub_glGetPixelMapusv)get_proc_address("glGetPixelMapusv");
			if(stub_glGetPixelMapusv){(*stub_glGetPixelMapusv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGetPixelMapusv"); 
			} 
		} else { 
			FailWith("glGetPixelMapusv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetPointerv */
typedef void APIENTRY (*pstub_glGetPointerv)(GLenum,GLvoid**);
static pstub_glGetPointerv stub_glGetPointerv = NULL;
static int loaded_glGetPointerv = 0;

value glstub_glGetPointerv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLvoid** lv1 =  Data_bigarray_val(v1);

	if(stub_glGetPointerv){(*stub_glGetPointerv)(lv0, lv1);}
	else { 
		if(!loaded_glGetPointerv) { 
			loaded_glGetPointerv = 1;
			init_lib(); 
			stub_glGetPointerv=(pstub_glGetPointerv)get_proc_address("glGetPointerv");
			if(stub_glGetPointerv){(*stub_glGetPointerv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGetPointerv"); 
			} 
		} else { 
			FailWith("glGetPointerv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetPolygonStipple */
typedef void APIENTRY (*pstub_glGetPolygonStipple)(GLubyte*);
static pstub_glGetPolygonStipple stub_glGetPolygonStipple = NULL;
static int loaded_glGetPolygonStipple = 0;

value glstub_glGetPolygonStipple(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLubyte* lv0 =  Data_bigarray_val(v0);

	if(stub_glGetPolygonStipple){(*stub_glGetPolygonStipple)(lv0);}
	else { 
		if(!loaded_glGetPolygonStipple) { 
			loaded_glGetPolygonStipple = 1;
			init_lib(); 
			stub_glGetPolygonStipple=(pstub_glGetPolygonStipple)get_proc_address("glGetPolygonStipple");
			if(stub_glGetPolygonStipple){(*stub_glGetPolygonStipple)(lv0);}
			else { 
				FailWith("Cannot load glGetPolygonStipple"); 
			} 
		} else { 
			FailWith("glGetPolygonStipple not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetString */
typedef GLstring APIENTRY (*pstub_glGetString)(GLenum);
static pstub_glGetString stub_glGetString = NULL;
static int loaded_glGetString = 0;

value glstub_glGetString(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLstring ret;
	if(stub_glGetString){ret = (*stub_glGetString)(lv0);}
	else { 
		if(!loaded_glGetString) { 
			loaded_glGetString = 1;
			init_lib(); 
			stub_glGetString=(pstub_glGetString)get_proc_address("glGetString");
			if(stub_glGetString){ret = (*stub_glGetString)(lv0);}
			else { 
				FailWith("Cannot load glGetString"); 
			} 
		} else { 
			FailWith("glGetString not available"); 
		}
	}
	result = caml_copy_string(ret);
	CAMLreturn (result);
}

/* glGetTexEnvfv */
typedef void APIENTRY (*pstub_glGetTexEnvfv)(GLenum,GLenum,GLfloat*);
static pstub_glGetTexEnvfv stub_glGetTexEnvfv = NULL;
static int loaded_glGetTexEnvfv = 0;

value glstub_glGetTexEnvfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetTexEnvfv){(*stub_glGetTexEnvfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetTexEnvfv) { 
			loaded_glGetTexEnvfv = 1;
			init_lib(); 
			stub_glGetTexEnvfv=(pstub_glGetTexEnvfv)get_proc_address("glGetTexEnvfv");
			if(stub_glGetTexEnvfv){(*stub_glGetTexEnvfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetTexEnvfv"); 
			} 
		} else { 
			FailWith("glGetTexEnvfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetTexEnviv */
typedef void APIENTRY (*pstub_glGetTexEnviv)(GLenum,GLenum,GLint*);
static pstub_glGetTexEnviv stub_glGetTexEnviv = NULL;
static int loaded_glGetTexEnviv = 0;

value glstub_glGetTexEnviv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetTexEnviv){(*stub_glGetTexEnviv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetTexEnviv) { 
			loaded_glGetTexEnviv = 1;
			init_lib(); 
			stub_glGetTexEnviv=(pstub_glGetTexEnviv)get_proc_address("glGetTexEnviv");
			if(stub_glGetTexEnviv){(*stub_glGetTexEnviv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetTexEnviv"); 
			} 
		} else { 
			FailWith("glGetTexEnviv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetTexGendv */
typedef void APIENTRY (*pstub_glGetTexGendv)(GLenum,GLenum,GLdouble*);
static pstub_glGetTexGendv stub_glGetTexGendv = NULL;
static int loaded_glGetTexGendv = 0;

value glstub_glGetTexGendv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLdouble* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetTexGendv){(*stub_glGetTexGendv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetTexGendv) { 
			loaded_glGetTexGendv = 1;
			init_lib(); 
			stub_glGetTexGendv=(pstub_glGetTexGendv)get_proc_address("glGetTexGendv");
			if(stub_glGetTexGendv){(*stub_glGetTexGendv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetTexGendv"); 
			} 
		} else { 
			FailWith("glGetTexGendv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetTexGenfv */
typedef void APIENTRY (*pstub_glGetTexGenfv)(GLenum,GLenum,GLfloat*);
static pstub_glGetTexGenfv stub_glGetTexGenfv = NULL;
static int loaded_glGetTexGenfv = 0;

value glstub_glGetTexGenfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetTexGenfv){(*stub_glGetTexGenfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetTexGenfv) { 
			loaded_glGetTexGenfv = 1;
			init_lib(); 
			stub_glGetTexGenfv=(pstub_glGetTexGenfv)get_proc_address("glGetTexGenfv");
			if(stub_glGetTexGenfv){(*stub_glGetTexGenfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetTexGenfv"); 
			} 
		} else { 
			FailWith("glGetTexGenfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetTexGeniv */
typedef void APIENTRY (*pstub_glGetTexGeniv)(GLenum,GLenum,GLint*);
static pstub_glGetTexGeniv stub_glGetTexGeniv = NULL;
static int loaded_glGetTexGeniv = 0;

value glstub_glGetTexGeniv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetTexGeniv){(*stub_glGetTexGeniv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetTexGeniv) { 
			loaded_glGetTexGeniv = 1;
			init_lib(); 
			stub_glGetTexGeniv=(pstub_glGetTexGeniv)get_proc_address("glGetTexGeniv");
			if(stub_glGetTexGeniv){(*stub_glGetTexGeniv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetTexGeniv"); 
			} 
		} else { 
			FailWith("glGetTexGeniv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetTexImage */
typedef void APIENTRY (*pstub_glGetTexImage)(GLenum,GLint,GLenum,GLenum,GLvoid*);
static pstub_glGetTexImage stub_glGetTexImage = NULL;
static int loaded_glGetTexImage = 0;

value glstub_glGetTexImage(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLenum lv3 =  get_glenum(v3);
	GLvoid* lv4 =  Data_bigarray_val(v4);

	if(stub_glGetTexImage){(*stub_glGetTexImage)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glGetTexImage) { 
			loaded_glGetTexImage = 1;
			init_lib(); 
			stub_glGetTexImage=(pstub_glGetTexImage)get_proc_address("glGetTexImage");
			if(stub_glGetTexImage){(*stub_glGetTexImage)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glGetTexImage"); 
			} 
		} else { 
			FailWith("glGetTexImage not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetTexLevelParameterfv */
typedef void APIENTRY (*pstub_glGetTexLevelParameterfv)(GLenum,GLint,GLenum,GLfloat*);
static pstub_glGetTexLevelParameterfv stub_glGetTexLevelParameterfv = NULL;
static int loaded_glGetTexLevelParameterfv = 0;

value glstub_glGetTexLevelParameterfv(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLfloat* lv3 =  Data_bigarray_val(v3);

	if(stub_glGetTexLevelParameterfv){(*stub_glGetTexLevelParameterfv)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glGetTexLevelParameterfv) { 
			loaded_glGetTexLevelParameterfv = 1;
			init_lib(); 
			stub_glGetTexLevelParameterfv=(pstub_glGetTexLevelParameterfv)get_proc_address("glGetTexLevelParameterfv");
			if(stub_glGetTexLevelParameterfv){(*stub_glGetTexLevelParameterfv)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glGetTexLevelParameterfv"); 
			} 
		} else { 
			FailWith("glGetTexLevelParameterfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetTexLevelParameteriv */
typedef void APIENTRY (*pstub_glGetTexLevelParameteriv)(GLenum,GLint,GLenum,GLint*);
static pstub_glGetTexLevelParameteriv stub_glGetTexLevelParameteriv = NULL;
static int loaded_glGetTexLevelParameteriv = 0;

value glstub_glGetTexLevelParameteriv(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLint* lv3 =  Data_bigarray_val(v3);

	if(stub_glGetTexLevelParameteriv){(*stub_glGetTexLevelParameteriv)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glGetTexLevelParameteriv) { 
			loaded_glGetTexLevelParameteriv = 1;
			init_lib(); 
			stub_glGetTexLevelParameteriv=(pstub_glGetTexLevelParameteriv)get_proc_address("glGetTexLevelParameteriv");
			if(stub_glGetTexLevelParameteriv){(*stub_glGetTexLevelParameteriv)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glGetTexLevelParameteriv"); 
			} 
		} else { 
			FailWith("glGetTexLevelParameteriv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetTexParameterfv */
typedef void APIENTRY (*pstub_glGetTexParameterfv)(GLenum,GLenum,GLfloat*);
static pstub_glGetTexParameterfv stub_glGetTexParameterfv = NULL;
static int loaded_glGetTexParameterfv = 0;

value glstub_glGetTexParameterfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetTexParameterfv){(*stub_glGetTexParameterfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetTexParameterfv) { 
			loaded_glGetTexParameterfv = 1;
			init_lib(); 
			stub_glGetTexParameterfv=(pstub_glGetTexParameterfv)get_proc_address("glGetTexParameterfv");
			if(stub_glGetTexParameterfv){(*stub_glGetTexParameterfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetTexParameterfv"); 
			} 
		} else { 
			FailWith("glGetTexParameterfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetTexParameteriv */
typedef void APIENTRY (*pstub_glGetTexParameteriv)(GLenum,GLenum,GLint*);
static pstub_glGetTexParameteriv stub_glGetTexParameteriv = NULL;
static int loaded_glGetTexParameteriv = 0;

value glstub_glGetTexParameteriv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetTexParameteriv){(*stub_glGetTexParameteriv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetTexParameteriv) { 
			loaded_glGetTexParameteriv = 1;
			init_lib(); 
			stub_glGetTexParameteriv=(pstub_glGetTexParameteriv)get_proc_address("glGetTexParameteriv");
			if(stub_glGetTexParameteriv){(*stub_glGetTexParameteriv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetTexParameteriv"); 
			} 
		} else { 
			FailWith("glGetTexParameteriv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glHint */
typedef void APIENTRY (*pstub_glHint)(GLenum,GLenum);
static pstub_glHint stub_glHint = NULL;
static int loaded_glHint = 0;

value glstub_glHint(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);

	if(stub_glHint){(*stub_glHint)(lv0, lv1);}
	else { 
		if(!loaded_glHint) { 
			loaded_glHint = 1;
			init_lib(); 
			stub_glHint=(pstub_glHint)get_proc_address("glHint");
			if(stub_glHint){(*stub_glHint)(lv0, lv1);}
			else { 
				FailWith("Cannot load glHint"); 
			} 
		} else { 
			FailWith("glHint not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIndexMask */
typedef void APIENTRY (*pstub_glIndexMask)(GLuint);
static pstub_glIndexMask stub_glIndexMask = NULL;
static int loaded_glIndexMask = 0;

value glstub_glIndexMask(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glIndexMask){(*stub_glIndexMask)(lv0);}
	else { 
		if(!loaded_glIndexMask) { 
			loaded_glIndexMask = 1;
			init_lib(); 
			stub_glIndexMask=(pstub_glIndexMask)get_proc_address("glIndexMask");
			if(stub_glIndexMask){(*stub_glIndexMask)(lv0);}
			else { 
				FailWith("Cannot load glIndexMask"); 
			} 
		} else { 
			FailWith("glIndexMask not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIndexPointer */
typedef void APIENTRY (*pstub_glIndexPointer)(GLenum,GLsizei,GLvoid*);
static pstub_glIndexPointer stub_glIndexPointer = NULL;
static int loaded_glIndexPointer = 0;

value glstub_glIndexPointer(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizei lv1 =  Int_val(v1);
	GLvoid* lv2 =  Data_bigarray_val(v2);

	if(stub_glIndexPointer){(*stub_glIndexPointer)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glIndexPointer) { 
			loaded_glIndexPointer = 1;
			init_lib(); 
			stub_glIndexPointer=(pstub_glIndexPointer)get_proc_address("glIndexPointer");
			if(stub_glIndexPointer){(*stub_glIndexPointer)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glIndexPointer"); 
			} 
		} else { 
			FailWith("glIndexPointer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIndexd */
typedef void APIENTRY (*pstub_glIndexd)(GLdouble);
static pstub_glIndexd stub_glIndexd = NULL;
static int loaded_glIndexd = 0;

value glstub_glIndexd(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);

	if(stub_glIndexd){(*stub_glIndexd)(lv0);}
	else { 
		if(!loaded_glIndexd) { 
			loaded_glIndexd = 1;
			init_lib(); 
			stub_glIndexd=(pstub_glIndexd)get_proc_address("glIndexd");
			if(stub_glIndexd){(*stub_glIndexd)(lv0);}
			else { 
				FailWith("Cannot load glIndexd"); 
			} 
		} else { 
			FailWith("glIndexd not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIndexdv */
typedef void APIENTRY (*pstub_glIndexdv)(GLdouble*);
static pstub_glIndexdv stub_glIndexdv = NULL;
static int loaded_glIndexdv = 0;

value glstub_glIndexdv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glIndexdv){(*stub_glIndexdv)(lv0);}
	else { 
		if(!loaded_glIndexdv) { 
			loaded_glIndexdv = 1;
			init_lib(); 
			stub_glIndexdv=(pstub_glIndexdv)get_proc_address("glIndexdv");
			if(stub_glIndexdv){(*stub_glIndexdv)(lv0);}
			else { 
				FailWith("Cannot load glIndexdv"); 
			} 
		} else { 
			FailWith("glIndexdv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIndexf */
typedef void APIENTRY (*pstub_glIndexf)(GLfloat);
static pstub_glIndexf stub_glIndexf = NULL;
static int loaded_glIndexf = 0;

value glstub_glIndexf(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);

	if(stub_glIndexf){(*stub_glIndexf)(lv0);}
	else { 
		if(!loaded_glIndexf) { 
			loaded_glIndexf = 1;
			init_lib(); 
			stub_glIndexf=(pstub_glIndexf)get_proc_address("glIndexf");
			if(stub_glIndexf){(*stub_glIndexf)(lv0);}
			else { 
				FailWith("Cannot load glIndexf"); 
			} 
		} else { 
			FailWith("glIndexf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIndexfv */
typedef void APIENTRY (*pstub_glIndexfv)(GLfloat*);
static pstub_glIndexfv stub_glIndexfv = NULL;
static int loaded_glIndexfv = 0;

value glstub_glIndexfv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glIndexfv){(*stub_glIndexfv)(lv0);}
	else { 
		if(!loaded_glIndexfv) { 
			loaded_glIndexfv = 1;
			init_lib(); 
			stub_glIndexfv=(pstub_glIndexfv)get_proc_address("glIndexfv");
			if(stub_glIndexfv){(*stub_glIndexfv)(lv0);}
			else { 
				FailWith("Cannot load glIndexfv"); 
			} 
		} else { 
			FailWith("glIndexfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIndexi */
typedef void APIENTRY (*pstub_glIndexi)(GLint);
static pstub_glIndexi stub_glIndexi = NULL;
static int loaded_glIndexi = 0;

value glstub_glIndexi(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);

	if(stub_glIndexi){(*stub_glIndexi)(lv0);}
	else { 
		if(!loaded_glIndexi) { 
			loaded_glIndexi = 1;
			init_lib(); 
			stub_glIndexi=(pstub_glIndexi)get_proc_address("glIndexi");
			if(stub_glIndexi){(*stub_glIndexi)(lv0);}
			else { 
				FailWith("Cannot load glIndexi"); 
			} 
		} else { 
			FailWith("glIndexi not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIndexiv */
typedef void APIENTRY (*pstub_glIndexiv)(GLint*);
static pstub_glIndexiv stub_glIndexiv = NULL;
static int loaded_glIndexiv = 0;

value glstub_glIndexiv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glIndexiv){(*stub_glIndexiv)(lv0);}
	else { 
		if(!loaded_glIndexiv) { 
			loaded_glIndexiv = 1;
			init_lib(); 
			stub_glIndexiv=(pstub_glIndexiv)get_proc_address("glIndexiv");
			if(stub_glIndexiv){(*stub_glIndexiv)(lv0);}
			else { 
				FailWith("Cannot load glIndexiv"); 
			} 
		} else { 
			FailWith("glIndexiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIndexs */
typedef void APIENTRY (*pstub_glIndexs)(GLshort);
static pstub_glIndexs stub_glIndexs = NULL;
static int loaded_glIndexs = 0;

value glstub_glIndexs(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);

	if(stub_glIndexs){(*stub_glIndexs)(lv0);}
	else { 
		if(!loaded_glIndexs) { 
			loaded_glIndexs = 1;
			init_lib(); 
			stub_glIndexs=(pstub_glIndexs)get_proc_address("glIndexs");
			if(stub_glIndexs){(*stub_glIndexs)(lv0);}
			else { 
				FailWith("Cannot load glIndexs"); 
			} 
		} else { 
			FailWith("glIndexs not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIndexsv */
typedef void APIENTRY (*pstub_glIndexsv)(GLshort*);
static pstub_glIndexsv stub_glIndexsv = NULL;
static int loaded_glIndexsv = 0;

value glstub_glIndexsv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glIndexsv){(*stub_glIndexsv)(lv0);}
	else { 
		if(!loaded_glIndexsv) { 
			loaded_glIndexsv = 1;
			init_lib(); 
			stub_glIndexsv=(pstub_glIndexsv)get_proc_address("glIndexsv");
			if(stub_glIndexsv){(*stub_glIndexsv)(lv0);}
			else { 
				FailWith("Cannot load glIndexsv"); 
			} 
		} else { 
			FailWith("glIndexsv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIndexub */
typedef void APIENTRY (*pstub_glIndexub)(GLubyte);
static pstub_glIndexub stub_glIndexub = NULL;
static int loaded_glIndexub = 0;

value glstub_glIndexub(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLubyte lv0 =  Int_val(v0);

	if(stub_glIndexub){(*stub_glIndexub)(lv0);}
	else { 
		if(!loaded_glIndexub) { 
			loaded_glIndexub = 1;
			init_lib(); 
			stub_glIndexub=(pstub_glIndexub)get_proc_address("glIndexub");
			if(stub_glIndexub){(*stub_glIndexub)(lv0);}
			else { 
				FailWith("Cannot load glIndexub"); 
			} 
		} else { 
			FailWith("glIndexub not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIndexubv */
typedef void APIENTRY (*pstub_glIndexubv)(GLubyte*);
static pstub_glIndexubv stub_glIndexubv = NULL;
static int loaded_glIndexubv = 0;

value glstub_glIndexubv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLubyte* lv0 =  Data_bigarray_val(v0);

	if(stub_glIndexubv){(*stub_glIndexubv)(lv0);}
	else { 
		if(!loaded_glIndexubv) { 
			loaded_glIndexubv = 1;
			init_lib(); 
			stub_glIndexubv=(pstub_glIndexubv)get_proc_address("glIndexubv");
			if(stub_glIndexubv){(*stub_glIndexubv)(lv0);}
			else { 
				FailWith("Cannot load glIndexubv"); 
			} 
		} else { 
			FailWith("glIndexubv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glInitNames */
typedef void APIENTRY (*pstub_glInitNames)(GLvoid);
static pstub_glInitNames stub_glInitNames = NULL;
static int loaded_glInitNames = 0;

value glstub_glInitNames(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);

	if(stub_glInitNames){(*stub_glInitNames)();}
	else { 
		if(!loaded_glInitNames) { 
			loaded_glInitNames = 1;
			init_lib(); 
			stub_glInitNames=(pstub_glInitNames)get_proc_address("glInitNames");
			if(stub_glInitNames){(*stub_glInitNames)();}
			else { 
				FailWith("Cannot load glInitNames"); 
			} 
		} else { 
			FailWith("glInitNames not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glInterleavedArrays */
typedef void APIENTRY (*pstub_glInterleavedArrays)(GLenum,GLsizei,GLvoid*);
static pstub_glInterleavedArrays stub_glInterleavedArrays = NULL;
static int loaded_glInterleavedArrays = 0;

value glstub_glInterleavedArrays(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizei lv1 =  Int_val(v1);
	GLvoid* lv2 =  Data_bigarray_val(v2);

	if(stub_glInterleavedArrays){(*stub_glInterleavedArrays)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glInterleavedArrays) { 
			loaded_glInterleavedArrays = 1;
			init_lib(); 
			stub_glInterleavedArrays=(pstub_glInterleavedArrays)get_proc_address("glInterleavedArrays");
			if(stub_glInterleavedArrays){(*stub_glInterleavedArrays)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glInterleavedArrays"); 
			} 
		} else { 
			FailWith("glInterleavedArrays not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIsEnabled */
typedef GLboolean APIENTRY (*pstub_glIsEnabled)(GLenum);
static pstub_glIsEnabled stub_glIsEnabled = NULL;
static int loaded_glIsEnabled = 0;

value glstub_glIsEnabled(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLboolean ret;
	if(stub_glIsEnabled){ret = (*stub_glIsEnabled)(lv0);}
	else { 
		if(!loaded_glIsEnabled) { 
			loaded_glIsEnabled = 1;
			init_lib(); 
			stub_glIsEnabled=(pstub_glIsEnabled)get_proc_address("glIsEnabled");
			if(stub_glIsEnabled){ret = (*stub_glIsEnabled)(lv0);}
			else { 
				FailWith("Cannot load glIsEnabled"); 
			} 
		} else { 
			FailWith("glIsEnabled not available"); 
		}
	}
	result = Bool_val(ret);
	CAMLreturn (result);
}

/* glIsList */
typedef GLboolean APIENTRY (*pstub_glIsList)(GLuint);
static pstub_glIsList stub_glIsList = NULL;
static int loaded_glIsList = 0;

value glstub_glIsList(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLboolean ret;
	if(stub_glIsList){ret = (*stub_glIsList)(lv0);}
	else { 
		if(!loaded_glIsList) { 
			loaded_glIsList = 1;
			init_lib(); 
			stub_glIsList=(pstub_glIsList)get_proc_address("glIsList");
			if(stub_glIsList){ret = (*stub_glIsList)(lv0);}
			else { 
				FailWith("Cannot load glIsList"); 
			} 
		} else { 
			FailWith("glIsList not available"); 
		}
	}
	result = Bool_val(ret);
	CAMLreturn (result);
}

/* glIsTexture */
typedef GLboolean APIENTRY (*pstub_glIsTexture)(GLuint);
static pstub_glIsTexture stub_glIsTexture = NULL;
static int loaded_glIsTexture = 0;

value glstub_glIsTexture(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLboolean ret;
	if(stub_glIsTexture){ret = (*stub_glIsTexture)(lv0);}
	else { 
		if(!loaded_glIsTexture) { 
			loaded_glIsTexture = 1;
			init_lib(); 
			stub_glIsTexture=(pstub_glIsTexture)get_proc_address("glIsTexture");
			if(stub_glIsTexture){ret = (*stub_glIsTexture)(lv0);}
			else { 
				FailWith("Cannot load glIsTexture"); 
			} 
		} else { 
			FailWith("glIsTexture not available"); 
		}
	}
	result = Bool_val(ret);
	CAMLreturn (result);
}

/* glLightModelf */
typedef void APIENTRY (*pstub_glLightModelf)(GLenum,GLfloat);
static pstub_glLightModelf stub_glLightModelf = NULL;
static int loaded_glLightModelf = 0;

value glstub_glLightModelf(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glLightModelf){(*stub_glLightModelf)(lv0, lv1);}
	else { 
		if(!loaded_glLightModelf) { 
			loaded_glLightModelf = 1;
			init_lib(); 
			stub_glLightModelf=(pstub_glLightModelf)get_proc_address("glLightModelf");
			if(stub_glLightModelf){(*stub_glLightModelf)(lv0, lv1);}
			else { 
				FailWith("Cannot load glLightModelf"); 
			} 
		} else { 
			FailWith("glLightModelf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLightModelfv */
typedef void APIENTRY (*pstub_glLightModelfv)(GLenum,GLfloat*);
static pstub_glLightModelfv stub_glLightModelfv = NULL;
static int loaded_glLightModelfv = 0;

value glstub_glLightModelfv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glLightModelfv){(*stub_glLightModelfv)(lv0, lv1);}
	else { 
		if(!loaded_glLightModelfv) { 
			loaded_glLightModelfv = 1;
			init_lib(); 
			stub_glLightModelfv=(pstub_glLightModelfv)get_proc_address("glLightModelfv");
			if(stub_glLightModelfv){(*stub_glLightModelfv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glLightModelfv"); 
			} 
		} else { 
			FailWith("glLightModelfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLightModeli */
typedef void APIENTRY (*pstub_glLightModeli)(GLenum,GLint);
static pstub_glLightModeli stub_glLightModeli = NULL;
static int loaded_glLightModeli = 0;

value glstub_glLightModeli(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glLightModeli){(*stub_glLightModeli)(lv0, lv1);}
	else { 
		if(!loaded_glLightModeli) { 
			loaded_glLightModeli = 1;
			init_lib(); 
			stub_glLightModeli=(pstub_glLightModeli)get_proc_address("glLightModeli");
			if(stub_glLightModeli){(*stub_glLightModeli)(lv0, lv1);}
			else { 
				FailWith("Cannot load glLightModeli"); 
			} 
		} else { 
			FailWith("glLightModeli not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLightModeliv */
typedef void APIENTRY (*pstub_glLightModeliv)(GLenum,GLint*);
static pstub_glLightModeliv stub_glLightModeliv = NULL;
static int loaded_glLightModeliv = 0;

value glstub_glLightModeliv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glLightModeliv){(*stub_glLightModeliv)(lv0, lv1);}
	else { 
		if(!loaded_glLightModeliv) { 
			loaded_glLightModeliv = 1;
			init_lib(); 
			stub_glLightModeliv=(pstub_glLightModeliv)get_proc_address("glLightModeliv");
			if(stub_glLightModeliv){(*stub_glLightModeliv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glLightModeliv"); 
			} 
		} else { 
			FailWith("glLightModeliv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLightf */
typedef void APIENTRY (*pstub_glLightf)(GLenum,GLenum,GLfloat);
static pstub_glLightf stub_glLightf = NULL;
static int loaded_glLightf = 0;

value glstub_glLightf(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glLightf){(*stub_glLightf)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glLightf) { 
			loaded_glLightf = 1;
			init_lib(); 
			stub_glLightf=(pstub_glLightf)get_proc_address("glLightf");
			if(stub_glLightf){(*stub_glLightf)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glLightf"); 
			} 
		} else { 
			FailWith("glLightf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLightfv */
typedef void APIENTRY (*pstub_glLightfv)(GLenum,GLenum,GLfloat*);
static pstub_glLightfv stub_glLightfv = NULL;
static int loaded_glLightfv = 0;

value glstub_glLightfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glLightfv){(*stub_glLightfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glLightfv) { 
			loaded_glLightfv = 1;
			init_lib(); 
			stub_glLightfv=(pstub_glLightfv)get_proc_address("glLightfv");
			if(stub_glLightfv){(*stub_glLightfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glLightfv"); 
			} 
		} else { 
			FailWith("glLightfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLighti */
typedef void APIENTRY (*pstub_glLighti)(GLenum,GLenum,GLint);
static pstub_glLighti stub_glLighti = NULL;
static int loaded_glLighti = 0;

value glstub_glLighti(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glLighti){(*stub_glLighti)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glLighti) { 
			loaded_glLighti = 1;
			init_lib(); 
			stub_glLighti=(pstub_glLighti)get_proc_address("glLighti");
			if(stub_glLighti){(*stub_glLighti)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glLighti"); 
			} 
		} else { 
			FailWith("glLighti not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLightiv */
typedef void APIENTRY (*pstub_glLightiv)(GLenum,GLenum,GLint*);
static pstub_glLightiv stub_glLightiv = NULL;
static int loaded_glLightiv = 0;

value glstub_glLightiv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glLightiv){(*stub_glLightiv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glLightiv) { 
			loaded_glLightiv = 1;
			init_lib(); 
			stub_glLightiv=(pstub_glLightiv)get_proc_address("glLightiv");
			if(stub_glLightiv){(*stub_glLightiv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glLightiv"); 
			} 
		} else { 
			FailWith("glLightiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLineStipple */
typedef void APIENTRY (*pstub_glLineStipple)(GLint,GLushort);
static pstub_glLineStipple stub_glLineStipple = NULL;
static int loaded_glLineStipple = 0;

value glstub_glLineStipple(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLushort lv1 =  Int_val(v1);

	if(stub_glLineStipple){(*stub_glLineStipple)(lv0, lv1);}
	else { 
		if(!loaded_glLineStipple) { 
			loaded_glLineStipple = 1;
			init_lib(); 
			stub_glLineStipple=(pstub_glLineStipple)get_proc_address("glLineStipple");
			if(stub_glLineStipple){(*stub_glLineStipple)(lv0, lv1);}
			else { 
				FailWith("Cannot load glLineStipple"); 
			} 
		} else { 
			FailWith("glLineStipple not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLineWidth */
typedef void APIENTRY (*pstub_glLineWidth)(GLfloat);
static pstub_glLineWidth stub_glLineWidth = NULL;
static int loaded_glLineWidth = 0;

value glstub_glLineWidth(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);

	if(stub_glLineWidth){(*stub_glLineWidth)(lv0);}
	else { 
		if(!loaded_glLineWidth) { 
			loaded_glLineWidth = 1;
			init_lib(); 
			stub_glLineWidth=(pstub_glLineWidth)get_proc_address("glLineWidth");
			if(stub_glLineWidth){(*stub_glLineWidth)(lv0);}
			else { 
				FailWith("Cannot load glLineWidth"); 
			} 
		} else { 
			FailWith("glLineWidth not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glListBase */
typedef void APIENTRY (*pstub_glListBase)(GLuint);
static pstub_glListBase stub_glListBase = NULL;
static int loaded_glListBase = 0;

value glstub_glListBase(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glListBase){(*stub_glListBase)(lv0);}
	else { 
		if(!loaded_glListBase) { 
			loaded_glListBase = 1;
			init_lib(); 
			stub_glListBase=(pstub_glListBase)get_proc_address("glListBase");
			if(stub_glListBase){(*stub_glListBase)(lv0);}
			else { 
				FailWith("Cannot load glListBase"); 
			} 
		} else { 
			FailWith("glListBase not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLoadIdentity */
typedef void APIENTRY (*pstub_glLoadIdentity)(GLvoid);
static pstub_glLoadIdentity stub_glLoadIdentity = NULL;
static int loaded_glLoadIdentity = 0;

value glstub_glLoadIdentity(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);

	if(stub_glLoadIdentity){(*stub_glLoadIdentity)();}
	else { 
		if(!loaded_glLoadIdentity) { 
			loaded_glLoadIdentity = 1;
			init_lib(); 
			stub_glLoadIdentity=(pstub_glLoadIdentity)get_proc_address("glLoadIdentity");
			if(stub_glLoadIdentity){(*stub_glLoadIdentity)();}
			else { 
				FailWith("Cannot load glLoadIdentity"); 
			} 
		} else { 
			FailWith("glLoadIdentity not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLoadMatrixd */
typedef void APIENTRY (*pstub_glLoadMatrixd)(GLdouble*);
static pstub_glLoadMatrixd stub_glLoadMatrixd = NULL;
static int loaded_glLoadMatrixd = 0;

value glstub_glLoadMatrixd(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glLoadMatrixd){(*stub_glLoadMatrixd)(lv0);}
	else { 
		if(!loaded_glLoadMatrixd) { 
			loaded_glLoadMatrixd = 1;
			init_lib(); 
			stub_glLoadMatrixd=(pstub_glLoadMatrixd)get_proc_address("glLoadMatrixd");
			if(stub_glLoadMatrixd){(*stub_glLoadMatrixd)(lv0);}
			else { 
				FailWith("Cannot load glLoadMatrixd"); 
			} 
		} else { 
			FailWith("glLoadMatrixd not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLoadMatrixf */
typedef void APIENTRY (*pstub_glLoadMatrixf)(GLfloat*);
static pstub_glLoadMatrixf stub_glLoadMatrixf = NULL;
static int loaded_glLoadMatrixf = 0;

value glstub_glLoadMatrixf(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glLoadMatrixf){(*stub_glLoadMatrixf)(lv0);}
	else { 
		if(!loaded_glLoadMatrixf) { 
			loaded_glLoadMatrixf = 1;
			init_lib(); 
			stub_glLoadMatrixf=(pstub_glLoadMatrixf)get_proc_address("glLoadMatrixf");
			if(stub_glLoadMatrixf){(*stub_glLoadMatrixf)(lv0);}
			else { 
				FailWith("Cannot load glLoadMatrixf"); 
			} 
		} else { 
			FailWith("glLoadMatrixf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLoadName */
typedef void APIENTRY (*pstub_glLoadName)(GLuint);
static pstub_glLoadName stub_glLoadName = NULL;
static int loaded_glLoadName = 0;

value glstub_glLoadName(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glLoadName){(*stub_glLoadName)(lv0);}
	else { 
		if(!loaded_glLoadName) { 
			loaded_glLoadName = 1;
			init_lib(); 
			stub_glLoadName=(pstub_glLoadName)get_proc_address("glLoadName");
			if(stub_glLoadName){(*stub_glLoadName)(lv0);}
			else { 
				FailWith("Cannot load glLoadName"); 
			} 
		} else { 
			FailWith("glLoadName not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLogicOp */
typedef void APIENTRY (*pstub_glLogicOp)(GLenum);
static pstub_glLogicOp stub_glLogicOp = NULL;
static int loaded_glLogicOp = 0;

value glstub_glLogicOp(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glLogicOp){(*stub_glLogicOp)(lv0);}
	else { 
		if(!loaded_glLogicOp) { 
			loaded_glLogicOp = 1;
			init_lib(); 
			stub_glLogicOp=(pstub_glLogicOp)get_proc_address("glLogicOp");
			if(stub_glLogicOp){(*stub_glLogicOp)(lv0);}
			else { 
				FailWith("Cannot load glLogicOp"); 
			} 
		} else { 
			FailWith("glLogicOp not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMap1d */
typedef void APIENTRY (*pstub_glMap1d)(GLenum,GLdouble,GLdouble,GLint,GLint,GLdouble*);
static pstub_glMap1d stub_glMap1d = NULL;
static int loaded_glMap1d = 0;

value glstub_glMap1d(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);
	GLdouble* lv5 =  Data_bigarray_val(v5);

	if(stub_glMap1d){(*stub_glMap1d)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glMap1d) { 
			loaded_glMap1d = 1;
			init_lib(); 
			stub_glMap1d=(pstub_glMap1d)get_proc_address("glMap1d");
			if(stub_glMap1d){(*stub_glMap1d)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glMap1d"); 
			} 
		} else { 
			FailWith("glMap1d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glMap1d_byte(value * argv, int n)
{
	return glstub_glMap1d(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glMap1f */
typedef void APIENTRY (*pstub_glMap1f)(GLenum,GLfloat,GLfloat,GLint,GLint,GLfloat*);
static pstub_glMap1f stub_glMap1f = NULL;
static int loaded_glMap1f = 0;

value glstub_glMap1f(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);
	GLfloat* lv5 =  Data_bigarray_val(v5);

	if(stub_glMap1f){(*stub_glMap1f)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glMap1f) { 
			loaded_glMap1f = 1;
			init_lib(); 
			stub_glMap1f=(pstub_glMap1f)get_proc_address("glMap1f");
			if(stub_glMap1f){(*stub_glMap1f)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glMap1f"); 
			} 
		} else { 
			FailWith("glMap1f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glMap1f_byte(value * argv, int n)
{
	return glstub_glMap1f(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glMap2d */
typedef void APIENTRY (*pstub_glMap2d)(GLenum,GLdouble,GLdouble,GLint,GLint,GLdouble,GLdouble,GLint,GLint,GLdouble*);
static pstub_glMap2d stub_glMap2d = NULL;
static int loaded_glMap2d = 0;

value glstub_glMap2d(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7, value v8, value v9)
{
	CAMLparam5(v0, v1, v2, v3, v4);
CAMLxparam5(v5, v6, v7, v8, v9);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);
	GLdouble lv5 =  Double_val(v5);
	GLdouble lv6 =  Double_val(v6);
	GLint lv7 =  Int_val(v7);
	GLint lv8 =  Int_val(v8);
	GLdouble* lv9 =  Data_bigarray_val(v9);

	if(stub_glMap2d){(*stub_glMap2d)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8, lv9);}
	else { 
		if(!loaded_glMap2d) { 
			loaded_glMap2d = 1;
			init_lib(); 
			stub_glMap2d=(pstub_glMap2d)get_proc_address("glMap2d");
			if(stub_glMap2d){(*stub_glMap2d)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8, lv9);}
			else { 
				FailWith("Cannot load glMap2d"); 
			} 
		} else { 
			FailWith("glMap2d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glMap2d_byte(value * argv, int n)
{
	return glstub_glMap2d(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8], argv[9]);
}

/* glMap2f */
typedef void APIENTRY (*pstub_glMap2f)(GLenum,GLfloat,GLfloat,GLint,GLint,GLfloat,GLfloat,GLint,GLint,GLfloat*);
static pstub_glMap2f stub_glMap2f = NULL;
static int loaded_glMap2f = 0;

value glstub_glMap2f(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7, value v8, value v9)
{
	CAMLparam5(v0, v1, v2, v3, v4);
CAMLxparam5(v5, v6, v7, v8, v9);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);
	GLfloat lv5 =  Double_val(v5);
	GLfloat lv6 =  Double_val(v6);
	GLint lv7 =  Int_val(v7);
	GLint lv8 =  Int_val(v8);
	GLfloat* lv9 =  Data_bigarray_val(v9);

	if(stub_glMap2f){(*stub_glMap2f)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8, lv9);}
	else { 
		if(!loaded_glMap2f) { 
			loaded_glMap2f = 1;
			init_lib(); 
			stub_glMap2f=(pstub_glMap2f)get_proc_address("glMap2f");
			if(stub_glMap2f){(*stub_glMap2f)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8, lv9);}
			else { 
				FailWith("Cannot load glMap2f"); 
			} 
		} else { 
			FailWith("glMap2f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glMap2f_byte(value * argv, int n)
{
	return glstub_glMap2f(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8], argv[9]);
}

/* glMapGrid1d */
typedef void APIENTRY (*pstub_glMapGrid1d)(GLint,GLdouble,GLdouble);
static pstub_glMapGrid1d stub_glMapGrid1d = NULL;
static int loaded_glMapGrid1d = 0;

value glstub_glMapGrid1d(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glMapGrid1d){(*stub_glMapGrid1d)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMapGrid1d) { 
			loaded_glMapGrid1d = 1;
			init_lib(); 
			stub_glMapGrid1d=(pstub_glMapGrid1d)get_proc_address("glMapGrid1d");
			if(stub_glMapGrid1d){(*stub_glMapGrid1d)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMapGrid1d"); 
			} 
		} else { 
			FailWith("glMapGrid1d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMapGrid1f */
typedef void APIENTRY (*pstub_glMapGrid1f)(GLint,GLfloat,GLfloat);
static pstub_glMapGrid1f stub_glMapGrid1f = NULL;
static int loaded_glMapGrid1f = 0;

value glstub_glMapGrid1f(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glMapGrid1f){(*stub_glMapGrid1f)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMapGrid1f) { 
			loaded_glMapGrid1f = 1;
			init_lib(); 
			stub_glMapGrid1f=(pstub_glMapGrid1f)get_proc_address("glMapGrid1f");
			if(stub_glMapGrid1f){(*stub_glMapGrid1f)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMapGrid1f"); 
			} 
		} else { 
			FailWith("glMapGrid1f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMapGrid2d */
typedef void APIENTRY (*pstub_glMapGrid2d)(GLint,GLdouble,GLdouble,GLint,GLdouble,GLdouble);
static pstub_glMapGrid2d stub_glMapGrid2d = NULL;
static int loaded_glMapGrid2d = 0;

value glstub_glMapGrid2d(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLint lv3 =  Int_val(v3);
	GLdouble lv4 =  Double_val(v4);
	GLdouble lv5 =  Double_val(v5);

	if(stub_glMapGrid2d){(*stub_glMapGrid2d)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glMapGrid2d) { 
			loaded_glMapGrid2d = 1;
			init_lib(); 
			stub_glMapGrid2d=(pstub_glMapGrid2d)get_proc_address("glMapGrid2d");
			if(stub_glMapGrid2d){(*stub_glMapGrid2d)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glMapGrid2d"); 
			} 
		} else { 
			FailWith("glMapGrid2d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glMapGrid2d_byte(value * argv, int n)
{
	return glstub_glMapGrid2d(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glMapGrid2f */
typedef void APIENTRY (*pstub_glMapGrid2f)(GLint,GLfloat,GLfloat,GLint,GLfloat,GLfloat);
static pstub_glMapGrid2f stub_glMapGrid2f = NULL;
static int loaded_glMapGrid2f = 0;

value glstub_glMapGrid2f(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLint lv3 =  Int_val(v3);
	GLfloat lv4 =  Double_val(v4);
	GLfloat lv5 =  Double_val(v5);

	if(stub_glMapGrid2f){(*stub_glMapGrid2f)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glMapGrid2f) { 
			loaded_glMapGrid2f = 1;
			init_lib(); 
			stub_glMapGrid2f=(pstub_glMapGrid2f)get_proc_address("glMapGrid2f");
			if(stub_glMapGrid2f){(*stub_glMapGrid2f)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glMapGrid2f"); 
			} 
		} else { 
			FailWith("glMapGrid2f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glMapGrid2f_byte(value * argv, int n)
{
	return glstub_glMapGrid2f(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glMaterialf */
typedef void APIENTRY (*pstub_glMaterialf)(GLenum,GLenum,GLfloat);
static pstub_glMaterialf stub_glMaterialf = NULL;
static int loaded_glMaterialf = 0;

value glstub_glMaterialf(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glMaterialf){(*stub_glMaterialf)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMaterialf) { 
			loaded_glMaterialf = 1;
			init_lib(); 
			stub_glMaterialf=(pstub_glMaterialf)get_proc_address("glMaterialf");
			if(stub_glMaterialf){(*stub_glMaterialf)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMaterialf"); 
			} 
		} else { 
			FailWith("glMaterialf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMaterialfv */
typedef void APIENTRY (*pstub_glMaterialfv)(GLenum,GLenum,GLfloat*);
static pstub_glMaterialfv stub_glMaterialfv = NULL;
static int loaded_glMaterialfv = 0;

value glstub_glMaterialfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glMaterialfv){(*stub_glMaterialfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMaterialfv) { 
			loaded_glMaterialfv = 1;
			init_lib(); 
			stub_glMaterialfv=(pstub_glMaterialfv)get_proc_address("glMaterialfv");
			if(stub_glMaterialfv){(*stub_glMaterialfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMaterialfv"); 
			} 
		} else { 
			FailWith("glMaterialfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMateriali */
typedef void APIENTRY (*pstub_glMateriali)(GLenum,GLenum,GLint);
static pstub_glMateriali stub_glMateriali = NULL;
static int loaded_glMateriali = 0;

value glstub_glMateriali(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glMateriali){(*stub_glMateriali)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMateriali) { 
			loaded_glMateriali = 1;
			init_lib(); 
			stub_glMateriali=(pstub_glMateriali)get_proc_address("glMateriali");
			if(stub_glMateriali){(*stub_glMateriali)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMateriali"); 
			} 
		} else { 
			FailWith("glMateriali not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMaterialiv */
typedef void APIENTRY (*pstub_glMaterialiv)(GLenum,GLenum,GLint*);
static pstub_glMaterialiv stub_glMaterialiv = NULL;
static int loaded_glMaterialiv = 0;

value glstub_glMaterialiv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glMaterialiv){(*stub_glMaterialiv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMaterialiv) { 
			loaded_glMaterialiv = 1;
			init_lib(); 
			stub_glMaterialiv=(pstub_glMaterialiv)get_proc_address("glMaterialiv");
			if(stub_glMaterialiv){(*stub_glMaterialiv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMaterialiv"); 
			} 
		} else { 
			FailWith("glMaterialiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMatrixMode */
typedef void APIENTRY (*pstub_glMatrixMode)(GLenum);
static pstub_glMatrixMode stub_glMatrixMode = NULL;
static int loaded_glMatrixMode = 0;

value glstub_glMatrixMode(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glMatrixMode){(*stub_glMatrixMode)(lv0);}
	else { 
		if(!loaded_glMatrixMode) { 
			loaded_glMatrixMode = 1;
			init_lib(); 
			stub_glMatrixMode=(pstub_glMatrixMode)get_proc_address("glMatrixMode");
			if(stub_glMatrixMode){(*stub_glMatrixMode)(lv0);}
			else { 
				FailWith("Cannot load glMatrixMode"); 
			} 
		} else { 
			FailWith("glMatrixMode not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultMatrixd */
typedef void APIENTRY (*pstub_glMultMatrixd)(GLdouble*);
static pstub_glMultMatrixd stub_glMultMatrixd = NULL;
static int loaded_glMultMatrixd = 0;

value glstub_glMultMatrixd(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glMultMatrixd){(*stub_glMultMatrixd)(lv0);}
	else { 
		if(!loaded_glMultMatrixd) { 
			loaded_glMultMatrixd = 1;
			init_lib(); 
			stub_glMultMatrixd=(pstub_glMultMatrixd)get_proc_address("glMultMatrixd");
			if(stub_glMultMatrixd){(*stub_glMultMatrixd)(lv0);}
			else { 
				FailWith("Cannot load glMultMatrixd"); 
			} 
		} else { 
			FailWith("glMultMatrixd not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultMatrixf */
typedef void APIENTRY (*pstub_glMultMatrixf)(GLfloat*);
static pstub_glMultMatrixf stub_glMultMatrixf = NULL;
static int loaded_glMultMatrixf = 0;

value glstub_glMultMatrixf(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glMultMatrixf){(*stub_glMultMatrixf)(lv0);}
	else { 
		if(!loaded_glMultMatrixf) { 
			loaded_glMultMatrixf = 1;
			init_lib(); 
			stub_glMultMatrixf=(pstub_glMultMatrixf)get_proc_address("glMultMatrixf");
			if(stub_glMultMatrixf){(*stub_glMultMatrixf)(lv0);}
			else { 
				FailWith("Cannot load glMultMatrixf"); 
			} 
		} else { 
			FailWith("glMultMatrixf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNewList */
typedef void APIENTRY (*pstub_glNewList)(GLuint,GLenum);
static pstub_glNewList stub_glNewList = NULL;
static int loaded_glNewList = 0;

value glstub_glNewList(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);

	if(stub_glNewList){(*stub_glNewList)(lv0, lv1);}
	else { 
		if(!loaded_glNewList) { 
			loaded_glNewList = 1;
			init_lib(); 
			stub_glNewList=(pstub_glNewList)get_proc_address("glNewList");
			if(stub_glNewList){(*stub_glNewList)(lv0, lv1);}
			else { 
				FailWith("Cannot load glNewList"); 
			} 
		} else { 
			FailWith("glNewList not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormal3b */
typedef void APIENTRY (*pstub_glNormal3b)(GLbyte,GLbyte,GLbyte);
static pstub_glNormal3b stub_glNormal3b = NULL;
static int loaded_glNormal3b = 0;

value glstub_glNormal3b(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLbyte lv0 =  Int_val(v0);
	GLbyte lv1 =  Int_val(v1);
	GLbyte lv2 =  Int_val(v2);

	if(stub_glNormal3b){(*stub_glNormal3b)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glNormal3b) { 
			loaded_glNormal3b = 1;
			init_lib(); 
			stub_glNormal3b=(pstub_glNormal3b)get_proc_address("glNormal3b");
			if(stub_glNormal3b){(*stub_glNormal3b)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glNormal3b"); 
			} 
		} else { 
			FailWith("glNormal3b not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormal3bv */
typedef void APIENTRY (*pstub_glNormal3bv)(GLbyte*);
static pstub_glNormal3bv stub_glNormal3bv = NULL;
static int loaded_glNormal3bv = 0;

value glstub_glNormal3bv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLbyte* lv0 =  Data_bigarray_val(v0);

	if(stub_glNormal3bv){(*stub_glNormal3bv)(lv0);}
	else { 
		if(!loaded_glNormal3bv) { 
			loaded_glNormal3bv = 1;
			init_lib(); 
			stub_glNormal3bv=(pstub_glNormal3bv)get_proc_address("glNormal3bv");
			if(stub_glNormal3bv){(*stub_glNormal3bv)(lv0);}
			else { 
				FailWith("Cannot load glNormal3bv"); 
			} 
		} else { 
			FailWith("glNormal3bv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormal3d */
typedef void APIENTRY (*pstub_glNormal3d)(GLdouble,GLdouble,GLdouble);
static pstub_glNormal3d stub_glNormal3d = NULL;
static int loaded_glNormal3d = 0;

value glstub_glNormal3d(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glNormal3d){(*stub_glNormal3d)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glNormal3d) { 
			loaded_glNormal3d = 1;
			init_lib(); 
			stub_glNormal3d=(pstub_glNormal3d)get_proc_address("glNormal3d");
			if(stub_glNormal3d){(*stub_glNormal3d)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glNormal3d"); 
			} 
		} else { 
			FailWith("glNormal3d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormal3dv */
typedef void APIENTRY (*pstub_glNormal3dv)(GLdouble*);
static pstub_glNormal3dv stub_glNormal3dv = NULL;
static int loaded_glNormal3dv = 0;

value glstub_glNormal3dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glNormal3dv){(*stub_glNormal3dv)(lv0);}
	else { 
		if(!loaded_glNormal3dv) { 
			loaded_glNormal3dv = 1;
			init_lib(); 
			stub_glNormal3dv=(pstub_glNormal3dv)get_proc_address("glNormal3dv");
			if(stub_glNormal3dv){(*stub_glNormal3dv)(lv0);}
			else { 
				FailWith("Cannot load glNormal3dv"); 
			} 
		} else { 
			FailWith("glNormal3dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormal3f */
typedef void APIENTRY (*pstub_glNormal3f)(GLfloat,GLfloat,GLfloat);
static pstub_glNormal3f stub_glNormal3f = NULL;
static int loaded_glNormal3f = 0;

value glstub_glNormal3f(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glNormal3f){(*stub_glNormal3f)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glNormal3f) { 
			loaded_glNormal3f = 1;
			init_lib(); 
			stub_glNormal3f=(pstub_glNormal3f)get_proc_address("glNormal3f");
			if(stub_glNormal3f){(*stub_glNormal3f)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glNormal3f"); 
			} 
		} else { 
			FailWith("glNormal3f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormal3fv */
typedef void APIENTRY (*pstub_glNormal3fv)(GLfloat*);
static pstub_glNormal3fv stub_glNormal3fv = NULL;
static int loaded_glNormal3fv = 0;

value glstub_glNormal3fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glNormal3fv){(*stub_glNormal3fv)(lv0);}
	else { 
		if(!loaded_glNormal3fv) { 
			loaded_glNormal3fv = 1;
			init_lib(); 
			stub_glNormal3fv=(pstub_glNormal3fv)get_proc_address("glNormal3fv");
			if(stub_glNormal3fv){(*stub_glNormal3fv)(lv0);}
			else { 
				FailWith("Cannot load glNormal3fv"); 
			} 
		} else { 
			FailWith("glNormal3fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormal3i */
typedef void APIENTRY (*pstub_glNormal3i)(GLint,GLint,GLint);
static pstub_glNormal3i stub_glNormal3i = NULL;
static int loaded_glNormal3i = 0;

value glstub_glNormal3i(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glNormal3i){(*stub_glNormal3i)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glNormal3i) { 
			loaded_glNormal3i = 1;
			init_lib(); 
			stub_glNormal3i=(pstub_glNormal3i)get_proc_address("glNormal3i");
			if(stub_glNormal3i){(*stub_glNormal3i)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glNormal3i"); 
			} 
		} else { 
			FailWith("glNormal3i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormal3iv */
typedef void APIENTRY (*pstub_glNormal3iv)(GLint*);
static pstub_glNormal3iv stub_glNormal3iv = NULL;
static int loaded_glNormal3iv = 0;

value glstub_glNormal3iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glNormal3iv){(*stub_glNormal3iv)(lv0);}
	else { 
		if(!loaded_glNormal3iv) { 
			loaded_glNormal3iv = 1;
			init_lib(); 
			stub_glNormal3iv=(pstub_glNormal3iv)get_proc_address("glNormal3iv");
			if(stub_glNormal3iv){(*stub_glNormal3iv)(lv0);}
			else { 
				FailWith("Cannot load glNormal3iv"); 
			} 
		} else { 
			FailWith("glNormal3iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormal3s */
typedef void APIENTRY (*pstub_glNormal3s)(GLshort,GLshort,GLshort);
static pstub_glNormal3s stub_glNormal3s = NULL;
static int loaded_glNormal3s = 0;

value glstub_glNormal3s(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);

	if(stub_glNormal3s){(*stub_glNormal3s)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glNormal3s) { 
			loaded_glNormal3s = 1;
			init_lib(); 
			stub_glNormal3s=(pstub_glNormal3s)get_proc_address("glNormal3s");
			if(stub_glNormal3s){(*stub_glNormal3s)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glNormal3s"); 
			} 
		} else { 
			FailWith("glNormal3s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormal3sv */
typedef void APIENTRY (*pstub_glNormal3sv)(GLshort*);
static pstub_glNormal3sv stub_glNormal3sv = NULL;
static int loaded_glNormal3sv = 0;

value glstub_glNormal3sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glNormal3sv){(*stub_glNormal3sv)(lv0);}
	else { 
		if(!loaded_glNormal3sv) { 
			loaded_glNormal3sv = 1;
			init_lib(); 
			stub_glNormal3sv=(pstub_glNormal3sv)get_proc_address("glNormal3sv");
			if(stub_glNormal3sv){(*stub_glNormal3sv)(lv0);}
			else { 
				FailWith("Cannot load glNormal3sv"); 
			} 
		} else { 
			FailWith("glNormal3sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glNormalPointer */
typedef void APIENTRY (*pstub_glNormalPointer)(GLenum,GLsizei,GLvoid*);
static pstub_glNormalPointer stub_glNormalPointer = NULL;
static int loaded_glNormalPointer = 0;

value glstub_glNormalPointer(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizei lv1 =  Int_val(v1);
	GLvoid* lv2 =  Data_bigarray_val(v2);

	if(stub_glNormalPointer){(*stub_glNormalPointer)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glNormalPointer) { 
			loaded_glNormalPointer = 1;
			init_lib(); 
			stub_glNormalPointer=(pstub_glNormalPointer)get_proc_address("glNormalPointer");
			if(stub_glNormalPointer){(*stub_glNormalPointer)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glNormalPointer"); 
			} 
		} else { 
			FailWith("glNormalPointer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glOrtho */
typedef void APIENTRY (*pstub_glOrtho)(GLdouble,GLdouble,GLdouble,GLdouble,GLdouble,GLdouble);
static pstub_glOrtho stub_glOrtho = NULL;
static int loaded_glOrtho = 0;

value glstub_glOrtho(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);
	GLdouble lv4 =  Double_val(v4);
	GLdouble lv5 =  Double_val(v5);

	if(stub_glOrtho){(*stub_glOrtho)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glOrtho) { 
			loaded_glOrtho = 1;
			init_lib(); 
			stub_glOrtho=(pstub_glOrtho)get_proc_address("glOrtho");
			if(stub_glOrtho){(*stub_glOrtho)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glOrtho"); 
			} 
		} else { 
			FailWith("glOrtho not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glOrtho_byte(value * argv, int n)
{
	return glstub_glOrtho(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glPassThrough */
typedef void APIENTRY (*pstub_glPassThrough)(GLfloat);
static pstub_glPassThrough stub_glPassThrough = NULL;
static int loaded_glPassThrough = 0;

value glstub_glPassThrough(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);

	if(stub_glPassThrough){(*stub_glPassThrough)(lv0);}
	else { 
		if(!loaded_glPassThrough) { 
			loaded_glPassThrough = 1;
			init_lib(); 
			stub_glPassThrough=(pstub_glPassThrough)get_proc_address("glPassThrough");
			if(stub_glPassThrough){(*stub_glPassThrough)(lv0);}
			else { 
				FailWith("Cannot load glPassThrough"); 
			} 
		} else { 
			FailWith("glPassThrough not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPixelMapfv */
typedef void APIENTRY (*pstub_glPixelMapfv)(GLenum,GLsizei,GLfloat*);
static pstub_glPixelMapfv stub_glPixelMapfv = NULL;
static int loaded_glPixelMapfv = 0;

value glstub_glPixelMapfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizei lv1 =  Int_val(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glPixelMapfv){(*stub_glPixelMapfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glPixelMapfv) { 
			loaded_glPixelMapfv = 1;
			init_lib(); 
			stub_glPixelMapfv=(pstub_glPixelMapfv)get_proc_address("glPixelMapfv");
			if(stub_glPixelMapfv){(*stub_glPixelMapfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glPixelMapfv"); 
			} 
		} else { 
			FailWith("glPixelMapfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPixelMapuiv */
typedef void APIENTRY (*pstub_glPixelMapuiv)(GLenum,GLsizei,GLuint*);
static pstub_glPixelMapuiv stub_glPixelMapuiv = NULL;
static int loaded_glPixelMapuiv = 0;

value glstub_glPixelMapuiv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizei lv1 =  Int_val(v1);
	GLuint* lv2 =  Data_bigarray_val(v2);

	if(stub_glPixelMapuiv){(*stub_glPixelMapuiv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glPixelMapuiv) { 
			loaded_glPixelMapuiv = 1;
			init_lib(); 
			stub_glPixelMapuiv=(pstub_glPixelMapuiv)get_proc_address("glPixelMapuiv");
			if(stub_glPixelMapuiv){(*stub_glPixelMapuiv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glPixelMapuiv"); 
			} 
		} else { 
			FailWith("glPixelMapuiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPixelMapusv */
typedef void APIENTRY (*pstub_glPixelMapusv)(GLenum,GLsizei,GLushort*);
static pstub_glPixelMapusv stub_glPixelMapusv = NULL;
static int loaded_glPixelMapusv = 0;

value glstub_glPixelMapusv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizei lv1 =  Int_val(v1);
	GLushort* lv2 =  Data_bigarray_val(v2);

	if(stub_glPixelMapusv){(*stub_glPixelMapusv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glPixelMapusv) { 
			loaded_glPixelMapusv = 1;
			init_lib(); 
			stub_glPixelMapusv=(pstub_glPixelMapusv)get_proc_address("glPixelMapusv");
			if(stub_glPixelMapusv){(*stub_glPixelMapusv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glPixelMapusv"); 
			} 
		} else { 
			FailWith("glPixelMapusv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPixelStoref */
typedef void APIENTRY (*pstub_glPixelStoref)(GLenum,GLfloat);
static pstub_glPixelStoref stub_glPixelStoref = NULL;
static int loaded_glPixelStoref = 0;

value glstub_glPixelStoref(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glPixelStoref){(*stub_glPixelStoref)(lv0, lv1);}
	else { 
		if(!loaded_glPixelStoref) { 
			loaded_glPixelStoref = 1;
			init_lib(); 
			stub_glPixelStoref=(pstub_glPixelStoref)get_proc_address("glPixelStoref");
			if(stub_glPixelStoref){(*stub_glPixelStoref)(lv0, lv1);}
			else { 
				FailWith("Cannot load glPixelStoref"); 
			} 
		} else { 
			FailWith("glPixelStoref not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPixelStorei */
typedef void APIENTRY (*pstub_glPixelStorei)(GLenum,GLint);
static pstub_glPixelStorei stub_glPixelStorei = NULL;
static int loaded_glPixelStorei = 0;

value glstub_glPixelStorei(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glPixelStorei){(*stub_glPixelStorei)(lv0, lv1);}
	else { 
		if(!loaded_glPixelStorei) { 
			loaded_glPixelStorei = 1;
			init_lib(); 
			stub_glPixelStorei=(pstub_glPixelStorei)get_proc_address("glPixelStorei");
			if(stub_glPixelStorei){(*stub_glPixelStorei)(lv0, lv1);}
			else { 
				FailWith("Cannot load glPixelStorei"); 
			} 
		} else { 
			FailWith("glPixelStorei not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPixelTransferf */
typedef void APIENTRY (*pstub_glPixelTransferf)(GLenum,GLfloat);
static pstub_glPixelTransferf stub_glPixelTransferf = NULL;
static int loaded_glPixelTransferf = 0;

value glstub_glPixelTransferf(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glPixelTransferf){(*stub_glPixelTransferf)(lv0, lv1);}
	else { 
		if(!loaded_glPixelTransferf) { 
			loaded_glPixelTransferf = 1;
			init_lib(); 
			stub_glPixelTransferf=(pstub_glPixelTransferf)get_proc_address("glPixelTransferf");
			if(stub_glPixelTransferf){(*stub_glPixelTransferf)(lv0, lv1);}
			else { 
				FailWith("Cannot load glPixelTransferf"); 
			} 
		} else { 
			FailWith("glPixelTransferf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPixelTransferi */
typedef void APIENTRY (*pstub_glPixelTransferi)(GLenum,GLint);
static pstub_glPixelTransferi stub_glPixelTransferi = NULL;
static int loaded_glPixelTransferi = 0;

value glstub_glPixelTransferi(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glPixelTransferi){(*stub_glPixelTransferi)(lv0, lv1);}
	else { 
		if(!loaded_glPixelTransferi) { 
			loaded_glPixelTransferi = 1;
			init_lib(); 
			stub_glPixelTransferi=(pstub_glPixelTransferi)get_proc_address("glPixelTransferi");
			if(stub_glPixelTransferi){(*stub_glPixelTransferi)(lv0, lv1);}
			else { 
				FailWith("Cannot load glPixelTransferi"); 
			} 
		} else { 
			FailWith("glPixelTransferi not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPixelZoom */
typedef void APIENTRY (*pstub_glPixelZoom)(GLfloat,GLfloat);
static pstub_glPixelZoom stub_glPixelZoom = NULL;
static int loaded_glPixelZoom = 0;

value glstub_glPixelZoom(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glPixelZoom){(*stub_glPixelZoom)(lv0, lv1);}
	else { 
		if(!loaded_glPixelZoom) { 
			loaded_glPixelZoom = 1;
			init_lib(); 
			stub_glPixelZoom=(pstub_glPixelZoom)get_proc_address("glPixelZoom");
			if(stub_glPixelZoom){(*stub_glPixelZoom)(lv0, lv1);}
			else { 
				FailWith("Cannot load glPixelZoom"); 
			} 
		} else { 
			FailWith("glPixelZoom not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPointSize */
typedef void APIENTRY (*pstub_glPointSize)(GLfloat);
static pstub_glPointSize stub_glPointSize = NULL;
static int loaded_glPointSize = 0;

value glstub_glPointSize(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);

	if(stub_glPointSize){(*stub_glPointSize)(lv0);}
	else { 
		if(!loaded_glPointSize) { 
			loaded_glPointSize = 1;
			init_lib(); 
			stub_glPointSize=(pstub_glPointSize)get_proc_address("glPointSize");
			if(stub_glPointSize){(*stub_glPointSize)(lv0);}
			else { 
				FailWith("Cannot load glPointSize"); 
			} 
		} else { 
			FailWith("glPointSize not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPolygonMode */
typedef void APIENTRY (*pstub_glPolygonMode)(GLenum,GLenum);
static pstub_glPolygonMode stub_glPolygonMode = NULL;
static int loaded_glPolygonMode = 0;

value glstub_glPolygonMode(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);

	if(stub_glPolygonMode){(*stub_glPolygonMode)(lv0, lv1);}
	else { 
		if(!loaded_glPolygonMode) { 
			loaded_glPolygonMode = 1;
			init_lib(); 
			stub_glPolygonMode=(pstub_glPolygonMode)get_proc_address("glPolygonMode");
			if(stub_glPolygonMode){(*stub_glPolygonMode)(lv0, lv1);}
			else { 
				FailWith("Cannot load glPolygonMode"); 
			} 
		} else { 
			FailWith("glPolygonMode not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPolygonOffset */
typedef void APIENTRY (*pstub_glPolygonOffset)(GLfloat,GLfloat);
static pstub_glPolygonOffset stub_glPolygonOffset = NULL;
static int loaded_glPolygonOffset = 0;

value glstub_glPolygonOffset(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glPolygonOffset){(*stub_glPolygonOffset)(lv0, lv1);}
	else { 
		if(!loaded_glPolygonOffset) { 
			loaded_glPolygonOffset = 1;
			init_lib(); 
			stub_glPolygonOffset=(pstub_glPolygonOffset)get_proc_address("glPolygonOffset");
			if(stub_glPolygonOffset){(*stub_glPolygonOffset)(lv0, lv1);}
			else { 
				FailWith("Cannot load glPolygonOffset"); 
			} 
		} else { 
			FailWith("glPolygonOffset not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPolygonStipple */
typedef void APIENTRY (*pstub_glPolygonStipple)(GLubyte*);
static pstub_glPolygonStipple stub_glPolygonStipple = NULL;
static int loaded_glPolygonStipple = 0;

value glstub_glPolygonStipple(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLubyte* lv0 =  Data_bigarray_val(v0);

	if(stub_glPolygonStipple){(*stub_glPolygonStipple)(lv0);}
	else { 
		if(!loaded_glPolygonStipple) { 
			loaded_glPolygonStipple = 1;
			init_lib(); 
			stub_glPolygonStipple=(pstub_glPolygonStipple)get_proc_address("glPolygonStipple");
			if(stub_glPolygonStipple){(*stub_glPolygonStipple)(lv0);}
			else { 
				FailWith("Cannot load glPolygonStipple"); 
			} 
		} else { 
			FailWith("glPolygonStipple not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPopAttrib */
typedef void APIENTRY (*pstub_glPopAttrib)(GLvoid);
static pstub_glPopAttrib stub_glPopAttrib = NULL;
static int loaded_glPopAttrib = 0;

value glstub_glPopAttrib(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);

	if(stub_glPopAttrib){(*stub_glPopAttrib)();}
	else { 
		if(!loaded_glPopAttrib) { 
			loaded_glPopAttrib = 1;
			init_lib(); 
			stub_glPopAttrib=(pstub_glPopAttrib)get_proc_address("glPopAttrib");
			if(stub_glPopAttrib){(*stub_glPopAttrib)();}
			else { 
				FailWith("Cannot load glPopAttrib"); 
			} 
		} else { 
			FailWith("glPopAttrib not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPopClientAttrib */
typedef void APIENTRY (*pstub_glPopClientAttrib)(GLvoid);
static pstub_glPopClientAttrib stub_glPopClientAttrib = NULL;
static int loaded_glPopClientAttrib = 0;

value glstub_glPopClientAttrib(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);

	if(stub_glPopClientAttrib){(*stub_glPopClientAttrib)();}
	else { 
		if(!loaded_glPopClientAttrib) { 
			loaded_glPopClientAttrib = 1;
			init_lib(); 
			stub_glPopClientAttrib=(pstub_glPopClientAttrib)get_proc_address("glPopClientAttrib");
			if(stub_glPopClientAttrib){(*stub_glPopClientAttrib)();}
			else { 
				FailWith("Cannot load glPopClientAttrib"); 
			} 
		} else { 
			FailWith("glPopClientAttrib not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPopMatrix */
typedef void APIENTRY (*pstub_glPopMatrix)(GLvoid);
static pstub_glPopMatrix stub_glPopMatrix = NULL;
static int loaded_glPopMatrix = 0;

value glstub_glPopMatrix(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);

	if(stub_glPopMatrix){(*stub_glPopMatrix)();}
	else { 
		if(!loaded_glPopMatrix) { 
			loaded_glPopMatrix = 1;
			init_lib(); 
			stub_glPopMatrix=(pstub_glPopMatrix)get_proc_address("glPopMatrix");
			if(stub_glPopMatrix){(*stub_glPopMatrix)();}
			else { 
				FailWith("Cannot load glPopMatrix"); 
			} 
		} else { 
			FailWith("glPopMatrix not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPopName */
typedef void APIENTRY (*pstub_glPopName)(GLvoid);
static pstub_glPopName stub_glPopName = NULL;
static int loaded_glPopName = 0;

value glstub_glPopName(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);

	if(stub_glPopName){(*stub_glPopName)();}
	else { 
		if(!loaded_glPopName) { 
			loaded_glPopName = 1;
			init_lib(); 
			stub_glPopName=(pstub_glPopName)get_proc_address("glPopName");
			if(stub_glPopName){(*stub_glPopName)();}
			else { 
				FailWith("Cannot load glPopName"); 
			} 
		} else { 
			FailWith("glPopName not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPrioritizeTextures */
typedef void APIENTRY (*pstub_glPrioritizeTextures)(GLsizei,GLuint*,GLclampf*);
static pstub_glPrioritizeTextures stub_glPrioritizeTextures = NULL;
static int loaded_glPrioritizeTextures = 0;

value glstub_glPrioritizeTextures(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);
	GLclampf* lv2 =  Data_bigarray_val(v2);

	if(stub_glPrioritizeTextures){(*stub_glPrioritizeTextures)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glPrioritizeTextures) { 
			loaded_glPrioritizeTextures = 1;
			init_lib(); 
			stub_glPrioritizeTextures=(pstub_glPrioritizeTextures)get_proc_address("glPrioritizeTextures");
			if(stub_glPrioritizeTextures){(*stub_glPrioritizeTextures)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glPrioritizeTextures"); 
			} 
		} else { 
			FailWith("glPrioritizeTextures not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPushAttrib */
typedef void APIENTRY (*pstub_glPushAttrib)(GLbitfield);
static pstub_glPushAttrib stub_glPushAttrib = NULL;
static int loaded_glPushAttrib = 0;

value glstub_glPushAttrib(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLbitfield lv0 =  get_glenum_list(v0);

	if(stub_glPushAttrib){(*stub_glPushAttrib)(lv0);}
	else { 
		if(!loaded_glPushAttrib) { 
			loaded_glPushAttrib = 1;
			init_lib(); 
			stub_glPushAttrib=(pstub_glPushAttrib)get_proc_address("glPushAttrib");
			if(stub_glPushAttrib){(*stub_glPushAttrib)(lv0);}
			else { 
				FailWith("Cannot load glPushAttrib"); 
			} 
		} else { 
			FailWith("glPushAttrib not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPushClientAttrib */
typedef void APIENTRY (*pstub_glPushClientAttrib)(GLbitfield);
static pstub_glPushClientAttrib stub_glPushClientAttrib = NULL;
static int loaded_glPushClientAttrib = 0;

value glstub_glPushClientAttrib(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLbitfield lv0 =  get_glenum_list(v0);

	if(stub_glPushClientAttrib){(*stub_glPushClientAttrib)(lv0);}
	else { 
		if(!loaded_glPushClientAttrib) { 
			loaded_glPushClientAttrib = 1;
			init_lib(); 
			stub_glPushClientAttrib=(pstub_glPushClientAttrib)get_proc_address("glPushClientAttrib");
			if(stub_glPushClientAttrib){(*stub_glPushClientAttrib)(lv0);}
			else { 
				FailWith("Cannot load glPushClientAttrib"); 
			} 
		} else { 
			FailWith("glPushClientAttrib not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPushMatrix */
typedef void APIENTRY (*pstub_glPushMatrix)(GLvoid);
static pstub_glPushMatrix stub_glPushMatrix = NULL;
static int loaded_glPushMatrix = 0;

value glstub_glPushMatrix(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);

	if(stub_glPushMatrix){(*stub_glPushMatrix)();}
	else { 
		if(!loaded_glPushMatrix) { 
			loaded_glPushMatrix = 1;
			init_lib(); 
			stub_glPushMatrix=(pstub_glPushMatrix)get_proc_address("glPushMatrix");
			if(stub_glPushMatrix){(*stub_glPushMatrix)();}
			else { 
				FailWith("Cannot load glPushMatrix"); 
			} 
		} else { 
			FailWith("glPushMatrix not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPushName */
typedef void APIENTRY (*pstub_glPushName)(GLuint);
static pstub_glPushName stub_glPushName = NULL;
static int loaded_glPushName = 0;

value glstub_glPushName(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glPushName){(*stub_glPushName)(lv0);}
	else { 
		if(!loaded_glPushName) { 
			loaded_glPushName = 1;
			init_lib(); 
			stub_glPushName=(pstub_glPushName)get_proc_address("glPushName");
			if(stub_glPushName){(*stub_glPushName)(lv0);}
			else { 
				FailWith("Cannot load glPushName"); 
			} 
		} else { 
			FailWith("glPushName not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos2d */
typedef void APIENTRY (*pstub_glRasterPos2d)(GLdouble,GLdouble);
static pstub_glRasterPos2d stub_glRasterPos2d = NULL;
static int loaded_glRasterPos2d = 0;

value glstub_glRasterPos2d(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);

	if(stub_glRasterPos2d){(*stub_glRasterPos2d)(lv0, lv1);}
	else { 
		if(!loaded_glRasterPos2d) { 
			loaded_glRasterPos2d = 1;
			init_lib(); 
			stub_glRasterPos2d=(pstub_glRasterPos2d)get_proc_address("glRasterPos2d");
			if(stub_glRasterPos2d){(*stub_glRasterPos2d)(lv0, lv1);}
			else { 
				FailWith("Cannot load glRasterPos2d"); 
			} 
		} else { 
			FailWith("glRasterPos2d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos2dv */
typedef void APIENTRY (*pstub_glRasterPos2dv)(GLdouble*);
static pstub_glRasterPos2dv stub_glRasterPos2dv = NULL;
static int loaded_glRasterPos2dv = 0;

value glstub_glRasterPos2dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glRasterPos2dv){(*stub_glRasterPos2dv)(lv0);}
	else { 
		if(!loaded_glRasterPos2dv) { 
			loaded_glRasterPos2dv = 1;
			init_lib(); 
			stub_glRasterPos2dv=(pstub_glRasterPos2dv)get_proc_address("glRasterPos2dv");
			if(stub_glRasterPos2dv){(*stub_glRasterPos2dv)(lv0);}
			else { 
				FailWith("Cannot load glRasterPos2dv"); 
			} 
		} else { 
			FailWith("glRasterPos2dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos2f */
typedef void APIENTRY (*pstub_glRasterPos2f)(GLfloat,GLfloat);
static pstub_glRasterPos2f stub_glRasterPos2f = NULL;
static int loaded_glRasterPos2f = 0;

value glstub_glRasterPos2f(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glRasterPos2f){(*stub_glRasterPos2f)(lv0, lv1);}
	else { 
		if(!loaded_glRasterPos2f) { 
			loaded_glRasterPos2f = 1;
			init_lib(); 
			stub_glRasterPos2f=(pstub_glRasterPos2f)get_proc_address("glRasterPos2f");
			if(stub_glRasterPos2f){(*stub_glRasterPos2f)(lv0, lv1);}
			else { 
				FailWith("Cannot load glRasterPos2f"); 
			} 
		} else { 
			FailWith("glRasterPos2f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos2fv */
typedef void APIENTRY (*pstub_glRasterPos2fv)(GLfloat*);
static pstub_glRasterPos2fv stub_glRasterPos2fv = NULL;
static int loaded_glRasterPos2fv = 0;

value glstub_glRasterPos2fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glRasterPos2fv){(*stub_glRasterPos2fv)(lv0);}
	else { 
		if(!loaded_glRasterPos2fv) { 
			loaded_glRasterPos2fv = 1;
			init_lib(); 
			stub_glRasterPos2fv=(pstub_glRasterPos2fv)get_proc_address("glRasterPos2fv");
			if(stub_glRasterPos2fv){(*stub_glRasterPos2fv)(lv0);}
			else { 
				FailWith("Cannot load glRasterPos2fv"); 
			} 
		} else { 
			FailWith("glRasterPos2fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos2i */
typedef void APIENTRY (*pstub_glRasterPos2i)(GLint,GLint);
static pstub_glRasterPos2i stub_glRasterPos2i = NULL;
static int loaded_glRasterPos2i = 0;

value glstub_glRasterPos2i(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glRasterPos2i){(*stub_glRasterPos2i)(lv0, lv1);}
	else { 
		if(!loaded_glRasterPos2i) { 
			loaded_glRasterPos2i = 1;
			init_lib(); 
			stub_glRasterPos2i=(pstub_glRasterPos2i)get_proc_address("glRasterPos2i");
			if(stub_glRasterPos2i){(*stub_glRasterPos2i)(lv0, lv1);}
			else { 
				FailWith("Cannot load glRasterPos2i"); 
			} 
		} else { 
			FailWith("glRasterPos2i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos2iv */
typedef void APIENTRY (*pstub_glRasterPos2iv)(GLint*);
static pstub_glRasterPos2iv stub_glRasterPos2iv = NULL;
static int loaded_glRasterPos2iv = 0;

value glstub_glRasterPos2iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glRasterPos2iv){(*stub_glRasterPos2iv)(lv0);}
	else { 
		if(!loaded_glRasterPos2iv) { 
			loaded_glRasterPos2iv = 1;
			init_lib(); 
			stub_glRasterPos2iv=(pstub_glRasterPos2iv)get_proc_address("glRasterPos2iv");
			if(stub_glRasterPos2iv){(*stub_glRasterPos2iv)(lv0);}
			else { 
				FailWith("Cannot load glRasterPos2iv"); 
			} 
		} else { 
			FailWith("glRasterPos2iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos2s */
typedef void APIENTRY (*pstub_glRasterPos2s)(GLshort,GLshort);
static pstub_glRasterPos2s stub_glRasterPos2s = NULL;
static int loaded_glRasterPos2s = 0;

value glstub_glRasterPos2s(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);

	if(stub_glRasterPos2s){(*stub_glRasterPos2s)(lv0, lv1);}
	else { 
		if(!loaded_glRasterPos2s) { 
			loaded_glRasterPos2s = 1;
			init_lib(); 
			stub_glRasterPos2s=(pstub_glRasterPos2s)get_proc_address("glRasterPos2s");
			if(stub_glRasterPos2s){(*stub_glRasterPos2s)(lv0, lv1);}
			else { 
				FailWith("Cannot load glRasterPos2s"); 
			} 
		} else { 
			FailWith("glRasterPos2s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos2sv */
typedef void APIENTRY (*pstub_glRasterPos2sv)(GLshort*);
static pstub_glRasterPos2sv stub_glRasterPos2sv = NULL;
static int loaded_glRasterPos2sv = 0;

value glstub_glRasterPos2sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glRasterPos2sv){(*stub_glRasterPos2sv)(lv0);}
	else { 
		if(!loaded_glRasterPos2sv) { 
			loaded_glRasterPos2sv = 1;
			init_lib(); 
			stub_glRasterPos2sv=(pstub_glRasterPos2sv)get_proc_address("glRasterPos2sv");
			if(stub_glRasterPos2sv){(*stub_glRasterPos2sv)(lv0);}
			else { 
				FailWith("Cannot load glRasterPos2sv"); 
			} 
		} else { 
			FailWith("glRasterPos2sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos3d */
typedef void APIENTRY (*pstub_glRasterPos3d)(GLdouble,GLdouble,GLdouble);
static pstub_glRasterPos3d stub_glRasterPos3d = NULL;
static int loaded_glRasterPos3d = 0;

value glstub_glRasterPos3d(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glRasterPos3d){(*stub_glRasterPos3d)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glRasterPos3d) { 
			loaded_glRasterPos3d = 1;
			init_lib(); 
			stub_glRasterPos3d=(pstub_glRasterPos3d)get_proc_address("glRasterPos3d");
			if(stub_glRasterPos3d){(*stub_glRasterPos3d)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glRasterPos3d"); 
			} 
		} else { 
			FailWith("glRasterPos3d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos3dv */
typedef void APIENTRY (*pstub_glRasterPos3dv)(GLdouble*);
static pstub_glRasterPos3dv stub_glRasterPos3dv = NULL;
static int loaded_glRasterPos3dv = 0;

value glstub_glRasterPos3dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glRasterPos3dv){(*stub_glRasterPos3dv)(lv0);}
	else { 
		if(!loaded_glRasterPos3dv) { 
			loaded_glRasterPos3dv = 1;
			init_lib(); 
			stub_glRasterPos3dv=(pstub_glRasterPos3dv)get_proc_address("glRasterPos3dv");
			if(stub_glRasterPos3dv){(*stub_glRasterPos3dv)(lv0);}
			else { 
				FailWith("Cannot load glRasterPos3dv"); 
			} 
		} else { 
			FailWith("glRasterPos3dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos3f */
typedef void APIENTRY (*pstub_glRasterPos3f)(GLfloat,GLfloat,GLfloat);
static pstub_glRasterPos3f stub_glRasterPos3f = NULL;
static int loaded_glRasterPos3f = 0;

value glstub_glRasterPos3f(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glRasterPos3f){(*stub_glRasterPos3f)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glRasterPos3f) { 
			loaded_glRasterPos3f = 1;
			init_lib(); 
			stub_glRasterPos3f=(pstub_glRasterPos3f)get_proc_address("glRasterPos3f");
			if(stub_glRasterPos3f){(*stub_glRasterPos3f)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glRasterPos3f"); 
			} 
		} else { 
			FailWith("glRasterPos3f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos3fv */
typedef void APIENTRY (*pstub_glRasterPos3fv)(GLfloat*);
static pstub_glRasterPos3fv stub_glRasterPos3fv = NULL;
static int loaded_glRasterPos3fv = 0;

value glstub_glRasterPos3fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glRasterPos3fv){(*stub_glRasterPos3fv)(lv0);}
	else { 
		if(!loaded_glRasterPos3fv) { 
			loaded_glRasterPos3fv = 1;
			init_lib(); 
			stub_glRasterPos3fv=(pstub_glRasterPos3fv)get_proc_address("glRasterPos3fv");
			if(stub_glRasterPos3fv){(*stub_glRasterPos3fv)(lv0);}
			else { 
				FailWith("Cannot load glRasterPos3fv"); 
			} 
		} else { 
			FailWith("glRasterPos3fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos3i */
typedef void APIENTRY (*pstub_glRasterPos3i)(GLint,GLint,GLint);
static pstub_glRasterPos3i stub_glRasterPos3i = NULL;
static int loaded_glRasterPos3i = 0;

value glstub_glRasterPos3i(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glRasterPos3i){(*stub_glRasterPos3i)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glRasterPos3i) { 
			loaded_glRasterPos3i = 1;
			init_lib(); 
			stub_glRasterPos3i=(pstub_glRasterPos3i)get_proc_address("glRasterPos3i");
			if(stub_glRasterPos3i){(*stub_glRasterPos3i)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glRasterPos3i"); 
			} 
		} else { 
			FailWith("glRasterPos3i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos3iv */
typedef void APIENTRY (*pstub_glRasterPos3iv)(GLint*);
static pstub_glRasterPos3iv stub_glRasterPos3iv = NULL;
static int loaded_glRasterPos3iv = 0;

value glstub_glRasterPos3iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glRasterPos3iv){(*stub_glRasterPos3iv)(lv0);}
	else { 
		if(!loaded_glRasterPos3iv) { 
			loaded_glRasterPos3iv = 1;
			init_lib(); 
			stub_glRasterPos3iv=(pstub_glRasterPos3iv)get_proc_address("glRasterPos3iv");
			if(stub_glRasterPos3iv){(*stub_glRasterPos3iv)(lv0);}
			else { 
				FailWith("Cannot load glRasterPos3iv"); 
			} 
		} else { 
			FailWith("glRasterPos3iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos3s */
typedef void APIENTRY (*pstub_glRasterPos3s)(GLshort,GLshort,GLshort);
static pstub_glRasterPos3s stub_glRasterPos3s = NULL;
static int loaded_glRasterPos3s = 0;

value glstub_glRasterPos3s(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);

	if(stub_glRasterPos3s){(*stub_glRasterPos3s)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glRasterPos3s) { 
			loaded_glRasterPos3s = 1;
			init_lib(); 
			stub_glRasterPos3s=(pstub_glRasterPos3s)get_proc_address("glRasterPos3s");
			if(stub_glRasterPos3s){(*stub_glRasterPos3s)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glRasterPos3s"); 
			} 
		} else { 
			FailWith("glRasterPos3s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos3sv */
typedef void APIENTRY (*pstub_glRasterPos3sv)(GLshort*);
static pstub_glRasterPos3sv stub_glRasterPos3sv = NULL;
static int loaded_glRasterPos3sv = 0;

value glstub_glRasterPos3sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glRasterPos3sv){(*stub_glRasterPos3sv)(lv0);}
	else { 
		if(!loaded_glRasterPos3sv) { 
			loaded_glRasterPos3sv = 1;
			init_lib(); 
			stub_glRasterPos3sv=(pstub_glRasterPos3sv)get_proc_address("glRasterPos3sv");
			if(stub_glRasterPos3sv){(*stub_glRasterPos3sv)(lv0);}
			else { 
				FailWith("Cannot load glRasterPos3sv"); 
			} 
		} else { 
			FailWith("glRasterPos3sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos4d */
typedef void APIENTRY (*pstub_glRasterPos4d)(GLdouble,GLdouble,GLdouble,GLdouble);
static pstub_glRasterPos4d stub_glRasterPos4d = NULL;
static int loaded_glRasterPos4d = 0;

value glstub_glRasterPos4d(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);

	if(stub_glRasterPos4d){(*stub_glRasterPos4d)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glRasterPos4d) { 
			loaded_glRasterPos4d = 1;
			init_lib(); 
			stub_glRasterPos4d=(pstub_glRasterPos4d)get_proc_address("glRasterPos4d");
			if(stub_glRasterPos4d){(*stub_glRasterPos4d)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glRasterPos4d"); 
			} 
		} else { 
			FailWith("glRasterPos4d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos4dv */
typedef void APIENTRY (*pstub_glRasterPos4dv)(GLdouble*);
static pstub_glRasterPos4dv stub_glRasterPos4dv = NULL;
static int loaded_glRasterPos4dv = 0;

value glstub_glRasterPos4dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glRasterPos4dv){(*stub_glRasterPos4dv)(lv0);}
	else { 
		if(!loaded_glRasterPos4dv) { 
			loaded_glRasterPos4dv = 1;
			init_lib(); 
			stub_glRasterPos4dv=(pstub_glRasterPos4dv)get_proc_address("glRasterPos4dv");
			if(stub_glRasterPos4dv){(*stub_glRasterPos4dv)(lv0);}
			else { 
				FailWith("Cannot load glRasterPos4dv"); 
			} 
		} else { 
			FailWith("glRasterPos4dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos4f */
typedef void APIENTRY (*pstub_glRasterPos4f)(GLfloat,GLfloat,GLfloat,GLfloat);
static pstub_glRasterPos4f stub_glRasterPos4f = NULL;
static int loaded_glRasterPos4f = 0;

value glstub_glRasterPos4f(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);

	if(stub_glRasterPos4f){(*stub_glRasterPos4f)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glRasterPos4f) { 
			loaded_glRasterPos4f = 1;
			init_lib(); 
			stub_glRasterPos4f=(pstub_glRasterPos4f)get_proc_address("glRasterPos4f");
			if(stub_glRasterPos4f){(*stub_glRasterPos4f)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glRasterPos4f"); 
			} 
		} else { 
			FailWith("glRasterPos4f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos4fv */
typedef void APIENTRY (*pstub_glRasterPos4fv)(GLfloat*);
static pstub_glRasterPos4fv stub_glRasterPos4fv = NULL;
static int loaded_glRasterPos4fv = 0;

value glstub_glRasterPos4fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glRasterPos4fv){(*stub_glRasterPos4fv)(lv0);}
	else { 
		if(!loaded_glRasterPos4fv) { 
			loaded_glRasterPos4fv = 1;
			init_lib(); 
			stub_glRasterPos4fv=(pstub_glRasterPos4fv)get_proc_address("glRasterPos4fv");
			if(stub_glRasterPos4fv){(*stub_glRasterPos4fv)(lv0);}
			else { 
				FailWith("Cannot load glRasterPos4fv"); 
			} 
		} else { 
			FailWith("glRasterPos4fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos4i */
typedef void APIENTRY (*pstub_glRasterPos4i)(GLint,GLint,GLint,GLint);
static pstub_glRasterPos4i stub_glRasterPos4i = NULL;
static int loaded_glRasterPos4i = 0;

value glstub_glRasterPos4i(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);

	if(stub_glRasterPos4i){(*stub_glRasterPos4i)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glRasterPos4i) { 
			loaded_glRasterPos4i = 1;
			init_lib(); 
			stub_glRasterPos4i=(pstub_glRasterPos4i)get_proc_address("glRasterPos4i");
			if(stub_glRasterPos4i){(*stub_glRasterPos4i)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glRasterPos4i"); 
			} 
		} else { 
			FailWith("glRasterPos4i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos4iv */
typedef void APIENTRY (*pstub_glRasterPos4iv)(GLint*);
static pstub_glRasterPos4iv stub_glRasterPos4iv = NULL;
static int loaded_glRasterPos4iv = 0;

value glstub_glRasterPos4iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glRasterPos4iv){(*stub_glRasterPos4iv)(lv0);}
	else { 
		if(!loaded_glRasterPos4iv) { 
			loaded_glRasterPos4iv = 1;
			init_lib(); 
			stub_glRasterPos4iv=(pstub_glRasterPos4iv)get_proc_address("glRasterPos4iv");
			if(stub_glRasterPos4iv){(*stub_glRasterPos4iv)(lv0);}
			else { 
				FailWith("Cannot load glRasterPos4iv"); 
			} 
		} else { 
			FailWith("glRasterPos4iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos4s */
typedef void APIENTRY (*pstub_glRasterPos4s)(GLshort,GLshort,GLshort,GLshort);
static pstub_glRasterPos4s stub_glRasterPos4s = NULL;
static int loaded_glRasterPos4s = 0;

value glstub_glRasterPos4s(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);

	if(stub_glRasterPos4s){(*stub_glRasterPos4s)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glRasterPos4s) { 
			loaded_glRasterPos4s = 1;
			init_lib(); 
			stub_glRasterPos4s=(pstub_glRasterPos4s)get_proc_address("glRasterPos4s");
			if(stub_glRasterPos4s){(*stub_glRasterPos4s)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glRasterPos4s"); 
			} 
		} else { 
			FailWith("glRasterPos4s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRasterPos4sv */
typedef void APIENTRY (*pstub_glRasterPos4sv)(GLshort*);
static pstub_glRasterPos4sv stub_glRasterPos4sv = NULL;
static int loaded_glRasterPos4sv = 0;

value glstub_glRasterPos4sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glRasterPos4sv){(*stub_glRasterPos4sv)(lv0);}
	else { 
		if(!loaded_glRasterPos4sv) { 
			loaded_glRasterPos4sv = 1;
			init_lib(); 
			stub_glRasterPos4sv=(pstub_glRasterPos4sv)get_proc_address("glRasterPos4sv");
			if(stub_glRasterPos4sv){(*stub_glRasterPos4sv)(lv0);}
			else { 
				FailWith("Cannot load glRasterPos4sv"); 
			} 
		} else { 
			FailWith("glRasterPos4sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glReadBuffer */
typedef void APIENTRY (*pstub_glReadBuffer)(GLenum);
static pstub_glReadBuffer stub_glReadBuffer = NULL;
static int loaded_glReadBuffer = 0;

value glstub_glReadBuffer(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glReadBuffer){(*stub_glReadBuffer)(lv0);}
	else { 
		if(!loaded_glReadBuffer) { 
			loaded_glReadBuffer = 1;
			init_lib(); 
			stub_glReadBuffer=(pstub_glReadBuffer)get_proc_address("glReadBuffer");
			if(stub_glReadBuffer){(*stub_glReadBuffer)(lv0);}
			else { 
				FailWith("Cannot load glReadBuffer"); 
			} 
		} else { 
			FailWith("glReadBuffer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glReadPixels */
typedef void APIENTRY (*pstub_glReadPixels)(GLint,GLint,GLsizei,GLsizei,GLenum,GLenum,GLvoid*);
static pstub_glReadPixels stub_glReadPixels = NULL;
static int loaded_glReadPixels = 0;

value glstub_glReadPixels(value v0, value v1, value v2, value v3, value v4, value v5, value v6)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam2(v5,v6);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLsizei lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);
	GLenum lv4 =  get_glenum(v4);
	GLenum lv5 =  get_glenum(v5);
	GLvoid* lv6 =  Data_bigarray_val(v6);

	if(stub_glReadPixels){(*stub_glReadPixels)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
	else { 
		if(!loaded_glReadPixels) { 
			loaded_glReadPixels = 1;
			init_lib(); 
			stub_glReadPixels=(pstub_glReadPixels)get_proc_address("glReadPixels");
			if(stub_glReadPixels){(*stub_glReadPixels)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
			else { 
				FailWith("Cannot load glReadPixels"); 
			} 
		} else { 
			FailWith("glReadPixels not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glReadPixels_byte(value * argv, int n)
{
	return glstub_glReadPixels(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]);
}

/* glRectd */
typedef void APIENTRY (*pstub_glRectd)(GLdouble,GLdouble,GLdouble,GLdouble);
static pstub_glRectd stub_glRectd = NULL;
static int loaded_glRectd = 0;

value glstub_glRectd(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);

	if(stub_glRectd){(*stub_glRectd)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glRectd) { 
			loaded_glRectd = 1;
			init_lib(); 
			stub_glRectd=(pstub_glRectd)get_proc_address("glRectd");
			if(stub_glRectd){(*stub_glRectd)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glRectd"); 
			} 
		} else { 
			FailWith("glRectd not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRectdv */
typedef void APIENTRY (*pstub_glRectdv)(GLdouble*,GLdouble*);
static pstub_glRectdv stub_glRectdv = NULL;
static int loaded_glRectdv = 0;

value glstub_glRectdv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glRectdv){(*stub_glRectdv)(lv0, lv1);}
	else { 
		if(!loaded_glRectdv) { 
			loaded_glRectdv = 1;
			init_lib(); 
			stub_glRectdv=(pstub_glRectdv)get_proc_address("glRectdv");
			if(stub_glRectdv){(*stub_glRectdv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glRectdv"); 
			} 
		} else { 
			FailWith("glRectdv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRectf */
typedef void APIENTRY (*pstub_glRectf)(GLfloat,GLfloat,GLfloat,GLfloat);
static pstub_glRectf stub_glRectf = NULL;
static int loaded_glRectf = 0;

value glstub_glRectf(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);

	if(stub_glRectf){(*stub_glRectf)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glRectf) { 
			loaded_glRectf = 1;
			init_lib(); 
			stub_glRectf=(pstub_glRectf)get_proc_address("glRectf");
			if(stub_glRectf){(*stub_glRectf)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glRectf"); 
			} 
		} else { 
			FailWith("glRectf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRectfv */
typedef void APIENTRY (*pstub_glRectfv)(GLfloat*,GLfloat*);
static pstub_glRectfv stub_glRectfv = NULL;
static int loaded_glRectfv = 0;

value glstub_glRectfv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glRectfv){(*stub_glRectfv)(lv0, lv1);}
	else { 
		if(!loaded_glRectfv) { 
			loaded_glRectfv = 1;
			init_lib(); 
			stub_glRectfv=(pstub_glRectfv)get_proc_address("glRectfv");
			if(stub_glRectfv){(*stub_glRectfv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glRectfv"); 
			} 
		} else { 
			FailWith("glRectfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRecti */
typedef void APIENTRY (*pstub_glRecti)(GLint,GLint,GLint,GLint);
static pstub_glRecti stub_glRecti = NULL;
static int loaded_glRecti = 0;

value glstub_glRecti(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);

	if(stub_glRecti){(*stub_glRecti)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glRecti) { 
			loaded_glRecti = 1;
			init_lib(); 
			stub_glRecti=(pstub_glRecti)get_proc_address("glRecti");
			if(stub_glRecti){(*stub_glRecti)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glRecti"); 
			} 
		} else { 
			FailWith("glRecti not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRectiv */
typedef void APIENTRY (*pstub_glRectiv)(GLint*,GLint*);
static pstub_glRectiv stub_glRectiv = NULL;
static int loaded_glRectiv = 0;

value glstub_glRectiv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glRectiv){(*stub_glRectiv)(lv0, lv1);}
	else { 
		if(!loaded_glRectiv) { 
			loaded_glRectiv = 1;
			init_lib(); 
			stub_glRectiv=(pstub_glRectiv)get_proc_address("glRectiv");
			if(stub_glRectiv){(*stub_glRectiv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glRectiv"); 
			} 
		} else { 
			FailWith("glRectiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRects */
typedef void APIENTRY (*pstub_glRects)(GLshort,GLshort,GLshort,GLshort);
static pstub_glRects stub_glRects = NULL;
static int loaded_glRects = 0;

value glstub_glRects(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);

	if(stub_glRects){(*stub_glRects)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glRects) { 
			loaded_glRects = 1;
			init_lib(); 
			stub_glRects=(pstub_glRects)get_proc_address("glRects");
			if(stub_glRects){(*stub_glRects)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glRects"); 
			} 
		} else { 
			FailWith("glRects not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRectsv */
typedef void APIENTRY (*pstub_glRectsv)(GLshort*,GLshort*);
static pstub_glRectsv stub_glRectsv = NULL;
static int loaded_glRectsv = 0;

value glstub_glRectsv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glRectsv){(*stub_glRectsv)(lv0, lv1);}
	else { 
		if(!loaded_glRectsv) { 
			loaded_glRectsv = 1;
			init_lib(); 
			stub_glRectsv=(pstub_glRectsv)get_proc_address("glRectsv");
			if(stub_glRectsv){(*stub_glRectsv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glRectsv"); 
			} 
		} else { 
			FailWith("glRectsv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRenderMode */
typedef GLint APIENTRY (*pstub_glRenderMode)(GLenum);
static pstub_glRenderMode stub_glRenderMode = NULL;
static int loaded_glRenderMode = 0;

value glstub_glRenderMode(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint ret;
	if(stub_glRenderMode){ret = (*stub_glRenderMode)(lv0);}
	else { 
		if(!loaded_glRenderMode) { 
			loaded_glRenderMode = 1;
			init_lib(); 
			stub_glRenderMode=(pstub_glRenderMode)get_proc_address("glRenderMode");
			if(stub_glRenderMode){ret = (*stub_glRenderMode)(lv0);}
			else { 
				FailWith("Cannot load glRenderMode"); 
			} 
		} else { 
			FailWith("glRenderMode not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glRotated */
typedef void APIENTRY (*pstub_glRotated)(GLdouble,GLdouble,GLdouble,GLdouble);
static pstub_glRotated stub_glRotated = NULL;
static int loaded_glRotated = 0;

value glstub_glRotated(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);

	if(stub_glRotated){(*stub_glRotated)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glRotated) { 
			loaded_glRotated = 1;
			init_lib(); 
			stub_glRotated=(pstub_glRotated)get_proc_address("glRotated");
			if(stub_glRotated){(*stub_glRotated)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glRotated"); 
			} 
		} else { 
			FailWith("glRotated not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glRotatef */
typedef void APIENTRY (*pstub_glRotatef)(GLfloat,GLfloat,GLfloat,GLfloat);
static pstub_glRotatef stub_glRotatef = NULL;
static int loaded_glRotatef = 0;

value glstub_glRotatef(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);

	if(stub_glRotatef){(*stub_glRotatef)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glRotatef) { 
			loaded_glRotatef = 1;
			init_lib(); 
			stub_glRotatef=(pstub_glRotatef)get_proc_address("glRotatef");
			if(stub_glRotatef){(*stub_glRotatef)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glRotatef"); 
			} 
		} else { 
			FailWith("glRotatef not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glScaled */
typedef void APIENTRY (*pstub_glScaled)(GLdouble,GLdouble,GLdouble);
static pstub_glScaled stub_glScaled = NULL;
static int loaded_glScaled = 0;

value glstub_glScaled(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glScaled){(*stub_glScaled)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glScaled) { 
			loaded_glScaled = 1;
			init_lib(); 
			stub_glScaled=(pstub_glScaled)get_proc_address("glScaled");
			if(stub_glScaled){(*stub_glScaled)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glScaled"); 
			} 
		} else { 
			FailWith("glScaled not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glScalef */
typedef void APIENTRY (*pstub_glScalef)(GLfloat,GLfloat,GLfloat);
static pstub_glScalef stub_glScalef = NULL;
static int loaded_glScalef = 0;

value glstub_glScalef(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glScalef){(*stub_glScalef)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glScalef) { 
			loaded_glScalef = 1;
			init_lib(); 
			stub_glScalef=(pstub_glScalef)get_proc_address("glScalef");
			if(stub_glScalef){(*stub_glScalef)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glScalef"); 
			} 
		} else { 
			FailWith("glScalef not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glScissor */
typedef void APIENTRY (*pstub_glScissor)(GLint,GLint,GLsizei,GLsizei);
static pstub_glScissor stub_glScissor = NULL;
static int loaded_glScissor = 0;

value glstub_glScissor(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLsizei lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);

	if(stub_glScissor){(*stub_glScissor)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glScissor) { 
			loaded_glScissor = 1;
			init_lib(); 
			stub_glScissor=(pstub_glScissor)get_proc_address("glScissor");
			if(stub_glScissor){(*stub_glScissor)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glScissor"); 
			} 
		} else { 
			FailWith("glScissor not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSelectBuffer */
typedef void APIENTRY (*pstub_glSelectBuffer)(GLsizei,GLuint*);
static pstub_glSelectBuffer stub_glSelectBuffer = NULL;
static int loaded_glSelectBuffer = 0;

value glstub_glSelectBuffer(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);

	if(stub_glSelectBuffer){(*stub_glSelectBuffer)(lv0, lv1);}
	else { 
		if(!loaded_glSelectBuffer) { 
			loaded_glSelectBuffer = 1;
			init_lib(); 
			stub_glSelectBuffer=(pstub_glSelectBuffer)get_proc_address("glSelectBuffer");
			if(stub_glSelectBuffer){(*stub_glSelectBuffer)(lv0, lv1);}
			else { 
				FailWith("Cannot load glSelectBuffer"); 
			} 
		} else { 
			FailWith("glSelectBuffer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glShadeModel */
typedef void APIENTRY (*pstub_glShadeModel)(GLenum);
static pstub_glShadeModel stub_glShadeModel = NULL;
static int loaded_glShadeModel = 0;

value glstub_glShadeModel(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glShadeModel){(*stub_glShadeModel)(lv0);}
	else { 
		if(!loaded_glShadeModel) { 
			loaded_glShadeModel = 1;
			init_lib(); 
			stub_glShadeModel=(pstub_glShadeModel)get_proc_address("glShadeModel");
			if(stub_glShadeModel){(*stub_glShadeModel)(lv0);}
			else { 
				FailWith("Cannot load glShadeModel"); 
			} 
		} else { 
			FailWith("glShadeModel not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glStencilFunc */
typedef void APIENTRY (*pstub_glStencilFunc)(GLenum,GLint,GLuint);
static pstub_glStencilFunc stub_glStencilFunc = NULL;
static int loaded_glStencilFunc = 0;

value glstub_glStencilFunc(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLuint lv2 =  Int_val(v2);

	if(stub_glStencilFunc){(*stub_glStencilFunc)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glStencilFunc) { 
			loaded_glStencilFunc = 1;
			init_lib(); 
			stub_glStencilFunc=(pstub_glStencilFunc)get_proc_address("glStencilFunc");
			if(stub_glStencilFunc){(*stub_glStencilFunc)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glStencilFunc"); 
			} 
		} else { 
			FailWith("glStencilFunc not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glStencilMask */
typedef void APIENTRY (*pstub_glStencilMask)(GLuint);
static pstub_glStencilMask stub_glStencilMask = NULL;
static int loaded_glStencilMask = 0;

value glstub_glStencilMask(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glStencilMask){(*stub_glStencilMask)(lv0);}
	else { 
		if(!loaded_glStencilMask) { 
			loaded_glStencilMask = 1;
			init_lib(); 
			stub_glStencilMask=(pstub_glStencilMask)get_proc_address("glStencilMask");
			if(stub_glStencilMask){(*stub_glStencilMask)(lv0);}
			else { 
				FailWith("Cannot load glStencilMask"); 
			} 
		} else { 
			FailWith("glStencilMask not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glStencilOp */
typedef void APIENTRY (*pstub_glStencilOp)(GLenum,GLenum,GLenum);
static pstub_glStencilOp stub_glStencilOp = NULL;
static int loaded_glStencilOp = 0;

value glstub_glStencilOp(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLenum lv2 =  get_glenum(v2);

	if(stub_glStencilOp){(*stub_glStencilOp)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glStencilOp) { 
			loaded_glStencilOp = 1;
			init_lib(); 
			stub_glStencilOp=(pstub_glStencilOp)get_proc_address("glStencilOp");
			if(stub_glStencilOp){(*stub_glStencilOp)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glStencilOp"); 
			} 
		} else { 
			FailWith("glStencilOp not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord1d */
typedef void APIENTRY (*pstub_glTexCoord1d)(GLdouble);
static pstub_glTexCoord1d stub_glTexCoord1d = NULL;
static int loaded_glTexCoord1d = 0;

value glstub_glTexCoord1d(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);

	if(stub_glTexCoord1d){(*stub_glTexCoord1d)(lv0);}
	else { 
		if(!loaded_glTexCoord1d) { 
			loaded_glTexCoord1d = 1;
			init_lib(); 
			stub_glTexCoord1d=(pstub_glTexCoord1d)get_proc_address("glTexCoord1d");
			if(stub_glTexCoord1d){(*stub_glTexCoord1d)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord1d"); 
			} 
		} else { 
			FailWith("glTexCoord1d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord1dv */
typedef void APIENTRY (*pstub_glTexCoord1dv)(GLdouble*);
static pstub_glTexCoord1dv stub_glTexCoord1dv = NULL;
static int loaded_glTexCoord1dv = 0;

value glstub_glTexCoord1dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord1dv){(*stub_glTexCoord1dv)(lv0);}
	else { 
		if(!loaded_glTexCoord1dv) { 
			loaded_glTexCoord1dv = 1;
			init_lib(); 
			stub_glTexCoord1dv=(pstub_glTexCoord1dv)get_proc_address("glTexCoord1dv");
			if(stub_glTexCoord1dv){(*stub_glTexCoord1dv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord1dv"); 
			} 
		} else { 
			FailWith("glTexCoord1dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord1f */
typedef void APIENTRY (*pstub_glTexCoord1f)(GLfloat);
static pstub_glTexCoord1f stub_glTexCoord1f = NULL;
static int loaded_glTexCoord1f = 0;

value glstub_glTexCoord1f(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);

	if(stub_glTexCoord1f){(*stub_glTexCoord1f)(lv0);}
	else { 
		if(!loaded_glTexCoord1f) { 
			loaded_glTexCoord1f = 1;
			init_lib(); 
			stub_glTexCoord1f=(pstub_glTexCoord1f)get_proc_address("glTexCoord1f");
			if(stub_glTexCoord1f){(*stub_glTexCoord1f)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord1f"); 
			} 
		} else { 
			FailWith("glTexCoord1f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord1fv */
typedef void APIENTRY (*pstub_glTexCoord1fv)(GLfloat*);
static pstub_glTexCoord1fv stub_glTexCoord1fv = NULL;
static int loaded_glTexCoord1fv = 0;

value glstub_glTexCoord1fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord1fv){(*stub_glTexCoord1fv)(lv0);}
	else { 
		if(!loaded_glTexCoord1fv) { 
			loaded_glTexCoord1fv = 1;
			init_lib(); 
			stub_glTexCoord1fv=(pstub_glTexCoord1fv)get_proc_address("glTexCoord1fv");
			if(stub_glTexCoord1fv){(*stub_glTexCoord1fv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord1fv"); 
			} 
		} else { 
			FailWith("glTexCoord1fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord1i */
typedef void APIENTRY (*pstub_glTexCoord1i)(GLint);
static pstub_glTexCoord1i stub_glTexCoord1i = NULL;
static int loaded_glTexCoord1i = 0;

value glstub_glTexCoord1i(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);

	if(stub_glTexCoord1i){(*stub_glTexCoord1i)(lv0);}
	else { 
		if(!loaded_glTexCoord1i) { 
			loaded_glTexCoord1i = 1;
			init_lib(); 
			stub_glTexCoord1i=(pstub_glTexCoord1i)get_proc_address("glTexCoord1i");
			if(stub_glTexCoord1i){(*stub_glTexCoord1i)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord1i"); 
			} 
		} else { 
			FailWith("glTexCoord1i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord1iv */
typedef void APIENTRY (*pstub_glTexCoord1iv)(GLint*);
static pstub_glTexCoord1iv stub_glTexCoord1iv = NULL;
static int loaded_glTexCoord1iv = 0;

value glstub_glTexCoord1iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord1iv){(*stub_glTexCoord1iv)(lv0);}
	else { 
		if(!loaded_glTexCoord1iv) { 
			loaded_glTexCoord1iv = 1;
			init_lib(); 
			stub_glTexCoord1iv=(pstub_glTexCoord1iv)get_proc_address("glTexCoord1iv");
			if(stub_glTexCoord1iv){(*stub_glTexCoord1iv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord1iv"); 
			} 
		} else { 
			FailWith("glTexCoord1iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord1s */
typedef void APIENTRY (*pstub_glTexCoord1s)(GLshort);
static pstub_glTexCoord1s stub_glTexCoord1s = NULL;
static int loaded_glTexCoord1s = 0;

value glstub_glTexCoord1s(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);

	if(stub_glTexCoord1s){(*stub_glTexCoord1s)(lv0);}
	else { 
		if(!loaded_glTexCoord1s) { 
			loaded_glTexCoord1s = 1;
			init_lib(); 
			stub_glTexCoord1s=(pstub_glTexCoord1s)get_proc_address("glTexCoord1s");
			if(stub_glTexCoord1s){(*stub_glTexCoord1s)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord1s"); 
			} 
		} else { 
			FailWith("glTexCoord1s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord1sv */
typedef void APIENTRY (*pstub_glTexCoord1sv)(GLshort*);
static pstub_glTexCoord1sv stub_glTexCoord1sv = NULL;
static int loaded_glTexCoord1sv = 0;

value glstub_glTexCoord1sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord1sv){(*stub_glTexCoord1sv)(lv0);}
	else { 
		if(!loaded_glTexCoord1sv) { 
			loaded_glTexCoord1sv = 1;
			init_lib(); 
			stub_glTexCoord1sv=(pstub_glTexCoord1sv)get_proc_address("glTexCoord1sv");
			if(stub_glTexCoord1sv){(*stub_glTexCoord1sv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord1sv"); 
			} 
		} else { 
			FailWith("glTexCoord1sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord2d */
typedef void APIENTRY (*pstub_glTexCoord2d)(GLdouble,GLdouble);
static pstub_glTexCoord2d stub_glTexCoord2d = NULL;
static int loaded_glTexCoord2d = 0;

value glstub_glTexCoord2d(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);

	if(stub_glTexCoord2d){(*stub_glTexCoord2d)(lv0, lv1);}
	else { 
		if(!loaded_glTexCoord2d) { 
			loaded_glTexCoord2d = 1;
			init_lib(); 
			stub_glTexCoord2d=(pstub_glTexCoord2d)get_proc_address("glTexCoord2d");
			if(stub_glTexCoord2d){(*stub_glTexCoord2d)(lv0, lv1);}
			else { 
				FailWith("Cannot load glTexCoord2d"); 
			} 
		} else { 
			FailWith("glTexCoord2d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord2dv */
typedef void APIENTRY (*pstub_glTexCoord2dv)(GLdouble*);
static pstub_glTexCoord2dv stub_glTexCoord2dv = NULL;
static int loaded_glTexCoord2dv = 0;

value glstub_glTexCoord2dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord2dv){(*stub_glTexCoord2dv)(lv0);}
	else { 
		if(!loaded_glTexCoord2dv) { 
			loaded_glTexCoord2dv = 1;
			init_lib(); 
			stub_glTexCoord2dv=(pstub_glTexCoord2dv)get_proc_address("glTexCoord2dv");
			if(stub_glTexCoord2dv){(*stub_glTexCoord2dv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord2dv"); 
			} 
		} else { 
			FailWith("glTexCoord2dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord2f */
typedef void APIENTRY (*pstub_glTexCoord2f)(GLfloat,GLfloat);
static pstub_glTexCoord2f stub_glTexCoord2f = NULL;
static int loaded_glTexCoord2f = 0;

value glstub_glTexCoord2f(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glTexCoord2f){(*stub_glTexCoord2f)(lv0, lv1);}
	else { 
		if(!loaded_glTexCoord2f) { 
			loaded_glTexCoord2f = 1;
			init_lib(); 
			stub_glTexCoord2f=(pstub_glTexCoord2f)get_proc_address("glTexCoord2f");
			if(stub_glTexCoord2f){(*stub_glTexCoord2f)(lv0, lv1);}
			else { 
				FailWith("Cannot load glTexCoord2f"); 
			} 
		} else { 
			FailWith("glTexCoord2f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord2fv */
typedef void APIENTRY (*pstub_glTexCoord2fv)(GLfloat*);
static pstub_glTexCoord2fv stub_glTexCoord2fv = NULL;
static int loaded_glTexCoord2fv = 0;

value glstub_glTexCoord2fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord2fv){(*stub_glTexCoord2fv)(lv0);}
	else { 
		if(!loaded_glTexCoord2fv) { 
			loaded_glTexCoord2fv = 1;
			init_lib(); 
			stub_glTexCoord2fv=(pstub_glTexCoord2fv)get_proc_address("glTexCoord2fv");
			if(stub_glTexCoord2fv){(*stub_glTexCoord2fv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord2fv"); 
			} 
		} else { 
			FailWith("glTexCoord2fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord2i */
typedef void APIENTRY (*pstub_glTexCoord2i)(GLint,GLint);
static pstub_glTexCoord2i stub_glTexCoord2i = NULL;
static int loaded_glTexCoord2i = 0;

value glstub_glTexCoord2i(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glTexCoord2i){(*stub_glTexCoord2i)(lv0, lv1);}
	else { 
		if(!loaded_glTexCoord2i) { 
			loaded_glTexCoord2i = 1;
			init_lib(); 
			stub_glTexCoord2i=(pstub_glTexCoord2i)get_proc_address("glTexCoord2i");
			if(stub_glTexCoord2i){(*stub_glTexCoord2i)(lv0, lv1);}
			else { 
				FailWith("Cannot load glTexCoord2i"); 
			} 
		} else { 
			FailWith("glTexCoord2i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord2iv */
typedef void APIENTRY (*pstub_glTexCoord2iv)(GLint*);
static pstub_glTexCoord2iv stub_glTexCoord2iv = NULL;
static int loaded_glTexCoord2iv = 0;

value glstub_glTexCoord2iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord2iv){(*stub_glTexCoord2iv)(lv0);}
	else { 
		if(!loaded_glTexCoord2iv) { 
			loaded_glTexCoord2iv = 1;
			init_lib(); 
			stub_glTexCoord2iv=(pstub_glTexCoord2iv)get_proc_address("glTexCoord2iv");
			if(stub_glTexCoord2iv){(*stub_glTexCoord2iv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord2iv"); 
			} 
		} else { 
			FailWith("glTexCoord2iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord2s */
typedef void APIENTRY (*pstub_glTexCoord2s)(GLshort,GLshort);
static pstub_glTexCoord2s stub_glTexCoord2s = NULL;
static int loaded_glTexCoord2s = 0;

value glstub_glTexCoord2s(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);

	if(stub_glTexCoord2s){(*stub_glTexCoord2s)(lv0, lv1);}
	else { 
		if(!loaded_glTexCoord2s) { 
			loaded_glTexCoord2s = 1;
			init_lib(); 
			stub_glTexCoord2s=(pstub_glTexCoord2s)get_proc_address("glTexCoord2s");
			if(stub_glTexCoord2s){(*stub_glTexCoord2s)(lv0, lv1);}
			else { 
				FailWith("Cannot load glTexCoord2s"); 
			} 
		} else { 
			FailWith("glTexCoord2s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord2sv */
typedef void APIENTRY (*pstub_glTexCoord2sv)(GLshort*);
static pstub_glTexCoord2sv stub_glTexCoord2sv = NULL;
static int loaded_glTexCoord2sv = 0;

value glstub_glTexCoord2sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord2sv){(*stub_glTexCoord2sv)(lv0);}
	else { 
		if(!loaded_glTexCoord2sv) { 
			loaded_glTexCoord2sv = 1;
			init_lib(); 
			stub_glTexCoord2sv=(pstub_glTexCoord2sv)get_proc_address("glTexCoord2sv");
			if(stub_glTexCoord2sv){(*stub_glTexCoord2sv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord2sv"); 
			} 
		} else { 
			FailWith("glTexCoord2sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord3d */
typedef void APIENTRY (*pstub_glTexCoord3d)(GLdouble,GLdouble,GLdouble);
static pstub_glTexCoord3d stub_glTexCoord3d = NULL;
static int loaded_glTexCoord3d = 0;

value glstub_glTexCoord3d(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glTexCoord3d){(*stub_glTexCoord3d)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexCoord3d) { 
			loaded_glTexCoord3d = 1;
			init_lib(); 
			stub_glTexCoord3d=(pstub_glTexCoord3d)get_proc_address("glTexCoord3d");
			if(stub_glTexCoord3d){(*stub_glTexCoord3d)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexCoord3d"); 
			} 
		} else { 
			FailWith("glTexCoord3d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord3dv */
typedef void APIENTRY (*pstub_glTexCoord3dv)(GLdouble*);
static pstub_glTexCoord3dv stub_glTexCoord3dv = NULL;
static int loaded_glTexCoord3dv = 0;

value glstub_glTexCoord3dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord3dv){(*stub_glTexCoord3dv)(lv0);}
	else { 
		if(!loaded_glTexCoord3dv) { 
			loaded_glTexCoord3dv = 1;
			init_lib(); 
			stub_glTexCoord3dv=(pstub_glTexCoord3dv)get_proc_address("glTexCoord3dv");
			if(stub_glTexCoord3dv){(*stub_glTexCoord3dv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord3dv"); 
			} 
		} else { 
			FailWith("glTexCoord3dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord3f */
typedef void APIENTRY (*pstub_glTexCoord3f)(GLfloat,GLfloat,GLfloat);
static pstub_glTexCoord3f stub_glTexCoord3f = NULL;
static int loaded_glTexCoord3f = 0;

value glstub_glTexCoord3f(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glTexCoord3f){(*stub_glTexCoord3f)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexCoord3f) { 
			loaded_glTexCoord3f = 1;
			init_lib(); 
			stub_glTexCoord3f=(pstub_glTexCoord3f)get_proc_address("glTexCoord3f");
			if(stub_glTexCoord3f){(*stub_glTexCoord3f)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexCoord3f"); 
			} 
		} else { 
			FailWith("glTexCoord3f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord3fv */
typedef void APIENTRY (*pstub_glTexCoord3fv)(GLfloat*);
static pstub_glTexCoord3fv stub_glTexCoord3fv = NULL;
static int loaded_glTexCoord3fv = 0;

value glstub_glTexCoord3fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord3fv){(*stub_glTexCoord3fv)(lv0);}
	else { 
		if(!loaded_glTexCoord3fv) { 
			loaded_glTexCoord3fv = 1;
			init_lib(); 
			stub_glTexCoord3fv=(pstub_glTexCoord3fv)get_proc_address("glTexCoord3fv");
			if(stub_glTexCoord3fv){(*stub_glTexCoord3fv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord3fv"); 
			} 
		} else { 
			FailWith("glTexCoord3fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord3i */
typedef void APIENTRY (*pstub_glTexCoord3i)(GLint,GLint,GLint);
static pstub_glTexCoord3i stub_glTexCoord3i = NULL;
static int loaded_glTexCoord3i = 0;

value glstub_glTexCoord3i(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glTexCoord3i){(*stub_glTexCoord3i)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexCoord3i) { 
			loaded_glTexCoord3i = 1;
			init_lib(); 
			stub_glTexCoord3i=(pstub_glTexCoord3i)get_proc_address("glTexCoord3i");
			if(stub_glTexCoord3i){(*stub_glTexCoord3i)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexCoord3i"); 
			} 
		} else { 
			FailWith("glTexCoord3i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord3iv */
typedef void APIENTRY (*pstub_glTexCoord3iv)(GLint*);
static pstub_glTexCoord3iv stub_glTexCoord3iv = NULL;
static int loaded_glTexCoord3iv = 0;

value glstub_glTexCoord3iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord3iv){(*stub_glTexCoord3iv)(lv0);}
	else { 
		if(!loaded_glTexCoord3iv) { 
			loaded_glTexCoord3iv = 1;
			init_lib(); 
			stub_glTexCoord3iv=(pstub_glTexCoord3iv)get_proc_address("glTexCoord3iv");
			if(stub_glTexCoord3iv){(*stub_glTexCoord3iv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord3iv"); 
			} 
		} else { 
			FailWith("glTexCoord3iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord3s */
typedef void APIENTRY (*pstub_glTexCoord3s)(GLshort,GLshort,GLshort);
static pstub_glTexCoord3s stub_glTexCoord3s = NULL;
static int loaded_glTexCoord3s = 0;

value glstub_glTexCoord3s(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);

	if(stub_glTexCoord3s){(*stub_glTexCoord3s)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexCoord3s) { 
			loaded_glTexCoord3s = 1;
			init_lib(); 
			stub_glTexCoord3s=(pstub_glTexCoord3s)get_proc_address("glTexCoord3s");
			if(stub_glTexCoord3s){(*stub_glTexCoord3s)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexCoord3s"); 
			} 
		} else { 
			FailWith("glTexCoord3s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord3sv */
typedef void APIENTRY (*pstub_glTexCoord3sv)(GLshort*);
static pstub_glTexCoord3sv stub_glTexCoord3sv = NULL;
static int loaded_glTexCoord3sv = 0;

value glstub_glTexCoord3sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord3sv){(*stub_glTexCoord3sv)(lv0);}
	else { 
		if(!loaded_glTexCoord3sv) { 
			loaded_glTexCoord3sv = 1;
			init_lib(); 
			stub_glTexCoord3sv=(pstub_glTexCoord3sv)get_proc_address("glTexCoord3sv");
			if(stub_glTexCoord3sv){(*stub_glTexCoord3sv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord3sv"); 
			} 
		} else { 
			FailWith("glTexCoord3sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord4d */
typedef void APIENTRY (*pstub_glTexCoord4d)(GLdouble,GLdouble,GLdouble,GLdouble);
static pstub_glTexCoord4d stub_glTexCoord4d = NULL;
static int loaded_glTexCoord4d = 0;

value glstub_glTexCoord4d(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);

	if(stub_glTexCoord4d){(*stub_glTexCoord4d)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glTexCoord4d) { 
			loaded_glTexCoord4d = 1;
			init_lib(); 
			stub_glTexCoord4d=(pstub_glTexCoord4d)get_proc_address("glTexCoord4d");
			if(stub_glTexCoord4d){(*stub_glTexCoord4d)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glTexCoord4d"); 
			} 
		} else { 
			FailWith("glTexCoord4d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord4dv */
typedef void APIENTRY (*pstub_glTexCoord4dv)(GLdouble*);
static pstub_glTexCoord4dv stub_glTexCoord4dv = NULL;
static int loaded_glTexCoord4dv = 0;

value glstub_glTexCoord4dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord4dv){(*stub_glTexCoord4dv)(lv0);}
	else { 
		if(!loaded_glTexCoord4dv) { 
			loaded_glTexCoord4dv = 1;
			init_lib(); 
			stub_glTexCoord4dv=(pstub_glTexCoord4dv)get_proc_address("glTexCoord4dv");
			if(stub_glTexCoord4dv){(*stub_glTexCoord4dv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord4dv"); 
			} 
		} else { 
			FailWith("glTexCoord4dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord4f */
typedef void APIENTRY (*pstub_glTexCoord4f)(GLfloat,GLfloat,GLfloat,GLfloat);
static pstub_glTexCoord4f stub_glTexCoord4f = NULL;
static int loaded_glTexCoord4f = 0;

value glstub_glTexCoord4f(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);

	if(stub_glTexCoord4f){(*stub_glTexCoord4f)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glTexCoord4f) { 
			loaded_glTexCoord4f = 1;
			init_lib(); 
			stub_glTexCoord4f=(pstub_glTexCoord4f)get_proc_address("glTexCoord4f");
			if(stub_glTexCoord4f){(*stub_glTexCoord4f)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glTexCoord4f"); 
			} 
		} else { 
			FailWith("glTexCoord4f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord4fv */
typedef void APIENTRY (*pstub_glTexCoord4fv)(GLfloat*);
static pstub_glTexCoord4fv stub_glTexCoord4fv = NULL;
static int loaded_glTexCoord4fv = 0;

value glstub_glTexCoord4fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord4fv){(*stub_glTexCoord4fv)(lv0);}
	else { 
		if(!loaded_glTexCoord4fv) { 
			loaded_glTexCoord4fv = 1;
			init_lib(); 
			stub_glTexCoord4fv=(pstub_glTexCoord4fv)get_proc_address("glTexCoord4fv");
			if(stub_glTexCoord4fv){(*stub_glTexCoord4fv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord4fv"); 
			} 
		} else { 
			FailWith("glTexCoord4fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord4i */
typedef void APIENTRY (*pstub_glTexCoord4i)(GLint,GLint,GLint,GLint);
static pstub_glTexCoord4i stub_glTexCoord4i = NULL;
static int loaded_glTexCoord4i = 0;

value glstub_glTexCoord4i(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);

	if(stub_glTexCoord4i){(*stub_glTexCoord4i)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glTexCoord4i) { 
			loaded_glTexCoord4i = 1;
			init_lib(); 
			stub_glTexCoord4i=(pstub_glTexCoord4i)get_proc_address("glTexCoord4i");
			if(stub_glTexCoord4i){(*stub_glTexCoord4i)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glTexCoord4i"); 
			} 
		} else { 
			FailWith("glTexCoord4i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord4iv */
typedef void APIENTRY (*pstub_glTexCoord4iv)(GLint*);
static pstub_glTexCoord4iv stub_glTexCoord4iv = NULL;
static int loaded_glTexCoord4iv = 0;

value glstub_glTexCoord4iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord4iv){(*stub_glTexCoord4iv)(lv0);}
	else { 
		if(!loaded_glTexCoord4iv) { 
			loaded_glTexCoord4iv = 1;
			init_lib(); 
			stub_glTexCoord4iv=(pstub_glTexCoord4iv)get_proc_address("glTexCoord4iv");
			if(stub_glTexCoord4iv){(*stub_glTexCoord4iv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord4iv"); 
			} 
		} else { 
			FailWith("glTexCoord4iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord4s */
typedef void APIENTRY (*pstub_glTexCoord4s)(GLshort,GLshort,GLshort,GLshort);
static pstub_glTexCoord4s stub_glTexCoord4s = NULL;
static int loaded_glTexCoord4s = 0;

value glstub_glTexCoord4s(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);

	if(stub_glTexCoord4s){(*stub_glTexCoord4s)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glTexCoord4s) { 
			loaded_glTexCoord4s = 1;
			init_lib(); 
			stub_glTexCoord4s=(pstub_glTexCoord4s)get_proc_address("glTexCoord4s");
			if(stub_glTexCoord4s){(*stub_glTexCoord4s)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glTexCoord4s"); 
			} 
		} else { 
			FailWith("glTexCoord4s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoord4sv */
typedef void APIENTRY (*pstub_glTexCoord4sv)(GLshort*);
static pstub_glTexCoord4sv stub_glTexCoord4sv = NULL;
static int loaded_glTexCoord4sv = 0;

value glstub_glTexCoord4sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glTexCoord4sv){(*stub_glTexCoord4sv)(lv0);}
	else { 
		if(!loaded_glTexCoord4sv) { 
			loaded_glTexCoord4sv = 1;
			init_lib(); 
			stub_glTexCoord4sv=(pstub_glTexCoord4sv)get_proc_address("glTexCoord4sv");
			if(stub_glTexCoord4sv){(*stub_glTexCoord4sv)(lv0);}
			else { 
				FailWith("Cannot load glTexCoord4sv"); 
			} 
		} else { 
			FailWith("glTexCoord4sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexCoordPointer */
typedef void APIENTRY (*pstub_glTexCoordPointer)(GLint,GLenum,GLsizei,GLvoid*);
static pstub_glTexCoordPointer stub_glTexCoordPointer = NULL;
static int loaded_glTexCoordPointer = 0;

value glstub_glTexCoordPointer(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLsizei lv2 =  Int_val(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);

	if(stub_glTexCoordPointer){(*stub_glTexCoordPointer)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glTexCoordPointer) { 
			loaded_glTexCoordPointer = 1;
			init_lib(); 
			stub_glTexCoordPointer=(pstub_glTexCoordPointer)get_proc_address("glTexCoordPointer");
			if(stub_glTexCoordPointer){(*stub_glTexCoordPointer)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glTexCoordPointer"); 
			} 
		} else { 
			FailWith("glTexCoordPointer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexEnvf */
typedef void APIENTRY (*pstub_glTexEnvf)(GLenum,GLenum,GLfloat);
static pstub_glTexEnvf stub_glTexEnvf = NULL;
static int loaded_glTexEnvf = 0;

value glstub_glTexEnvf(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glTexEnvf){(*stub_glTexEnvf)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexEnvf) { 
			loaded_glTexEnvf = 1;
			init_lib(); 
			stub_glTexEnvf=(pstub_glTexEnvf)get_proc_address("glTexEnvf");
			if(stub_glTexEnvf){(*stub_glTexEnvf)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexEnvf"); 
			} 
		} else { 
			FailWith("glTexEnvf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexEnvfv */
typedef void APIENTRY (*pstub_glTexEnvfv)(GLenum,GLenum,GLfloat*);
static pstub_glTexEnvfv stub_glTexEnvfv = NULL;
static int loaded_glTexEnvfv = 0;

value glstub_glTexEnvfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glTexEnvfv){(*stub_glTexEnvfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexEnvfv) { 
			loaded_glTexEnvfv = 1;
			init_lib(); 
			stub_glTexEnvfv=(pstub_glTexEnvfv)get_proc_address("glTexEnvfv");
			if(stub_glTexEnvfv){(*stub_glTexEnvfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexEnvfv"); 
			} 
		} else { 
			FailWith("glTexEnvfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexEnvi */
typedef void APIENTRY (*pstub_glTexEnvi)(GLenum,GLenum,GLint);
static pstub_glTexEnvi stub_glTexEnvi = NULL;
static int loaded_glTexEnvi = 0;

value glstub_glTexEnvi(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glTexEnvi){(*stub_glTexEnvi)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexEnvi) { 
			loaded_glTexEnvi = 1;
			init_lib(); 
			stub_glTexEnvi=(pstub_glTexEnvi)get_proc_address("glTexEnvi");
			if(stub_glTexEnvi){(*stub_glTexEnvi)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexEnvi"); 
			} 
		} else { 
			FailWith("glTexEnvi not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexEnviv */
typedef void APIENTRY (*pstub_glTexEnviv)(GLenum,GLenum,GLint*);
static pstub_glTexEnviv stub_glTexEnviv = NULL;
static int loaded_glTexEnviv = 0;

value glstub_glTexEnviv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glTexEnviv){(*stub_glTexEnviv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexEnviv) { 
			loaded_glTexEnviv = 1;
			init_lib(); 
			stub_glTexEnviv=(pstub_glTexEnviv)get_proc_address("glTexEnviv");
			if(stub_glTexEnviv){(*stub_glTexEnviv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexEnviv"); 
			} 
		} else { 
			FailWith("glTexEnviv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexGend */
typedef void APIENTRY (*pstub_glTexGend)(GLenum,GLenum,GLdouble);
static pstub_glTexGend stub_glTexGend = NULL;
static int loaded_glTexGend = 0;

value glstub_glTexGend(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glTexGend){(*stub_glTexGend)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexGend) { 
			loaded_glTexGend = 1;
			init_lib(); 
			stub_glTexGend=(pstub_glTexGend)get_proc_address("glTexGend");
			if(stub_glTexGend){(*stub_glTexGend)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexGend"); 
			} 
		} else { 
			FailWith("glTexGend not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexGendv */
typedef void APIENTRY (*pstub_glTexGendv)(GLenum,GLenum,GLdouble*);
static pstub_glTexGendv stub_glTexGendv = NULL;
static int loaded_glTexGendv = 0;

value glstub_glTexGendv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLdouble* lv2 =  Data_bigarray_val(v2);

	if(stub_glTexGendv){(*stub_glTexGendv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexGendv) { 
			loaded_glTexGendv = 1;
			init_lib(); 
			stub_glTexGendv=(pstub_glTexGendv)get_proc_address("glTexGendv");
			if(stub_glTexGendv){(*stub_glTexGendv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexGendv"); 
			} 
		} else { 
			FailWith("glTexGendv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexGenf */
typedef void APIENTRY (*pstub_glTexGenf)(GLenum,GLenum,GLfloat);
static pstub_glTexGenf stub_glTexGenf = NULL;
static int loaded_glTexGenf = 0;

value glstub_glTexGenf(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glTexGenf){(*stub_glTexGenf)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexGenf) { 
			loaded_glTexGenf = 1;
			init_lib(); 
			stub_glTexGenf=(pstub_glTexGenf)get_proc_address("glTexGenf");
			if(stub_glTexGenf){(*stub_glTexGenf)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexGenf"); 
			} 
		} else { 
			FailWith("glTexGenf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexGenfv */
typedef void APIENTRY (*pstub_glTexGenfv)(GLenum,GLenum,GLfloat*);
static pstub_glTexGenfv stub_glTexGenfv = NULL;
static int loaded_glTexGenfv = 0;

value glstub_glTexGenfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glTexGenfv){(*stub_glTexGenfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexGenfv) { 
			loaded_glTexGenfv = 1;
			init_lib(); 
			stub_glTexGenfv=(pstub_glTexGenfv)get_proc_address("glTexGenfv");
			if(stub_glTexGenfv){(*stub_glTexGenfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexGenfv"); 
			} 
		} else { 
			FailWith("glTexGenfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexGeni */
typedef void APIENTRY (*pstub_glTexGeni)(GLenum,GLenum,GLint);
static pstub_glTexGeni stub_glTexGeni = NULL;
static int loaded_glTexGeni = 0;

value glstub_glTexGeni(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glTexGeni){(*stub_glTexGeni)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexGeni) { 
			loaded_glTexGeni = 1;
			init_lib(); 
			stub_glTexGeni=(pstub_glTexGeni)get_proc_address("glTexGeni");
			if(stub_glTexGeni){(*stub_glTexGeni)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexGeni"); 
			} 
		} else { 
			FailWith("glTexGeni not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexGeniv */
typedef void APIENTRY (*pstub_glTexGeniv)(GLenum,GLenum,GLint*);
static pstub_glTexGeniv stub_glTexGeniv = NULL;
static int loaded_glTexGeniv = 0;

value glstub_glTexGeniv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glTexGeniv){(*stub_glTexGeniv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexGeniv) { 
			loaded_glTexGeniv = 1;
			init_lib(); 
			stub_glTexGeniv=(pstub_glTexGeniv)get_proc_address("glTexGeniv");
			if(stub_glTexGeniv){(*stub_glTexGeniv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexGeniv"); 
			} 
		} else { 
			FailWith("glTexGeniv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexImage1D */
typedef void APIENTRY (*pstub_glTexImage1D)(GLenum,GLint,GLint,GLsizei,GLint,GLenum,GLenum,GLvoid*);
static pstub_glTexImage1D stub_glTexImage1D = NULL;
static int loaded_glTexImage1D = 0;

value glstub_glTexImage1D(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam3(v5,v6,v7);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);
	GLenum lv5 =  get_glenum(v5);
	GLenum lv6 =  get_glenum(v6);
	GLvoid* lv7 =  Data_bigarray_val(v7);

	if(stub_glTexImage1D){(*stub_glTexImage1D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7);}
	else { 
		if(!loaded_glTexImage1D) { 
			loaded_glTexImage1D = 1;
			init_lib(); 
			stub_glTexImage1D=(pstub_glTexImage1D)get_proc_address("glTexImage1D");
			if(stub_glTexImage1D){(*stub_glTexImage1D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7);}
			else { 
				FailWith("Cannot load glTexImage1D"); 
			} 
		} else { 
			FailWith("glTexImage1D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glTexImage1D_byte(value * argv, int n)
{
	return glstub_glTexImage1D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7]);
}

/* glTexImage2D */
typedef void APIENTRY (*pstub_glTexImage2D)(GLenum,GLint,GLint,GLsizei,GLsizei,GLint,GLenum,GLenum,GLvoid*);
static pstub_glTexImage2D stub_glTexImage2D = NULL;
static int loaded_glTexImage2D = 0;

value glstub_glTexImage2D(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7, value v8)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam4(v5,v6,v7,v8);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);
	GLsizei lv4 =  Int_val(v4);
	GLint lv5 =  Int_val(v5);
	GLenum lv6 =  get_glenum(v6);
	GLenum lv7 =  get_glenum(v7);
	GLvoid* lv8 =  Data_bigarray_val(v8);

	if(stub_glTexImage2D){(*stub_glTexImage2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8);}
	else { 
		if(!loaded_glTexImage2D) { 
			loaded_glTexImage2D = 1;
			init_lib(); 
			stub_glTexImage2D=(pstub_glTexImage2D)get_proc_address("glTexImage2D");
			if(stub_glTexImage2D){(*stub_glTexImage2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8);}
			else { 
				FailWith("Cannot load glTexImage2D"); 
			} 
		} else { 
			FailWith("glTexImage2D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glTexImage2D_byte(value * argv, int n)
{
	return glstub_glTexImage2D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8]);
}

/* glTexParameterf */
typedef void APIENTRY (*pstub_glTexParameterf)(GLenum,GLenum,GLfloat);
static pstub_glTexParameterf stub_glTexParameterf = NULL;
static int loaded_glTexParameterf = 0;

value glstub_glTexParameterf(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glTexParameterf){(*stub_glTexParameterf)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexParameterf) { 
			loaded_glTexParameterf = 1;
			init_lib(); 
			stub_glTexParameterf=(pstub_glTexParameterf)get_proc_address("glTexParameterf");
			if(stub_glTexParameterf){(*stub_glTexParameterf)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexParameterf"); 
			} 
		} else { 
			FailWith("glTexParameterf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexParameterfv */
typedef void APIENTRY (*pstub_glTexParameterfv)(GLenum,GLenum,GLfloat*);
static pstub_glTexParameterfv stub_glTexParameterfv = NULL;
static int loaded_glTexParameterfv = 0;

value glstub_glTexParameterfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glTexParameterfv){(*stub_glTexParameterfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexParameterfv) { 
			loaded_glTexParameterfv = 1;
			init_lib(); 
			stub_glTexParameterfv=(pstub_glTexParameterfv)get_proc_address("glTexParameterfv");
			if(stub_glTexParameterfv){(*stub_glTexParameterfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexParameterfv"); 
			} 
		} else { 
			FailWith("glTexParameterfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexParameteri */
typedef void APIENTRY (*pstub_glTexParameteri)(GLenum,GLenum,GLint);
static pstub_glTexParameteri stub_glTexParameteri = NULL;
static int loaded_glTexParameteri = 0;

value glstub_glTexParameteri(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glTexParameteri){(*stub_glTexParameteri)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexParameteri) { 
			loaded_glTexParameteri = 1;
			init_lib(); 
			stub_glTexParameteri=(pstub_glTexParameteri)get_proc_address("glTexParameteri");
			if(stub_glTexParameteri){(*stub_glTexParameteri)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexParameteri"); 
			} 
		} else { 
			FailWith("glTexParameteri not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexParameteriv */
typedef void APIENTRY (*pstub_glTexParameteriv)(GLenum,GLenum,GLint*);
static pstub_glTexParameteriv stub_glTexParameteriv = NULL;
static int loaded_glTexParameteriv = 0;

value glstub_glTexParameteriv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glTexParameteriv){(*stub_glTexParameteriv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTexParameteriv) { 
			loaded_glTexParameteriv = 1;
			init_lib(); 
			stub_glTexParameteriv=(pstub_glTexParameteriv)get_proc_address("glTexParameteriv");
			if(stub_glTexParameteriv){(*stub_glTexParameteriv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTexParameteriv"); 
			} 
		} else { 
			FailWith("glTexParameteriv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTexSubImage1D */
typedef void APIENTRY (*pstub_glTexSubImage1D)(GLenum,GLint,GLint,GLsizei,GLenum,GLenum,GLvoid*);
static pstub_glTexSubImage1D stub_glTexSubImage1D = NULL;
static int loaded_glTexSubImage1D = 0;

value glstub_glTexSubImage1D(value v0, value v1, value v2, value v3, value v4, value v5, value v6)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam2(v5,v6);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);
	GLenum lv4 =  get_glenum(v4);
	GLenum lv5 =  get_glenum(v5);
	GLvoid* lv6 =  Data_bigarray_val(v6);

	if(stub_glTexSubImage1D){(*stub_glTexSubImage1D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
	else { 
		if(!loaded_glTexSubImage1D) { 
			loaded_glTexSubImage1D = 1;
			init_lib(); 
			stub_glTexSubImage1D=(pstub_glTexSubImage1D)get_proc_address("glTexSubImage1D");
			if(stub_glTexSubImage1D){(*stub_glTexSubImage1D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
			else { 
				FailWith("Cannot load glTexSubImage1D"); 
			} 
		} else { 
			FailWith("glTexSubImage1D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glTexSubImage1D_byte(value * argv, int n)
{
	return glstub_glTexSubImage1D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]);
}

/* glTexSubImage2D */
typedef void APIENTRY (*pstub_glTexSubImage2D)(GLenum,GLint,GLint,GLint,GLsizei,GLsizei,GLenum,GLenum,GLvoid*);
static pstub_glTexSubImage2D stub_glTexSubImage2D = NULL;
static int loaded_glTexSubImage2D = 0;

value glstub_glTexSubImage2D(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7, value v8)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam4(v5,v6,v7,v8);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLsizei lv4 =  Int_val(v4);
	GLsizei lv5 =  Int_val(v5);
	GLenum lv6 =  get_glenum(v6);
	GLenum lv7 =  get_glenum(v7);
	GLvoid* lv8 =  Data_bigarray_val(v8);

	if(stub_glTexSubImage2D){(*stub_glTexSubImage2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8);}
	else { 
		if(!loaded_glTexSubImage2D) { 
			loaded_glTexSubImage2D = 1;
			init_lib(); 
			stub_glTexSubImage2D=(pstub_glTexSubImage2D)get_proc_address("glTexSubImage2D");
			if(stub_glTexSubImage2D){(*stub_glTexSubImage2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8);}
			else { 
				FailWith("Cannot load glTexSubImage2D"); 
			} 
		} else { 
			FailWith("glTexSubImage2D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glTexSubImage2D_byte(value * argv, int n)
{
	return glstub_glTexSubImage2D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8]);
}

/* glTranslated */
typedef void APIENTRY (*pstub_glTranslated)(GLdouble,GLdouble,GLdouble);
static pstub_glTranslated stub_glTranslated = NULL;
static int loaded_glTranslated = 0;

value glstub_glTranslated(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glTranslated){(*stub_glTranslated)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTranslated) { 
			loaded_glTranslated = 1;
			init_lib(); 
			stub_glTranslated=(pstub_glTranslated)get_proc_address("glTranslated");
			if(stub_glTranslated){(*stub_glTranslated)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTranslated"); 
			} 
		} else { 
			FailWith("glTranslated not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glTranslatef */
typedef void APIENTRY (*pstub_glTranslatef)(GLfloat,GLfloat,GLfloat);
static pstub_glTranslatef stub_glTranslatef = NULL;
static int loaded_glTranslatef = 0;

value glstub_glTranslatef(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glTranslatef){(*stub_glTranslatef)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glTranslatef) { 
			loaded_glTranslatef = 1;
			init_lib(); 
			stub_glTranslatef=(pstub_glTranslatef)get_proc_address("glTranslatef");
			if(stub_glTranslatef){(*stub_glTranslatef)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glTranslatef"); 
			} 
		} else { 
			FailWith("glTranslatef not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex2d */
typedef void APIENTRY (*pstub_glVertex2d)(GLdouble,GLdouble);
static pstub_glVertex2d stub_glVertex2d = NULL;
static int loaded_glVertex2d = 0;

value glstub_glVertex2d(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);

	if(stub_glVertex2d){(*stub_glVertex2d)(lv0, lv1);}
	else { 
		if(!loaded_glVertex2d) { 
			loaded_glVertex2d = 1;
			init_lib(); 
			stub_glVertex2d=(pstub_glVertex2d)get_proc_address("glVertex2d");
			if(stub_glVertex2d){(*stub_glVertex2d)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertex2d"); 
			} 
		} else { 
			FailWith("glVertex2d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex2dv */
typedef void APIENTRY (*pstub_glVertex2dv)(GLdouble*);
static pstub_glVertex2dv stub_glVertex2dv = NULL;
static int loaded_glVertex2dv = 0;

value glstub_glVertex2dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glVertex2dv){(*stub_glVertex2dv)(lv0);}
	else { 
		if(!loaded_glVertex2dv) { 
			loaded_glVertex2dv = 1;
			init_lib(); 
			stub_glVertex2dv=(pstub_glVertex2dv)get_proc_address("glVertex2dv");
			if(stub_glVertex2dv){(*stub_glVertex2dv)(lv0);}
			else { 
				FailWith("Cannot load glVertex2dv"); 
			} 
		} else { 
			FailWith("glVertex2dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex2f */
typedef void APIENTRY (*pstub_glVertex2f)(GLfloat,GLfloat);
static pstub_glVertex2f stub_glVertex2f = NULL;
static int loaded_glVertex2f = 0;

value glstub_glVertex2f(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glVertex2f){(*stub_glVertex2f)(lv0, lv1);}
	else { 
		if(!loaded_glVertex2f) { 
			loaded_glVertex2f = 1;
			init_lib(); 
			stub_glVertex2f=(pstub_glVertex2f)get_proc_address("glVertex2f");
			if(stub_glVertex2f){(*stub_glVertex2f)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertex2f"); 
			} 
		} else { 
			FailWith("glVertex2f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex2fv */
typedef void APIENTRY (*pstub_glVertex2fv)(GLfloat*);
static pstub_glVertex2fv stub_glVertex2fv = NULL;
static int loaded_glVertex2fv = 0;

value glstub_glVertex2fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glVertex2fv){(*stub_glVertex2fv)(lv0);}
	else { 
		if(!loaded_glVertex2fv) { 
			loaded_glVertex2fv = 1;
			init_lib(); 
			stub_glVertex2fv=(pstub_glVertex2fv)get_proc_address("glVertex2fv");
			if(stub_glVertex2fv){(*stub_glVertex2fv)(lv0);}
			else { 
				FailWith("Cannot load glVertex2fv"); 
			} 
		} else { 
			FailWith("glVertex2fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex2i */
typedef void APIENTRY (*pstub_glVertex2i)(GLint,GLint);
static pstub_glVertex2i stub_glVertex2i = NULL;
static int loaded_glVertex2i = 0;

value glstub_glVertex2i(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glVertex2i){(*stub_glVertex2i)(lv0, lv1);}
	else { 
		if(!loaded_glVertex2i) { 
			loaded_glVertex2i = 1;
			init_lib(); 
			stub_glVertex2i=(pstub_glVertex2i)get_proc_address("glVertex2i");
			if(stub_glVertex2i){(*stub_glVertex2i)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertex2i"); 
			} 
		} else { 
			FailWith("glVertex2i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex2iv */
typedef void APIENTRY (*pstub_glVertex2iv)(GLint*);
static pstub_glVertex2iv stub_glVertex2iv = NULL;
static int loaded_glVertex2iv = 0;

value glstub_glVertex2iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glVertex2iv){(*stub_glVertex2iv)(lv0);}
	else { 
		if(!loaded_glVertex2iv) { 
			loaded_glVertex2iv = 1;
			init_lib(); 
			stub_glVertex2iv=(pstub_glVertex2iv)get_proc_address("glVertex2iv");
			if(stub_glVertex2iv){(*stub_glVertex2iv)(lv0);}
			else { 
				FailWith("Cannot load glVertex2iv"); 
			} 
		} else { 
			FailWith("glVertex2iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex2s */
typedef void APIENTRY (*pstub_glVertex2s)(GLshort,GLshort);
static pstub_glVertex2s stub_glVertex2s = NULL;
static int loaded_glVertex2s = 0;

value glstub_glVertex2s(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);

	if(stub_glVertex2s){(*stub_glVertex2s)(lv0, lv1);}
	else { 
		if(!loaded_glVertex2s) { 
			loaded_glVertex2s = 1;
			init_lib(); 
			stub_glVertex2s=(pstub_glVertex2s)get_proc_address("glVertex2s");
			if(stub_glVertex2s){(*stub_glVertex2s)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertex2s"); 
			} 
		} else { 
			FailWith("glVertex2s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex2sv */
typedef void APIENTRY (*pstub_glVertex2sv)(GLshort*);
static pstub_glVertex2sv stub_glVertex2sv = NULL;
static int loaded_glVertex2sv = 0;

value glstub_glVertex2sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glVertex2sv){(*stub_glVertex2sv)(lv0);}
	else { 
		if(!loaded_glVertex2sv) { 
			loaded_glVertex2sv = 1;
			init_lib(); 
			stub_glVertex2sv=(pstub_glVertex2sv)get_proc_address("glVertex2sv");
			if(stub_glVertex2sv){(*stub_glVertex2sv)(lv0);}
			else { 
				FailWith("Cannot load glVertex2sv"); 
			} 
		} else { 
			FailWith("glVertex2sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex3d */
typedef void APIENTRY (*pstub_glVertex3d)(GLdouble,GLdouble,GLdouble);
static pstub_glVertex3d stub_glVertex3d = NULL;
static int loaded_glVertex3d = 0;

value glstub_glVertex3d(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glVertex3d){(*stub_glVertex3d)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glVertex3d) { 
			loaded_glVertex3d = 1;
			init_lib(); 
			stub_glVertex3d=(pstub_glVertex3d)get_proc_address("glVertex3d");
			if(stub_glVertex3d){(*stub_glVertex3d)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glVertex3d"); 
			} 
		} else { 
			FailWith("glVertex3d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex3dv */
typedef void APIENTRY (*pstub_glVertex3dv)(GLdouble*);
static pstub_glVertex3dv stub_glVertex3dv = NULL;
static int loaded_glVertex3dv = 0;

value glstub_glVertex3dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glVertex3dv){(*stub_glVertex3dv)(lv0);}
	else { 
		if(!loaded_glVertex3dv) { 
			loaded_glVertex3dv = 1;
			init_lib(); 
			stub_glVertex3dv=(pstub_glVertex3dv)get_proc_address("glVertex3dv");
			if(stub_glVertex3dv){(*stub_glVertex3dv)(lv0);}
			else { 
				FailWith("Cannot load glVertex3dv"); 
			} 
		} else { 
			FailWith("glVertex3dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex3f */
typedef void APIENTRY (*pstub_glVertex3f)(GLfloat,GLfloat,GLfloat);
static pstub_glVertex3f stub_glVertex3f = NULL;
static int loaded_glVertex3f = 0;

value glstub_glVertex3f(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glVertex3f){(*stub_glVertex3f)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glVertex3f) { 
			loaded_glVertex3f = 1;
			init_lib(); 
			stub_glVertex3f=(pstub_glVertex3f)get_proc_address("glVertex3f");
			if(stub_glVertex3f){(*stub_glVertex3f)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glVertex3f"); 
			} 
		} else { 
			FailWith("glVertex3f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex3fv */
typedef void APIENTRY (*pstub_glVertex3fv)(GLfloat*);
static pstub_glVertex3fv stub_glVertex3fv = NULL;
static int loaded_glVertex3fv = 0;

value glstub_glVertex3fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glVertex3fv){(*stub_glVertex3fv)(lv0);}
	else { 
		if(!loaded_glVertex3fv) { 
			loaded_glVertex3fv = 1;
			init_lib(); 
			stub_glVertex3fv=(pstub_glVertex3fv)get_proc_address("glVertex3fv");
			if(stub_glVertex3fv){(*stub_glVertex3fv)(lv0);}
			else { 
				FailWith("Cannot load glVertex3fv"); 
			} 
		} else { 
			FailWith("glVertex3fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex3i */
typedef void APIENTRY (*pstub_glVertex3i)(GLint,GLint,GLint);
static pstub_glVertex3i stub_glVertex3i = NULL;
static int loaded_glVertex3i = 0;

value glstub_glVertex3i(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glVertex3i){(*stub_glVertex3i)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glVertex3i) { 
			loaded_glVertex3i = 1;
			init_lib(); 
			stub_glVertex3i=(pstub_glVertex3i)get_proc_address("glVertex3i");
			if(stub_glVertex3i){(*stub_glVertex3i)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glVertex3i"); 
			} 
		} else { 
			FailWith("glVertex3i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex3iv */
typedef void APIENTRY (*pstub_glVertex3iv)(GLint*);
static pstub_glVertex3iv stub_glVertex3iv = NULL;
static int loaded_glVertex3iv = 0;

value glstub_glVertex3iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glVertex3iv){(*stub_glVertex3iv)(lv0);}
	else { 
		if(!loaded_glVertex3iv) { 
			loaded_glVertex3iv = 1;
			init_lib(); 
			stub_glVertex3iv=(pstub_glVertex3iv)get_proc_address("glVertex3iv");
			if(stub_glVertex3iv){(*stub_glVertex3iv)(lv0);}
			else { 
				FailWith("Cannot load glVertex3iv"); 
			} 
		} else { 
			FailWith("glVertex3iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex3s */
typedef void APIENTRY (*pstub_glVertex3s)(GLshort,GLshort,GLshort);
static pstub_glVertex3s stub_glVertex3s = NULL;
static int loaded_glVertex3s = 0;

value glstub_glVertex3s(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);

	if(stub_glVertex3s){(*stub_glVertex3s)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glVertex3s) { 
			loaded_glVertex3s = 1;
			init_lib(); 
			stub_glVertex3s=(pstub_glVertex3s)get_proc_address("glVertex3s");
			if(stub_glVertex3s){(*stub_glVertex3s)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glVertex3s"); 
			} 
		} else { 
			FailWith("glVertex3s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex3sv */
typedef void APIENTRY (*pstub_glVertex3sv)(GLshort*);
static pstub_glVertex3sv stub_glVertex3sv = NULL;
static int loaded_glVertex3sv = 0;

value glstub_glVertex3sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glVertex3sv){(*stub_glVertex3sv)(lv0);}
	else { 
		if(!loaded_glVertex3sv) { 
			loaded_glVertex3sv = 1;
			init_lib(); 
			stub_glVertex3sv=(pstub_glVertex3sv)get_proc_address("glVertex3sv");
			if(stub_glVertex3sv){(*stub_glVertex3sv)(lv0);}
			else { 
				FailWith("Cannot load glVertex3sv"); 
			} 
		} else { 
			FailWith("glVertex3sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex4d */
typedef void APIENTRY (*pstub_glVertex4d)(GLdouble,GLdouble,GLdouble,GLdouble);
static pstub_glVertex4d stub_glVertex4d = NULL;
static int loaded_glVertex4d = 0;

value glstub_glVertex4d(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);

	if(stub_glVertex4d){(*stub_glVertex4d)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glVertex4d) { 
			loaded_glVertex4d = 1;
			init_lib(); 
			stub_glVertex4d=(pstub_glVertex4d)get_proc_address("glVertex4d");
			if(stub_glVertex4d){(*stub_glVertex4d)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glVertex4d"); 
			} 
		} else { 
			FailWith("glVertex4d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex4dv */
typedef void APIENTRY (*pstub_glVertex4dv)(GLdouble*);
static pstub_glVertex4dv stub_glVertex4dv = NULL;
static int loaded_glVertex4dv = 0;

value glstub_glVertex4dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glVertex4dv){(*stub_glVertex4dv)(lv0);}
	else { 
		if(!loaded_glVertex4dv) { 
			loaded_glVertex4dv = 1;
			init_lib(); 
			stub_glVertex4dv=(pstub_glVertex4dv)get_proc_address("glVertex4dv");
			if(stub_glVertex4dv){(*stub_glVertex4dv)(lv0);}
			else { 
				FailWith("Cannot load glVertex4dv"); 
			} 
		} else { 
			FailWith("glVertex4dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex4f */
typedef void APIENTRY (*pstub_glVertex4f)(GLfloat,GLfloat,GLfloat,GLfloat);
static pstub_glVertex4f stub_glVertex4f = NULL;
static int loaded_glVertex4f = 0;

value glstub_glVertex4f(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);

	if(stub_glVertex4f){(*stub_glVertex4f)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glVertex4f) { 
			loaded_glVertex4f = 1;
			init_lib(); 
			stub_glVertex4f=(pstub_glVertex4f)get_proc_address("glVertex4f");
			if(stub_glVertex4f){(*stub_glVertex4f)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glVertex4f"); 
			} 
		} else { 
			FailWith("glVertex4f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex4fv */
typedef void APIENTRY (*pstub_glVertex4fv)(GLfloat*);
static pstub_glVertex4fv stub_glVertex4fv = NULL;
static int loaded_glVertex4fv = 0;

value glstub_glVertex4fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glVertex4fv){(*stub_glVertex4fv)(lv0);}
	else { 
		if(!loaded_glVertex4fv) { 
			loaded_glVertex4fv = 1;
			init_lib(); 
			stub_glVertex4fv=(pstub_glVertex4fv)get_proc_address("glVertex4fv");
			if(stub_glVertex4fv){(*stub_glVertex4fv)(lv0);}
			else { 
				FailWith("Cannot load glVertex4fv"); 
			} 
		} else { 
			FailWith("glVertex4fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex4i */
typedef void APIENTRY (*pstub_glVertex4i)(GLint,GLint,GLint,GLint);
static pstub_glVertex4i stub_glVertex4i = NULL;
static int loaded_glVertex4i = 0;

value glstub_glVertex4i(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);

	if(stub_glVertex4i){(*stub_glVertex4i)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glVertex4i) { 
			loaded_glVertex4i = 1;
			init_lib(); 
			stub_glVertex4i=(pstub_glVertex4i)get_proc_address("glVertex4i");
			if(stub_glVertex4i){(*stub_glVertex4i)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glVertex4i"); 
			} 
		} else { 
			FailWith("glVertex4i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex4iv */
typedef void APIENTRY (*pstub_glVertex4iv)(GLint*);
static pstub_glVertex4iv stub_glVertex4iv = NULL;
static int loaded_glVertex4iv = 0;

value glstub_glVertex4iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glVertex4iv){(*stub_glVertex4iv)(lv0);}
	else { 
		if(!loaded_glVertex4iv) { 
			loaded_glVertex4iv = 1;
			init_lib(); 
			stub_glVertex4iv=(pstub_glVertex4iv)get_proc_address("glVertex4iv");
			if(stub_glVertex4iv){(*stub_glVertex4iv)(lv0);}
			else { 
				FailWith("Cannot load glVertex4iv"); 
			} 
		} else { 
			FailWith("glVertex4iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex4s */
typedef void APIENTRY (*pstub_glVertex4s)(GLshort,GLshort,GLshort,GLshort);
static pstub_glVertex4s stub_glVertex4s = NULL;
static int loaded_glVertex4s = 0;

value glstub_glVertex4s(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);

	if(stub_glVertex4s){(*stub_glVertex4s)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glVertex4s) { 
			loaded_glVertex4s = 1;
			init_lib(); 
			stub_glVertex4s=(pstub_glVertex4s)get_proc_address("glVertex4s");
			if(stub_glVertex4s){(*stub_glVertex4s)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glVertex4s"); 
			} 
		} else { 
			FailWith("glVertex4s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertex4sv */
typedef void APIENTRY (*pstub_glVertex4sv)(GLshort*);
static pstub_glVertex4sv stub_glVertex4sv = NULL;
static int loaded_glVertex4sv = 0;

value glstub_glVertex4sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glVertex4sv){(*stub_glVertex4sv)(lv0);}
	else { 
		if(!loaded_glVertex4sv) { 
			loaded_glVertex4sv = 1;
			init_lib(); 
			stub_glVertex4sv=(pstub_glVertex4sv)get_proc_address("glVertex4sv");
			if(stub_glVertex4sv){(*stub_glVertex4sv)(lv0);}
			else { 
				FailWith("Cannot load glVertex4sv"); 
			} 
		} else { 
			FailWith("glVertex4sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexPointer */
typedef void APIENTRY (*pstub_glVertexPointer)(GLint,GLenum,GLsizei,GLvoid*);
static pstub_glVertexPointer stub_glVertexPointer = NULL;
static int loaded_glVertexPointer = 0;

value glstub_glVertexPointer(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLsizei lv2 =  Int_val(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);

	if(stub_glVertexPointer){(*stub_glVertexPointer)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glVertexPointer) { 
			loaded_glVertexPointer = 1;
			init_lib(); 
			stub_glVertexPointer=(pstub_glVertexPointer)get_proc_address("glVertexPointer");
			if(stub_glVertexPointer){(*stub_glVertexPointer)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glVertexPointer"); 
			} 
		} else { 
			FailWith("glVertexPointer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glViewport */
typedef void APIENTRY (*pstub_glViewport)(GLint,GLint,GLsizei,GLsizei);
static pstub_glViewport stub_glViewport = NULL;
static int loaded_glViewport = 0;

value glstub_glViewport(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLsizei lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);

	if(stub_glViewport){(*stub_glViewport)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glViewport) { 
			loaded_glViewport = 1;
			init_lib(); 
			stub_glViewport=(pstub_glViewport)get_proc_address("glViewport");
			if(stub_glViewport){(*stub_glViewport)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glViewport"); 
			} 
		} else { 
			FailWith("glViewport not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDrawRangeElements */
typedef void APIENTRY (*pstub_glDrawRangeElements)(GLenum,GLuint,GLuint,GLsizei,GLenum,GLvoid*);
static pstub_glDrawRangeElements stub_glDrawRangeElements = NULL;
static int loaded_glDrawRangeElements = 0;

value glstub_glDrawRangeElements(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLuint lv1 =  Int_val(v1);
	GLuint lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);
	GLenum lv4 =  get_glenum(v4);
	GLvoid* lv5 =  Data_bigarray_val(v5);

	if(stub_glDrawRangeElements){(*stub_glDrawRangeElements)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glDrawRangeElements) { 
			loaded_glDrawRangeElements = 1;
			init_lib(); 
			stub_glDrawRangeElements=(pstub_glDrawRangeElements)get_proc_address("glDrawRangeElements");
			if(stub_glDrawRangeElements){(*stub_glDrawRangeElements)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glDrawRangeElements"); 
			} 
		} else { 
			FailWith("glDrawRangeElements not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glDrawRangeElements_byte(value * argv, int n)
{
	return glstub_glDrawRangeElements(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glTexImage3D */
typedef void APIENTRY (*pstub_glTexImage3D)(GLenum,GLint,GLint,GLsizei,GLsizei,GLsizei,GLint,GLenum,GLenum,GLvoid*);
static pstub_glTexImage3D stub_glTexImage3D = NULL;
static int loaded_glTexImage3D = 0;

value glstub_glTexImage3D(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7, value v8, value v9)
{
	CAMLparam5(v0, v1, v2, v3, v4);
CAMLxparam5(v5, v6, v7, v8, v9);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);
	GLsizei lv4 =  Int_val(v4);
	GLsizei lv5 =  Int_val(v5);
	GLint lv6 =  Int_val(v6);
	GLenum lv7 =  get_glenum(v7);
	GLenum lv8 =  get_glenum(v8);
	GLvoid* lv9 =  Data_bigarray_val(v9);

	if(stub_glTexImage3D){(*stub_glTexImage3D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8, lv9);}
	else { 
		if(!loaded_glTexImage3D) { 
			loaded_glTexImage3D = 1;
			init_lib(); 
			stub_glTexImage3D=(pstub_glTexImage3D)get_proc_address("glTexImage3D");
			if(stub_glTexImage3D){(*stub_glTexImage3D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8, lv9);}
			else { 
				FailWith("Cannot load glTexImage3D"); 
			} 
		} else { 
			FailWith("glTexImage3D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glTexImage3D_byte(value * argv, int n)
{
	return glstub_glTexImage3D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8], argv[9]);
}

/* glTexSubImage3D */
typedef void APIENTRY (*pstub_glTexSubImage3D)(GLenum,GLint,GLint,GLint,GLint,GLsizei,GLsizei,GLsizei,GLenum,GLenum,GLvoid*);
static pstub_glTexSubImage3D stub_glTexSubImage3D = NULL;
static int loaded_glTexSubImage3D = 0;

value glstub_glTexSubImage3D(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7, value v8, value v9, value v10)
{
	CAMLparam5(v0, v1, v2, v3, v4);
CAMLxparam5(v5, v6, v7, v8, v9);
	CAMLxparam1(v10);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);
	GLsizei lv5 =  Int_val(v5);
	GLsizei lv6 =  Int_val(v6);
	GLsizei lv7 =  Int_val(v7);
	GLenum lv8 =  get_glenum(v8);
	GLenum lv9 =  get_glenum(v9);
	GLvoid* lv10 =  Data_bigarray_val(v10);

	if(stub_glTexSubImage3D){(*stub_glTexSubImage3D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8, lv9, lv10);}
	else { 
		if(!loaded_glTexSubImage3D) { 
			loaded_glTexSubImage3D = 1;
			init_lib(); 
			stub_glTexSubImage3D=(pstub_glTexSubImage3D)get_proc_address("glTexSubImage3D");
			if(stub_glTexSubImage3D){(*stub_glTexSubImage3D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8, lv9, lv10);}
			else { 
				FailWith("Cannot load glTexSubImage3D"); 
			} 
		} else { 
			FailWith("glTexSubImage3D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glTexSubImage3D_byte(value * argv, int n)
{
	return glstub_glTexSubImage3D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8], argv[9], argv[10]);
}

/* glCopyTexSubImage3D */
typedef void APIENTRY (*pstub_glCopyTexSubImage3D)(GLenum,GLint,GLint,GLint,GLint,GLint,GLint,GLsizei,GLsizei);
static pstub_glCopyTexSubImage3D stub_glCopyTexSubImage3D = NULL;
static int loaded_glCopyTexSubImage3D = 0;

value glstub_glCopyTexSubImage3D(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7, value v8)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam4(v5,v6,v7,v8);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);
	GLint lv5 =  Int_val(v5);
	GLint lv6 =  Int_val(v6);
	GLsizei lv7 =  Int_val(v7);
	GLsizei lv8 =  Int_val(v8);

	if(stub_glCopyTexSubImage3D){(*stub_glCopyTexSubImage3D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8);}
	else { 
		if(!loaded_glCopyTexSubImage3D) { 
			loaded_glCopyTexSubImage3D = 1;
			init_lib(); 
			stub_glCopyTexSubImage3D=(pstub_glCopyTexSubImage3D)get_proc_address("glCopyTexSubImage3D");
			if(stub_glCopyTexSubImage3D){(*stub_glCopyTexSubImage3D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8);}
			else { 
				FailWith("Cannot load glCopyTexSubImage3D"); 
			} 
		} else { 
			FailWith("glCopyTexSubImage3D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glCopyTexSubImage3D_byte(value * argv, int n)
{
	return glstub_glCopyTexSubImage3D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8]);
}

/* glActiveTexture */
typedef void APIENTRY (*pstub_glActiveTexture)(GLenum);
static pstub_glActiveTexture stub_glActiveTexture = NULL;
static int loaded_glActiveTexture = 0;

value glstub_glActiveTexture(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glActiveTexture){(*stub_glActiveTexture)(lv0);}
	else { 
		if(!loaded_glActiveTexture) { 
			loaded_glActiveTexture = 1;
			init_lib(); 
			stub_glActiveTexture=(pstub_glActiveTexture)get_proc_address("glActiveTexture");
			if(stub_glActiveTexture){(*stub_glActiveTexture)(lv0);}
			else { 
				FailWith("Cannot load glActiveTexture"); 
			} 
		} else { 
			FailWith("glActiveTexture not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glClientActiveTexture */
typedef void APIENTRY (*pstub_glClientActiveTexture)(GLenum);
static pstub_glClientActiveTexture stub_glClientActiveTexture = NULL;
static int loaded_glClientActiveTexture = 0;

value glstub_glClientActiveTexture(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glClientActiveTexture){(*stub_glClientActiveTexture)(lv0);}
	else { 
		if(!loaded_glClientActiveTexture) { 
			loaded_glClientActiveTexture = 1;
			init_lib(); 
			stub_glClientActiveTexture=(pstub_glClientActiveTexture)get_proc_address("glClientActiveTexture");
			if(stub_glClientActiveTexture){(*stub_glClientActiveTexture)(lv0);}
			else { 
				FailWith("Cannot load glClientActiveTexture"); 
			} 
		} else { 
			FailWith("glClientActiveTexture not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glCompressedTexImage1D */
typedef void APIENTRY (*pstub_glCompressedTexImage1D)(GLenum,GLint,GLenum,GLsizei,GLint,GLsizei,GLvoid*);
static pstub_glCompressedTexImage1D stub_glCompressedTexImage1D = NULL;
static int loaded_glCompressedTexImage1D = 0;

value glstub_glCompressedTexImage1D(value v0, value v1, value v2, value v3, value v4, value v5, value v6)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam2(v5,v6);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLsizei lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);
	GLsizei lv5 =  Int_val(v5);
	GLvoid* lv6 =  Data_bigarray_val(v6);

	if(stub_glCompressedTexImage1D){(*stub_glCompressedTexImage1D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
	else { 
		if(!loaded_glCompressedTexImage1D) { 
			loaded_glCompressedTexImage1D = 1;
			init_lib(); 
			stub_glCompressedTexImage1D=(pstub_glCompressedTexImage1D)get_proc_address("glCompressedTexImage1D");
			if(stub_glCompressedTexImage1D){(*stub_glCompressedTexImage1D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
			else { 
				FailWith("Cannot load glCompressedTexImage1D"); 
			} 
		} else { 
			FailWith("glCompressedTexImage1D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glCompressedTexImage1D_byte(value * argv, int n)
{
	return glstub_glCompressedTexImage1D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]);
}

/* glCompressedTexImage2D */
typedef void APIENTRY (*pstub_glCompressedTexImage2D)(GLenum,GLint,GLenum,GLsizei,GLsizei,GLint,GLsizei,GLvoid*);
static pstub_glCompressedTexImage2D stub_glCompressedTexImage2D = NULL;
static int loaded_glCompressedTexImage2D = 0;

value glstub_glCompressedTexImage2D(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam3(v5,v6,v7);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLsizei lv3 =  Int_val(v3);
	GLsizei lv4 =  Int_val(v4);
	GLint lv5 =  Int_val(v5);
	GLsizei lv6 =  Int_val(v6);
	GLvoid* lv7 =  Data_bigarray_val(v7);

	if(stub_glCompressedTexImage2D){(*stub_glCompressedTexImage2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7);}
	else { 
		if(!loaded_glCompressedTexImage2D) { 
			loaded_glCompressedTexImage2D = 1;
			init_lib(); 
			stub_glCompressedTexImage2D=(pstub_glCompressedTexImage2D)get_proc_address("glCompressedTexImage2D");
			if(stub_glCompressedTexImage2D){(*stub_glCompressedTexImage2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7);}
			else { 
				FailWith("Cannot load glCompressedTexImage2D"); 
			} 
		} else { 
			FailWith("glCompressedTexImage2D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glCompressedTexImage2D_byte(value * argv, int n)
{
	return glstub_glCompressedTexImage2D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7]);
}

/* glCompressedTexImage3D */
typedef void APIENTRY (*pstub_glCompressedTexImage3D)(GLenum,GLint,GLenum,GLsizei,GLsizei,GLsizei,GLint,GLsizei,GLvoid*);
static pstub_glCompressedTexImage3D stub_glCompressedTexImage3D = NULL;
static int loaded_glCompressedTexImage3D = 0;

value glstub_glCompressedTexImage3D(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7, value v8)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam4(v5,v6,v7,v8);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLsizei lv3 =  Int_val(v3);
	GLsizei lv4 =  Int_val(v4);
	GLsizei lv5 =  Int_val(v5);
	GLint lv6 =  Int_val(v6);
	GLsizei lv7 =  Int_val(v7);
	GLvoid* lv8 =  Data_bigarray_val(v8);

	if(stub_glCompressedTexImage3D){(*stub_glCompressedTexImage3D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8);}
	else { 
		if(!loaded_glCompressedTexImage3D) { 
			loaded_glCompressedTexImage3D = 1;
			init_lib(); 
			stub_glCompressedTexImage3D=(pstub_glCompressedTexImage3D)get_proc_address("glCompressedTexImage3D");
			if(stub_glCompressedTexImage3D){(*stub_glCompressedTexImage3D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8);}
			else { 
				FailWith("Cannot load glCompressedTexImage3D"); 
			} 
		} else { 
			FailWith("glCompressedTexImage3D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glCompressedTexImage3D_byte(value * argv, int n)
{
	return glstub_glCompressedTexImage3D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8]);
}

/* glCompressedTexSubImage1D */
typedef void APIENTRY (*pstub_glCompressedTexSubImage1D)(GLenum,GLint,GLint,GLsizei,GLenum,GLsizei,GLvoid*);
static pstub_glCompressedTexSubImage1D stub_glCompressedTexSubImage1D = NULL;
static int loaded_glCompressedTexSubImage1D = 0;

value glstub_glCompressedTexSubImage1D(value v0, value v1, value v2, value v3, value v4, value v5, value v6)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam2(v5,v6);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLsizei lv3 =  Int_val(v3);
	GLenum lv4 =  get_glenum(v4);
	GLsizei lv5 =  Int_val(v5);
	GLvoid* lv6 =  Data_bigarray_val(v6);

	if(stub_glCompressedTexSubImage1D){(*stub_glCompressedTexSubImage1D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
	else { 
		if(!loaded_glCompressedTexSubImage1D) { 
			loaded_glCompressedTexSubImage1D = 1;
			init_lib(); 
			stub_glCompressedTexSubImage1D=(pstub_glCompressedTexSubImage1D)get_proc_address("glCompressedTexSubImage1D");
			if(stub_glCompressedTexSubImage1D){(*stub_glCompressedTexSubImage1D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
			else { 
				FailWith("Cannot load glCompressedTexSubImage1D"); 
			} 
		} else { 
			FailWith("glCompressedTexSubImage1D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glCompressedTexSubImage1D_byte(value * argv, int n)
{
	return glstub_glCompressedTexSubImage1D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]);
}

/* glCompressedTexSubImage2D */
typedef void APIENTRY (*pstub_glCompressedTexSubImage2D)(GLenum,GLint,GLint,GLint,GLsizei,GLsizei,GLenum,GLsizei,GLvoid*);
static pstub_glCompressedTexSubImage2D stub_glCompressedTexSubImage2D = NULL;
static int loaded_glCompressedTexSubImage2D = 0;

value glstub_glCompressedTexSubImage2D(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7, value v8)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam4(v5,v6,v7,v8);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLsizei lv4 =  Int_val(v4);
	GLsizei lv5 =  Int_val(v5);
	GLenum lv6 =  get_glenum(v6);
	GLsizei lv7 =  Int_val(v7);
	GLvoid* lv8 =  Data_bigarray_val(v8);

	if(stub_glCompressedTexSubImage2D){(*stub_glCompressedTexSubImage2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8);}
	else { 
		if(!loaded_glCompressedTexSubImage2D) { 
			loaded_glCompressedTexSubImage2D = 1;
			init_lib(); 
			stub_glCompressedTexSubImage2D=(pstub_glCompressedTexSubImage2D)get_proc_address("glCompressedTexSubImage2D");
			if(stub_glCompressedTexSubImage2D){(*stub_glCompressedTexSubImage2D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8);}
			else { 
				FailWith("Cannot load glCompressedTexSubImage2D"); 
			} 
		} else { 
			FailWith("glCompressedTexSubImage2D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glCompressedTexSubImage2D_byte(value * argv, int n)
{
	return glstub_glCompressedTexSubImage2D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8]);
}

/* glCompressedTexSubImage3D */
typedef void APIENTRY (*pstub_glCompressedTexSubImage3D)(GLenum,GLint,GLint,GLint,GLint,GLsizei,GLsizei,GLsizei,GLenum,GLsizei,GLvoid*);
static pstub_glCompressedTexSubImage3D stub_glCompressedTexSubImage3D = NULL;
static int loaded_glCompressedTexSubImage3D = 0;

value glstub_glCompressedTexSubImage3D(value v0, value v1, value v2, value v3, value v4, value v5, value v6, value v7, value v8, value v9, value v10)
{
	CAMLparam5(v0, v1, v2, v3, v4);
CAMLxparam5(v5, v6, v7, v8, v9);
	CAMLxparam1(v10);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);
	GLsizei lv5 =  Int_val(v5);
	GLsizei lv6 =  Int_val(v6);
	GLsizei lv7 =  Int_val(v7);
	GLenum lv8 =  get_glenum(v8);
	GLsizei lv9 =  Int_val(v9);
	GLvoid* lv10 =  Data_bigarray_val(v10);

	if(stub_glCompressedTexSubImage3D){(*stub_glCompressedTexSubImage3D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8, lv9, lv10);}
	else { 
		if(!loaded_glCompressedTexSubImage3D) { 
			loaded_glCompressedTexSubImage3D = 1;
			init_lib(); 
			stub_glCompressedTexSubImage3D=(pstub_glCompressedTexSubImage3D)get_proc_address("glCompressedTexSubImage3D");
			if(stub_glCompressedTexSubImage3D){(*stub_glCompressedTexSubImage3D)(lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8, lv9, lv10);}
			else { 
				FailWith("Cannot load glCompressedTexSubImage3D"); 
			} 
		} else { 
			FailWith("glCompressedTexSubImage3D not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glCompressedTexSubImage3D_byte(value * argv, int n)
{
	return glstub_glCompressedTexSubImage3D(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8], argv[9], argv[10]);
}

/* glGetCompressedTexImage */
typedef void APIENTRY (*pstub_glGetCompressedTexImage)(GLenum,GLint,GLvoid*);
static pstub_glGetCompressedTexImage stub_glGetCompressedTexImage = NULL;
static int loaded_glGetCompressedTexImage = 0;

value glstub_glGetCompressedTexImage(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLvoid* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetCompressedTexImage){(*stub_glGetCompressedTexImage)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetCompressedTexImage) { 
			loaded_glGetCompressedTexImage = 1;
			init_lib(); 
			stub_glGetCompressedTexImage=(pstub_glGetCompressedTexImage)get_proc_address("glGetCompressedTexImage");
			if(stub_glGetCompressedTexImage){(*stub_glGetCompressedTexImage)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetCompressedTexImage"); 
			} 
		} else { 
			FailWith("glGetCompressedTexImage not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLoadTransposeMatrixd */
typedef void APIENTRY (*pstub_glLoadTransposeMatrixd)(GLdouble);
static pstub_glLoadTransposeMatrixd stub_glLoadTransposeMatrixd = NULL;
static int loaded_glLoadTransposeMatrixd = 0;

value glstub_glLoadTransposeMatrixd(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);

	if(stub_glLoadTransposeMatrixd){(*stub_glLoadTransposeMatrixd)(lv0);}
	else { 
		if(!loaded_glLoadTransposeMatrixd) { 
			loaded_glLoadTransposeMatrixd = 1;
			init_lib(); 
			stub_glLoadTransposeMatrixd=(pstub_glLoadTransposeMatrixd)get_proc_address("glLoadTransposeMatrixd");
			if(stub_glLoadTransposeMatrixd){(*stub_glLoadTransposeMatrixd)(lv0);}
			else { 
				FailWith("Cannot load glLoadTransposeMatrixd"); 
			} 
		} else { 
			FailWith("glLoadTransposeMatrixd not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glLoadTransposeMatrixf */
typedef void APIENTRY (*pstub_glLoadTransposeMatrixf)(GLfloat);
static pstub_glLoadTransposeMatrixf stub_glLoadTransposeMatrixf = NULL;
static int loaded_glLoadTransposeMatrixf = 0;

value glstub_glLoadTransposeMatrixf(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);

	if(stub_glLoadTransposeMatrixf){(*stub_glLoadTransposeMatrixf)(lv0);}
	else { 
		if(!loaded_glLoadTransposeMatrixf) { 
			loaded_glLoadTransposeMatrixf = 1;
			init_lib(); 
			stub_glLoadTransposeMatrixf=(pstub_glLoadTransposeMatrixf)get_proc_address("glLoadTransposeMatrixf");
			if(stub_glLoadTransposeMatrixf){(*stub_glLoadTransposeMatrixf)(lv0);}
			else { 
				FailWith("Cannot load glLoadTransposeMatrixf"); 
			} 
		} else { 
			FailWith("glLoadTransposeMatrixf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultTransposeMatrixd */
typedef void APIENTRY (*pstub_glMultTransposeMatrixd)(GLdouble);
static pstub_glMultTransposeMatrixd stub_glMultTransposeMatrixd = NULL;
static int loaded_glMultTransposeMatrixd = 0;

value glstub_glMultTransposeMatrixd(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);

	if(stub_glMultTransposeMatrixd){(*stub_glMultTransposeMatrixd)(lv0);}
	else { 
		if(!loaded_glMultTransposeMatrixd) { 
			loaded_glMultTransposeMatrixd = 1;
			init_lib(); 
			stub_glMultTransposeMatrixd=(pstub_glMultTransposeMatrixd)get_proc_address("glMultTransposeMatrixd");
			if(stub_glMultTransposeMatrixd){(*stub_glMultTransposeMatrixd)(lv0);}
			else { 
				FailWith("Cannot load glMultTransposeMatrixd"); 
			} 
		} else { 
			FailWith("glMultTransposeMatrixd not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultTransposeMatrixf */
typedef void APIENTRY (*pstub_glMultTransposeMatrixf)(GLfloat);
static pstub_glMultTransposeMatrixf stub_glMultTransposeMatrixf = NULL;
static int loaded_glMultTransposeMatrixf = 0;

value glstub_glMultTransposeMatrixf(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);

	if(stub_glMultTransposeMatrixf){(*stub_glMultTransposeMatrixf)(lv0);}
	else { 
		if(!loaded_glMultTransposeMatrixf) { 
			loaded_glMultTransposeMatrixf = 1;
			init_lib(); 
			stub_glMultTransposeMatrixf=(pstub_glMultTransposeMatrixf)get_proc_address("glMultTransposeMatrixf");
			if(stub_glMultTransposeMatrixf){(*stub_glMultTransposeMatrixf)(lv0);}
			else { 
				FailWith("Cannot load glMultTransposeMatrixf"); 
			} 
		} else { 
			FailWith("glMultTransposeMatrixf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1d */
typedef void APIENTRY (*pstub_glMultiTexCoord1d)(GLenum,GLdouble);
static pstub_glMultiTexCoord1d stub_glMultiTexCoord1d = NULL;
static int loaded_glMultiTexCoord1d = 0;

value glstub_glMultiTexCoord1d(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);

	if(stub_glMultiTexCoord1d){(*stub_glMultiTexCoord1d)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1d) { 
			loaded_glMultiTexCoord1d = 1;
			init_lib(); 
			stub_glMultiTexCoord1d=(pstub_glMultiTexCoord1d)get_proc_address("glMultiTexCoord1d");
			if(stub_glMultiTexCoord1d){(*stub_glMultiTexCoord1d)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1d"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1dv */
typedef void APIENTRY (*pstub_glMultiTexCoord1dv)(GLenum,GLdouble*);
static pstub_glMultiTexCoord1dv stub_glMultiTexCoord1dv = NULL;
static int loaded_glMultiTexCoord1dv = 0;

value glstub_glMultiTexCoord1dv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord1dv){(*stub_glMultiTexCoord1dv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1dv) { 
			loaded_glMultiTexCoord1dv = 1;
			init_lib(); 
			stub_glMultiTexCoord1dv=(pstub_glMultiTexCoord1dv)get_proc_address("glMultiTexCoord1dv");
			if(stub_glMultiTexCoord1dv){(*stub_glMultiTexCoord1dv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1dv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1f */
typedef void APIENTRY (*pstub_glMultiTexCoord1f)(GLenum,GLfloat);
static pstub_glMultiTexCoord1f stub_glMultiTexCoord1f = NULL;
static int loaded_glMultiTexCoord1f = 0;

value glstub_glMultiTexCoord1f(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glMultiTexCoord1f){(*stub_glMultiTexCoord1f)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1f) { 
			loaded_glMultiTexCoord1f = 1;
			init_lib(); 
			stub_glMultiTexCoord1f=(pstub_glMultiTexCoord1f)get_proc_address("glMultiTexCoord1f");
			if(stub_glMultiTexCoord1f){(*stub_glMultiTexCoord1f)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1f"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1fv */
typedef void APIENTRY (*pstub_glMultiTexCoord1fv)(GLenum,GLfloat*);
static pstub_glMultiTexCoord1fv stub_glMultiTexCoord1fv = NULL;
static int loaded_glMultiTexCoord1fv = 0;

value glstub_glMultiTexCoord1fv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord1fv){(*stub_glMultiTexCoord1fv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1fv) { 
			loaded_glMultiTexCoord1fv = 1;
			init_lib(); 
			stub_glMultiTexCoord1fv=(pstub_glMultiTexCoord1fv)get_proc_address("glMultiTexCoord1fv");
			if(stub_glMultiTexCoord1fv){(*stub_glMultiTexCoord1fv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1fv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1i */
typedef void APIENTRY (*pstub_glMultiTexCoord1i)(GLenum,GLint);
static pstub_glMultiTexCoord1i stub_glMultiTexCoord1i = NULL;
static int loaded_glMultiTexCoord1i = 0;

value glstub_glMultiTexCoord1i(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glMultiTexCoord1i){(*stub_glMultiTexCoord1i)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1i) { 
			loaded_glMultiTexCoord1i = 1;
			init_lib(); 
			stub_glMultiTexCoord1i=(pstub_glMultiTexCoord1i)get_proc_address("glMultiTexCoord1i");
			if(stub_glMultiTexCoord1i){(*stub_glMultiTexCoord1i)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1i"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1iv */
typedef void APIENTRY (*pstub_glMultiTexCoord1iv)(GLenum,GLint*);
static pstub_glMultiTexCoord1iv stub_glMultiTexCoord1iv = NULL;
static int loaded_glMultiTexCoord1iv = 0;

value glstub_glMultiTexCoord1iv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord1iv){(*stub_glMultiTexCoord1iv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1iv) { 
			loaded_glMultiTexCoord1iv = 1;
			init_lib(); 
			stub_glMultiTexCoord1iv=(pstub_glMultiTexCoord1iv)get_proc_address("glMultiTexCoord1iv");
			if(stub_glMultiTexCoord1iv){(*stub_glMultiTexCoord1iv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1iv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1s */
typedef void APIENTRY (*pstub_glMultiTexCoord1s)(GLenum,GLshort);
static pstub_glMultiTexCoord1s stub_glMultiTexCoord1s = NULL;
static int loaded_glMultiTexCoord1s = 0;

value glstub_glMultiTexCoord1s(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort lv1 =  Int_val(v1);

	if(stub_glMultiTexCoord1s){(*stub_glMultiTexCoord1s)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1s) { 
			loaded_glMultiTexCoord1s = 1;
			init_lib(); 
			stub_glMultiTexCoord1s=(pstub_glMultiTexCoord1s)get_proc_address("glMultiTexCoord1s");
			if(stub_glMultiTexCoord1s){(*stub_glMultiTexCoord1s)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1s"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord1sv */
typedef void APIENTRY (*pstub_glMultiTexCoord1sv)(GLenum,GLshort*);
static pstub_glMultiTexCoord1sv stub_glMultiTexCoord1sv = NULL;
static int loaded_glMultiTexCoord1sv = 0;

value glstub_glMultiTexCoord1sv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord1sv){(*stub_glMultiTexCoord1sv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord1sv) { 
			loaded_glMultiTexCoord1sv = 1;
			init_lib(); 
			stub_glMultiTexCoord1sv=(pstub_glMultiTexCoord1sv)get_proc_address("glMultiTexCoord1sv");
			if(stub_glMultiTexCoord1sv){(*stub_glMultiTexCoord1sv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord1sv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord1sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2d */
typedef void APIENTRY (*pstub_glMultiTexCoord2d)(GLenum,GLdouble,GLdouble);
static pstub_glMultiTexCoord2d stub_glMultiTexCoord2d = NULL;
static int loaded_glMultiTexCoord2d = 0;

value glstub_glMultiTexCoord2d(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glMultiTexCoord2d){(*stub_glMultiTexCoord2d)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMultiTexCoord2d) { 
			loaded_glMultiTexCoord2d = 1;
			init_lib(); 
			stub_glMultiTexCoord2d=(pstub_glMultiTexCoord2d)get_proc_address("glMultiTexCoord2d");
			if(stub_glMultiTexCoord2d){(*stub_glMultiTexCoord2d)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMultiTexCoord2d"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2dv */
typedef void APIENTRY (*pstub_glMultiTexCoord2dv)(GLenum,GLdouble*);
static pstub_glMultiTexCoord2dv stub_glMultiTexCoord2dv = NULL;
static int loaded_glMultiTexCoord2dv = 0;

value glstub_glMultiTexCoord2dv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord2dv){(*stub_glMultiTexCoord2dv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord2dv) { 
			loaded_glMultiTexCoord2dv = 1;
			init_lib(); 
			stub_glMultiTexCoord2dv=(pstub_glMultiTexCoord2dv)get_proc_address("glMultiTexCoord2dv");
			if(stub_glMultiTexCoord2dv){(*stub_glMultiTexCoord2dv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord2dv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2f */
typedef void APIENTRY (*pstub_glMultiTexCoord2f)(GLenum,GLfloat,GLfloat);
static pstub_glMultiTexCoord2f stub_glMultiTexCoord2f = NULL;
static int loaded_glMultiTexCoord2f = 0;

value glstub_glMultiTexCoord2f(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glMultiTexCoord2f){(*stub_glMultiTexCoord2f)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMultiTexCoord2f) { 
			loaded_glMultiTexCoord2f = 1;
			init_lib(); 
			stub_glMultiTexCoord2f=(pstub_glMultiTexCoord2f)get_proc_address("glMultiTexCoord2f");
			if(stub_glMultiTexCoord2f){(*stub_glMultiTexCoord2f)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMultiTexCoord2f"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2fv */
typedef void APIENTRY (*pstub_glMultiTexCoord2fv)(GLenum,GLfloat*);
static pstub_glMultiTexCoord2fv stub_glMultiTexCoord2fv = NULL;
static int loaded_glMultiTexCoord2fv = 0;

value glstub_glMultiTexCoord2fv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord2fv){(*stub_glMultiTexCoord2fv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord2fv) { 
			loaded_glMultiTexCoord2fv = 1;
			init_lib(); 
			stub_glMultiTexCoord2fv=(pstub_glMultiTexCoord2fv)get_proc_address("glMultiTexCoord2fv");
			if(stub_glMultiTexCoord2fv){(*stub_glMultiTexCoord2fv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord2fv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2i */
typedef void APIENTRY (*pstub_glMultiTexCoord2i)(GLenum,GLint,GLint);
static pstub_glMultiTexCoord2i stub_glMultiTexCoord2i = NULL;
static int loaded_glMultiTexCoord2i = 0;

value glstub_glMultiTexCoord2i(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glMultiTexCoord2i){(*stub_glMultiTexCoord2i)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMultiTexCoord2i) { 
			loaded_glMultiTexCoord2i = 1;
			init_lib(); 
			stub_glMultiTexCoord2i=(pstub_glMultiTexCoord2i)get_proc_address("glMultiTexCoord2i");
			if(stub_glMultiTexCoord2i){(*stub_glMultiTexCoord2i)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMultiTexCoord2i"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2iv */
typedef void APIENTRY (*pstub_glMultiTexCoord2iv)(GLenum,GLint*);
static pstub_glMultiTexCoord2iv stub_glMultiTexCoord2iv = NULL;
static int loaded_glMultiTexCoord2iv = 0;

value glstub_glMultiTexCoord2iv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord2iv){(*stub_glMultiTexCoord2iv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord2iv) { 
			loaded_glMultiTexCoord2iv = 1;
			init_lib(); 
			stub_glMultiTexCoord2iv=(pstub_glMultiTexCoord2iv)get_proc_address("glMultiTexCoord2iv");
			if(stub_glMultiTexCoord2iv){(*stub_glMultiTexCoord2iv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord2iv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2s */
typedef void APIENTRY (*pstub_glMultiTexCoord2s)(GLenum,GLshort,GLshort);
static pstub_glMultiTexCoord2s stub_glMultiTexCoord2s = NULL;
static int loaded_glMultiTexCoord2s = 0;

value glstub_glMultiTexCoord2s(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);

	if(stub_glMultiTexCoord2s){(*stub_glMultiTexCoord2s)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glMultiTexCoord2s) { 
			loaded_glMultiTexCoord2s = 1;
			init_lib(); 
			stub_glMultiTexCoord2s=(pstub_glMultiTexCoord2s)get_proc_address("glMultiTexCoord2s");
			if(stub_glMultiTexCoord2s){(*stub_glMultiTexCoord2s)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glMultiTexCoord2s"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord2sv */
typedef void APIENTRY (*pstub_glMultiTexCoord2sv)(GLenum,GLshort*);
static pstub_glMultiTexCoord2sv stub_glMultiTexCoord2sv = NULL;
static int loaded_glMultiTexCoord2sv = 0;

value glstub_glMultiTexCoord2sv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord2sv){(*stub_glMultiTexCoord2sv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord2sv) { 
			loaded_glMultiTexCoord2sv = 1;
			init_lib(); 
			stub_glMultiTexCoord2sv=(pstub_glMultiTexCoord2sv)get_proc_address("glMultiTexCoord2sv");
			if(stub_glMultiTexCoord2sv){(*stub_glMultiTexCoord2sv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord2sv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord2sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3d */
typedef void APIENTRY (*pstub_glMultiTexCoord3d)(GLenum,GLdouble,GLdouble,GLdouble);
static pstub_glMultiTexCoord3d stub_glMultiTexCoord3d = NULL;
static int loaded_glMultiTexCoord3d = 0;

value glstub_glMultiTexCoord3d(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);

	if(stub_glMultiTexCoord3d){(*stub_glMultiTexCoord3d)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glMultiTexCoord3d) { 
			loaded_glMultiTexCoord3d = 1;
			init_lib(); 
			stub_glMultiTexCoord3d=(pstub_glMultiTexCoord3d)get_proc_address("glMultiTexCoord3d");
			if(stub_glMultiTexCoord3d){(*stub_glMultiTexCoord3d)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glMultiTexCoord3d"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3dv */
typedef void APIENTRY (*pstub_glMultiTexCoord3dv)(GLenum,GLdouble*);
static pstub_glMultiTexCoord3dv stub_glMultiTexCoord3dv = NULL;
static int loaded_glMultiTexCoord3dv = 0;

value glstub_glMultiTexCoord3dv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord3dv){(*stub_glMultiTexCoord3dv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord3dv) { 
			loaded_glMultiTexCoord3dv = 1;
			init_lib(); 
			stub_glMultiTexCoord3dv=(pstub_glMultiTexCoord3dv)get_proc_address("glMultiTexCoord3dv");
			if(stub_glMultiTexCoord3dv){(*stub_glMultiTexCoord3dv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord3dv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3f */
typedef void APIENTRY (*pstub_glMultiTexCoord3f)(GLenum,GLfloat,GLfloat,GLfloat);
static pstub_glMultiTexCoord3f stub_glMultiTexCoord3f = NULL;
static int loaded_glMultiTexCoord3f = 0;

value glstub_glMultiTexCoord3f(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);

	if(stub_glMultiTexCoord3f){(*stub_glMultiTexCoord3f)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glMultiTexCoord3f) { 
			loaded_glMultiTexCoord3f = 1;
			init_lib(); 
			stub_glMultiTexCoord3f=(pstub_glMultiTexCoord3f)get_proc_address("glMultiTexCoord3f");
			if(stub_glMultiTexCoord3f){(*stub_glMultiTexCoord3f)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glMultiTexCoord3f"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3fv */
typedef void APIENTRY (*pstub_glMultiTexCoord3fv)(GLenum,GLfloat*);
static pstub_glMultiTexCoord3fv stub_glMultiTexCoord3fv = NULL;
static int loaded_glMultiTexCoord3fv = 0;

value glstub_glMultiTexCoord3fv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord3fv){(*stub_glMultiTexCoord3fv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord3fv) { 
			loaded_glMultiTexCoord3fv = 1;
			init_lib(); 
			stub_glMultiTexCoord3fv=(pstub_glMultiTexCoord3fv)get_proc_address("glMultiTexCoord3fv");
			if(stub_glMultiTexCoord3fv){(*stub_glMultiTexCoord3fv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord3fv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3i */
typedef void APIENTRY (*pstub_glMultiTexCoord3i)(GLenum,GLint,GLint,GLint);
static pstub_glMultiTexCoord3i stub_glMultiTexCoord3i = NULL;
static int loaded_glMultiTexCoord3i = 0;

value glstub_glMultiTexCoord3i(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);

	if(stub_glMultiTexCoord3i){(*stub_glMultiTexCoord3i)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glMultiTexCoord3i) { 
			loaded_glMultiTexCoord3i = 1;
			init_lib(); 
			stub_glMultiTexCoord3i=(pstub_glMultiTexCoord3i)get_proc_address("glMultiTexCoord3i");
			if(stub_glMultiTexCoord3i){(*stub_glMultiTexCoord3i)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glMultiTexCoord3i"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3iv */
typedef void APIENTRY (*pstub_glMultiTexCoord3iv)(GLenum,GLint*);
static pstub_glMultiTexCoord3iv stub_glMultiTexCoord3iv = NULL;
static int loaded_glMultiTexCoord3iv = 0;

value glstub_glMultiTexCoord3iv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord3iv){(*stub_glMultiTexCoord3iv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord3iv) { 
			loaded_glMultiTexCoord3iv = 1;
			init_lib(); 
			stub_glMultiTexCoord3iv=(pstub_glMultiTexCoord3iv)get_proc_address("glMultiTexCoord3iv");
			if(stub_glMultiTexCoord3iv){(*stub_glMultiTexCoord3iv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord3iv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3s */
typedef void APIENTRY (*pstub_glMultiTexCoord3s)(GLenum,GLshort,GLshort,GLshort);
static pstub_glMultiTexCoord3s stub_glMultiTexCoord3s = NULL;
static int loaded_glMultiTexCoord3s = 0;

value glstub_glMultiTexCoord3s(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);

	if(stub_glMultiTexCoord3s){(*stub_glMultiTexCoord3s)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glMultiTexCoord3s) { 
			loaded_glMultiTexCoord3s = 1;
			init_lib(); 
			stub_glMultiTexCoord3s=(pstub_glMultiTexCoord3s)get_proc_address("glMultiTexCoord3s");
			if(stub_glMultiTexCoord3s){(*stub_glMultiTexCoord3s)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glMultiTexCoord3s"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord3sv */
typedef void APIENTRY (*pstub_glMultiTexCoord3sv)(GLenum,GLshort*);
static pstub_glMultiTexCoord3sv stub_glMultiTexCoord3sv = NULL;
static int loaded_glMultiTexCoord3sv = 0;

value glstub_glMultiTexCoord3sv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord3sv){(*stub_glMultiTexCoord3sv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord3sv) { 
			loaded_glMultiTexCoord3sv = 1;
			init_lib(); 
			stub_glMultiTexCoord3sv=(pstub_glMultiTexCoord3sv)get_proc_address("glMultiTexCoord3sv");
			if(stub_glMultiTexCoord3sv){(*stub_glMultiTexCoord3sv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord3sv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord3sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4d */
typedef void APIENTRY (*pstub_glMultiTexCoord4d)(GLenum,GLdouble,GLdouble,GLdouble,GLdouble);
static pstub_glMultiTexCoord4d stub_glMultiTexCoord4d = NULL;
static int loaded_glMultiTexCoord4d = 0;

value glstub_glMultiTexCoord4d(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);
	GLdouble lv4 =  Double_val(v4);

	if(stub_glMultiTexCoord4d){(*stub_glMultiTexCoord4d)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glMultiTexCoord4d) { 
			loaded_glMultiTexCoord4d = 1;
			init_lib(); 
			stub_glMultiTexCoord4d=(pstub_glMultiTexCoord4d)get_proc_address("glMultiTexCoord4d");
			if(stub_glMultiTexCoord4d){(*stub_glMultiTexCoord4d)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glMultiTexCoord4d"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4dv */
typedef void APIENTRY (*pstub_glMultiTexCoord4dv)(GLenum,GLdouble*);
static pstub_glMultiTexCoord4dv stub_glMultiTexCoord4dv = NULL;
static int loaded_glMultiTexCoord4dv = 0;

value glstub_glMultiTexCoord4dv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord4dv){(*stub_glMultiTexCoord4dv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord4dv) { 
			loaded_glMultiTexCoord4dv = 1;
			init_lib(); 
			stub_glMultiTexCoord4dv=(pstub_glMultiTexCoord4dv)get_proc_address("glMultiTexCoord4dv");
			if(stub_glMultiTexCoord4dv){(*stub_glMultiTexCoord4dv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord4dv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4f */
typedef void APIENTRY (*pstub_glMultiTexCoord4f)(GLenum,GLfloat,GLfloat,GLfloat,GLfloat);
static pstub_glMultiTexCoord4f stub_glMultiTexCoord4f = NULL;
static int loaded_glMultiTexCoord4f = 0;

value glstub_glMultiTexCoord4f(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);
	GLfloat lv4 =  Double_val(v4);

	if(stub_glMultiTexCoord4f){(*stub_glMultiTexCoord4f)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glMultiTexCoord4f) { 
			loaded_glMultiTexCoord4f = 1;
			init_lib(); 
			stub_glMultiTexCoord4f=(pstub_glMultiTexCoord4f)get_proc_address("glMultiTexCoord4f");
			if(stub_glMultiTexCoord4f){(*stub_glMultiTexCoord4f)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glMultiTexCoord4f"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4fv */
typedef void APIENTRY (*pstub_glMultiTexCoord4fv)(GLenum,GLfloat*);
static pstub_glMultiTexCoord4fv stub_glMultiTexCoord4fv = NULL;
static int loaded_glMultiTexCoord4fv = 0;

value glstub_glMultiTexCoord4fv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord4fv){(*stub_glMultiTexCoord4fv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord4fv) { 
			loaded_glMultiTexCoord4fv = 1;
			init_lib(); 
			stub_glMultiTexCoord4fv=(pstub_glMultiTexCoord4fv)get_proc_address("glMultiTexCoord4fv");
			if(stub_glMultiTexCoord4fv){(*stub_glMultiTexCoord4fv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord4fv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4i */
typedef void APIENTRY (*pstub_glMultiTexCoord4i)(GLenum,GLint,GLint,GLint,GLint);
static pstub_glMultiTexCoord4i stub_glMultiTexCoord4i = NULL;
static int loaded_glMultiTexCoord4i = 0;

value glstub_glMultiTexCoord4i(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);

	if(stub_glMultiTexCoord4i){(*stub_glMultiTexCoord4i)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glMultiTexCoord4i) { 
			loaded_glMultiTexCoord4i = 1;
			init_lib(); 
			stub_glMultiTexCoord4i=(pstub_glMultiTexCoord4i)get_proc_address("glMultiTexCoord4i");
			if(stub_glMultiTexCoord4i){(*stub_glMultiTexCoord4i)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glMultiTexCoord4i"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4iv */
typedef void APIENTRY (*pstub_glMultiTexCoord4iv)(GLenum,GLint*);
static pstub_glMultiTexCoord4iv stub_glMultiTexCoord4iv = NULL;
static int loaded_glMultiTexCoord4iv = 0;

value glstub_glMultiTexCoord4iv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord4iv){(*stub_glMultiTexCoord4iv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord4iv) { 
			loaded_glMultiTexCoord4iv = 1;
			init_lib(); 
			stub_glMultiTexCoord4iv=(pstub_glMultiTexCoord4iv)get_proc_address("glMultiTexCoord4iv");
			if(stub_glMultiTexCoord4iv){(*stub_glMultiTexCoord4iv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord4iv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4s */
typedef void APIENTRY (*pstub_glMultiTexCoord4s)(GLenum,GLshort,GLshort,GLshort,GLshort);
static pstub_glMultiTexCoord4s stub_glMultiTexCoord4s = NULL;
static int loaded_glMultiTexCoord4s = 0;

value glstub_glMultiTexCoord4s(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);
	GLshort lv4 =  Int_val(v4);

	if(stub_glMultiTexCoord4s){(*stub_glMultiTexCoord4s)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glMultiTexCoord4s) { 
			loaded_glMultiTexCoord4s = 1;
			init_lib(); 
			stub_glMultiTexCoord4s=(pstub_glMultiTexCoord4s)get_proc_address("glMultiTexCoord4s");
			if(stub_glMultiTexCoord4s){(*stub_glMultiTexCoord4s)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glMultiTexCoord4s"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiTexCoord4sv */
typedef void APIENTRY (*pstub_glMultiTexCoord4sv)(GLenum,GLshort*);
static pstub_glMultiTexCoord4sv stub_glMultiTexCoord4sv = NULL;
static int loaded_glMultiTexCoord4sv = 0;

value glstub_glMultiTexCoord4sv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glMultiTexCoord4sv){(*stub_glMultiTexCoord4sv)(lv0, lv1);}
	else { 
		if(!loaded_glMultiTexCoord4sv) { 
			loaded_glMultiTexCoord4sv = 1;
			init_lib(); 
			stub_glMultiTexCoord4sv=(pstub_glMultiTexCoord4sv)get_proc_address("glMultiTexCoord4sv");
			if(stub_glMultiTexCoord4sv){(*stub_glMultiTexCoord4sv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMultiTexCoord4sv"); 
			} 
		} else { 
			FailWith("glMultiTexCoord4sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSampleCoverage */
typedef void APIENTRY (*pstub_glSampleCoverage)(GLclampf,GLboolean);
static pstub_glSampleCoverage stub_glSampleCoverage = NULL;
static int loaded_glSampleCoverage = 0;

value glstub_glSampleCoverage(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLclampf lv0 =  Double_val(v0);
	GLboolean lv1 =  Bool_val(v1);

	if(stub_glSampleCoverage){(*stub_glSampleCoverage)(lv0, lv1);}
	else { 
		if(!loaded_glSampleCoverage) { 
			loaded_glSampleCoverage = 1;
			init_lib(); 
			stub_glSampleCoverage=(pstub_glSampleCoverage)get_proc_address("glSampleCoverage");
			if(stub_glSampleCoverage){(*stub_glSampleCoverage)(lv0, lv1);}
			else { 
				FailWith("Cannot load glSampleCoverage"); 
			} 
		} else { 
			FailWith("glSampleCoverage not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBlendEquation */
typedef void APIENTRY (*pstub_glBlendEquation)(GLenum);
static pstub_glBlendEquation stub_glBlendEquation = NULL;
static int loaded_glBlendEquation = 0;

value glstub_glBlendEquation(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glBlendEquation){(*stub_glBlendEquation)(lv0);}
	else { 
		if(!loaded_glBlendEquation) { 
			loaded_glBlendEquation = 1;
			init_lib(); 
			stub_glBlendEquation=(pstub_glBlendEquation)get_proc_address("glBlendEquation");
			if(stub_glBlendEquation){(*stub_glBlendEquation)(lv0);}
			else { 
				FailWith("Cannot load glBlendEquation"); 
			} 
		} else { 
			FailWith("glBlendEquation not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBlendColor */
typedef void APIENTRY (*pstub_glBlendColor)(GLclampf,GLclampf,GLclampf,GLclampf);
static pstub_glBlendColor stub_glBlendColor = NULL;
static int loaded_glBlendColor = 0;

value glstub_glBlendColor(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLclampf lv0 =  Double_val(v0);
	GLclampf lv1 =  Double_val(v1);
	GLclampf lv2 =  Double_val(v2);
	GLclampf lv3 =  Double_val(v3);

	if(stub_glBlendColor){(*stub_glBlendColor)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glBlendColor) { 
			loaded_glBlendColor = 1;
			init_lib(); 
			stub_glBlendColor=(pstub_glBlendColor)get_proc_address("glBlendColor");
			if(stub_glBlendColor){(*stub_glBlendColor)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glBlendColor"); 
			} 
		} else { 
			FailWith("glBlendColor not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFogCoordf */
typedef void APIENTRY (*pstub_glFogCoordf)(GLfloat);
static pstub_glFogCoordf stub_glFogCoordf = NULL;
static int loaded_glFogCoordf = 0;

value glstub_glFogCoordf(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);

	if(stub_glFogCoordf){(*stub_glFogCoordf)(lv0);}
	else { 
		if(!loaded_glFogCoordf) { 
			loaded_glFogCoordf = 1;
			init_lib(); 
			stub_glFogCoordf=(pstub_glFogCoordf)get_proc_address("glFogCoordf");
			if(stub_glFogCoordf){(*stub_glFogCoordf)(lv0);}
			else { 
				FailWith("Cannot load glFogCoordf"); 
			} 
		} else { 
			FailWith("glFogCoordf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFogCoordfv */
typedef void APIENTRY (*pstub_glFogCoordfv)(GLfloat*);
static pstub_glFogCoordfv stub_glFogCoordfv = NULL;
static int loaded_glFogCoordfv = 0;

value glstub_glFogCoordfv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glFogCoordfv){(*stub_glFogCoordfv)(lv0);}
	else { 
		if(!loaded_glFogCoordfv) { 
			loaded_glFogCoordfv = 1;
			init_lib(); 
			stub_glFogCoordfv=(pstub_glFogCoordfv)get_proc_address("glFogCoordfv");
			if(stub_glFogCoordfv){(*stub_glFogCoordfv)(lv0);}
			else { 
				FailWith("Cannot load glFogCoordfv"); 
			} 
		} else { 
			FailWith("glFogCoordfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFogCoordd */
typedef void APIENTRY (*pstub_glFogCoordd)(GLdouble);
static pstub_glFogCoordd stub_glFogCoordd = NULL;
static int loaded_glFogCoordd = 0;

value glstub_glFogCoordd(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);

	if(stub_glFogCoordd){(*stub_glFogCoordd)(lv0);}
	else { 
		if(!loaded_glFogCoordd) { 
			loaded_glFogCoordd = 1;
			init_lib(); 
			stub_glFogCoordd=(pstub_glFogCoordd)get_proc_address("glFogCoordd");
			if(stub_glFogCoordd){(*stub_glFogCoordd)(lv0);}
			else { 
				FailWith("Cannot load glFogCoordd"); 
			} 
		} else { 
			FailWith("glFogCoordd not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFogCoorddv */
typedef void APIENTRY (*pstub_glFogCoorddv)(GLdouble*);
static pstub_glFogCoorddv stub_glFogCoorddv = NULL;
static int loaded_glFogCoorddv = 0;

value glstub_glFogCoorddv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glFogCoorddv){(*stub_glFogCoorddv)(lv0);}
	else { 
		if(!loaded_glFogCoorddv) { 
			loaded_glFogCoorddv = 1;
			init_lib(); 
			stub_glFogCoorddv=(pstub_glFogCoorddv)get_proc_address("glFogCoorddv");
			if(stub_glFogCoorddv){(*stub_glFogCoorddv)(lv0);}
			else { 
				FailWith("Cannot load glFogCoorddv"); 
			} 
		} else { 
			FailWith("glFogCoorddv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glFogCoordPointer */
typedef void APIENTRY (*pstub_glFogCoordPointer)(GLenum,GLsizei,GLvoid*);
static pstub_glFogCoordPointer stub_glFogCoordPointer = NULL;
static int loaded_glFogCoordPointer = 0;

value glstub_glFogCoordPointer(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizei lv1 =  Int_val(v1);
	GLvoid* lv2 =  Data_bigarray_val(v2);

	if(stub_glFogCoordPointer){(*stub_glFogCoordPointer)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glFogCoordPointer) { 
			loaded_glFogCoordPointer = 1;
			init_lib(); 
			stub_glFogCoordPointer=(pstub_glFogCoordPointer)get_proc_address("glFogCoordPointer");
			if(stub_glFogCoordPointer){(*stub_glFogCoordPointer)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glFogCoordPointer"); 
			} 
		} else { 
			FailWith("glFogCoordPointer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiDrawArrays */
typedef void APIENTRY (*pstub_glMultiDrawArrays)(GLenum,GLint*,GLsizei*,GLsizei);
static pstub_glMultiDrawArrays stub_glMultiDrawArrays = NULL;
static int loaded_glMultiDrawArrays = 0;

value glstub_glMultiDrawArrays(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLint* lv1 =  Data_bigarray_val(v1);
	GLsizei* lv2 =  Data_bigarray_val(v2);
	GLsizei lv3 =  Int_val(v3);

	if(stub_glMultiDrawArrays){(*stub_glMultiDrawArrays)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glMultiDrawArrays) { 
			loaded_glMultiDrawArrays = 1;
			init_lib(); 
			stub_glMultiDrawArrays=(pstub_glMultiDrawArrays)get_proc_address("glMultiDrawArrays");
			if(stub_glMultiDrawArrays){(*stub_glMultiDrawArrays)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glMultiDrawArrays"); 
			} 
		} else { 
			FailWith("glMultiDrawArrays not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMultiDrawElements */
typedef void APIENTRY (*pstub_glMultiDrawElements)(GLenum,GLsizei*,GLenum,GLvoid**,GLsizei);
static pstub_glMultiDrawElements stub_glMultiDrawElements = NULL;
static int loaded_glMultiDrawElements = 0;

value glstub_glMultiDrawElements(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizei* lv1 =  Data_bigarray_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLvoid** lv3 =  Data_bigarray_val(v3);
	GLsizei lv4 =  Int_val(v4);

	if(stub_glMultiDrawElements){(*stub_glMultiDrawElements)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glMultiDrawElements) { 
			loaded_glMultiDrawElements = 1;
			init_lib(); 
			stub_glMultiDrawElements=(pstub_glMultiDrawElements)get_proc_address("glMultiDrawElements");
			if(stub_glMultiDrawElements){(*stub_glMultiDrawElements)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glMultiDrawElements"); 
			} 
		} else { 
			FailWith("glMultiDrawElements not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPointParameterf */
typedef void APIENTRY (*pstub_glPointParameterf)(GLenum,GLfloat);
static pstub_glPointParameterf stub_glPointParameterf = NULL;
static int loaded_glPointParameterf = 0;

value glstub_glPointParameterf(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glPointParameterf){(*stub_glPointParameterf)(lv0, lv1);}
	else { 
		if(!loaded_glPointParameterf) { 
			loaded_glPointParameterf = 1;
			init_lib(); 
			stub_glPointParameterf=(pstub_glPointParameterf)get_proc_address("glPointParameterf");
			if(stub_glPointParameterf){(*stub_glPointParameterf)(lv0, lv1);}
			else { 
				FailWith("Cannot load glPointParameterf"); 
			} 
		} else { 
			FailWith("glPointParameterf not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glPointParameterfv */
typedef void APIENTRY (*pstub_glPointParameterfv)(GLenum,GLfloat*);
static pstub_glPointParameterfv stub_glPointParameterfv = NULL;
static int loaded_glPointParameterfv = 0;

value glstub_glPointParameterfv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glPointParameterfv){(*stub_glPointParameterfv)(lv0, lv1);}
	else { 
		if(!loaded_glPointParameterfv) { 
			loaded_glPointParameterfv = 1;
			init_lib(); 
			stub_glPointParameterfv=(pstub_glPointParameterfv)get_proc_address("glPointParameterfv");
			if(stub_glPointParameterfv){(*stub_glPointParameterfv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glPointParameterfv"); 
			} 
		} else { 
			FailWith("glPointParameterfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3b */
typedef void APIENTRY (*pstub_glSecondaryColor3b)(GLbyte,GLbyte,GLbyte);
static pstub_glSecondaryColor3b stub_glSecondaryColor3b = NULL;
static int loaded_glSecondaryColor3b = 0;

value glstub_glSecondaryColor3b(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLbyte lv0 =  Int_val(v0);
	GLbyte lv1 =  Int_val(v1);
	GLbyte lv2 =  Int_val(v2);

	if(stub_glSecondaryColor3b){(*stub_glSecondaryColor3b)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3b) { 
			loaded_glSecondaryColor3b = 1;
			init_lib(); 
			stub_glSecondaryColor3b=(pstub_glSecondaryColor3b)get_proc_address("glSecondaryColor3b");
			if(stub_glSecondaryColor3b){(*stub_glSecondaryColor3b)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3b"); 
			} 
		} else { 
			FailWith("glSecondaryColor3b not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3bv */
typedef void APIENTRY (*pstub_glSecondaryColor3bv)(GLbyte*);
static pstub_glSecondaryColor3bv stub_glSecondaryColor3bv = NULL;
static int loaded_glSecondaryColor3bv = 0;

value glstub_glSecondaryColor3bv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLbyte* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3bv){(*stub_glSecondaryColor3bv)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3bv) { 
			loaded_glSecondaryColor3bv = 1;
			init_lib(); 
			stub_glSecondaryColor3bv=(pstub_glSecondaryColor3bv)get_proc_address("glSecondaryColor3bv");
			if(stub_glSecondaryColor3bv){(*stub_glSecondaryColor3bv)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3bv"); 
			} 
		} else { 
			FailWith("glSecondaryColor3bv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3d */
typedef void APIENTRY (*pstub_glSecondaryColor3d)(GLdouble,GLdouble,GLdouble);
static pstub_glSecondaryColor3d stub_glSecondaryColor3d = NULL;
static int loaded_glSecondaryColor3d = 0;

value glstub_glSecondaryColor3d(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glSecondaryColor3d){(*stub_glSecondaryColor3d)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3d) { 
			loaded_glSecondaryColor3d = 1;
			init_lib(); 
			stub_glSecondaryColor3d=(pstub_glSecondaryColor3d)get_proc_address("glSecondaryColor3d");
			if(stub_glSecondaryColor3d){(*stub_glSecondaryColor3d)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3d"); 
			} 
		} else { 
			FailWith("glSecondaryColor3d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3dv */
typedef void APIENTRY (*pstub_glSecondaryColor3dv)(GLdouble*);
static pstub_glSecondaryColor3dv stub_glSecondaryColor3dv = NULL;
static int loaded_glSecondaryColor3dv = 0;

value glstub_glSecondaryColor3dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3dv){(*stub_glSecondaryColor3dv)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3dv) { 
			loaded_glSecondaryColor3dv = 1;
			init_lib(); 
			stub_glSecondaryColor3dv=(pstub_glSecondaryColor3dv)get_proc_address("glSecondaryColor3dv");
			if(stub_glSecondaryColor3dv){(*stub_glSecondaryColor3dv)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3dv"); 
			} 
		} else { 
			FailWith("glSecondaryColor3dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3f */
typedef void APIENTRY (*pstub_glSecondaryColor3f)(GLfloat,GLfloat,GLfloat);
static pstub_glSecondaryColor3f stub_glSecondaryColor3f = NULL;
static int loaded_glSecondaryColor3f = 0;

value glstub_glSecondaryColor3f(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glSecondaryColor3f){(*stub_glSecondaryColor3f)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3f) { 
			loaded_glSecondaryColor3f = 1;
			init_lib(); 
			stub_glSecondaryColor3f=(pstub_glSecondaryColor3f)get_proc_address("glSecondaryColor3f");
			if(stub_glSecondaryColor3f){(*stub_glSecondaryColor3f)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3f"); 
			} 
		} else { 
			FailWith("glSecondaryColor3f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3fv */
typedef void APIENTRY (*pstub_glSecondaryColor3fv)(GLfloat*);
static pstub_glSecondaryColor3fv stub_glSecondaryColor3fv = NULL;
static int loaded_glSecondaryColor3fv = 0;

value glstub_glSecondaryColor3fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3fv){(*stub_glSecondaryColor3fv)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3fv) { 
			loaded_glSecondaryColor3fv = 1;
			init_lib(); 
			stub_glSecondaryColor3fv=(pstub_glSecondaryColor3fv)get_proc_address("glSecondaryColor3fv");
			if(stub_glSecondaryColor3fv){(*stub_glSecondaryColor3fv)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3fv"); 
			} 
		} else { 
			FailWith("glSecondaryColor3fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3i */
typedef void APIENTRY (*pstub_glSecondaryColor3i)(GLint,GLint,GLint);
static pstub_glSecondaryColor3i stub_glSecondaryColor3i = NULL;
static int loaded_glSecondaryColor3i = 0;

value glstub_glSecondaryColor3i(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glSecondaryColor3i){(*stub_glSecondaryColor3i)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3i) { 
			loaded_glSecondaryColor3i = 1;
			init_lib(); 
			stub_glSecondaryColor3i=(pstub_glSecondaryColor3i)get_proc_address("glSecondaryColor3i");
			if(stub_glSecondaryColor3i){(*stub_glSecondaryColor3i)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3i"); 
			} 
		} else { 
			FailWith("glSecondaryColor3i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3iv */
typedef void APIENTRY (*pstub_glSecondaryColor3iv)(GLint*);
static pstub_glSecondaryColor3iv stub_glSecondaryColor3iv = NULL;
static int loaded_glSecondaryColor3iv = 0;

value glstub_glSecondaryColor3iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3iv){(*stub_glSecondaryColor3iv)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3iv) { 
			loaded_glSecondaryColor3iv = 1;
			init_lib(); 
			stub_glSecondaryColor3iv=(pstub_glSecondaryColor3iv)get_proc_address("glSecondaryColor3iv");
			if(stub_glSecondaryColor3iv){(*stub_glSecondaryColor3iv)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3iv"); 
			} 
		} else { 
			FailWith("glSecondaryColor3iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3s */
typedef void APIENTRY (*pstub_glSecondaryColor3s)(GLshort,GLshort,GLshort);
static pstub_glSecondaryColor3s stub_glSecondaryColor3s = NULL;
static int loaded_glSecondaryColor3s = 0;

value glstub_glSecondaryColor3s(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);

	if(stub_glSecondaryColor3s){(*stub_glSecondaryColor3s)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3s) { 
			loaded_glSecondaryColor3s = 1;
			init_lib(); 
			stub_glSecondaryColor3s=(pstub_glSecondaryColor3s)get_proc_address("glSecondaryColor3s");
			if(stub_glSecondaryColor3s){(*stub_glSecondaryColor3s)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3s"); 
			} 
		} else { 
			FailWith("glSecondaryColor3s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3sv */
typedef void APIENTRY (*pstub_glSecondaryColor3sv)(GLshort*);
static pstub_glSecondaryColor3sv stub_glSecondaryColor3sv = NULL;
static int loaded_glSecondaryColor3sv = 0;

value glstub_glSecondaryColor3sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3sv){(*stub_glSecondaryColor3sv)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3sv) { 
			loaded_glSecondaryColor3sv = 1;
			init_lib(); 
			stub_glSecondaryColor3sv=(pstub_glSecondaryColor3sv)get_proc_address("glSecondaryColor3sv");
			if(stub_glSecondaryColor3sv){(*stub_glSecondaryColor3sv)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3sv"); 
			} 
		} else { 
			FailWith("glSecondaryColor3sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3ub */
typedef void APIENTRY (*pstub_glSecondaryColor3ub)(GLubyte,GLubyte,GLubyte);
static pstub_glSecondaryColor3ub stub_glSecondaryColor3ub = NULL;
static int loaded_glSecondaryColor3ub = 0;

value glstub_glSecondaryColor3ub(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLubyte lv0 =  Int_val(v0);
	GLubyte lv1 =  Int_val(v1);
	GLubyte lv2 =  Int_val(v2);

	if(stub_glSecondaryColor3ub){(*stub_glSecondaryColor3ub)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3ub) { 
			loaded_glSecondaryColor3ub = 1;
			init_lib(); 
			stub_glSecondaryColor3ub=(pstub_glSecondaryColor3ub)get_proc_address("glSecondaryColor3ub");
			if(stub_glSecondaryColor3ub){(*stub_glSecondaryColor3ub)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3ub"); 
			} 
		} else { 
			FailWith("glSecondaryColor3ub not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3ubv */
typedef void APIENTRY (*pstub_glSecondaryColor3ubv)(GLubyte*);
static pstub_glSecondaryColor3ubv stub_glSecondaryColor3ubv = NULL;
static int loaded_glSecondaryColor3ubv = 0;

value glstub_glSecondaryColor3ubv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLubyte* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3ubv){(*stub_glSecondaryColor3ubv)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3ubv) { 
			loaded_glSecondaryColor3ubv = 1;
			init_lib(); 
			stub_glSecondaryColor3ubv=(pstub_glSecondaryColor3ubv)get_proc_address("glSecondaryColor3ubv");
			if(stub_glSecondaryColor3ubv){(*stub_glSecondaryColor3ubv)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3ubv"); 
			} 
		} else { 
			FailWith("glSecondaryColor3ubv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3ui */
typedef void APIENTRY (*pstub_glSecondaryColor3ui)(GLuint,GLuint,GLuint);
static pstub_glSecondaryColor3ui stub_glSecondaryColor3ui = NULL;
static int loaded_glSecondaryColor3ui = 0;

value glstub_glSecondaryColor3ui(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint lv1 =  Int_val(v1);
	GLuint lv2 =  Int_val(v2);

	if(stub_glSecondaryColor3ui){(*stub_glSecondaryColor3ui)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3ui) { 
			loaded_glSecondaryColor3ui = 1;
			init_lib(); 
			stub_glSecondaryColor3ui=(pstub_glSecondaryColor3ui)get_proc_address("glSecondaryColor3ui");
			if(stub_glSecondaryColor3ui){(*stub_glSecondaryColor3ui)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3ui"); 
			} 
		} else { 
			FailWith("glSecondaryColor3ui not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3uiv */
typedef void APIENTRY (*pstub_glSecondaryColor3uiv)(GLuint*);
static pstub_glSecondaryColor3uiv stub_glSecondaryColor3uiv = NULL;
static int loaded_glSecondaryColor3uiv = 0;

value glstub_glSecondaryColor3uiv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3uiv){(*stub_glSecondaryColor3uiv)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3uiv) { 
			loaded_glSecondaryColor3uiv = 1;
			init_lib(); 
			stub_glSecondaryColor3uiv=(pstub_glSecondaryColor3uiv)get_proc_address("glSecondaryColor3uiv");
			if(stub_glSecondaryColor3uiv){(*stub_glSecondaryColor3uiv)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3uiv"); 
			} 
		} else { 
			FailWith("glSecondaryColor3uiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3us */
typedef void APIENTRY (*pstub_glSecondaryColor3us)(GLushort,GLushort,GLushort);
static pstub_glSecondaryColor3us stub_glSecondaryColor3us = NULL;
static int loaded_glSecondaryColor3us = 0;

value glstub_glSecondaryColor3us(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLushort lv0 =  Int_val(v0);
	GLushort lv1 =  Int_val(v1);
	GLushort lv2 =  Int_val(v2);

	if(stub_glSecondaryColor3us){(*stub_glSecondaryColor3us)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glSecondaryColor3us) { 
			loaded_glSecondaryColor3us = 1;
			init_lib(); 
			stub_glSecondaryColor3us=(pstub_glSecondaryColor3us)get_proc_address("glSecondaryColor3us");
			if(stub_glSecondaryColor3us){(*stub_glSecondaryColor3us)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glSecondaryColor3us"); 
			} 
		} else { 
			FailWith("glSecondaryColor3us not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColor3usv */
typedef void APIENTRY (*pstub_glSecondaryColor3usv)(GLushort*);
static pstub_glSecondaryColor3usv stub_glSecondaryColor3usv = NULL;
static int loaded_glSecondaryColor3usv = 0;

value glstub_glSecondaryColor3usv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLushort* lv0 =  Data_bigarray_val(v0);

	if(stub_glSecondaryColor3usv){(*stub_glSecondaryColor3usv)(lv0);}
	else { 
		if(!loaded_glSecondaryColor3usv) { 
			loaded_glSecondaryColor3usv = 1;
			init_lib(); 
			stub_glSecondaryColor3usv=(pstub_glSecondaryColor3usv)get_proc_address("glSecondaryColor3usv");
			if(stub_glSecondaryColor3usv){(*stub_glSecondaryColor3usv)(lv0);}
			else { 
				FailWith("Cannot load glSecondaryColor3usv"); 
			} 
		} else { 
			FailWith("glSecondaryColor3usv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glSecondaryColorPointer */
typedef void APIENTRY (*pstub_glSecondaryColorPointer)(GLint,GLenum,GLsizei,GLvoid*);
static pstub_glSecondaryColorPointer stub_glSecondaryColorPointer = NULL;
static int loaded_glSecondaryColorPointer = 0;

value glstub_glSecondaryColorPointer(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLsizei lv2 =  Int_val(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);

	if(stub_glSecondaryColorPointer){(*stub_glSecondaryColorPointer)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glSecondaryColorPointer) { 
			loaded_glSecondaryColorPointer = 1;
			init_lib(); 
			stub_glSecondaryColorPointer=(pstub_glSecondaryColorPointer)get_proc_address("glSecondaryColorPointer");
			if(stub_glSecondaryColorPointer){(*stub_glSecondaryColorPointer)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glSecondaryColorPointer"); 
			} 
		} else { 
			FailWith("glSecondaryColorPointer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBlendFuncSeparate */
typedef void APIENTRY (*pstub_glBlendFuncSeparate)(GLenum,GLenum,GLenum,GLenum);
static pstub_glBlendFuncSeparate stub_glBlendFuncSeparate = NULL;
static int loaded_glBlendFuncSeparate = 0;

value glstub_glBlendFuncSeparate(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLenum lv2 =  get_glenum(v2);
	GLenum lv3 =  get_glenum(v3);

	if(stub_glBlendFuncSeparate){(*stub_glBlendFuncSeparate)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glBlendFuncSeparate) { 
			loaded_glBlendFuncSeparate = 1;
			init_lib(); 
			stub_glBlendFuncSeparate=(pstub_glBlendFuncSeparate)get_proc_address("glBlendFuncSeparate");
			if(stub_glBlendFuncSeparate){(*stub_glBlendFuncSeparate)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glBlendFuncSeparate"); 
			} 
		} else { 
			FailWith("glBlendFuncSeparate not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos2d */
typedef void APIENTRY (*pstub_glWindowPos2d)(GLdouble,GLdouble);
static pstub_glWindowPos2d stub_glWindowPos2d = NULL;
static int loaded_glWindowPos2d = 0;

value glstub_glWindowPos2d(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);

	if(stub_glWindowPos2d){(*stub_glWindowPos2d)(lv0, lv1);}
	else { 
		if(!loaded_glWindowPos2d) { 
			loaded_glWindowPos2d = 1;
			init_lib(); 
			stub_glWindowPos2d=(pstub_glWindowPos2d)get_proc_address("glWindowPos2d");
			if(stub_glWindowPos2d){(*stub_glWindowPos2d)(lv0, lv1);}
			else { 
				FailWith("Cannot load glWindowPos2d"); 
			} 
		} else { 
			FailWith("glWindowPos2d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos2f */
typedef void APIENTRY (*pstub_glWindowPos2f)(GLfloat,GLfloat);
static pstub_glWindowPos2f stub_glWindowPos2f = NULL;
static int loaded_glWindowPos2f = 0;

value glstub_glWindowPos2f(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glWindowPos2f){(*stub_glWindowPos2f)(lv0, lv1);}
	else { 
		if(!loaded_glWindowPos2f) { 
			loaded_glWindowPos2f = 1;
			init_lib(); 
			stub_glWindowPos2f=(pstub_glWindowPos2f)get_proc_address("glWindowPos2f");
			if(stub_glWindowPos2f){(*stub_glWindowPos2f)(lv0, lv1);}
			else { 
				FailWith("Cannot load glWindowPos2f"); 
			} 
		} else { 
			FailWith("glWindowPos2f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos2i */
typedef void APIENTRY (*pstub_glWindowPos2i)(GLint,GLint);
static pstub_glWindowPos2i stub_glWindowPos2i = NULL;
static int loaded_glWindowPos2i = 0;

value glstub_glWindowPos2i(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glWindowPos2i){(*stub_glWindowPos2i)(lv0, lv1);}
	else { 
		if(!loaded_glWindowPos2i) { 
			loaded_glWindowPos2i = 1;
			init_lib(); 
			stub_glWindowPos2i=(pstub_glWindowPos2i)get_proc_address("glWindowPos2i");
			if(stub_glWindowPos2i){(*stub_glWindowPos2i)(lv0, lv1);}
			else { 
				FailWith("Cannot load glWindowPos2i"); 
			} 
		} else { 
			FailWith("glWindowPos2i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos2s */
typedef void APIENTRY (*pstub_glWindowPos2s)(GLshort,GLshort);
static pstub_glWindowPos2s stub_glWindowPos2s = NULL;
static int loaded_glWindowPos2s = 0;

value glstub_glWindowPos2s(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);

	if(stub_glWindowPos2s){(*stub_glWindowPos2s)(lv0, lv1);}
	else { 
		if(!loaded_glWindowPos2s) { 
			loaded_glWindowPos2s = 1;
			init_lib(); 
			stub_glWindowPos2s=(pstub_glWindowPos2s)get_proc_address("glWindowPos2s");
			if(stub_glWindowPos2s){(*stub_glWindowPos2s)(lv0, lv1);}
			else { 
				FailWith("Cannot load glWindowPos2s"); 
			} 
		} else { 
			FailWith("glWindowPos2s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos2dv */
typedef void APIENTRY (*pstub_glWindowPos2dv)(GLdouble*);
static pstub_glWindowPos2dv stub_glWindowPos2dv = NULL;
static int loaded_glWindowPos2dv = 0;

value glstub_glWindowPos2dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glWindowPos2dv){(*stub_glWindowPos2dv)(lv0);}
	else { 
		if(!loaded_glWindowPos2dv) { 
			loaded_glWindowPos2dv = 1;
			init_lib(); 
			stub_glWindowPos2dv=(pstub_glWindowPos2dv)get_proc_address("glWindowPos2dv");
			if(stub_glWindowPos2dv){(*stub_glWindowPos2dv)(lv0);}
			else { 
				FailWith("Cannot load glWindowPos2dv"); 
			} 
		} else { 
			FailWith("glWindowPos2dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos2fv */
typedef void APIENTRY (*pstub_glWindowPos2fv)(GLfloat*);
static pstub_glWindowPos2fv stub_glWindowPos2fv = NULL;
static int loaded_glWindowPos2fv = 0;

value glstub_glWindowPos2fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glWindowPos2fv){(*stub_glWindowPos2fv)(lv0);}
	else { 
		if(!loaded_glWindowPos2fv) { 
			loaded_glWindowPos2fv = 1;
			init_lib(); 
			stub_glWindowPos2fv=(pstub_glWindowPos2fv)get_proc_address("glWindowPos2fv");
			if(stub_glWindowPos2fv){(*stub_glWindowPos2fv)(lv0);}
			else { 
				FailWith("Cannot load glWindowPos2fv"); 
			} 
		} else { 
			FailWith("glWindowPos2fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos2iv */
typedef void APIENTRY (*pstub_glWindowPos2iv)(GLint*);
static pstub_glWindowPos2iv stub_glWindowPos2iv = NULL;
static int loaded_glWindowPos2iv = 0;

value glstub_glWindowPos2iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glWindowPos2iv){(*stub_glWindowPos2iv)(lv0);}
	else { 
		if(!loaded_glWindowPos2iv) { 
			loaded_glWindowPos2iv = 1;
			init_lib(); 
			stub_glWindowPos2iv=(pstub_glWindowPos2iv)get_proc_address("glWindowPos2iv");
			if(stub_glWindowPos2iv){(*stub_glWindowPos2iv)(lv0);}
			else { 
				FailWith("Cannot load glWindowPos2iv"); 
			} 
		} else { 
			FailWith("glWindowPos2iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos2sv */
typedef void APIENTRY (*pstub_glWindowPos2sv)(GLshort*);
static pstub_glWindowPos2sv stub_glWindowPos2sv = NULL;
static int loaded_glWindowPos2sv = 0;

value glstub_glWindowPos2sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glWindowPos2sv){(*stub_glWindowPos2sv)(lv0);}
	else { 
		if(!loaded_glWindowPos2sv) { 
			loaded_glWindowPos2sv = 1;
			init_lib(); 
			stub_glWindowPos2sv=(pstub_glWindowPos2sv)get_proc_address("glWindowPos2sv");
			if(stub_glWindowPos2sv){(*stub_glWindowPos2sv)(lv0);}
			else { 
				FailWith("Cannot load glWindowPos2sv"); 
			} 
		} else { 
			FailWith("glWindowPos2sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos3d */
typedef void APIENTRY (*pstub_glWindowPos3d)(GLdouble,GLdouble,GLdouble);
static pstub_glWindowPos3d stub_glWindowPos3d = NULL;
static int loaded_glWindowPos3d = 0;

value glstub_glWindowPos3d(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLdouble lv0 =  Double_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glWindowPos3d){(*stub_glWindowPos3d)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glWindowPos3d) { 
			loaded_glWindowPos3d = 1;
			init_lib(); 
			stub_glWindowPos3d=(pstub_glWindowPos3d)get_proc_address("glWindowPos3d");
			if(stub_glWindowPos3d){(*stub_glWindowPos3d)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glWindowPos3d"); 
			} 
		} else { 
			FailWith("glWindowPos3d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos3f */
typedef void APIENTRY (*pstub_glWindowPos3f)(GLfloat,GLfloat,GLfloat);
static pstub_glWindowPos3f stub_glWindowPos3f = NULL;
static int loaded_glWindowPos3f = 0;

value glstub_glWindowPos3f(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLfloat lv0 =  Double_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glWindowPos3f){(*stub_glWindowPos3f)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glWindowPos3f) { 
			loaded_glWindowPos3f = 1;
			init_lib(); 
			stub_glWindowPos3f=(pstub_glWindowPos3f)get_proc_address("glWindowPos3f");
			if(stub_glWindowPos3f){(*stub_glWindowPos3f)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glWindowPos3f"); 
			} 
		} else { 
			FailWith("glWindowPos3f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos3i */
typedef void APIENTRY (*pstub_glWindowPos3i)(GLint,GLint,GLint);
static pstub_glWindowPos3i stub_glWindowPos3i = NULL;
static int loaded_glWindowPos3i = 0;

value glstub_glWindowPos3i(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glWindowPos3i){(*stub_glWindowPos3i)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glWindowPos3i) { 
			loaded_glWindowPos3i = 1;
			init_lib(); 
			stub_glWindowPos3i=(pstub_glWindowPos3i)get_proc_address("glWindowPos3i");
			if(stub_glWindowPos3i){(*stub_glWindowPos3i)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glWindowPos3i"); 
			} 
		} else { 
			FailWith("glWindowPos3i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos3s */
typedef void APIENTRY (*pstub_glWindowPos3s)(GLshort,GLshort,GLshort);
static pstub_glWindowPos3s stub_glWindowPos3s = NULL;
static int loaded_glWindowPos3s = 0;

value glstub_glWindowPos3s(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLshort lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);

	if(stub_glWindowPos3s){(*stub_glWindowPos3s)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glWindowPos3s) { 
			loaded_glWindowPos3s = 1;
			init_lib(); 
			stub_glWindowPos3s=(pstub_glWindowPos3s)get_proc_address("glWindowPos3s");
			if(stub_glWindowPos3s){(*stub_glWindowPos3s)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glWindowPos3s"); 
			} 
		} else { 
			FailWith("glWindowPos3s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos3dv */
typedef void APIENTRY (*pstub_glWindowPos3dv)(GLdouble*);
static pstub_glWindowPos3dv stub_glWindowPos3dv = NULL;
static int loaded_glWindowPos3dv = 0;

value glstub_glWindowPos3dv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLdouble* lv0 =  Data_bigarray_val(v0);

	if(stub_glWindowPos3dv){(*stub_glWindowPos3dv)(lv0);}
	else { 
		if(!loaded_glWindowPos3dv) { 
			loaded_glWindowPos3dv = 1;
			init_lib(); 
			stub_glWindowPos3dv=(pstub_glWindowPos3dv)get_proc_address("glWindowPos3dv");
			if(stub_glWindowPos3dv){(*stub_glWindowPos3dv)(lv0);}
			else { 
				FailWith("Cannot load glWindowPos3dv"); 
			} 
		} else { 
			FailWith("glWindowPos3dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos3fv */
typedef void APIENTRY (*pstub_glWindowPos3fv)(GLfloat*);
static pstub_glWindowPos3fv stub_glWindowPos3fv = NULL;
static int loaded_glWindowPos3fv = 0;

value glstub_glWindowPos3fv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLfloat* lv0 =  Data_bigarray_val(v0);

	if(stub_glWindowPos3fv){(*stub_glWindowPos3fv)(lv0);}
	else { 
		if(!loaded_glWindowPos3fv) { 
			loaded_glWindowPos3fv = 1;
			init_lib(); 
			stub_glWindowPos3fv=(pstub_glWindowPos3fv)get_proc_address("glWindowPos3fv");
			if(stub_glWindowPos3fv){(*stub_glWindowPos3fv)(lv0);}
			else { 
				FailWith("Cannot load glWindowPos3fv"); 
			} 
		} else { 
			FailWith("glWindowPos3fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos3iv */
typedef void APIENTRY (*pstub_glWindowPos3iv)(GLint*);
static pstub_glWindowPos3iv stub_glWindowPos3iv = NULL;
static int loaded_glWindowPos3iv = 0;

value glstub_glWindowPos3iv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLint* lv0 =  Data_bigarray_val(v0);

	if(stub_glWindowPos3iv){(*stub_glWindowPos3iv)(lv0);}
	else { 
		if(!loaded_glWindowPos3iv) { 
			loaded_glWindowPos3iv = 1;
			init_lib(); 
			stub_glWindowPos3iv=(pstub_glWindowPos3iv)get_proc_address("glWindowPos3iv");
			if(stub_glWindowPos3iv){(*stub_glWindowPos3iv)(lv0);}
			else { 
				FailWith("Cannot load glWindowPos3iv"); 
			} 
		} else { 
			FailWith("glWindowPos3iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glWindowPos3sv */
typedef void APIENTRY (*pstub_glWindowPos3sv)(GLshort*);
static pstub_glWindowPos3sv stub_glWindowPos3sv = NULL;
static int loaded_glWindowPos3sv = 0;

value glstub_glWindowPos3sv(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLshort* lv0 =  Data_bigarray_val(v0);

	if(stub_glWindowPos3sv){(*stub_glWindowPos3sv)(lv0);}
	else { 
		if(!loaded_glWindowPos3sv) { 
			loaded_glWindowPos3sv = 1;
			init_lib(); 
			stub_glWindowPos3sv=(pstub_glWindowPos3sv)get_proc_address("glWindowPos3sv");
			if(stub_glWindowPos3sv){(*stub_glWindowPos3sv)(lv0);}
			else { 
				FailWith("Cannot load glWindowPos3sv"); 
			} 
		} else { 
			FailWith("glWindowPos3sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGenQueries */
typedef void APIENTRY (*pstub_glGenQueries)(GLsizei,GLuint*);
static pstub_glGenQueries stub_glGenQueries = NULL;
static int loaded_glGenQueries = 0;

value glstub_glGenQueries(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);

	if(stub_glGenQueries){(*stub_glGenQueries)(lv0, lv1);}
	else { 
		if(!loaded_glGenQueries) { 
			loaded_glGenQueries = 1;
			init_lib(); 
			stub_glGenQueries=(pstub_glGenQueries)get_proc_address("glGenQueries");
			if(stub_glGenQueries){(*stub_glGenQueries)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGenQueries"); 
			} 
		} else { 
			FailWith("glGenQueries not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDeleteQueries */
typedef void APIENTRY (*pstub_glDeleteQueries)(GLsizei,GLuint*);
static pstub_glDeleteQueries stub_glDeleteQueries = NULL;
static int loaded_glDeleteQueries = 0;

value glstub_glDeleteQueries(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);

	if(stub_glDeleteQueries){(*stub_glDeleteQueries)(lv0, lv1);}
	else { 
		if(!loaded_glDeleteQueries) { 
			loaded_glDeleteQueries = 1;
			init_lib(); 
			stub_glDeleteQueries=(pstub_glDeleteQueries)get_proc_address("glDeleteQueries");
			if(stub_glDeleteQueries){(*stub_glDeleteQueries)(lv0, lv1);}
			else { 
				FailWith("Cannot load glDeleteQueries"); 
			} 
		} else { 
			FailWith("glDeleteQueries not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIsQuery */
typedef GLboolean APIENTRY (*pstub_glIsQuery)(GLuint);
static pstub_glIsQuery stub_glIsQuery = NULL;
static int loaded_glIsQuery = 0;

value glstub_glIsQuery(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLboolean ret;
	if(stub_glIsQuery){ret = (*stub_glIsQuery)(lv0);}
	else { 
		if(!loaded_glIsQuery) { 
			loaded_glIsQuery = 1;
			init_lib(); 
			stub_glIsQuery=(pstub_glIsQuery)get_proc_address("glIsQuery");
			if(stub_glIsQuery){ret = (*stub_glIsQuery)(lv0);}
			else { 
				FailWith("Cannot load glIsQuery"); 
			} 
		} else { 
			FailWith("glIsQuery not available"); 
		}
	}
	result = Bool_val(ret);
	CAMLreturn (result);
}

/* glBeginQuery */
typedef void APIENTRY (*pstub_glBeginQuery)(GLenum,GLuint);
static pstub_glBeginQuery stub_glBeginQuery = NULL;
static int loaded_glBeginQuery = 0;

value glstub_glBeginQuery(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLuint lv1 =  Int_val(v1);

	if(stub_glBeginQuery){(*stub_glBeginQuery)(lv0, lv1);}
	else { 
		if(!loaded_glBeginQuery) { 
			loaded_glBeginQuery = 1;
			init_lib(); 
			stub_glBeginQuery=(pstub_glBeginQuery)get_proc_address("glBeginQuery");
			if(stub_glBeginQuery){(*stub_glBeginQuery)(lv0, lv1);}
			else { 
				FailWith("Cannot load glBeginQuery"); 
			} 
		} else { 
			FailWith("glBeginQuery not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEndQuery */
typedef void APIENTRY (*pstub_glEndQuery)(GLenum);
static pstub_glEndQuery stub_glEndQuery = NULL;
static int loaded_glEndQuery = 0;

value glstub_glEndQuery(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);

	if(stub_glEndQuery){(*stub_glEndQuery)(lv0);}
	else { 
		if(!loaded_glEndQuery) { 
			loaded_glEndQuery = 1;
			init_lib(); 
			stub_glEndQuery=(pstub_glEndQuery)get_proc_address("glEndQuery");
			if(stub_glEndQuery){(*stub_glEndQuery)(lv0);}
			else { 
				FailWith("Cannot load glEndQuery"); 
			} 
		} else { 
			FailWith("glEndQuery not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetQueryiv */
typedef void APIENTRY (*pstub_glGetQueryiv)(GLenum,GLenum,GLint*);
static pstub_glGetQueryiv stub_glGetQueryiv = NULL;
static int loaded_glGetQueryiv = 0;

value glstub_glGetQueryiv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetQueryiv){(*stub_glGetQueryiv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetQueryiv) { 
			loaded_glGetQueryiv = 1;
			init_lib(); 
			stub_glGetQueryiv=(pstub_glGetQueryiv)get_proc_address("glGetQueryiv");
			if(stub_glGetQueryiv){(*stub_glGetQueryiv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetQueryiv"); 
			} 
		} else { 
			FailWith("glGetQueryiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetQueryObjectiv */
typedef void APIENTRY (*pstub_glGetQueryObjectiv)(GLuint,GLenum,GLint*);
static pstub_glGetQueryObjectiv stub_glGetQueryObjectiv = NULL;
static int loaded_glGetQueryObjectiv = 0;

value glstub_glGetQueryObjectiv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetQueryObjectiv){(*stub_glGetQueryObjectiv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetQueryObjectiv) { 
			loaded_glGetQueryObjectiv = 1;
			init_lib(); 
			stub_glGetQueryObjectiv=(pstub_glGetQueryObjectiv)get_proc_address("glGetQueryObjectiv");
			if(stub_glGetQueryObjectiv){(*stub_glGetQueryObjectiv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetQueryObjectiv"); 
			} 
		} else { 
			FailWith("glGetQueryObjectiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetQueryObjectuiv */
typedef void APIENTRY (*pstub_glGetQueryObjectuiv)(GLuint,GLenum,GLuint*);
static pstub_glGetQueryObjectuiv stub_glGetQueryObjectuiv = NULL;
static int loaded_glGetQueryObjectuiv = 0;

value glstub_glGetQueryObjectuiv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLuint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetQueryObjectuiv){(*stub_glGetQueryObjectuiv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetQueryObjectuiv) { 
			loaded_glGetQueryObjectuiv = 1;
			init_lib(); 
			stub_glGetQueryObjectuiv=(pstub_glGetQueryObjectuiv)get_proc_address("glGetQueryObjectuiv");
			if(stub_glGetQueryObjectuiv){(*stub_glGetQueryObjectuiv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetQueryObjectuiv"); 
			} 
		} else { 
			FailWith("glGetQueryObjectuiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBindBuffer */
typedef void APIENTRY (*pstub_glBindBuffer)(GLenum,GLuint);
static pstub_glBindBuffer stub_glBindBuffer = NULL;
static int loaded_glBindBuffer = 0;

value glstub_glBindBuffer(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLuint lv1 =  Int_val(v1);

	if(stub_glBindBuffer){(*stub_glBindBuffer)(lv0, lv1);}
	else { 
		if(!loaded_glBindBuffer) { 
			loaded_glBindBuffer = 1;
			init_lib(); 
			stub_glBindBuffer=(pstub_glBindBuffer)get_proc_address("glBindBuffer");
			if(stub_glBindBuffer){(*stub_glBindBuffer)(lv0, lv1);}
			else { 
				FailWith("Cannot load glBindBuffer"); 
			} 
		} else { 
			FailWith("glBindBuffer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDeleteBuffers */
typedef void APIENTRY (*pstub_glDeleteBuffers)(GLsizei,GLuint*);
static pstub_glDeleteBuffers stub_glDeleteBuffers = NULL;
static int loaded_glDeleteBuffers = 0;

value glstub_glDeleteBuffers(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);

	if(stub_glDeleteBuffers){(*stub_glDeleteBuffers)(lv0, lv1);}
	else { 
		if(!loaded_glDeleteBuffers) { 
			loaded_glDeleteBuffers = 1;
			init_lib(); 
			stub_glDeleteBuffers=(pstub_glDeleteBuffers)get_proc_address("glDeleteBuffers");
			if(stub_glDeleteBuffers){(*stub_glDeleteBuffers)(lv0, lv1);}
			else { 
				FailWith("Cannot load glDeleteBuffers"); 
			} 
		} else { 
			FailWith("glDeleteBuffers not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGenBuffers */
typedef void APIENTRY (*pstub_glGenBuffers)(GLsizei,GLuint*);
static pstub_glGenBuffers stub_glGenBuffers = NULL;
static int loaded_glGenBuffers = 0;

value glstub_glGenBuffers(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);

	if(stub_glGenBuffers){(*stub_glGenBuffers)(lv0, lv1);}
	else { 
		if(!loaded_glGenBuffers) { 
			loaded_glGenBuffers = 1;
			init_lib(); 
			stub_glGenBuffers=(pstub_glGenBuffers)get_proc_address("glGenBuffers");
			if(stub_glGenBuffers){(*stub_glGenBuffers)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGenBuffers"); 
			} 
		} else { 
			FailWith("glGenBuffers not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIsBuffer */
typedef GLboolean APIENTRY (*pstub_glIsBuffer)(GLuint);
static pstub_glIsBuffer stub_glIsBuffer = NULL;
static int loaded_glIsBuffer = 0;

value glstub_glIsBuffer(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLboolean ret;
	if(stub_glIsBuffer){ret = (*stub_glIsBuffer)(lv0);}
	else { 
		if(!loaded_glIsBuffer) { 
			loaded_glIsBuffer = 1;
			init_lib(); 
			stub_glIsBuffer=(pstub_glIsBuffer)get_proc_address("glIsBuffer");
			if(stub_glIsBuffer){ret = (*stub_glIsBuffer)(lv0);}
			else { 
				FailWith("Cannot load glIsBuffer"); 
			} 
		} else { 
			FailWith("glIsBuffer not available"); 
		}
	}
	result = Bool_val(ret);
	CAMLreturn (result);
}

/* glBufferData */
typedef void APIENTRY (*pstub_glBufferData)(GLenum,GLsizeiptr,GLvoid*,GLenum);
static pstub_glBufferData stub_glBufferData = NULL;
static int loaded_glBufferData = 0;

value glstub_glBufferData(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLsizeiptr lv1 =  Int_val(v1);
	GLvoid* lv2 =  Data_bigarray_val(v2);
	GLenum lv3 =  get_glenum(v3);

	if(stub_glBufferData){(*stub_glBufferData)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glBufferData) { 
			loaded_glBufferData = 1;
			init_lib(); 
			stub_glBufferData=(pstub_glBufferData)get_proc_address("glBufferData");
			if(stub_glBufferData){(*stub_glBufferData)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glBufferData"); 
			} 
		} else { 
			FailWith("glBufferData not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBufferSubData */
typedef void APIENTRY (*pstub_glBufferSubData)(GLenum,GLintptr,GLsizeiptr,GLvoid*);
static pstub_glBufferSubData stub_glBufferSubData = NULL;
static int loaded_glBufferSubData = 0;

value glstub_glBufferSubData(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLintptr lv1 =  Int_val(v1);
	GLsizeiptr lv2 =  Int_val(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);

	if(stub_glBufferSubData){(*stub_glBufferSubData)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glBufferSubData) { 
			loaded_glBufferSubData = 1;
			init_lib(); 
			stub_glBufferSubData=(pstub_glBufferSubData)get_proc_address("glBufferSubData");
			if(stub_glBufferSubData){(*stub_glBufferSubData)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glBufferSubData"); 
			} 
		} else { 
			FailWith("glBufferSubData not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetBufferSubData */
typedef void APIENTRY (*pstub_glGetBufferSubData)(GLenum,GLintptr,GLsizeiptr,GLvoid*);
static pstub_glGetBufferSubData stub_glGetBufferSubData = NULL;
static int loaded_glGetBufferSubData = 0;

value glstub_glGetBufferSubData(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLintptr lv1 =  Int_val(v1);
	GLsizeiptr lv2 =  Int_val(v2);
	GLvoid* lv3 =  Data_bigarray_val(v3);

	if(stub_glGetBufferSubData){(*stub_glGetBufferSubData)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glGetBufferSubData) { 
			loaded_glGetBufferSubData = 1;
			init_lib(); 
			stub_glGetBufferSubData=(pstub_glGetBufferSubData)get_proc_address("glGetBufferSubData");
			if(stub_glGetBufferSubData){(*stub_glGetBufferSubData)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glGetBufferSubData"); 
			} 
		} else { 
			FailWith("glGetBufferSubData not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glMapBuffer */
typedef GLvoid* APIENTRY (*pstub_glMapBuffer)(GLenum,GLenum);
static pstub_glMapBuffer stub_glMapBuffer = NULL;
static int loaded_glMapBuffer = 0;

value glstub_glMapBuffer(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLvoid* ret;
	if(stub_glMapBuffer){ret = (*stub_glMapBuffer)(lv0, lv1);}
	else { 
		if(!loaded_glMapBuffer) { 
			loaded_glMapBuffer = 1;
			init_lib(); 
			stub_glMapBuffer=(pstub_glMapBuffer)get_proc_address("glMapBuffer");
			if(stub_glMapBuffer){ret = (*stub_glMapBuffer)(lv0, lv1);}
			else { 
				FailWith("Cannot load glMapBuffer"); 
			} 
		} else { 
			FailWith("glMapBuffer not available"); 
		}
	}
	result = (value)(ret);
	CAMLreturn (result);
}

/* glUnmapBuffer */
typedef GLboolean APIENTRY (*pstub_glUnmapBuffer)(GLenum);
static pstub_glUnmapBuffer stub_glUnmapBuffer = NULL;
static int loaded_glUnmapBuffer = 0;

value glstub_glUnmapBuffer(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLboolean ret;
	if(stub_glUnmapBuffer){ret = (*stub_glUnmapBuffer)(lv0);}
	else { 
		if(!loaded_glUnmapBuffer) { 
			loaded_glUnmapBuffer = 1;
			init_lib(); 
			stub_glUnmapBuffer=(pstub_glUnmapBuffer)get_proc_address("glUnmapBuffer");
			if(stub_glUnmapBuffer){ret = (*stub_glUnmapBuffer)(lv0);}
			else { 
				FailWith("Cannot load glUnmapBuffer"); 
			} 
		} else { 
			FailWith("glUnmapBuffer not available"); 
		}
	}
	result = Bool_val(ret);
	CAMLreturn (result);
}

/* glGetBufferParameteriv */
typedef void APIENTRY (*pstub_glGetBufferParameteriv)(GLenum,GLenum,GLint*);
static pstub_glGetBufferParameteriv stub_glGetBufferParameteriv = NULL;
static int loaded_glGetBufferParameteriv = 0;

value glstub_glGetBufferParameteriv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetBufferParameteriv){(*stub_glGetBufferParameteriv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetBufferParameteriv) { 
			loaded_glGetBufferParameteriv = 1;
			init_lib(); 
			stub_glGetBufferParameteriv=(pstub_glGetBufferParameteriv)get_proc_address("glGetBufferParameteriv");
			if(stub_glGetBufferParameteriv){(*stub_glGetBufferParameteriv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetBufferParameteriv"); 
			} 
		} else { 
			FailWith("glGetBufferParameteriv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetBufferPointerv */
typedef void APIENTRY (*pstub_glGetBufferPointerv)(GLenum,GLenum,GLvoid**);
static pstub_glGetBufferPointerv stub_glGetBufferPointerv = NULL;
static int loaded_glGetBufferPointerv = 0;

value glstub_glGetBufferPointerv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLvoid** lv2 =  Data_bigarray_val(v2);

	if(stub_glGetBufferPointerv){(*stub_glGetBufferPointerv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetBufferPointerv) { 
			loaded_glGetBufferPointerv = 1;
			init_lib(); 
			stub_glGetBufferPointerv=(pstub_glGetBufferPointerv)get_proc_address("glGetBufferPointerv");
			if(stub_glGetBufferPointerv){(*stub_glGetBufferPointerv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetBufferPointerv"); 
			} 
		} else { 
			FailWith("glGetBufferPointerv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBlendEquationSeparate */
typedef void APIENTRY (*pstub_glBlendEquationSeparate)(GLenum,GLenum);
static pstub_glBlendEquationSeparate stub_glBlendEquationSeparate = NULL;
static int loaded_glBlendEquationSeparate = 0;

value glstub_glBlendEquationSeparate(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);

	if(stub_glBlendEquationSeparate){(*stub_glBlendEquationSeparate)(lv0, lv1);}
	else { 
		if(!loaded_glBlendEquationSeparate) { 
			loaded_glBlendEquationSeparate = 1;
			init_lib(); 
			stub_glBlendEquationSeparate=(pstub_glBlendEquationSeparate)get_proc_address("glBlendEquationSeparate");
			if(stub_glBlendEquationSeparate){(*stub_glBlendEquationSeparate)(lv0, lv1);}
			else { 
				FailWith("Cannot load glBlendEquationSeparate"); 
			} 
		} else { 
			FailWith("glBlendEquationSeparate not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDrawBuffers */
typedef void APIENTRY (*pstub_glDrawBuffers)(GLsizei,GLenum*);
static pstub_glDrawBuffers stub_glDrawBuffers = NULL;
static int loaded_glDrawBuffers = 0;

value glstub_glDrawBuffers(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLsizei lv0 =  Int_val(v0);
	GLenum* lv1 =  Data_bigarray_val(v1);

	if(stub_glDrawBuffers){(*stub_glDrawBuffers)(lv0, lv1);}
	else { 
		if(!loaded_glDrawBuffers) { 
			loaded_glDrawBuffers = 1;
			init_lib(); 
			stub_glDrawBuffers=(pstub_glDrawBuffers)get_proc_address("glDrawBuffers");
			if(stub_glDrawBuffers){(*stub_glDrawBuffers)(lv0, lv1);}
			else { 
				FailWith("Cannot load glDrawBuffers"); 
			} 
		} else { 
			FailWith("glDrawBuffers not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glStencilOpSeparate */
typedef void APIENTRY (*pstub_glStencilOpSeparate)(GLenum,GLenum,GLenum,GLenum);
static pstub_glStencilOpSeparate stub_glStencilOpSeparate = NULL;
static int loaded_glStencilOpSeparate = 0;

value glstub_glStencilOpSeparate(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLenum lv2 =  get_glenum(v2);
	GLenum lv3 =  get_glenum(v3);

	if(stub_glStencilOpSeparate){(*stub_glStencilOpSeparate)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glStencilOpSeparate) { 
			loaded_glStencilOpSeparate = 1;
			init_lib(); 
			stub_glStencilOpSeparate=(pstub_glStencilOpSeparate)get_proc_address("glStencilOpSeparate");
			if(stub_glStencilOpSeparate){(*stub_glStencilOpSeparate)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glStencilOpSeparate"); 
			} 
		} else { 
			FailWith("glStencilOpSeparate not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glStencilFuncSeparate */
typedef void APIENTRY (*pstub_glStencilFuncSeparate)(GLenum,GLenum,GLint,GLuint);
static pstub_glStencilFuncSeparate stub_glStencilFuncSeparate = NULL;
static int loaded_glStencilFuncSeparate = 0;

value glstub_glStencilFuncSeparate(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint lv2 =  Int_val(v2);
	GLuint lv3 =  Int_val(v3);

	if(stub_glStencilFuncSeparate){(*stub_glStencilFuncSeparate)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glStencilFuncSeparate) { 
			loaded_glStencilFuncSeparate = 1;
			init_lib(); 
			stub_glStencilFuncSeparate=(pstub_glStencilFuncSeparate)get_proc_address("glStencilFuncSeparate");
			if(stub_glStencilFuncSeparate){(*stub_glStencilFuncSeparate)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glStencilFuncSeparate"); 
			} 
		} else { 
			FailWith("glStencilFuncSeparate not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glStencilMaskSeparate */
typedef void APIENTRY (*pstub_glStencilMaskSeparate)(GLenum,GLuint);
static pstub_glStencilMaskSeparate stub_glStencilMaskSeparate = NULL;
static int loaded_glStencilMaskSeparate = 0;

value glstub_glStencilMaskSeparate(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLuint lv1 =  Int_val(v1);

	if(stub_glStencilMaskSeparate){(*stub_glStencilMaskSeparate)(lv0, lv1);}
	else { 
		if(!loaded_glStencilMaskSeparate) { 
			loaded_glStencilMaskSeparate = 1;
			init_lib(); 
			stub_glStencilMaskSeparate=(pstub_glStencilMaskSeparate)get_proc_address("glStencilMaskSeparate");
			if(stub_glStencilMaskSeparate){(*stub_glStencilMaskSeparate)(lv0, lv1);}
			else { 
				FailWith("Cannot load glStencilMaskSeparate"); 
			} 
		} else { 
			FailWith("glStencilMaskSeparate not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glAttachShader */
typedef void APIENTRY (*pstub_glAttachShader)(GLuint,GLuint);
static pstub_glAttachShader stub_glAttachShader = NULL;
static int loaded_glAttachShader = 0;

value glstub_glAttachShader(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint lv1 =  Int_val(v1);

	if(stub_glAttachShader){(*stub_glAttachShader)(lv0, lv1);}
	else { 
		if(!loaded_glAttachShader) { 
			loaded_glAttachShader = 1;
			init_lib(); 
			stub_glAttachShader=(pstub_glAttachShader)get_proc_address("glAttachShader");
			if(stub_glAttachShader){(*stub_glAttachShader)(lv0, lv1);}
			else { 
				FailWith("Cannot load glAttachShader"); 
			} 
		} else { 
			FailWith("glAttachShader not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glBindAttribLocation */
typedef void APIENTRY (*pstub_glBindAttribLocation)(GLuint,GLuint,GLchar*);
static pstub_glBindAttribLocation stub_glBindAttribLocation = NULL;
static int loaded_glBindAttribLocation = 0;

value glstub_glBindAttribLocation(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint lv1 =  Int_val(v1);
	GLchar* lv2 =  Data_bigarray_val(v2);

	if(stub_glBindAttribLocation){(*stub_glBindAttribLocation)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glBindAttribLocation) { 
			loaded_glBindAttribLocation = 1;
			init_lib(); 
			stub_glBindAttribLocation=(pstub_glBindAttribLocation)get_proc_address("glBindAttribLocation");
			if(stub_glBindAttribLocation){(*stub_glBindAttribLocation)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glBindAttribLocation"); 
			} 
		} else { 
			FailWith("glBindAttribLocation not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glCompileShader */
typedef void APIENTRY (*pstub_glCompileShader)(GLuint);
static pstub_glCompileShader stub_glCompileShader = NULL;
static int loaded_glCompileShader = 0;

value glstub_glCompileShader(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glCompileShader){(*stub_glCompileShader)(lv0);}
	else { 
		if(!loaded_glCompileShader) { 
			loaded_glCompileShader = 1;
			init_lib(); 
			stub_glCompileShader=(pstub_glCompileShader)get_proc_address("glCompileShader");
			if(stub_glCompileShader){(*stub_glCompileShader)(lv0);}
			else { 
				FailWith("Cannot load glCompileShader"); 
			} 
		} else { 
			FailWith("glCompileShader not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glCreateProgram */
typedef GLuint APIENTRY (*pstub_glCreateProgram)(GLvoid);
static pstub_glCreateProgram stub_glCreateProgram = NULL;
static int loaded_glCreateProgram = 0;

value glstub_glCreateProgram(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint ret;
	if(stub_glCreateProgram){ret = (*stub_glCreateProgram)();}
	else { 
		if(!loaded_glCreateProgram) { 
			loaded_glCreateProgram = 1;
			init_lib(); 
			stub_glCreateProgram=(pstub_glCreateProgram)get_proc_address("glCreateProgram");
			if(stub_glCreateProgram){ret = (*stub_glCreateProgram)();}
			else { 
				FailWith("Cannot load glCreateProgram"); 
			} 
		} else { 
			FailWith("glCreateProgram not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glCreateShader */
typedef GLuint APIENTRY (*pstub_glCreateShader)(GLenum);
static pstub_glCreateShader stub_glCreateShader = NULL;
static int loaded_glCreateShader = 0;

value glstub_glCreateShader(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLenum lv0 =  get_glenum(v0);
	GLuint ret;
	if(stub_glCreateShader){ret = (*stub_glCreateShader)(lv0);}
	else { 
		if(!loaded_glCreateShader) { 
			loaded_glCreateShader = 1;
			init_lib(); 
			stub_glCreateShader=(pstub_glCreateShader)get_proc_address("glCreateShader");
			if(stub_glCreateShader){ret = (*stub_glCreateShader)(lv0);}
			else { 
				FailWith("Cannot load glCreateShader"); 
			} 
		} else { 
			FailWith("glCreateShader not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glDeleteProgram */
typedef void APIENTRY (*pstub_glDeleteProgram)(GLuint);
static pstub_glDeleteProgram stub_glDeleteProgram = NULL;
static int loaded_glDeleteProgram = 0;

value glstub_glDeleteProgram(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glDeleteProgram){(*stub_glDeleteProgram)(lv0);}
	else { 
		if(!loaded_glDeleteProgram) { 
			loaded_glDeleteProgram = 1;
			init_lib(); 
			stub_glDeleteProgram=(pstub_glDeleteProgram)get_proc_address("glDeleteProgram");
			if(stub_glDeleteProgram){(*stub_glDeleteProgram)(lv0);}
			else { 
				FailWith("Cannot load glDeleteProgram"); 
			} 
		} else { 
			FailWith("glDeleteProgram not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDeleteShader */
typedef void APIENTRY (*pstub_glDeleteShader)(GLuint);
static pstub_glDeleteShader stub_glDeleteShader = NULL;
static int loaded_glDeleteShader = 0;

value glstub_glDeleteShader(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glDeleteShader){(*stub_glDeleteShader)(lv0);}
	else { 
		if(!loaded_glDeleteShader) { 
			loaded_glDeleteShader = 1;
			init_lib(); 
			stub_glDeleteShader=(pstub_glDeleteShader)get_proc_address("glDeleteShader");
			if(stub_glDeleteShader){(*stub_glDeleteShader)(lv0);}
			else { 
				FailWith("Cannot load glDeleteShader"); 
			} 
		} else { 
			FailWith("glDeleteShader not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDetachShader */
typedef void APIENTRY (*pstub_glDetachShader)(GLuint,GLuint);
static pstub_glDetachShader stub_glDetachShader = NULL;
static int loaded_glDetachShader = 0;

value glstub_glDetachShader(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint lv1 =  Int_val(v1);

	if(stub_glDetachShader){(*stub_glDetachShader)(lv0, lv1);}
	else { 
		if(!loaded_glDetachShader) { 
			loaded_glDetachShader = 1;
			init_lib(); 
			stub_glDetachShader=(pstub_glDetachShader)get_proc_address("glDetachShader");
			if(stub_glDetachShader){(*stub_glDetachShader)(lv0, lv1);}
			else { 
				FailWith("Cannot load glDetachShader"); 
			} 
		} else { 
			FailWith("glDetachShader not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glDisableVertexAttribArray */
typedef void APIENTRY (*pstub_glDisableVertexAttribArray)(GLuint);
static pstub_glDisableVertexAttribArray stub_glDisableVertexAttribArray = NULL;
static int loaded_glDisableVertexAttribArray = 0;

value glstub_glDisableVertexAttribArray(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glDisableVertexAttribArray){(*stub_glDisableVertexAttribArray)(lv0);}
	else { 
		if(!loaded_glDisableVertexAttribArray) { 
			loaded_glDisableVertexAttribArray = 1;
			init_lib(); 
			stub_glDisableVertexAttribArray=(pstub_glDisableVertexAttribArray)get_proc_address("glDisableVertexAttribArray");
			if(stub_glDisableVertexAttribArray){(*stub_glDisableVertexAttribArray)(lv0);}
			else { 
				FailWith("Cannot load glDisableVertexAttribArray"); 
			} 
		} else { 
			FailWith("glDisableVertexAttribArray not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glEnableVertexAttribArray */
typedef void APIENTRY (*pstub_glEnableVertexAttribArray)(GLuint);
static pstub_glEnableVertexAttribArray stub_glEnableVertexAttribArray = NULL;
static int loaded_glEnableVertexAttribArray = 0;

value glstub_glEnableVertexAttribArray(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glEnableVertexAttribArray){(*stub_glEnableVertexAttribArray)(lv0);}
	else { 
		if(!loaded_glEnableVertexAttribArray) { 
			loaded_glEnableVertexAttribArray = 1;
			init_lib(); 
			stub_glEnableVertexAttribArray=(pstub_glEnableVertexAttribArray)get_proc_address("glEnableVertexAttribArray");
			if(stub_glEnableVertexAttribArray){(*stub_glEnableVertexAttribArray)(lv0);}
			else { 
				FailWith("Cannot load glEnableVertexAttribArray"); 
			} 
		} else { 
			FailWith("glEnableVertexAttribArray not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetActiveAttrib */
typedef void APIENTRY (*pstub_glGetActiveAttrib)(GLuint,GLuint,GLsizei,GLsizei*,GLint*,GLenum*,GLchar*);
static pstub_glGetActiveAttrib stub_glGetActiveAttrib = NULL;
static int loaded_glGetActiveAttrib = 0;

value glstub_glGetActiveAttrib(value v0, value v1, value v2, value v3, value v4, value v5, value v6)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam2(v5,v6);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint lv1 =  Int_val(v1);
	GLsizei lv2 =  Int_val(v2);
	GLsizei* lv3 =  Data_bigarray_val(v3);
	GLint* lv4 =  Data_bigarray_val(v4);
	GLenum* lv5 =  Data_bigarray_val(v5);
	GLchar* lv6 =  Data_bigarray_val(v6);

	if(stub_glGetActiveAttrib){(*stub_glGetActiveAttrib)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
	else { 
		if(!loaded_glGetActiveAttrib) { 
			loaded_glGetActiveAttrib = 1;
			init_lib(); 
			stub_glGetActiveAttrib=(pstub_glGetActiveAttrib)get_proc_address("glGetActiveAttrib");
			if(stub_glGetActiveAttrib){(*stub_glGetActiveAttrib)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
			else { 
				FailWith("Cannot load glGetActiveAttrib"); 
			} 
		} else { 
			FailWith("glGetActiveAttrib not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glGetActiveAttrib_byte(value * argv, int n)
{
	return glstub_glGetActiveAttrib(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]);
}

/* glGetActiveUniform */
typedef void APIENTRY (*pstub_glGetActiveUniform)(GLuint,GLuint,GLsizei,GLsizei*,GLint*,GLenum*,GLchar*);
static pstub_glGetActiveUniform stub_glGetActiveUniform = NULL;
static int loaded_glGetActiveUniform = 0;

value glstub_glGetActiveUniform(value v0, value v1, value v2, value v3, value v4, value v5, value v6)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam2(v5,v6);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint lv1 =  Int_val(v1);
	GLsizei lv2 =  Int_val(v2);
	GLsizei* lv3 =  Data_bigarray_val(v3);
	GLint* lv4 =  Data_bigarray_val(v4);
	GLenum* lv5 =  Data_bigarray_val(v5);
	GLchar* lv6 =  Data_bigarray_val(v6);

	if(stub_glGetActiveUniform){(*stub_glGetActiveUniform)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
	else { 
		if(!loaded_glGetActiveUniform) { 
			loaded_glGetActiveUniform = 1;
			init_lib(); 
			stub_glGetActiveUniform=(pstub_glGetActiveUniform)get_proc_address("glGetActiveUniform");
			if(stub_glGetActiveUniform){(*stub_glGetActiveUniform)(lv0, lv1, lv2, lv3, lv4, lv5, lv6);}
			else { 
				FailWith("Cannot load glGetActiveUniform"); 
			} 
		} else { 
			FailWith("glGetActiveUniform not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glGetActiveUniform_byte(value * argv, int n)
{
	return glstub_glGetActiveUniform(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]);
}

/* glGetAttachedShaders */
typedef void APIENTRY (*pstub_glGetAttachedShaders)(GLuint,GLsizei,GLsizei*,GLuint*);
static pstub_glGetAttachedShaders stub_glGetAttachedShaders = NULL;
static int loaded_glGetAttachedShaders = 0;

value glstub_glGetAttachedShaders(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLsizei* lv2 =  Data_bigarray_val(v2);
	GLuint* lv3 =  Data_bigarray_val(v3);

	if(stub_glGetAttachedShaders){(*stub_glGetAttachedShaders)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glGetAttachedShaders) { 
			loaded_glGetAttachedShaders = 1;
			init_lib(); 
			stub_glGetAttachedShaders=(pstub_glGetAttachedShaders)get_proc_address("glGetAttachedShaders");
			if(stub_glGetAttachedShaders){(*stub_glGetAttachedShaders)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glGetAttachedShaders"); 
			} 
		} else { 
			FailWith("glGetAttachedShaders not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetAttribLocation */
typedef GLint APIENTRY (*pstub_glGetAttribLocation)(GLuint,GLchar*);
static pstub_glGetAttribLocation stub_glGetAttribLocation = NULL;
static int loaded_glGetAttribLocation = 0;

value glstub_glGetAttribLocation(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLchar* lv1 =  Data_bigarray_val(v1);
	GLint ret;
	if(stub_glGetAttribLocation){ret = (*stub_glGetAttribLocation)(lv0, lv1);}
	else { 
		if(!loaded_glGetAttribLocation) { 
			loaded_glGetAttribLocation = 1;
			init_lib(); 
			stub_glGetAttribLocation=(pstub_glGetAttribLocation)get_proc_address("glGetAttribLocation");
			if(stub_glGetAttribLocation){ret = (*stub_glGetAttribLocation)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGetAttribLocation"); 
			} 
		} else { 
			FailWith("glGetAttribLocation not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glGetProgramiv */
typedef void APIENTRY (*pstub_glGetProgramiv)(GLuint,GLenum,GLint*);
static pstub_glGetProgramiv stub_glGetProgramiv = NULL;
static int loaded_glGetProgramiv = 0;

value glstub_glGetProgramiv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetProgramiv){(*stub_glGetProgramiv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetProgramiv) { 
			loaded_glGetProgramiv = 1;
			init_lib(); 
			stub_glGetProgramiv=(pstub_glGetProgramiv)get_proc_address("glGetProgramiv");
			if(stub_glGetProgramiv){(*stub_glGetProgramiv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetProgramiv"); 
			} 
		} else { 
			FailWith("glGetProgramiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetProgramInfoLog */
typedef void APIENTRY (*pstub_glGetProgramInfoLog)(GLuint,GLsizei,GLsizei*,GLchar*);
static pstub_glGetProgramInfoLog stub_glGetProgramInfoLog = NULL;
static int loaded_glGetProgramInfoLog = 0;

value glstub_glGetProgramInfoLog(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLsizei* lv2 =  Data_bigarray_val(v2);
	GLchar* lv3 =  Data_bigarray_val(v3);

	if(stub_glGetProgramInfoLog){(*stub_glGetProgramInfoLog)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glGetProgramInfoLog) { 
			loaded_glGetProgramInfoLog = 1;
			init_lib(); 
			stub_glGetProgramInfoLog=(pstub_glGetProgramInfoLog)get_proc_address("glGetProgramInfoLog");
			if(stub_glGetProgramInfoLog){(*stub_glGetProgramInfoLog)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glGetProgramInfoLog"); 
			} 
		} else { 
			FailWith("glGetProgramInfoLog not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetShaderiv */
typedef void APIENTRY (*pstub_glGetShaderiv)(GLuint,GLenum,GLint*);
static pstub_glGetShaderiv stub_glGetShaderiv = NULL;
static int loaded_glGetShaderiv = 0;

value glstub_glGetShaderiv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetShaderiv){(*stub_glGetShaderiv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetShaderiv) { 
			loaded_glGetShaderiv = 1;
			init_lib(); 
			stub_glGetShaderiv=(pstub_glGetShaderiv)get_proc_address("glGetShaderiv");
			if(stub_glGetShaderiv){(*stub_glGetShaderiv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetShaderiv"); 
			} 
		} else { 
			FailWith("glGetShaderiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetShaderInfoLog */
typedef void APIENTRY (*pstub_glGetShaderInfoLog)(GLuint,GLsizei,GLsizei*,GLchar*);
static pstub_glGetShaderInfoLog stub_glGetShaderInfoLog = NULL;
static int loaded_glGetShaderInfoLog = 0;

value glstub_glGetShaderInfoLog(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLsizei* lv2 =  Data_bigarray_val(v2);
	GLchar* lv3 =  Data_bigarray_val(v3);

	if(stub_glGetShaderInfoLog){(*stub_glGetShaderInfoLog)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glGetShaderInfoLog) { 
			loaded_glGetShaderInfoLog = 1;
			init_lib(); 
			stub_glGetShaderInfoLog=(pstub_glGetShaderInfoLog)get_proc_address("glGetShaderInfoLog");
			if(stub_glGetShaderInfoLog){(*stub_glGetShaderInfoLog)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glGetShaderInfoLog"); 
			} 
		} else { 
			FailWith("glGetShaderInfoLog not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glShaderSource */
typedef void APIENTRY (*pstub_glShaderSource)(GLuint,GLsizei,GLchar**,GLint*);
static pstub_glShaderSource stub_glShaderSource = NULL;
static int loaded_glShaderSource = 0;

value glstub_glShaderSource(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLchar** lv2 =  Data_bigarray_val(v2);
	GLint* lv3 =  Data_bigarray_val(v3);

	if(stub_glShaderSource){(*stub_glShaderSource)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glShaderSource) { 
			loaded_glShaderSource = 1;
			init_lib(); 
			stub_glShaderSource=(pstub_glShaderSource)get_proc_address("glShaderSource");
			if(stub_glShaderSource){(*stub_glShaderSource)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glShaderSource"); 
			} 
		} else { 
			FailWith("glShaderSource not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetUniformLocation */
typedef GLint APIENTRY (*pstub_glGetUniformLocation)(GLint,GLchar*);
static pstub_glGetUniformLocation stub_glGetUniformLocation = NULL;
static int loaded_glGetUniformLocation = 0;

value glstub_glGetUniformLocation(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLchar* lv1 =  Data_bigarray_val(v1);
	GLint ret;
	if(stub_glGetUniformLocation){ret = (*stub_glGetUniformLocation)(lv0, lv1);}
	else { 
		if(!loaded_glGetUniformLocation) { 
			loaded_glGetUniformLocation = 1;
			init_lib(); 
			stub_glGetUniformLocation=(pstub_glGetUniformLocation)get_proc_address("glGetUniformLocation");
			if(stub_glGetUniformLocation){ret = (*stub_glGetUniformLocation)(lv0, lv1);}
			else { 
				FailWith("Cannot load glGetUniformLocation"); 
			} 
		} else { 
			FailWith("glGetUniformLocation not available"); 
		}
	}
	result = Int_val(ret);
	CAMLreturn (result);
}

/* glGetUniformfv */
typedef void APIENTRY (*pstub_glGetUniformfv)(GLuint,GLint,GLfloat*);
static pstub_glGetUniformfv stub_glGetUniformfv = NULL;
static int loaded_glGetUniformfv = 0;

value glstub_glGetUniformfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetUniformfv){(*stub_glGetUniformfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetUniformfv) { 
			loaded_glGetUniformfv = 1;
			init_lib(); 
			stub_glGetUniformfv=(pstub_glGetUniformfv)get_proc_address("glGetUniformfv");
			if(stub_glGetUniformfv){(*stub_glGetUniformfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetUniformfv"); 
			} 
		} else { 
			FailWith("glGetUniformfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetUniformiv */
typedef void APIENTRY (*pstub_glGetUniformiv)(GLuint,GLint,GLint*);
static pstub_glGetUniformiv stub_glGetUniformiv = NULL;
static int loaded_glGetUniformiv = 0;

value glstub_glGetUniformiv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetUniformiv){(*stub_glGetUniformiv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetUniformiv) { 
			loaded_glGetUniformiv = 1;
			init_lib(); 
			stub_glGetUniformiv=(pstub_glGetUniformiv)get_proc_address("glGetUniformiv");
			if(stub_glGetUniformiv){(*stub_glGetUniformiv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetUniformiv"); 
			} 
		} else { 
			FailWith("glGetUniformiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetVertexAttribdv */
typedef void APIENTRY (*pstub_glGetVertexAttribdv)(GLuint,GLenum,GLdouble*);
static pstub_glGetVertexAttribdv stub_glGetVertexAttribdv = NULL;
static int loaded_glGetVertexAttribdv = 0;

value glstub_glGetVertexAttribdv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLdouble* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetVertexAttribdv){(*stub_glGetVertexAttribdv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetVertexAttribdv) { 
			loaded_glGetVertexAttribdv = 1;
			init_lib(); 
			stub_glGetVertexAttribdv=(pstub_glGetVertexAttribdv)get_proc_address("glGetVertexAttribdv");
			if(stub_glGetVertexAttribdv){(*stub_glGetVertexAttribdv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetVertexAttribdv"); 
			} 
		} else { 
			FailWith("glGetVertexAttribdv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetVertexAttribfv */
typedef void APIENTRY (*pstub_glGetVertexAttribfv)(GLuint,GLenum,GLfloat*);
static pstub_glGetVertexAttribfv stub_glGetVertexAttribfv = NULL;
static int loaded_glGetVertexAttribfv = 0;

value glstub_glGetVertexAttribfv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetVertexAttribfv){(*stub_glGetVertexAttribfv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetVertexAttribfv) { 
			loaded_glGetVertexAttribfv = 1;
			init_lib(); 
			stub_glGetVertexAttribfv=(pstub_glGetVertexAttribfv)get_proc_address("glGetVertexAttribfv");
			if(stub_glGetVertexAttribfv){(*stub_glGetVertexAttribfv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetVertexAttribfv"); 
			} 
		} else { 
			FailWith("glGetVertexAttribfv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetVertexAttribiv */
typedef void APIENTRY (*pstub_glGetVertexAttribiv)(GLuint,GLenum,GLint*);
static pstub_glGetVertexAttribiv stub_glGetVertexAttribiv = NULL;
static int loaded_glGetVertexAttribiv = 0;

value glstub_glGetVertexAttribiv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetVertexAttribiv){(*stub_glGetVertexAttribiv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetVertexAttribiv) { 
			loaded_glGetVertexAttribiv = 1;
			init_lib(); 
			stub_glGetVertexAttribiv=(pstub_glGetVertexAttribiv)get_proc_address("glGetVertexAttribiv");
			if(stub_glGetVertexAttribiv){(*stub_glGetVertexAttribiv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetVertexAttribiv"); 
			} 
		} else { 
			FailWith("glGetVertexAttribiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetVertexAttribPointerv */
typedef void APIENTRY (*pstub_glGetVertexAttribPointerv)(GLuint,GLenum,GLvoid*);
static pstub_glGetVertexAttribPointerv stub_glGetVertexAttribPointerv = NULL;
static int loaded_glGetVertexAttribPointerv = 0;

value glstub_glGetVertexAttribPointerv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLenum lv1 =  get_glenum(v1);
	GLvoid* lv2 =  Data_bigarray_val(v2);

	if(stub_glGetVertexAttribPointerv){(*stub_glGetVertexAttribPointerv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glGetVertexAttribPointerv) { 
			loaded_glGetVertexAttribPointerv = 1;
			init_lib(); 
			stub_glGetVertexAttribPointerv=(pstub_glGetVertexAttribPointerv)get_proc_address("glGetVertexAttribPointerv");
			if(stub_glGetVertexAttribPointerv){(*stub_glGetVertexAttribPointerv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glGetVertexAttribPointerv"); 
			} 
		} else { 
			FailWith("glGetVertexAttribPointerv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glIsProgram */
typedef GLboolean APIENTRY (*pstub_glIsProgram)(GLuint);
static pstub_glIsProgram stub_glIsProgram = NULL;
static int loaded_glIsProgram = 0;

value glstub_glIsProgram(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLboolean ret;
	if(stub_glIsProgram){ret = (*stub_glIsProgram)(lv0);}
	else { 
		if(!loaded_glIsProgram) { 
			loaded_glIsProgram = 1;
			init_lib(); 
			stub_glIsProgram=(pstub_glIsProgram)get_proc_address("glIsProgram");
			if(stub_glIsProgram){ret = (*stub_glIsProgram)(lv0);}
			else { 
				FailWith("Cannot load glIsProgram"); 
			} 
		} else { 
			FailWith("glIsProgram not available"); 
		}
	}
	result = Bool_val(ret);
	CAMLreturn (result);
}

/* glIsShader */
typedef GLboolean APIENTRY (*pstub_glIsShader)(GLuint);
static pstub_glIsShader stub_glIsShader = NULL;
static int loaded_glIsShader = 0;

value glstub_glIsShader(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLboolean ret;
	if(stub_glIsShader){ret = (*stub_glIsShader)(lv0);}
	else { 
		if(!loaded_glIsShader) { 
			loaded_glIsShader = 1;
			init_lib(); 
			stub_glIsShader=(pstub_glIsShader)get_proc_address("glIsShader");
			if(stub_glIsShader){ret = (*stub_glIsShader)(lv0);}
			else { 
				FailWith("Cannot load glIsShader"); 
			} 
		} else { 
			FailWith("glIsShader not available"); 
		}
	}
	result = Bool_val(ret);
	CAMLreturn (result);
}

/* glLinkProgram */
typedef void APIENTRY (*pstub_glLinkProgram)(GLuint);
static pstub_glLinkProgram stub_glLinkProgram = NULL;
static int loaded_glLinkProgram = 0;

value glstub_glLinkProgram(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glLinkProgram){(*stub_glLinkProgram)(lv0);}
	else { 
		if(!loaded_glLinkProgram) { 
			loaded_glLinkProgram = 1;
			init_lib(); 
			stub_glLinkProgram=(pstub_glLinkProgram)get_proc_address("glLinkProgram");
			if(stub_glLinkProgram){(*stub_glLinkProgram)(lv0);}
			else { 
				FailWith("Cannot load glLinkProgram"); 
			} 
		} else { 
			FailWith("glLinkProgram not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glGetShaderSource */
typedef void APIENTRY (*pstub_glGetShaderSource)(GLint,GLsizei,GLsizei*,GLchar*);
static pstub_glGetShaderSource stub_glGetShaderSource = NULL;
static int loaded_glGetShaderSource = 0;

value glstub_glGetShaderSource(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLsizei* lv2 =  Data_bigarray_val(v2);
	GLchar* lv3 =  Data_bigarray_val(v3);

	if(stub_glGetShaderSource){(*stub_glGetShaderSource)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glGetShaderSource) { 
			loaded_glGetShaderSource = 1;
			init_lib(); 
			stub_glGetShaderSource=(pstub_glGetShaderSource)get_proc_address("glGetShaderSource");
			if(stub_glGetShaderSource){(*stub_glGetShaderSource)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glGetShaderSource"); 
			} 
		} else { 
			FailWith("glGetShaderSource not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUseProgram */
typedef void APIENTRY (*pstub_glUseProgram)(GLuint);
static pstub_glUseProgram stub_glUseProgram = NULL;
static int loaded_glUseProgram = 0;

value glstub_glUseProgram(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glUseProgram){(*stub_glUseProgram)(lv0);}
	else { 
		if(!loaded_glUseProgram) { 
			loaded_glUseProgram = 1;
			init_lib(); 
			stub_glUseProgram=(pstub_glUseProgram)get_proc_address("glUseProgram");
			if(stub_glUseProgram){(*stub_glUseProgram)(lv0);}
			else { 
				FailWith("Cannot load glUseProgram"); 
			} 
		} else { 
			FailWith("glUseProgram not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform1f */
typedef void APIENTRY (*pstub_glUniform1f)(GLint,GLfloat);
static pstub_glUniform1f stub_glUniform1f = NULL;
static int loaded_glUniform1f = 0;

value glstub_glUniform1f(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glUniform1f){(*stub_glUniform1f)(lv0, lv1);}
	else { 
		if(!loaded_glUniform1f) { 
			loaded_glUniform1f = 1;
			init_lib(); 
			stub_glUniform1f=(pstub_glUniform1f)get_proc_address("glUniform1f");
			if(stub_glUniform1f){(*stub_glUniform1f)(lv0, lv1);}
			else { 
				FailWith("Cannot load glUniform1f"); 
			} 
		} else { 
			FailWith("glUniform1f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform1fv */
typedef void APIENTRY (*pstub_glUniform1fv)(GLint,GLsizei,GLfloat*);
static pstub_glUniform1fv stub_glUniform1fv = NULL;
static int loaded_glUniform1fv = 0;

value glstub_glUniform1fv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glUniform1fv){(*stub_glUniform1fv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glUniform1fv) { 
			loaded_glUniform1fv = 1;
			init_lib(); 
			stub_glUniform1fv=(pstub_glUniform1fv)get_proc_address("glUniform1fv");
			if(stub_glUniform1fv){(*stub_glUniform1fv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glUniform1fv"); 
			} 
		} else { 
			FailWith("glUniform1fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform1i */
typedef void APIENTRY (*pstub_glUniform1i)(GLint,GLint);
static pstub_glUniform1i stub_glUniform1i = NULL;
static int loaded_glUniform1i = 0;

value glstub_glUniform1i(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);

	if(stub_glUniform1i){(*stub_glUniform1i)(lv0, lv1);}
	else { 
		if(!loaded_glUniform1i) { 
			loaded_glUniform1i = 1;
			init_lib(); 
			stub_glUniform1i=(pstub_glUniform1i)get_proc_address("glUniform1i");
			if(stub_glUniform1i){(*stub_glUniform1i)(lv0, lv1);}
			else { 
				FailWith("Cannot load glUniform1i"); 
			} 
		} else { 
			FailWith("glUniform1i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform1iv */
typedef void APIENTRY (*pstub_glUniform1iv)(GLint,GLsizei,GLint*);
static pstub_glUniform1iv stub_glUniform1iv = NULL;
static int loaded_glUniform1iv = 0;

value glstub_glUniform1iv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glUniform1iv){(*stub_glUniform1iv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glUniform1iv) { 
			loaded_glUniform1iv = 1;
			init_lib(); 
			stub_glUniform1iv=(pstub_glUniform1iv)get_proc_address("glUniform1iv");
			if(stub_glUniform1iv){(*stub_glUniform1iv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glUniform1iv"); 
			} 
		} else { 
			FailWith("glUniform1iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform2f */
typedef void APIENTRY (*pstub_glUniform2f)(GLint,GLfloat,GLfloat);
static pstub_glUniform2f stub_glUniform2f = NULL;
static int loaded_glUniform2f = 0;

value glstub_glUniform2f(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glUniform2f){(*stub_glUniform2f)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glUniform2f) { 
			loaded_glUniform2f = 1;
			init_lib(); 
			stub_glUniform2f=(pstub_glUniform2f)get_proc_address("glUniform2f");
			if(stub_glUniform2f){(*stub_glUniform2f)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glUniform2f"); 
			} 
		} else { 
			FailWith("glUniform2f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform2fv */
typedef void APIENTRY (*pstub_glUniform2fv)(GLint,GLsizei,GLfloat*);
static pstub_glUniform2fv stub_glUniform2fv = NULL;
static int loaded_glUniform2fv = 0;

value glstub_glUniform2fv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glUniform2fv){(*stub_glUniform2fv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glUniform2fv) { 
			loaded_glUniform2fv = 1;
			init_lib(); 
			stub_glUniform2fv=(pstub_glUniform2fv)get_proc_address("glUniform2fv");
			if(stub_glUniform2fv){(*stub_glUniform2fv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glUniform2fv"); 
			} 
		} else { 
			FailWith("glUniform2fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform2i */
typedef void APIENTRY (*pstub_glUniform2i)(GLint,GLint,GLint);
static pstub_glUniform2i stub_glUniform2i = NULL;
static int loaded_glUniform2i = 0;

value glstub_glUniform2i(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);

	if(stub_glUniform2i){(*stub_glUniform2i)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glUniform2i) { 
			loaded_glUniform2i = 1;
			init_lib(); 
			stub_glUniform2i=(pstub_glUniform2i)get_proc_address("glUniform2i");
			if(stub_glUniform2i){(*stub_glUniform2i)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glUniform2i"); 
			} 
		} else { 
			FailWith("glUniform2i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform2iv */
typedef void APIENTRY (*pstub_glUniform2iv)(GLint,GLsizei,GLint*);
static pstub_glUniform2iv stub_glUniform2iv = NULL;
static int loaded_glUniform2iv = 0;

value glstub_glUniform2iv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glUniform2iv){(*stub_glUniform2iv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glUniform2iv) { 
			loaded_glUniform2iv = 1;
			init_lib(); 
			stub_glUniform2iv=(pstub_glUniform2iv)get_proc_address("glUniform2iv");
			if(stub_glUniform2iv){(*stub_glUniform2iv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glUniform2iv"); 
			} 
		} else { 
			FailWith("glUniform2iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform3f */
typedef void APIENTRY (*pstub_glUniform3f)(GLint,GLfloat,GLfloat,GLfloat);
static pstub_glUniform3f stub_glUniform3f = NULL;
static int loaded_glUniform3f = 0;

value glstub_glUniform3f(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);

	if(stub_glUniform3f){(*stub_glUniform3f)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glUniform3f) { 
			loaded_glUniform3f = 1;
			init_lib(); 
			stub_glUniform3f=(pstub_glUniform3f)get_proc_address("glUniform3f");
			if(stub_glUniform3f){(*stub_glUniform3f)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glUniform3f"); 
			} 
		} else { 
			FailWith("glUniform3f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform3fv */
typedef void APIENTRY (*pstub_glUniform3fv)(GLint,GLsizei,GLfloat*);
static pstub_glUniform3fv stub_glUniform3fv = NULL;
static int loaded_glUniform3fv = 0;

value glstub_glUniform3fv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glUniform3fv){(*stub_glUniform3fv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glUniform3fv) { 
			loaded_glUniform3fv = 1;
			init_lib(); 
			stub_glUniform3fv=(pstub_glUniform3fv)get_proc_address("glUniform3fv");
			if(stub_glUniform3fv){(*stub_glUniform3fv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glUniform3fv"); 
			} 
		} else { 
			FailWith("glUniform3fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform3i */
typedef void APIENTRY (*pstub_glUniform3i)(GLint,GLint,GLint,GLint);
static pstub_glUniform3i stub_glUniform3i = NULL;
static int loaded_glUniform3i = 0;

value glstub_glUniform3i(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);

	if(stub_glUniform3i){(*stub_glUniform3i)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glUniform3i) { 
			loaded_glUniform3i = 1;
			init_lib(); 
			stub_glUniform3i=(pstub_glUniform3i)get_proc_address("glUniform3i");
			if(stub_glUniform3i){(*stub_glUniform3i)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glUniform3i"); 
			} 
		} else { 
			FailWith("glUniform3i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform3iv */
typedef void APIENTRY (*pstub_glUniform3iv)(GLint,GLsizei,GLint*);
static pstub_glUniform3iv stub_glUniform3iv = NULL;
static int loaded_glUniform3iv = 0;

value glstub_glUniform3iv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glUniform3iv){(*stub_glUniform3iv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glUniform3iv) { 
			loaded_glUniform3iv = 1;
			init_lib(); 
			stub_glUniform3iv=(pstub_glUniform3iv)get_proc_address("glUniform3iv");
			if(stub_glUniform3iv){(*stub_glUniform3iv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glUniform3iv"); 
			} 
		} else { 
			FailWith("glUniform3iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform4f */
typedef void APIENTRY (*pstub_glUniform4f)(GLint,GLfloat,GLfloat,GLfloat,GLfloat);
static pstub_glUniform4f stub_glUniform4f = NULL;
static int loaded_glUniform4f = 0;

value glstub_glUniform4f(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);
	GLfloat lv4 =  Double_val(v4);

	if(stub_glUniform4f){(*stub_glUniform4f)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glUniform4f) { 
			loaded_glUniform4f = 1;
			init_lib(); 
			stub_glUniform4f=(pstub_glUniform4f)get_proc_address("glUniform4f");
			if(stub_glUniform4f){(*stub_glUniform4f)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glUniform4f"); 
			} 
		} else { 
			FailWith("glUniform4f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform4fv */
typedef void APIENTRY (*pstub_glUniform4fv)(GLint,GLsizei,GLfloat*);
static pstub_glUniform4fv stub_glUniform4fv = NULL;
static int loaded_glUniform4fv = 0;

value glstub_glUniform4fv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLfloat* lv2 =  Data_bigarray_val(v2);

	if(stub_glUniform4fv){(*stub_glUniform4fv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glUniform4fv) { 
			loaded_glUniform4fv = 1;
			init_lib(); 
			stub_glUniform4fv=(pstub_glUniform4fv)get_proc_address("glUniform4fv");
			if(stub_glUniform4fv){(*stub_glUniform4fv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glUniform4fv"); 
			} 
		} else { 
			FailWith("glUniform4fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform4i */
typedef void APIENTRY (*pstub_glUniform4i)(GLint,GLint,GLint,GLint,GLint);
static pstub_glUniform4i stub_glUniform4i = NULL;
static int loaded_glUniform4i = 0;

value glstub_glUniform4i(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLint lv2 =  Int_val(v2);
	GLint lv3 =  Int_val(v3);
	GLint lv4 =  Int_val(v4);

	if(stub_glUniform4i){(*stub_glUniform4i)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glUniform4i) { 
			loaded_glUniform4i = 1;
			init_lib(); 
			stub_glUniform4i=(pstub_glUniform4i)get_proc_address("glUniform4i");
			if(stub_glUniform4i){(*stub_glUniform4i)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glUniform4i"); 
			} 
		} else { 
			FailWith("glUniform4i not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniform4iv */
typedef void APIENTRY (*pstub_glUniform4iv)(GLint,GLsizei,GLint*);
static pstub_glUniform4iv stub_glUniform4iv = NULL;
static int loaded_glUniform4iv = 0;

value glstub_glUniform4iv(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLint* lv2 =  Data_bigarray_val(v2);

	if(stub_glUniform4iv){(*stub_glUniform4iv)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glUniform4iv) { 
			loaded_glUniform4iv = 1;
			init_lib(); 
			stub_glUniform4iv=(pstub_glUniform4iv)get_proc_address("glUniform4iv");
			if(stub_glUniform4iv){(*stub_glUniform4iv)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glUniform4iv"); 
			} 
		} else { 
			FailWith("glUniform4iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniformMatrix2fv */
typedef void APIENTRY (*pstub_glUniformMatrix2fv)(GLint,GLsizei,GLboolean,GLfloat*);
static pstub_glUniformMatrix2fv stub_glUniformMatrix2fv = NULL;
static int loaded_glUniformMatrix2fv = 0;

value glstub_glUniformMatrix2fv(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLboolean lv2 =  Bool_val(v2);
	GLfloat* lv3 =  Data_bigarray_val(v3);

	if(stub_glUniformMatrix2fv){(*stub_glUniformMatrix2fv)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glUniformMatrix2fv) { 
			loaded_glUniformMatrix2fv = 1;
			init_lib(); 
			stub_glUniformMatrix2fv=(pstub_glUniformMatrix2fv)get_proc_address("glUniformMatrix2fv");
			if(stub_glUniformMatrix2fv){(*stub_glUniformMatrix2fv)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glUniformMatrix2fv"); 
			} 
		} else { 
			FailWith("glUniformMatrix2fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniformMatrix3fv */
typedef void APIENTRY (*pstub_glUniformMatrix3fv)(GLint,GLsizei,GLboolean,GLfloat*);
static pstub_glUniformMatrix3fv stub_glUniformMatrix3fv = NULL;
static int loaded_glUniformMatrix3fv = 0;

value glstub_glUniformMatrix3fv(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLboolean lv2 =  Bool_val(v2);
	GLfloat* lv3 =  Data_bigarray_val(v3);

	if(stub_glUniformMatrix3fv){(*stub_glUniformMatrix3fv)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glUniformMatrix3fv) { 
			loaded_glUniformMatrix3fv = 1;
			init_lib(); 
			stub_glUniformMatrix3fv=(pstub_glUniformMatrix3fv)get_proc_address("glUniformMatrix3fv");
			if(stub_glUniformMatrix3fv){(*stub_glUniformMatrix3fv)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glUniformMatrix3fv"); 
			} 
		} else { 
			FailWith("glUniformMatrix3fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniformMatrix4fv */
typedef void APIENTRY (*pstub_glUniformMatrix4fv)(GLint,GLsizei,GLboolean,GLfloat*);
static pstub_glUniformMatrix4fv stub_glUniformMatrix4fv = NULL;
static int loaded_glUniformMatrix4fv = 0;

value glstub_glUniformMatrix4fv(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLboolean lv2 =  Bool_val(v2);
	GLfloat* lv3 =  Data_bigarray_val(v3);

	if(stub_glUniformMatrix4fv){(*stub_glUniformMatrix4fv)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glUniformMatrix4fv) { 
			loaded_glUniformMatrix4fv = 1;
			init_lib(); 
			stub_glUniformMatrix4fv=(pstub_glUniformMatrix4fv)get_proc_address("glUniformMatrix4fv");
			if(stub_glUniformMatrix4fv){(*stub_glUniformMatrix4fv)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glUniformMatrix4fv"); 
			} 
		} else { 
			FailWith("glUniformMatrix4fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glValidateProgram */
typedef void APIENTRY (*pstub_glValidateProgram)(GLuint);
static pstub_glValidateProgram stub_glValidateProgram = NULL;
static int loaded_glValidateProgram = 0;

value glstub_glValidateProgram(value v0)
{
	CAMLparam1(v0);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);

	if(stub_glValidateProgram){(*stub_glValidateProgram)(lv0);}
	else { 
		if(!loaded_glValidateProgram) { 
			loaded_glValidateProgram = 1;
			init_lib(); 
			stub_glValidateProgram=(pstub_glValidateProgram)get_proc_address("glValidateProgram");
			if(stub_glValidateProgram){(*stub_glValidateProgram)(lv0);}
			else { 
				FailWith("Cannot load glValidateProgram"); 
			} 
		} else { 
			FailWith("glValidateProgram not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib1d */
typedef void APIENTRY (*pstub_glVertexAttrib1d)(GLuint,GLdouble);
static pstub_glVertexAttrib1d stub_glVertexAttrib1d = NULL;
static int loaded_glVertexAttrib1d = 0;

value glstub_glVertexAttrib1d(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLdouble lv1 =  Double_val(v1);

	if(stub_glVertexAttrib1d){(*stub_glVertexAttrib1d)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib1d) { 
			loaded_glVertexAttrib1d = 1;
			init_lib(); 
			stub_glVertexAttrib1d=(pstub_glVertexAttrib1d)get_proc_address("glVertexAttrib1d");
			if(stub_glVertexAttrib1d){(*stub_glVertexAttrib1d)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib1d"); 
			} 
		} else { 
			FailWith("glVertexAttrib1d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib1dv */
typedef void APIENTRY (*pstub_glVertexAttrib1dv)(GLuint,GLdouble*);
static pstub_glVertexAttrib1dv stub_glVertexAttrib1dv = NULL;
static int loaded_glVertexAttrib1dv = 0;

value glstub_glVertexAttrib1dv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib1dv){(*stub_glVertexAttrib1dv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib1dv) { 
			loaded_glVertexAttrib1dv = 1;
			init_lib(); 
			stub_glVertexAttrib1dv=(pstub_glVertexAttrib1dv)get_proc_address("glVertexAttrib1dv");
			if(stub_glVertexAttrib1dv){(*stub_glVertexAttrib1dv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib1dv"); 
			} 
		} else { 
			FailWith("glVertexAttrib1dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib1f */
typedef void APIENTRY (*pstub_glVertexAttrib1f)(GLuint,GLfloat);
static pstub_glVertexAttrib1f stub_glVertexAttrib1f = NULL;
static int loaded_glVertexAttrib1f = 0;

value glstub_glVertexAttrib1f(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLfloat lv1 =  Double_val(v1);

	if(stub_glVertexAttrib1f){(*stub_glVertexAttrib1f)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib1f) { 
			loaded_glVertexAttrib1f = 1;
			init_lib(); 
			stub_glVertexAttrib1f=(pstub_glVertexAttrib1f)get_proc_address("glVertexAttrib1f");
			if(stub_glVertexAttrib1f){(*stub_glVertexAttrib1f)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib1f"); 
			} 
		} else { 
			FailWith("glVertexAttrib1f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib1fv */
typedef void APIENTRY (*pstub_glVertexAttrib1fv)(GLuint,GLfloat*);
static pstub_glVertexAttrib1fv stub_glVertexAttrib1fv = NULL;
static int loaded_glVertexAttrib1fv = 0;

value glstub_glVertexAttrib1fv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib1fv){(*stub_glVertexAttrib1fv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib1fv) { 
			loaded_glVertexAttrib1fv = 1;
			init_lib(); 
			stub_glVertexAttrib1fv=(pstub_glVertexAttrib1fv)get_proc_address("glVertexAttrib1fv");
			if(stub_glVertexAttrib1fv){(*stub_glVertexAttrib1fv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib1fv"); 
			} 
		} else { 
			FailWith("glVertexAttrib1fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib1s */
typedef void APIENTRY (*pstub_glVertexAttrib1s)(GLuint,GLshort);
static pstub_glVertexAttrib1s stub_glVertexAttrib1s = NULL;
static int loaded_glVertexAttrib1s = 0;

value glstub_glVertexAttrib1s(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);

	if(stub_glVertexAttrib1s){(*stub_glVertexAttrib1s)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib1s) { 
			loaded_glVertexAttrib1s = 1;
			init_lib(); 
			stub_glVertexAttrib1s=(pstub_glVertexAttrib1s)get_proc_address("glVertexAttrib1s");
			if(stub_glVertexAttrib1s){(*stub_glVertexAttrib1s)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib1s"); 
			} 
		} else { 
			FailWith("glVertexAttrib1s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib1sv */
typedef void APIENTRY (*pstub_glVertexAttrib1sv)(GLuint,GLshort*);
static pstub_glVertexAttrib1sv stub_glVertexAttrib1sv = NULL;
static int loaded_glVertexAttrib1sv = 0;

value glstub_glVertexAttrib1sv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib1sv){(*stub_glVertexAttrib1sv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib1sv) { 
			loaded_glVertexAttrib1sv = 1;
			init_lib(); 
			stub_glVertexAttrib1sv=(pstub_glVertexAttrib1sv)get_proc_address("glVertexAttrib1sv");
			if(stub_glVertexAttrib1sv){(*stub_glVertexAttrib1sv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib1sv"); 
			} 
		} else { 
			FailWith("glVertexAttrib1sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib2d */
typedef void APIENTRY (*pstub_glVertexAttrib2d)(GLuint,GLdouble,GLdouble);
static pstub_glVertexAttrib2d stub_glVertexAttrib2d = NULL;
static int loaded_glVertexAttrib2d = 0;

value glstub_glVertexAttrib2d(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);

	if(stub_glVertexAttrib2d){(*stub_glVertexAttrib2d)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glVertexAttrib2d) { 
			loaded_glVertexAttrib2d = 1;
			init_lib(); 
			stub_glVertexAttrib2d=(pstub_glVertexAttrib2d)get_proc_address("glVertexAttrib2d");
			if(stub_glVertexAttrib2d){(*stub_glVertexAttrib2d)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glVertexAttrib2d"); 
			} 
		} else { 
			FailWith("glVertexAttrib2d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib2dv */
typedef void APIENTRY (*pstub_glVertexAttrib2dv)(GLuint,GLdouble*);
static pstub_glVertexAttrib2dv stub_glVertexAttrib2dv = NULL;
static int loaded_glVertexAttrib2dv = 0;

value glstub_glVertexAttrib2dv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib2dv){(*stub_glVertexAttrib2dv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib2dv) { 
			loaded_glVertexAttrib2dv = 1;
			init_lib(); 
			stub_glVertexAttrib2dv=(pstub_glVertexAttrib2dv)get_proc_address("glVertexAttrib2dv");
			if(stub_glVertexAttrib2dv){(*stub_glVertexAttrib2dv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib2dv"); 
			} 
		} else { 
			FailWith("glVertexAttrib2dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib2f */
typedef void APIENTRY (*pstub_glVertexAttrib2f)(GLuint,GLfloat,GLfloat);
static pstub_glVertexAttrib2f stub_glVertexAttrib2f = NULL;
static int loaded_glVertexAttrib2f = 0;

value glstub_glVertexAttrib2f(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);

	if(stub_glVertexAttrib2f){(*stub_glVertexAttrib2f)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glVertexAttrib2f) { 
			loaded_glVertexAttrib2f = 1;
			init_lib(); 
			stub_glVertexAttrib2f=(pstub_glVertexAttrib2f)get_proc_address("glVertexAttrib2f");
			if(stub_glVertexAttrib2f){(*stub_glVertexAttrib2f)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glVertexAttrib2f"); 
			} 
		} else { 
			FailWith("glVertexAttrib2f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib2fv */
typedef void APIENTRY (*pstub_glVertexAttrib2fv)(GLuint,GLfloat*);
static pstub_glVertexAttrib2fv stub_glVertexAttrib2fv = NULL;
static int loaded_glVertexAttrib2fv = 0;

value glstub_glVertexAttrib2fv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib2fv){(*stub_glVertexAttrib2fv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib2fv) { 
			loaded_glVertexAttrib2fv = 1;
			init_lib(); 
			stub_glVertexAttrib2fv=(pstub_glVertexAttrib2fv)get_proc_address("glVertexAttrib2fv");
			if(stub_glVertexAttrib2fv){(*stub_glVertexAttrib2fv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib2fv"); 
			} 
		} else { 
			FailWith("glVertexAttrib2fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib2s */
typedef void APIENTRY (*pstub_glVertexAttrib2s)(GLuint,GLshort,GLshort);
static pstub_glVertexAttrib2s stub_glVertexAttrib2s = NULL;
static int loaded_glVertexAttrib2s = 0;

value glstub_glVertexAttrib2s(value v0, value v1, value v2)
{
	CAMLparam3(v0,v1,v2);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);

	if(stub_glVertexAttrib2s){(*stub_glVertexAttrib2s)(lv0, lv1, lv2);}
	else { 
		if(!loaded_glVertexAttrib2s) { 
			loaded_glVertexAttrib2s = 1;
			init_lib(); 
			stub_glVertexAttrib2s=(pstub_glVertexAttrib2s)get_proc_address("glVertexAttrib2s");
			if(stub_glVertexAttrib2s){(*stub_glVertexAttrib2s)(lv0, lv1, lv2);}
			else { 
				FailWith("Cannot load glVertexAttrib2s"); 
			} 
		} else { 
			FailWith("glVertexAttrib2s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib2sv */
typedef void APIENTRY (*pstub_glVertexAttrib2sv)(GLuint,GLshort*);
static pstub_glVertexAttrib2sv stub_glVertexAttrib2sv = NULL;
static int loaded_glVertexAttrib2sv = 0;

value glstub_glVertexAttrib2sv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib2sv){(*stub_glVertexAttrib2sv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib2sv) { 
			loaded_glVertexAttrib2sv = 1;
			init_lib(); 
			stub_glVertexAttrib2sv=(pstub_glVertexAttrib2sv)get_proc_address("glVertexAttrib2sv");
			if(stub_glVertexAttrib2sv){(*stub_glVertexAttrib2sv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib2sv"); 
			} 
		} else { 
			FailWith("glVertexAttrib2sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib3d */
typedef void APIENTRY (*pstub_glVertexAttrib3d)(GLuint,GLdouble,GLdouble,GLdouble);
static pstub_glVertexAttrib3d stub_glVertexAttrib3d = NULL;
static int loaded_glVertexAttrib3d = 0;

value glstub_glVertexAttrib3d(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);

	if(stub_glVertexAttrib3d){(*stub_glVertexAttrib3d)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glVertexAttrib3d) { 
			loaded_glVertexAttrib3d = 1;
			init_lib(); 
			stub_glVertexAttrib3d=(pstub_glVertexAttrib3d)get_proc_address("glVertexAttrib3d");
			if(stub_glVertexAttrib3d){(*stub_glVertexAttrib3d)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glVertexAttrib3d"); 
			} 
		} else { 
			FailWith("glVertexAttrib3d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib3dv */
typedef void APIENTRY (*pstub_glVertexAttrib3dv)(GLuint,GLdouble*);
static pstub_glVertexAttrib3dv stub_glVertexAttrib3dv = NULL;
static int loaded_glVertexAttrib3dv = 0;

value glstub_glVertexAttrib3dv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib3dv){(*stub_glVertexAttrib3dv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib3dv) { 
			loaded_glVertexAttrib3dv = 1;
			init_lib(); 
			stub_glVertexAttrib3dv=(pstub_glVertexAttrib3dv)get_proc_address("glVertexAttrib3dv");
			if(stub_glVertexAttrib3dv){(*stub_glVertexAttrib3dv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib3dv"); 
			} 
		} else { 
			FailWith("glVertexAttrib3dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib3f */
typedef void APIENTRY (*pstub_glVertexAttrib3f)(GLuint,GLfloat,GLfloat,GLfloat);
static pstub_glVertexAttrib3f stub_glVertexAttrib3f = NULL;
static int loaded_glVertexAttrib3f = 0;

value glstub_glVertexAttrib3f(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);

	if(stub_glVertexAttrib3f){(*stub_glVertexAttrib3f)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glVertexAttrib3f) { 
			loaded_glVertexAttrib3f = 1;
			init_lib(); 
			stub_glVertexAttrib3f=(pstub_glVertexAttrib3f)get_proc_address("glVertexAttrib3f");
			if(stub_glVertexAttrib3f){(*stub_glVertexAttrib3f)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glVertexAttrib3f"); 
			} 
		} else { 
			FailWith("glVertexAttrib3f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib3fv */
typedef void APIENTRY (*pstub_glVertexAttrib3fv)(GLuint,GLfloat*);
static pstub_glVertexAttrib3fv stub_glVertexAttrib3fv = NULL;
static int loaded_glVertexAttrib3fv = 0;

value glstub_glVertexAttrib3fv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib3fv){(*stub_glVertexAttrib3fv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib3fv) { 
			loaded_glVertexAttrib3fv = 1;
			init_lib(); 
			stub_glVertexAttrib3fv=(pstub_glVertexAttrib3fv)get_proc_address("glVertexAttrib3fv");
			if(stub_glVertexAttrib3fv){(*stub_glVertexAttrib3fv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib3fv"); 
			} 
		} else { 
			FailWith("glVertexAttrib3fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib3s */
typedef void APIENTRY (*pstub_glVertexAttrib3s)(GLuint,GLshort,GLshort,GLshort);
static pstub_glVertexAttrib3s stub_glVertexAttrib3s = NULL;
static int loaded_glVertexAttrib3s = 0;

value glstub_glVertexAttrib3s(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);

	if(stub_glVertexAttrib3s){(*stub_glVertexAttrib3s)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glVertexAttrib3s) { 
			loaded_glVertexAttrib3s = 1;
			init_lib(); 
			stub_glVertexAttrib3s=(pstub_glVertexAttrib3s)get_proc_address("glVertexAttrib3s");
			if(stub_glVertexAttrib3s){(*stub_glVertexAttrib3s)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glVertexAttrib3s"); 
			} 
		} else { 
			FailWith("glVertexAttrib3s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib3sv */
typedef void APIENTRY (*pstub_glVertexAttrib3sv)(GLuint,GLshort*);
static pstub_glVertexAttrib3sv stub_glVertexAttrib3sv = NULL;
static int loaded_glVertexAttrib3sv = 0;

value glstub_glVertexAttrib3sv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib3sv){(*stub_glVertexAttrib3sv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib3sv) { 
			loaded_glVertexAttrib3sv = 1;
			init_lib(); 
			stub_glVertexAttrib3sv=(pstub_glVertexAttrib3sv)get_proc_address("glVertexAttrib3sv");
			if(stub_glVertexAttrib3sv){(*stub_glVertexAttrib3sv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib3sv"); 
			} 
		} else { 
			FailWith("glVertexAttrib3sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4Nbv */
typedef void APIENTRY (*pstub_glVertexAttrib4Nbv)(GLuint,GLbyte*);
static pstub_glVertexAttrib4Nbv stub_glVertexAttrib4Nbv = NULL;
static int loaded_glVertexAttrib4Nbv = 0;

value glstub_glVertexAttrib4Nbv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLbyte* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4Nbv){(*stub_glVertexAttrib4Nbv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4Nbv) { 
			loaded_glVertexAttrib4Nbv = 1;
			init_lib(); 
			stub_glVertexAttrib4Nbv=(pstub_glVertexAttrib4Nbv)get_proc_address("glVertexAttrib4Nbv");
			if(stub_glVertexAttrib4Nbv){(*stub_glVertexAttrib4Nbv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4Nbv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4Nbv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4Niv */
typedef void APIENTRY (*pstub_glVertexAttrib4Niv)(GLuint,GLint*);
static pstub_glVertexAttrib4Niv stub_glVertexAttrib4Niv = NULL;
static int loaded_glVertexAttrib4Niv = 0;

value glstub_glVertexAttrib4Niv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4Niv){(*stub_glVertexAttrib4Niv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4Niv) { 
			loaded_glVertexAttrib4Niv = 1;
			init_lib(); 
			stub_glVertexAttrib4Niv=(pstub_glVertexAttrib4Niv)get_proc_address("glVertexAttrib4Niv");
			if(stub_glVertexAttrib4Niv){(*stub_glVertexAttrib4Niv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4Niv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4Niv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4Nsv */
typedef void APIENTRY (*pstub_glVertexAttrib4Nsv)(GLuint,GLshort*);
static pstub_glVertexAttrib4Nsv stub_glVertexAttrib4Nsv = NULL;
static int loaded_glVertexAttrib4Nsv = 0;

value glstub_glVertexAttrib4Nsv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4Nsv){(*stub_glVertexAttrib4Nsv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4Nsv) { 
			loaded_glVertexAttrib4Nsv = 1;
			init_lib(); 
			stub_glVertexAttrib4Nsv=(pstub_glVertexAttrib4Nsv)get_proc_address("glVertexAttrib4Nsv");
			if(stub_glVertexAttrib4Nsv){(*stub_glVertexAttrib4Nsv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4Nsv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4Nsv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4Nub */
typedef void APIENTRY (*pstub_glVertexAttrib4Nub)(GLuint,GLubyte,GLubyte,GLubyte,GLubyte);
static pstub_glVertexAttrib4Nub stub_glVertexAttrib4Nub = NULL;
static int loaded_glVertexAttrib4Nub = 0;

value glstub_glVertexAttrib4Nub(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLubyte lv1 =  Int_val(v1);
	GLubyte lv2 =  Int_val(v2);
	GLubyte lv3 =  Int_val(v3);
	GLubyte lv4 =  Int_val(v4);

	if(stub_glVertexAttrib4Nub){(*stub_glVertexAttrib4Nub)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glVertexAttrib4Nub) { 
			loaded_glVertexAttrib4Nub = 1;
			init_lib(); 
			stub_glVertexAttrib4Nub=(pstub_glVertexAttrib4Nub)get_proc_address("glVertexAttrib4Nub");
			if(stub_glVertexAttrib4Nub){(*stub_glVertexAttrib4Nub)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glVertexAttrib4Nub"); 
			} 
		} else { 
			FailWith("glVertexAttrib4Nub not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4Nubv */
typedef void APIENTRY (*pstub_glVertexAttrib4Nubv)(GLuint,GLubyte*);
static pstub_glVertexAttrib4Nubv stub_glVertexAttrib4Nubv = NULL;
static int loaded_glVertexAttrib4Nubv = 0;

value glstub_glVertexAttrib4Nubv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLubyte* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4Nubv){(*stub_glVertexAttrib4Nubv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4Nubv) { 
			loaded_glVertexAttrib4Nubv = 1;
			init_lib(); 
			stub_glVertexAttrib4Nubv=(pstub_glVertexAttrib4Nubv)get_proc_address("glVertexAttrib4Nubv");
			if(stub_glVertexAttrib4Nubv){(*stub_glVertexAttrib4Nubv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4Nubv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4Nubv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4Nuiv */
typedef void APIENTRY (*pstub_glVertexAttrib4Nuiv)(GLuint,GLuint*);
static pstub_glVertexAttrib4Nuiv stub_glVertexAttrib4Nuiv = NULL;
static int loaded_glVertexAttrib4Nuiv = 0;

value glstub_glVertexAttrib4Nuiv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4Nuiv){(*stub_glVertexAttrib4Nuiv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4Nuiv) { 
			loaded_glVertexAttrib4Nuiv = 1;
			init_lib(); 
			stub_glVertexAttrib4Nuiv=(pstub_glVertexAttrib4Nuiv)get_proc_address("glVertexAttrib4Nuiv");
			if(stub_glVertexAttrib4Nuiv){(*stub_glVertexAttrib4Nuiv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4Nuiv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4Nuiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4Nusv */
typedef void APIENTRY (*pstub_glVertexAttrib4Nusv)(GLuint,GLushort*);
static pstub_glVertexAttrib4Nusv stub_glVertexAttrib4Nusv = NULL;
static int loaded_glVertexAttrib4Nusv = 0;

value glstub_glVertexAttrib4Nusv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLushort* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4Nusv){(*stub_glVertexAttrib4Nusv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4Nusv) { 
			loaded_glVertexAttrib4Nusv = 1;
			init_lib(); 
			stub_glVertexAttrib4Nusv=(pstub_glVertexAttrib4Nusv)get_proc_address("glVertexAttrib4Nusv");
			if(stub_glVertexAttrib4Nusv){(*stub_glVertexAttrib4Nusv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4Nusv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4Nusv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4bv */
typedef void APIENTRY (*pstub_glVertexAttrib4bv)(GLuint,GLbyte*);
static pstub_glVertexAttrib4bv stub_glVertexAttrib4bv = NULL;
static int loaded_glVertexAttrib4bv = 0;

value glstub_glVertexAttrib4bv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLbyte* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4bv){(*stub_glVertexAttrib4bv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4bv) { 
			loaded_glVertexAttrib4bv = 1;
			init_lib(); 
			stub_glVertexAttrib4bv=(pstub_glVertexAttrib4bv)get_proc_address("glVertexAttrib4bv");
			if(stub_glVertexAttrib4bv){(*stub_glVertexAttrib4bv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4bv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4bv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4d */
typedef void APIENTRY (*pstub_glVertexAttrib4d)(GLuint,GLdouble,GLdouble,GLdouble,GLdouble);
static pstub_glVertexAttrib4d stub_glVertexAttrib4d = NULL;
static int loaded_glVertexAttrib4d = 0;

value glstub_glVertexAttrib4d(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLdouble lv1 =  Double_val(v1);
	GLdouble lv2 =  Double_val(v2);
	GLdouble lv3 =  Double_val(v3);
	GLdouble lv4 =  Double_val(v4);

	if(stub_glVertexAttrib4d){(*stub_glVertexAttrib4d)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glVertexAttrib4d) { 
			loaded_glVertexAttrib4d = 1;
			init_lib(); 
			stub_glVertexAttrib4d=(pstub_glVertexAttrib4d)get_proc_address("glVertexAttrib4d");
			if(stub_glVertexAttrib4d){(*stub_glVertexAttrib4d)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glVertexAttrib4d"); 
			} 
		} else { 
			FailWith("glVertexAttrib4d not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4dv */
typedef void APIENTRY (*pstub_glVertexAttrib4dv)(GLuint,GLdouble*);
static pstub_glVertexAttrib4dv stub_glVertexAttrib4dv = NULL;
static int loaded_glVertexAttrib4dv = 0;

value glstub_glVertexAttrib4dv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLdouble* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4dv){(*stub_glVertexAttrib4dv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4dv) { 
			loaded_glVertexAttrib4dv = 1;
			init_lib(); 
			stub_glVertexAttrib4dv=(pstub_glVertexAttrib4dv)get_proc_address("glVertexAttrib4dv");
			if(stub_glVertexAttrib4dv){(*stub_glVertexAttrib4dv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4dv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4dv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4f */
typedef void APIENTRY (*pstub_glVertexAttrib4f)(GLuint,GLfloat,GLfloat,GLfloat,GLfloat);
static pstub_glVertexAttrib4f stub_glVertexAttrib4f = NULL;
static int loaded_glVertexAttrib4f = 0;

value glstub_glVertexAttrib4f(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLfloat lv1 =  Double_val(v1);
	GLfloat lv2 =  Double_val(v2);
	GLfloat lv3 =  Double_val(v3);
	GLfloat lv4 =  Double_val(v4);

	if(stub_glVertexAttrib4f){(*stub_glVertexAttrib4f)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glVertexAttrib4f) { 
			loaded_glVertexAttrib4f = 1;
			init_lib(); 
			stub_glVertexAttrib4f=(pstub_glVertexAttrib4f)get_proc_address("glVertexAttrib4f");
			if(stub_glVertexAttrib4f){(*stub_glVertexAttrib4f)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glVertexAttrib4f"); 
			} 
		} else { 
			FailWith("glVertexAttrib4f not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4fv */
typedef void APIENTRY (*pstub_glVertexAttrib4fv)(GLuint,GLfloat*);
static pstub_glVertexAttrib4fv stub_glVertexAttrib4fv = NULL;
static int loaded_glVertexAttrib4fv = 0;

value glstub_glVertexAttrib4fv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLfloat* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4fv){(*stub_glVertexAttrib4fv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4fv) { 
			loaded_glVertexAttrib4fv = 1;
			init_lib(); 
			stub_glVertexAttrib4fv=(pstub_glVertexAttrib4fv)get_proc_address("glVertexAttrib4fv");
			if(stub_glVertexAttrib4fv){(*stub_glVertexAttrib4fv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4fv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4iv */
typedef void APIENTRY (*pstub_glVertexAttrib4iv)(GLuint,GLint*);
static pstub_glVertexAttrib4iv stub_glVertexAttrib4iv = NULL;
static int loaded_glVertexAttrib4iv = 0;

value glstub_glVertexAttrib4iv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLint* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4iv){(*stub_glVertexAttrib4iv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4iv) { 
			loaded_glVertexAttrib4iv = 1;
			init_lib(); 
			stub_glVertexAttrib4iv=(pstub_glVertexAttrib4iv)get_proc_address("glVertexAttrib4iv");
			if(stub_glVertexAttrib4iv){(*stub_glVertexAttrib4iv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4iv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4iv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4s */
typedef void APIENTRY (*pstub_glVertexAttrib4s)(GLuint,GLshort,GLshort,GLshort,GLshort);
static pstub_glVertexAttrib4s stub_glVertexAttrib4s = NULL;
static int loaded_glVertexAttrib4s = 0;

value glstub_glVertexAttrib4s(value v0, value v1, value v2, value v3, value v4)
{
	CAMLparam5(v0, v1, v2, v3, v4);

	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLshort lv1 =  Int_val(v1);
	GLshort lv2 =  Int_val(v2);
	GLshort lv3 =  Int_val(v3);
	GLshort lv4 =  Int_val(v4);

	if(stub_glVertexAttrib4s){(*stub_glVertexAttrib4s)(lv0, lv1, lv2, lv3, lv4);}
	else { 
		if(!loaded_glVertexAttrib4s) { 
			loaded_glVertexAttrib4s = 1;
			init_lib(); 
			stub_glVertexAttrib4s=(pstub_glVertexAttrib4s)get_proc_address("glVertexAttrib4s");
			if(stub_glVertexAttrib4s){(*stub_glVertexAttrib4s)(lv0, lv1, lv2, lv3, lv4);}
			else { 
				FailWith("Cannot load glVertexAttrib4s"); 
			} 
		} else { 
			FailWith("glVertexAttrib4s not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4sv */
typedef void APIENTRY (*pstub_glVertexAttrib4sv)(GLuint,GLshort*);
static pstub_glVertexAttrib4sv stub_glVertexAttrib4sv = NULL;
static int loaded_glVertexAttrib4sv = 0;

value glstub_glVertexAttrib4sv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLshort* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4sv){(*stub_glVertexAttrib4sv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4sv) { 
			loaded_glVertexAttrib4sv = 1;
			init_lib(); 
			stub_glVertexAttrib4sv=(pstub_glVertexAttrib4sv)get_proc_address("glVertexAttrib4sv");
			if(stub_glVertexAttrib4sv){(*stub_glVertexAttrib4sv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4sv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4sv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4ubv */
typedef void APIENTRY (*pstub_glVertexAttrib4ubv)(GLuint,GLubyte*);
static pstub_glVertexAttrib4ubv stub_glVertexAttrib4ubv = NULL;
static int loaded_glVertexAttrib4ubv = 0;

value glstub_glVertexAttrib4ubv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLubyte* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4ubv){(*stub_glVertexAttrib4ubv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4ubv) { 
			loaded_glVertexAttrib4ubv = 1;
			init_lib(); 
			stub_glVertexAttrib4ubv=(pstub_glVertexAttrib4ubv)get_proc_address("glVertexAttrib4ubv");
			if(stub_glVertexAttrib4ubv){(*stub_glVertexAttrib4ubv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4ubv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4ubv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4uiv */
typedef void APIENTRY (*pstub_glVertexAttrib4uiv)(GLuint,GLuint*);
static pstub_glVertexAttrib4uiv stub_glVertexAttrib4uiv = NULL;
static int loaded_glVertexAttrib4uiv = 0;

value glstub_glVertexAttrib4uiv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLuint* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4uiv){(*stub_glVertexAttrib4uiv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4uiv) { 
			loaded_glVertexAttrib4uiv = 1;
			init_lib(); 
			stub_glVertexAttrib4uiv=(pstub_glVertexAttrib4uiv)get_proc_address("glVertexAttrib4uiv");
			if(stub_glVertexAttrib4uiv){(*stub_glVertexAttrib4uiv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4uiv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4uiv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttrib4usv */
typedef void APIENTRY (*pstub_glVertexAttrib4usv)(GLuint,GLushort*);
static pstub_glVertexAttrib4usv stub_glVertexAttrib4usv = NULL;
static int loaded_glVertexAttrib4usv = 0;

value glstub_glVertexAttrib4usv(value v0, value v1)
{
	CAMLparam2(v0,v1);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLushort* lv1 =  Data_bigarray_val(v1);

	if(stub_glVertexAttrib4usv){(*stub_glVertexAttrib4usv)(lv0, lv1);}
	else { 
		if(!loaded_glVertexAttrib4usv) { 
			loaded_glVertexAttrib4usv = 1;
			init_lib(); 
			stub_glVertexAttrib4usv=(pstub_glVertexAttrib4usv)get_proc_address("glVertexAttrib4usv");
			if(stub_glVertexAttrib4usv){(*stub_glVertexAttrib4usv)(lv0, lv1);}
			else { 
				FailWith("Cannot load glVertexAttrib4usv"); 
			} 
		} else { 
			FailWith("glVertexAttrib4usv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glVertexAttribPointer */
typedef void APIENTRY (*pstub_glVertexAttribPointer)(GLuint,GLint,GLenum,GLboolean,GLsizei,GLvoid*);
static pstub_glVertexAttribPointer stub_glVertexAttribPointer = NULL;
static int loaded_glVertexAttribPointer = 0;

value glstub_glVertexAttribPointer(value v0, value v1, value v2, value v3, value v4, value v5)
{
	CAMLparam5(v0, v1, v2, v3, v4);
	CAMLxparam1(v5);
	CAMLlocal1(result);
	GLuint lv0 =  Int_val(v0);
	GLint lv1 =  Int_val(v1);
	GLenum lv2 =  get_glenum(v2);
	GLboolean lv3 =  Bool_val(v3);
	GLsizei lv4 =  Int_val(v4);
	GLvoid* lv5 =  Data_bigarray_val(v5);

	if(stub_glVertexAttribPointer){(*stub_glVertexAttribPointer)(lv0, lv1, lv2, lv3, lv4, lv5);}
	else { 
		if(!loaded_glVertexAttribPointer) { 
			loaded_glVertexAttribPointer = 1;
			init_lib(); 
			stub_glVertexAttribPointer=(pstub_glVertexAttribPointer)get_proc_address("glVertexAttribPointer");
			if(stub_glVertexAttribPointer){(*stub_glVertexAttribPointer)(lv0, lv1, lv2, lv3, lv4, lv5);}
			else { 
				FailWith("Cannot load glVertexAttribPointer"); 
			} 
		} else { 
			FailWith("glVertexAttribPointer not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}


value glstub_glVertexAttribPointer_byte(value * argv, int n)
{
	return glstub_glVertexAttribPointer(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5]);
}

/* glUniformMatrix2x3fv */
typedef void APIENTRY (*pstub_glUniformMatrix2x3fv)(GLint,GLsizei,GLboolean,GLfloat*);
static pstub_glUniformMatrix2x3fv stub_glUniformMatrix2x3fv = NULL;
static int loaded_glUniformMatrix2x3fv = 0;

value glstub_glUniformMatrix2x3fv(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLboolean lv2 =  Bool_val(v2);
	GLfloat* lv3 =  Data_bigarray_val(v3);

	if(stub_glUniformMatrix2x3fv){(*stub_glUniformMatrix2x3fv)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glUniformMatrix2x3fv) { 
			loaded_glUniformMatrix2x3fv = 1;
			init_lib(); 
			stub_glUniformMatrix2x3fv=(pstub_glUniformMatrix2x3fv)get_proc_address("glUniformMatrix2x3fv");
			if(stub_glUniformMatrix2x3fv){(*stub_glUniformMatrix2x3fv)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glUniformMatrix2x3fv"); 
			} 
		} else { 
			FailWith("glUniformMatrix2x3fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniformMatrix2x4fv */
typedef void APIENTRY (*pstub_glUniformMatrix2x4fv)(GLint,GLsizei,GLboolean,GLfloat*);
static pstub_glUniformMatrix2x4fv stub_glUniformMatrix2x4fv = NULL;
static int loaded_glUniformMatrix2x4fv = 0;

value glstub_glUniformMatrix2x4fv(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLboolean lv2 =  Bool_val(v2);
	GLfloat* lv3 =  Data_bigarray_val(v3);

	if(stub_glUniformMatrix2x4fv){(*stub_glUniformMatrix2x4fv)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glUniformMatrix2x4fv) { 
			loaded_glUniformMatrix2x4fv = 1;
			init_lib(); 
			stub_glUniformMatrix2x4fv=(pstub_glUniformMatrix2x4fv)get_proc_address("glUniformMatrix2x4fv");
			if(stub_glUniformMatrix2x4fv){(*stub_glUniformMatrix2x4fv)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glUniformMatrix2x4fv"); 
			} 
		} else { 
			FailWith("glUniformMatrix2x4fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniformMatrix3x2fv */
typedef void APIENTRY (*pstub_glUniformMatrix3x2fv)(GLint,GLsizei,GLboolean,GLfloat*);
static pstub_glUniformMatrix3x2fv stub_glUniformMatrix3x2fv = NULL;
static int loaded_glUniformMatrix3x2fv = 0;

value glstub_glUniformMatrix3x2fv(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLboolean lv2 =  Bool_val(v2);
	GLfloat* lv3 =  Data_bigarray_val(v3);

	if(stub_glUniformMatrix3x2fv){(*stub_glUniformMatrix3x2fv)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glUniformMatrix3x2fv) { 
			loaded_glUniformMatrix3x2fv = 1;
			init_lib(); 
			stub_glUniformMatrix3x2fv=(pstub_glUniformMatrix3x2fv)get_proc_address("glUniformMatrix3x2fv");
			if(stub_glUniformMatrix3x2fv){(*stub_glUniformMatrix3x2fv)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glUniformMatrix3x2fv"); 
			} 
		} else { 
			FailWith("glUniformMatrix3x2fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniformMatrix3x4fv */
typedef void APIENTRY (*pstub_glUniformMatrix3x4fv)(GLint,GLsizei,GLboolean,GLfloat*);
static pstub_glUniformMatrix3x4fv stub_glUniformMatrix3x4fv = NULL;
static int loaded_glUniformMatrix3x4fv = 0;

value glstub_glUniformMatrix3x4fv(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLboolean lv2 =  Bool_val(v2);
	GLfloat* lv3 =  Data_bigarray_val(v3);

	if(stub_glUniformMatrix3x4fv){(*stub_glUniformMatrix3x4fv)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glUniformMatrix3x4fv) { 
			loaded_glUniformMatrix3x4fv = 1;
			init_lib(); 
			stub_glUniformMatrix3x4fv=(pstub_glUniformMatrix3x4fv)get_proc_address("glUniformMatrix3x4fv");
			if(stub_glUniformMatrix3x4fv){(*stub_glUniformMatrix3x4fv)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glUniformMatrix3x4fv"); 
			} 
		} else { 
			FailWith("glUniformMatrix3x4fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniformMatrix4x2fv */
typedef void APIENTRY (*pstub_glUniformMatrix4x2fv)(GLint,GLsizei,GLboolean,GLfloat*);
static pstub_glUniformMatrix4x2fv stub_glUniformMatrix4x2fv = NULL;
static int loaded_glUniformMatrix4x2fv = 0;

value glstub_glUniformMatrix4x2fv(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLboolean lv2 =  Bool_val(v2);
	GLfloat* lv3 =  Data_bigarray_val(v3);

	if(stub_glUniformMatrix4x2fv){(*stub_glUniformMatrix4x2fv)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glUniformMatrix4x2fv) { 
			loaded_glUniformMatrix4x2fv = 1;
			init_lib(); 
			stub_glUniformMatrix4x2fv=(pstub_glUniformMatrix4x2fv)get_proc_address("glUniformMatrix4x2fv");
			if(stub_glUniformMatrix4x2fv){(*stub_glUniformMatrix4x2fv)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glUniformMatrix4x2fv"); 
			} 
		} else { 
			FailWith("glUniformMatrix4x2fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}

/* glUniformMatrix4x3fv */
typedef void APIENTRY (*pstub_glUniformMatrix4x3fv)(GLint,GLsizei,GLboolean,GLfloat*);
static pstub_glUniformMatrix4x3fv stub_glUniformMatrix4x3fv = NULL;
static int loaded_glUniformMatrix4x3fv = 0;

value glstub_glUniformMatrix4x3fv(value v0, value v1, value v2, value v3)
{
	CAMLparam4(v0,v1,v2,v3);
	CAMLlocal1(result);
	GLint lv0 =  Int_val(v0);
	GLsizei lv1 =  Int_val(v1);
	GLboolean lv2 =  Bool_val(v2);
	GLfloat* lv3 =  Data_bigarray_val(v3);

	if(stub_glUniformMatrix4x3fv){(*stub_glUniformMatrix4x3fv)(lv0, lv1, lv2, lv3);}
	else { 
		if(!loaded_glUniformMatrix4x3fv) { 
			loaded_glUniformMatrix4x3fv = 1;
			init_lib(); 
			stub_glUniformMatrix4x3fv=(pstub_glUniformMatrix4x3fv)get_proc_address("glUniformMatrix4x3fv");
			if(stub_glUniformMatrix4x3fv){(*stub_glUniformMatrix4x3fv)(lv0, lv1, lv2, lv3);}
			else { 
				FailWith("Cannot load glUniformMatrix4x3fv"); 
			} 
		} else { 
			FailWith("glUniformMatrix4x3fv not available"); 
		}
	}
	result = Val_unit;
	CAMLreturn (result);
}
